/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGPREFERENCESMANAGER_H
#define NMGPREFERENCESMANAGER_H

#include "nmgconfigurationfilemanager.h"
#include "nmgapplicationinformation.h"


#define PREFERENCES_FILE_NAME	"preferences.nmgui.xml"

#define TAG_PREFERENCES	"preferences"
#define TAG_PREFERENCE	"preference"
#define ATTR_PREF_KEY	"key"
#define ATTR_PREF_VALUE	"value"
#define ID_LOAD_ALL_MODULES	"LoadAllModules"
#define ID_LOAD_ALL_MODULES_TRUE	"True"
#define ID_LOAD_ALL_MODULES_FALSE	"False"
#define ID_MODULES_TO_LOAD	"ModulesToLoad"


#define NMG_IMAGES_PATH		"nmg_images_path"

#define KEY_CONF_DIR	"configuration_directory"
#define KEY_XML_DIR	"xml_plugin_directory"
#define KEY_RESULT_DIR  "result_directory"

#define NETMETER_SERVERPORT	"netmeter_sevrver_port"

#include <qobject.h>
#include <qmap.h>
#include <qfiledialog.h>

typedef QMap<QString,QString> NMGPreferencesManagerMap;
typedef NMGPreferencesManagerMap::iterator NMGPreferencesManagerMapIterator;


#define TAB_GENERAL_PREFERENCES		0
#define TAB_MODULE_PREFERENCES		1
#define TAB_CONNECTION_PREFERENCES	2


class NMGPreferencesBaseWidget;


class NMGPreferencesManager : public QObject
{
Q_OBJECT
private:
    NMGPreferencesManager(QObject *parent = 0, const char *name = 0);

public:
    ~NMGPreferencesManager();
    
    static NMGPreferencesManager * self();
    
    void addValue(QString key, QString value);
    
    QString getValue(QString key);
    
    void removeValue(QString key);
    
    bool existsValue(QString key);
    
    QString getConfigurationDirectory();
    QString getResultDirectory();
    QString getXMLDirectory();
    
    int getNetMeterPort();
    

public slots:    
    void showGeneralPreferencesWidget();
    void showModulePreferencesWidget();
    void showConnectionPreferencesWidget();
    void setNetMeterPort(int port);
    void setConfigurationDirectory(QString dir);
    void setXMLDirectory(QString dir);
    void setResultDirectory(QString dir);
    void selectImageDirectoryClicked();
    void selectResultDirectoryClicked();
    void selectLogFilePath();

private slots:
    void updateWidgetsFromValues();
    void updateDiskFromWidget();
    void showModulesToLoadWidget();
    
private:
    void loadPreferencesFromDisk();
    void updateWidgetsFromAddedValue(QString key, QString value);
    void createResultDirectory(QString dirPath);
    
    static NMGPreferencesManager * instance;
    NMGPreferencesBaseWidget * widget;
    NMGPreferencesManagerMap preferencesMap;
    NMGXMLConfigurationFile * preferencesFile;
    bool mapLoadedFromDisk;

};

#endif
