/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGHOSTINFORMATIONWIDGET_H
#define NMGHOSTINFORMATIONWIDGET_H

#include "../nmgstation.h"



#include <qvariant.h>
#include <qwidget.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QGroupBox;
class QComboBox;
class QLabel;
class QToolBox;
class QLineEdit;

class NMGHostInformationMainWidget;

typedef enum {NMGHostInfoTestAddress=0, NMGHostInfoTestAlias=1, NMGHostInfoTestProtocol=2, NMGHostInfoManagementAddress=3,
	NMGHostInfoManagementUsername=4, NMGHostInfoManagementPassword=5, NMGHostInfoManagementProtocol=6, 
	NMGHostInfoManagementAddressPort=7} NMGHostInfoBaseWidgetFocus;

	
/**
@author Ernest Nieto
*/
class NMGHostInformationWidget : public QWidget
{
Q_OBJECT
public:
    NMGHostInformationWidget(QWidget *parent = 0, const char *name = 0, WFlags fl = 0 );

    ~NMGHostInformationWidget();
    
    QString getCurrentModule();
    
    void setCurrentModule(QString mod);
    
    void updateCurrentModuleView();
    
    void addTestHostAddressToList(QString hostAddress);
    
    void addTestHostAliasToList(QString hostAlias);
    
    QString getTestHostAddress();
    
    QString getTestHostAlias();
    
    bool getTestIsIPv4();
    
    QString getManagementHostAddress();
    
    QString getManagementHostAlias();
    
    bool getManagementIsIPv4();
    
    QString getManagementUsername();
    
    QString getManagementPassword();

    int getManagementAddressPort();
    
    void setManagementAddressPort(int port);
   
    void setTestHostAddress(QString testAddr);
    
    void setTestHostAlias(QString testAlias);
    
    void setTestIsIPv4(bool testIPv4);
    
    void setManagementHostAddress(QString managAddr);
    
    void setManagementHostAlias(QString managAlias);
    
    void setManagementIsIPv4(bool managIPv4);
    
    void setManagementUsername(QString user);
    
    void setManagementPassword(QString pass);
    
    NMGStation * getStation();
    
    void setWidgetFocus(NMGHostInfoBaseWidgetFocus focus);
    
    void setFieldFocus(const QString & field);

    
signals:
    void widgetHidden();
    
public slots:
    void addHostField(QString module, QString fieldName, QString field);
    
    void buttonOkClicked();
    
    void buttonCancelClicked();
    
    void comboAddressAliasActivated(const QString & string);
    
    void updateWidgetWithStation(NMGStation * station);

protected:
    void keyReleaseEvent(QKeyEvent *e);

private:
    void createModuleTabs();
    
    void showEvent(QShowEvent *);
    
    void hideEvent(QHideEvent *);
    
    int indexModule(QString name);
    
    QLineEdit * getLineEdit(QString name);
    
    void updateWidgetWithStationFields(NMGStation * s=0);
   
private:
    NMGHostInformationMainWidget * mainWidget;
    
    QString currentModule;
    
    NMGStation * station;
    
};
#endif
