/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgmainoptionswidget.h"

#include "../nmghostinformationmanager.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

#include <iostream>
using namespace std;

static const unsigned char img0_NMGuiOptions[] = { 
    0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
    0x49, 0x48, 0x44, 0x52, 0x00, 0x00, 0x00, 0x0f, 0x00, 0x00, 0x00, 0x0f,
    0x08, 0x06, 0x00, 0x00, 0x00, 0x3b, 0xd6, 0x95, 0x4a, 0x00, 0x00, 0x01,
    0xa0, 0x49, 0x44, 0x41, 0x54, 0x28, 0x91, 0x9d, 0x92, 0x31, 0x48, 0x5b,
    0x51, 0x14, 0x86, 0xbf, 0x17, 0x1d, 0x04, 0x71, 0x09, 0x91, 0x47, 0x4b,
    0x8a, 0x08, 0x41, 0x87, 0xd8, 0x82, 0x50, 0x82, 0xcd, 0xd4, 0x82, 0x25,
    0xe0, 0x52, 0xdc, 0x4a, 0x1d, 0x52, 0x70, 0x88, 0x6b, 0xa7, 0xea, 0x24,
    0x4e, 0x8d, 0x4b, 0xba, 0xd6, 0xad, 0x20, 0x85, 0x4e, 0x96, 0x62, 0x54,
    0xc2, 0x13, 0x5e, 0xa7, 0x2a, 0xb1, 0x20, 0x54, 0x4b, 0xb0, 0x79, 0x58,
    0xa5, 0xa5, 0xe4, 0x61, 0xf3, 0x1c, 0x44, 0x70, 0x91, 0xbf, 0x83, 0xbe,
    0xdb, 0x24, 0xda, 0x40, 0x7b, 0xe0, 0x70, 0x39, 0xe7, 0xde, 0x7b, 0xee,
    0x7f, 0xbe, 0x73, 0x2d, 0x49, 0xfc, 0xaf, 0x75, 0xb6, 0xc4, 0xf2, 0x02,
    0x9f, 0xaf, 0x27, 0xdf, 0xd8, 0x3b, 0x3a, 0xe3, 0xcb, 0xaf, 0x1f, 0x24,
    0x63, 0x71, 0x06, 0x7b, 0xbb, 0x18, 0xe8, 0xe9, 0x27, 0x11, 0xb5, 0x01,
    0xac, 0xf0, 0xb0, 0x15, 0xbe, 0xec, 0x05, 0xbe, 0xf2, 0x65, 0x87, 0x9d,
    0xf3, 0x75, 0x7e, 0x7e, 0x7e, 0xd4, 0x54, 0xf1, 0xe6, 0x9d, 0xf7, 0x00,
    0x3c, 0xb6, 0x9f, 0xf2, 0xec, 0xee, 0x7d, 0x53, 0xc0, 0x92, 0x84, 0x17,
    0xf8, 0x9a, 0xd8, 0x78, 0xce, 0xec, 0xd0, 0x14, 0x00, 0x7b, 0x47, 0x67,
    0x0c, 0xf6, 0x76, 0xb1, 0x54, 0xf1, 0x00, 0x28, 0x6d, 0x77, 0xe3, 0xe6,
    0xd2, 0x2c, 0xef, 0x57, 0x78, 0xeb, 0xbf, 0xe6, 0xcd, 0xbd, 0x79, 0x12,
    0x51, 0xdb, 0xa2, 0x5a, 0xaf, 0x29, 0x55, 0xcc, 0x6a, 0xe5, 0x60, 0x43,
    0x92, 0x34, 0xb9, 0xb6, 0xa8, 0x6a, 0xbd, 0xa6, 0x30, 0x9f, 0x2a, 0x66,
    0x35, 0xb9, 0xb6, 0xa8, 0xc2, 0x96, 0x2b, 0x49, 0x2a, 0x6c, 0xb9, 0x4a,
    0x15, 0xb3, 0x92, 0xa4, 0xc8, 0xf2, 0x7e, 0x05, 0x80, 0xb1, 0xbe, 0x11,
    0x23, 0x33, 0x5f, 0x76, 0xc8, 0x97, 0x1d, 0x13, 0x27, 0x63, 0x71, 0xd3,
    0xe6, 0xa5, 0x6c, 0x56, 0x0f, 0x37, 0x89, 0x34, 0x06, 0x61, 0x2f, 0xc9,
    0x58, 0x9c, 0x9d, 0xf3, 0xf5, 0x6b, 0x09, 0xbf, 0xfc, 0xf4, 0x01, 0x80,
    0x81, 0x9e, 0xfe, 0xab, 0x3d, 0x37, 0x52, 0x0e, 0xd7, 0x82, 0x73, 0xcc,
    0xc2, 0x93, 0x1b, 0x00, 0xcc, 0xed, 0xbe, 0x62, 0x76, 0x68, 0x8a, 0xb1,
    0xbe, 0x91, 0xf6, 0xb4, 0x33, 0xc3, 0xa7, 0x94, 0xb6, 0xbb, 0x0d, 0xf1,
    0xdb, 0x1d, 0xa3, 0x4c, 0xa7, 0x1e, 0x9a, 0x91, 0x59, 0x2d, 0x9f, 0x44,
    0xab, 0x87, 0x9b, 0x2c, 0x55, 0xbc, 0x26, 0xd9, 0x61, 0x31, 0x37, 0x97,
    0xbe, 0xa0, 0xdc, 0x3a, 0xe7, 0x16, 0x6b, 0x4a, 0x7a, 0x81, 0xcf, 0x83,
    0x85, 0x8f, 0x00, 0x7c, 0x9f, 0x19, 0x37, 0x6c, 0x22, 0xd7, 0x52, 0xb9,
    0xd8, 0x34, 0x9e, 0x88, 0xda, 0x96, 0x9b, 0x4b, 0x37, 0x02, 0x53, 0xbb,
    0xcb, 0x57, 0x2c, 0x11, 0xb5, 0xc9, 0x0c, 0x9f, 0x52, 0x70, 0x8e, 0xf1,
    0x02, 0xbf, 0xad, 0xec, 0xbf, 0x99, 0x6e, 0xbd, 0x78, 0xf7, 0x47, 0xbe,
    0xa4, 0x7f, 0xf2, 0x6a, 0xbd, 0x66, 0x7e, 0xdb, 0x6f, 0xf6, 0x9b, 0x00,
    0x99, 0x0c, 0x9a, 0x8c, 0x10, 0x00, 0x00, 0x00, 0x00, 0x49, 0x45, 0x4e,
    0x44, 0xae, 0x42, 0x60, 0x82
};


/**
 *  Constructs a NMGOptionsWidget as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 */
NMGMainOptionsWidget::NMGMainOptionsWidget( QWidget* parent, const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    if ( !name )
	setName( "NMGOptionsWidget" );
	createWidgets();
	connectWidgets();
}

/**
 *  Destroys the object and frees any allocated resources
 */
NMGMainOptionsWidget::~NMGMainOptionsWidget()
{
    // no need to delete child widgets, Qt does it all for us
}

/**
 * Creates all the widgets of NMGMainOptionsWidget
 */
void NMGMainOptionsWidget::createWidgets()
{
    QImage img;
    img.loadFromData( img0_NMGuiOptions, sizeof( img0_NMGuiOptions ), "PNG" );
    image0 = img;
    
    NMGOptionsWidgetLayout = new QGridLayout( this, 1, 1, 11, 6, "NMGOptionsWidgetLayout"); 

      
    groupOptions = new QGroupBox( this, "groupOptions" );
    groupOptions->setTitle("Options");
    groupOptions->setColumnLayout(0, Qt::Vertical );
    groupOptions->layout()->setSpacing( 6 );
    groupOptions->layout()->setMargin( 11 );
    groupOptionsLayout = new QGridLayout( groupOptions->layout() );
    groupOptionsLayout->setAlignment( Qt::AlignTop );

    lblProtocol = new QLabel( groupOptions, "lblProtocol" );
    lblProtocol->setText("Protocol" );

    groupOptionsLayout->addWidget( lblProtocol, 0, 0 );

    btnReceiver = new QPushButton( groupOptions, "btnSender" );
    btnReceiver->setText( QString::null );
    btnReceiver->setMaximumSize( QSize( 32, 32767));
    btnReceiver->setPixmap( image0 );

    groupOptionsLayout->addWidget( btnReceiver, 3, 2 );

    comboProtocol = new QComboBox( FALSE, groupOptions, "comboProtocol" );
    comboProtocol->clear();
    comboProtocol->insertItem("IPv4");
    comboProtocol->insertItem("IPv6");

    groupOptionsLayout->addWidget( comboProtocol, 0, 1 );

    lblAdressType = new QLabel( groupOptions, "lblAdressType" );
    lblAdressType->setText("Adress type");
    
 
    lblAdressType->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)5, 0, 0, lblAdressType->sizePolicy().hasHeightForWidth()));
    lblAdressType->setMaximumSize( QSize( 32767, 32767));
    lblAdressType->setScaledContents( FALSE );

    groupOptionsLayout->addWidget( lblAdressType, 1, 0 );

    comboSender = new QComboBox( FALSE, groupOptions, "comboSender" );
    comboSender->setEditable( TRUE );
    comboSender->clear();
    comboSender->insertItem("localhost");
    comboSender->setCurrentText("");
    
    groupOptionsLayout->addWidget( comboSender, 2, 1 );

    lblSender = new QLabel( groupOptions, "lblSender" );
    lblSender->setText("Sender");

    groupOptionsLayout->addWidget( lblSender, 2, 0 );

    btnSender = new QPushButton( groupOptions, "btnReceiver" );
    btnSender->setText( QString::null );
    btnSender->setMaximumSize( QSize( 32, 32767));
    btnSender->setPixmap( image0 );

    groupOptionsLayout->addWidget( btnSender, 2, 2 );

    comboReceiver = new QComboBox( FALSE, groupOptions, "comboReceiver" );
    comboReceiver->setEditable( TRUE );
    comboReceiver->clear();
    comboReceiver->insertItem("localhost");
    comboReceiver->setCurrentText("");

    groupOptionsLayout->addWidget( comboReceiver, 3, 1 );

    comboAdressType = new QComboBox( FALSE, groupOptions, "comboAdressType" );
    comboAdressType->clear();
    comboAdressType->insertItem("Unicast");
    comboAdressType->insertItem("Multicast");

    groupOptionsLayout->addWidget( comboAdressType, 1, 1 );

    lblReceiver = new QLabel( groupOptions, "lblReceiver" );
    lblReceiver->setText("Receiver");

    groupOptionsLayout->addWidget( lblReceiver, 3, 0 );

    NMGOptionsWidgetLayout->addWidget( groupOptions, 0, 0 );
    spacerH = new QSpacerItem( 20, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
    NMGOptionsWidgetLayout->addItem( spacerH, 0, 1 );

    groupConfiguration = new QGroupBox( this, "groupConfiguration" );
    groupConfiguration->setTitle("Configuration");
    groupConfiguration->setColumnLayout(0, Qt::Vertical );
    groupConfiguration->layout()->setSpacing( 6 );
    groupConfiguration->layout()->setMargin( 11 );
    groupConfigurationLayout = new QVBoxLayout( groupConfiguration->layout() );
    groupConfigurationLayout->setAlignment( Qt::AlignTop );

    layoutNameFile = new QGridLayout( 0, 1, 1, 0, 6, "layoutNameFile"); 

    lblConfigurationFile = new QLabel( groupConfiguration, "lblConfigurationFile" );
    lblConfigurationFile->setText("netmeter.cfg");

    layoutNameFile->addWidget( lblConfigurationFile, 1, 1 );

    lblConfigurationName = new QLabel( groupConfiguration, "lblConfigurationName" );
    lblConfigurationName->setText("Name");

    layoutNameFile->addWidget( lblConfigurationName, 0, 0 );

    lblConfigurationFileLabel = new QLabel( groupConfiguration, "lblConfigurationFileLabel" );
    lblConfigurationFileLabel->setText("File");

    layoutNameFile->addWidget( lblConfigurationFileLabel, 1, 0 );

    txtConfigurationName = new QLineEdit( groupConfiguration, "txtConfigurationName" );
    txtConfigurationName->setText("Netmeter IPV4/IPv6 Scenario");

    layoutNameFile->addWidget( txtConfigurationName, 0, 1 );
    groupConfigurationLayout->addLayout( layoutNameFile );

    layoutButtonsFile = new QHBoxLayout( 0, 0, 6, "layoutButtonsFile"); 

    btnConfigurationLoad = new QPushButton( groupConfiguration, "btnConfigurationLoad" );
    btnConfigurationLoad->setText("Load");
    layoutButtonsFile->addWidget( btnConfigurationLoad );

    btnConfigurationSave = new QPushButton( groupConfiguration, "btnConfigurationSave" );
    btnConfigurationSave->setText("Save");
    layoutButtonsFile->addWidget( btnConfigurationSave );

    btnConfigurationSaveAs = new QPushButton( groupConfiguration, "btnConfigurationSaveAs" );
    btnConfigurationSaveAs->setText("Save As");
    layoutButtonsFile->addWidget( btnConfigurationSaveAs );
    groupConfigurationLayout->addLayout( layoutButtonsFile );

    NMGOptionsWidgetLayout->addWidget( groupConfiguration, 1, 0 );
    spacerV = new QSpacerItem( 21, 16, QSizePolicy::Minimum, QSizePolicy::Expanding );
    NMGOptionsWidgetLayout->addItem( spacerV, 2, 0 );
    resize( QSize(314, 328).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );
}

/**
 * Connects the widgets
 */
void NMGMainOptionsWidget::connectWidgets()
{
	connect(comboSender,SIGNAL(activated(int)),this,SLOT(senderActivated(int )));
	connect(comboReceiver,SIGNAL(activated(int)),this,SLOT(receiverActivated(int )));
	connect(btnReceiver,SIGNAL(clicked()),this,SLOT(receiverButtonClicked()));
	connect(btnSender,SIGNAL(clicked()),this,SLOT(senderButtonClicked()));
	connect(this,SIGNAL(showHostInformation(QString)),NMGHostInformationManager::self(),SLOT(showHostWidget(QString)));
}

/**
 * Slot executed when the sender button is clicked
 */
void NMGMainOptionsWidget::senderButtonClicked()
{
	emit showHostInformation(comboSender->currentText());	
}

/**
 * Slot executed when the receiver button is clicked
 */
void NMGMainOptionsWidget::receiverButtonClicked()
{
	emit showHostInformation(comboReceiver->currentText());	
}

/**
 * Slot executed when an item of the sender combo box is selected
 * @param index 
 */
void NMGMainOptionsWidget::senderActivated(int index)
{
	cout << "Sender: " << index << endl;
}
    
/**
 * Slot executed when an item of the receiver combo box is selected
 * @param index 
 */
void NMGMainOptionsWidget::receiverActivated(int index)
{
	cout << "Receiver: " << index << endl;
}

#include "nmgmainoptionswidget.moc"
