/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgmaintabwidget.h"

#include "nmgmaintabbar.h"

#include "../nmgimagemanager.h"

#include <qscrollview.h>
#include <qobjectlist.h>
#include <qlayout.h>

#include <iostream>
using namespace std;


/**
 * Contructor of the class
 * @param parent QT's parent parameter
 * @param name QT's name parameter
 * @return 
 */
NMGMainTabWidget::NMGMainTabWidget(QWidget *parent, const char *name)
 : QTabWidget(parent, name)
{
	setSizePolicy(QSizePolicy(QSizePolicy::Maximum,QSizePolicy::Maximum));	
	NMGMainTabBar * tabBar = new NMGMainTabBar(this,"tabBar");
	connect(tabBar,SIGNAL(deleteCurrentTab()),this,SLOT(deleteCurrentTab()));
	connect(tabBar,SIGNAL(deleteAllTabs()),this,SLOT(deleteAllTabs()));
	connect(tabBar,SIGNAL(deleteOthersTabs()),this,SLOT(deleteOtherTabs()));
	setTabBar(tabBar);
}

/**
 * Destructor of the class
 * @return 
 */
NMGMainTabWidget::~NMGMainTabWidget()
{
}

/**
 * Adds a tab to the tabWidget
 * @param child 
 * @param label 
 */
void NMGMainTabWidget::addTab(QWidget * child, const QString & label,const QString & icon)
{
	QScrollView * scrollView = new QScrollView(this);
	
	QPoint p;
	
	child->reparent(scrollView->viewport(),p);
	QGridLayout * layout = new QGridLayout(scrollView->viewport(),1,1,0,0);
	layout->addWidget(child,0,0);
	
	scrollView->setResizePolicy( QScrollView::AutoOneFit );
	scrollView->addChild(child);
	scrollView->viewport()->setBackgroundColor(backgroundColor());
	if(icon and icon!="") QTabWidget::addTab(scrollView, QIconSet(*NMGImageManager::loadPixmap(icon)),label);
	else QTabWidget::addTab(scrollView,label);
	setCurrentPage(indexOf(scrollView));
	scrollView->show();
}

/**
 * Returns the currenmt page widget of a module
 * @return 
 */
QWidget * NMGMainTabWidget::getCurrentPageWidget()
{
	QWidget * current = currentPage();
	if(!current) return 0;
	
	if(QString(currentPage()->className())=="QScrollView")
	{
		const QObjectList * list = ((QScrollView*) current)->viewport()->children();
		if(list->count()>2) cerr << "[panic] More than two childs at the scroll view ("<<list->count()<<")" << endl;
		QObjectListIt it( *list );
    		QObject *obj;
		while ( (obj = it.current()) != 0 )
			if(QString(obj->className())!="QGridLayout") return (QWidget * )*it;
	}
	cerr << "[panic] No Scroll View child at QTabWidget" << endl;
	return 0;
}

/**
 * Returns the widget of a module  at the i page
 * @param i Number of the page (first is 0)
 * @return 
 */
QWidget * NMGMainTabWidget::getPageWidget(int i)
{
	QWidget * current = page(i);
	if(!current) return 0;
	
	if(QString(currentPage()->className())=="QScrollView")
	{
		const QObjectList * list = ((QScrollView*) current)->viewport()->children();
		if(list->count()>2) cerr << "[panic] More than two childs at the scroll view ("<<list->count()<<")" << endl;
		QObjectListIt it( *list );
    		QObject *obj;
		while ( (obj = it.current()) != 0 )
			if(QString(obj->className())!="QGridLayout") return (QWidget * )*it;
	}
	cerr << "[panic] No Scroll View child at QTabWidget" << endl;
	return 0;
}

/**
 * Removes and deletes the current page widget
 */
void NMGMainTabWidget::deleteCurrentPage()
{
	QWidget * currentScroll = currentPage();
	removePage(currentScroll);
	delete currentScroll;
}

/**
 * Deletes and removes the page i
 * @param i Number of the page (first is 0)
 */
void NMGMainTabWidget::deletePage(int i)
{
	QWidget * scroll = page(i);
	removePage(scroll);
	delete scroll;	
}

/**
 * Slot executed when the user wants to close the current tab
 */
void NMGMainTabWidget::deleteCurrentTab()
{
	emit closeCurrentTab();
}
    
/**
 * Slot executed when the user wants to close all the tabs
 */
void NMGMainTabWidget::deleteAllTabs()
{
	emit closeAllTabs();
}

/**
 * Slot executed when the user wants to close all the tabs but not hte current
 */
void NMGMainTabWidget::deleteOtherTabs()
{
	emit closeOtherTabs();
}

/**
 * Sets the image of the i tab
 * @param i Index of the tab
 * @param image Path of the image
 */
void NMGMainTabWidget::setImage(int i, const QString & image)
{
	QTab * tab = tabBar()->tabAt(i);
	if(tab) tab->setIconSet(QIconSet(*NMGImageManager::loadPixmap(image)));
	tabBar()->repaint();
}

/**
 * Removes the image of the i tab
 * @param i Index of the tab
 */
void NMGMainTabWidget::removeImage(int i)
{
	QTab * tab = tabBar()->tabAt(i);
	if(tab)
	{
cout << "TODO: NMGMainTabWidget::removeImage" << endl;
	}
	tabBar()->repaint();
}

/**
 * Returns the index of the tab of the widget
 * @param w Widget inside the scroll view
 * @return Index of the tab or -1 if there is no such tab
 */
int NMGMainTabWidget::getIndexFromWidgetInsideScrollView(QWidget *w)
{
	for(int i=0; i<count(); i++)
		if(getPageWidget(i)==w) return i;
	
	return -1;
}


#include "nmgmaintabwidget.moc"
