/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgmainwindowmenubar.h"

#include "nmgmainwindow.h"
#include "../nmgpreferencesmanager.h"
#include "nmgaboutwidget.h"

#include <qobjectlist.h>
#include <qmap.h>
#include <qaccel.h>

#define CAPABILITIES_POPUPMENU_SUFFIX	"CapabilityMenu"

#include  <iostream>
using namespace std;

/**
 * Contructor of the class
 * @param parent 
 * @param name 
 * @return 
 */
NMGMainWindowMenuBar::NMGMainWindowMenuBar(NMGMainWindow *parent, const char *name)
 : QMenuBar((QMainWindow *)parent, name)
{
	idMenuConnectToServer = 0;
	idMenuDisconnectFromServer = 0;

	fileMenu = new QPopupMenu(this);
	insertItem("&File",fileMenu,1);
/*	fileMenu->insertItem("&Nothing");
	fileMenu->insertSeparator();    */
	idMenuConnectToServer  = fileMenu->insertItem("&Connect to server",parent,SLOT(connectToServer()));
	idMenuDisconnectFromServer = fileMenu->insertItem("&Disconnect from server",parent,SLOT(disconnectFromServer()));
	fileMenu->setItemEnabled(idMenuDisconnectFromServer,false);	
	

	fileMenu->insertItem("&Quit",parent,SLOT(quitApplication()),CTRL+Key_Q);
	
	createMenuPlugins(parent);
	
	settingsMenu = new QPopupMenu(this);
	insertItem("&Settings",settingsMenu,3);
	settingsMenu->insertItem("&General preferences",NMGPreferencesManager::self(),SLOT(showGeneralPreferencesWidget()));
	settingsMenu->insertItem("&Plugins preferences",NMGPreferencesManager::self(),SLOT(showModulePreferencesWidget()));
	
	aboutMenu = new QPopupMenu(this);
	insertItem("&Help",aboutMenu,4);
	aboutMenu->insertItem("&About Netmeter",this,SLOT(showAboutWidget()));	
}


/**
 * Destroyer of the class
 * @return 
 */
NMGMainWindowMenuBar::~NMGMainWindowMenuBar()
{
}

/**
 * Sets if the application is connected to the server
 * @param connected 
 */
void NMGMainWindowMenuBar::setConnected(bool connected)
{
	if(connected)
	{
		fileMenu->setItemEnabled(idMenuConnectToServer,false);
		fileMenu->setItemEnabled(idMenuDisconnectFromServer,true);	
	}
	else
	{
		fileMenu->setItemEnabled(idMenuConnectToServer,true);
		fileMenu->setItemEnabled(idMenuDisconnectFromServer,false);
	}
}

/**
 * Creates the plugins menu adding al the modules loaded 
 */
void NMGMainWindowMenuBar::createMenuPlugins(NMGMainWindow *parent)
{
	pluginsMenu = new QPopupMenu(this);
	insertItem("&Plugins",pluginsMenu,2);
	pluginsMenu->insertItem("&Load new module",parent->getModuleToolBox(),SLOT(menuLoadModule()));
	pluginsMenu->insertItem("Close &current tab",this,SIGNAL(closeCurrentModuleTab()),CTRL+Key_W);
	pluginsMenu->insertItem("Close &all tabs",this,SIGNAL(closeAllModuleTabs()));
	pluginsMenu->insertItem("Close &other tabs",this,SIGNAL(closeAllModuleTabsButCurrent()));
	pluginsMenu->insertSeparator();
	
	pluginsSignalMapper = new QSignalMapper(this);
	connect(pluginsSignalMapper, SIGNAL( mapped(const QString & )), parent, SLOT(createNewInstanceModule(const QString &)));

	QMap<QString,QPopupMenu *> capabilitiesMap; /* key->capability */
	
	QValueList<NMModule *> l = NMGModuleManager::self()->getModuleList();
	for(QValueList<NMModule *>::iterator it = l.begin(); it!=l.end(); ++it)
	{
		NMGModuleParent * mod = (NMGModuleParent *) *it;
		QString moduleName = mod->getPluginNameGUI().c_str();
		QString moduleCapabilities = transformCapabilities(mod->getCapabilities().c_str());
		QMap<QString,QPopupMenu *>::iterator it = capabilitiesMap.find(moduleCapabilities);
		QPopupMenu * menu;
		if(it==capabilitiesMap.end())
		{
			menu = new QPopupMenu(pluginsMenu,moduleCapabilities+CAPABILITIES_POPUPMENU_SUFFIX); 
			pluginsMenu->insertItem(moduleCapabilities,menu);
			capabilitiesMap.insert(moduleCapabilities,menu);
		}
		else menu = *it;
		
		QAction * pluginAction = new QAction(moduleName, 0, this);
		pluginsSignalMapper->setMapping(pluginAction,moduleName);
		connect(pluginAction, SIGNAL(activated()), pluginsSignalMapper, SLOT(map()));
		pluginAction->addTo(menu);
	}
}

/**
 * Adds a module to the menu
 * @param moduleName 
 * @param capabilities 
 */
void NMGMainWindowMenuBar::addNewModuleAddedToMenu(QString moduleName, QString capabilities)
{
	capabilities = transformCapabilities(capabilities);
	
	QPopupMenu * capabilityMenu = (QPopupMenu *) child(capabilities+CAPABILITIES_POPUPMENU_SUFFIX,"QPopupMenu");
	if(!capabilityMenu)
	{
		capabilityMenu = new QPopupMenu(pluginsMenu,capabilities+CAPABILITIES_POPUPMENU_SUFFIX); 
		pluginsMenu->insertItem(capabilities,capabilityMenu);	
	}
	
	QAction * pluginAction = new QAction(moduleName, 0, this);
	pluginsSignalMapper->setMapping(pluginAction,moduleName);
	connect(pluginAction, SIGNAL(activated()), pluginsSignalMapper, SLOT(map()));
	pluginAction->addTo(capabilityMenu);
}


/**
 * Deletes the specified module from the plugin menu
 * @param moduleName 
 */
void NMGMainWindowMenuBar::deleteModuleFromPluginMenu(QString moduleName)
{
	QObjectList *l = pluginsMenu->queryList( "QPopupMenu" );
	QObjectListIt it( *l );
	QObject *obj;

	while ( (obj = it.current()) != 0 ) 
	{
		QPopupMenu * menu = (QPopupMenu *) obj;
		for(uint i = 0; i<menu->count(); ++i)
		{
			if(menu->text(menu->idAt(i))==moduleName)
			{
				menu->removeItemAt(i);
				if(menu->count()==0) delete menu;
				return; 
			}
		}
		++it;
	}
	delete l; // delete the list, not the objects
}

/**
 * Block/Unblocks the signals of the Qaccel childs of the menu
 * @param menu Menu to block its QAccel childs
 * @param blockSignals If true the signals are blocked, if false, the signals are unblocked
 */
void NMGMainWindowMenuBar::blockQAccelSignalsOfMenu(QPopupMenu * menu, bool blockSignals)
{
	if(!menu) return;
	const QObjectList * objList = menu->queryList("QAccel");
	if(!objList) return;
	else
	{
		QObjectListIt it(*objList);
		QObject * obj;
		while((obj = (QObject *) it.current())!=0)
		{
			QAccel * accel = (QAccel *) obj;
			accel->blockSignals(blockSignals);
			accel->setEnabled(not blockSignals);
			++it;
		}
	}
}

/**
 * Transform the string capabilites to be shown correctly to the user
 * @param capabilities 
 * @return 
 */
QString NMGMainWindowMenuBar::transformCapabilities(QString capabilities)
{
	return capabilities.lower().replace(0,1,capabilities.left(1).upper());
}

/**
 * Display About to Widget
 */
void NMGMainWindowMenuBar::showAboutWidget()
{
	NMGAboutWidget * aboutWidget;
	aboutWidget = new NMGAboutWidget(0,"NMGAboutWidget");
	aboutWidget->show();
}


#include "nmgmainwindowmenubar.moc"
