/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgmodulelistviewwidget.h"

#include <qfiledialog.h>

#include <iostream>
using namespace std;

/**
 * Constructor of the class
 * @param parent QT's parent parameter
 * @param name QT's name paremeter
 * @return 
 */
NMGModuleListViewWidget::NMGModuleListViewWidget(QWidget *parent, const char *name)
 : QListView(parent, name), menu(this,"ModuleListViewMenu")
{
	addColumn( "Plugins" );
	setRootIsDecorated(true);
	
	map = new QMap<QString,NMGModuleListViewItem *>();
	
	loadExistingModules();
	
	idMenuCreateInstance = menu.insertItem("&New instance",this,SLOT(menuCreateInstance()),0,1);
	idMenuUnloadModule = menu.insertItem("&Unload module",this,SLOT(menuUnloadModule()),0,2);
	menu.insertSeparator();
	menu.insertItem("&LoaaddModuled new module",this,SLOT(menuLoadModule()));
	
	connect(this,SIGNAL(returnPressed(QListViewItem*)),this,SLOT(returnPressed(QListViewItem* )));
	connect(this,SIGNAL(spacePressed(QListViewItem*)),this,SLOT(spacePressed(QListViewItem* )));
	connect(this,SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int )),this,SLOT(rightButtonPressed(QListViewItem*, const QPoint&, int )));
	connect(this,SIGNAL(mouseButtonClicked(int , QListViewItem * , const QPoint & , int)),this,SLOT(mouseButtonClicked(int, QListViewItem*, const QPoint&, int )));
}


/**
 * Destructor fot the class
 * @return 
 */
NMGModuleListViewWidget::~NMGModuleListViewWidget()
{
	//TODO: hay que borrar todas las listas que haya y todo lo creado!!
	delete map;
}

/**

 * Adds a capability ot the widget
 * @param capabilityName 
 */
void NMGModuleListViewWidget::addCapability(QString capabilityName)
{
	QMap<QString,NMGModuleListViewItem *>::iterator it = map->find(capabilityName);
	if(it==map->end())
	{
		NMGModuleListViewItem * listItem = new NMGModuleListViewItem(this);
		listItem->setExpandable(true);
		setOpen(listItem,true);
		listItem->setText(0,capabilityName);
		map->insert(capabilityName, listItem);
	}
}

/**
 * Adds a module to the widget specifying the name and its capability
 * @param moduleName Name of the module
 * @param capabilityName Capability of the module
 */
void NMGModuleListViewWidget::addModule(QString moduleName, QString capabilityName)
{
	QMap<QString,NMGModuleListViewItem *>::iterator it = map->find(capabilityName);
	NMGModuleListViewItem * listItem;
	if(it==map->end())
	{
		listItem = new NMGModuleListViewItem(this);
		listItem->setExpandable(true);
		setOpen(listItem,true);
		listItem->setText(0,capabilityName);
		map->insert(capabilityName, listItem);
	}
	else listItem = it.data();
	
	NMGModuleListViewItem * modItem = new NMGModuleListViewItem(listItem);
	modItem->setText(0,moduleName);
	modItem->setCapabilityIcon(capabilityName);
}

/**
 * Slot whichs adds a module
 * @param m 
 */
void NMGModuleListViewWidget::newModule(NMGModule *m)
{
	addModule(m->getModuleGUIName(),m->getModuleCapabilities());
}

/**
 * Slot executed whan a module is selected
 * @param s 
 */
void NMGModuleListViewWidget::moduleSelected(QString name)
{
	emit moduleActivated(name);
}

/**
 * Loads the existing modules in the NMGModuleManager
 */
void NMGModuleListViewWidget::loadExistingModules()
{
	QValueList<NMModule *> l = NMGModuleManager::self()->getModuleList();
	for(QValueList<NMModule *>::iterator it = l.begin(); it!=l.end(); ++it) 
		addModule(getModuleName(*it),(*it)->getCapabilities().c_str());
}

/**
 * Returns the name associated to the module (first the gui, after the core and after the name)
 * @param m 
 * @return 
 */
QString NMGModuleListViewWidget::getModuleName(NMModule * m)
{
	NMGModuleParent * mod = (NMGModuleParent *) m;
	QString nameGUI(mod->getPluginNameGUI().c_str());
	if(nameGUI!="") return nameGUI;
	QString nameCore(mod->getPluginNameCore().c_str());
	if(nameCore!="") return nameCore;
	QString name(mod->getPluginName().c_str());
	if(name!="") return name;
	return "null";
}

/**
 * Slot executed when the user presses a return
 * @param item Item pressed on
 */
void NMGModuleListViewWidget::returnPressed ( QListViewItem * item)
{
	if(item and !item->firstChild() and item->parent()) moduleSelected(item->text(0));
}

/**
 * Slot executed whan the user presse a space
 * @param item Item pressed on
 */
void NMGModuleListViewWidget::spacePressed ( QListViewItem * item)
{
	if(item and !item->firstChild() and item->parent()) moduleSelected(item->text(0));
}

/**
 * Slot executed when the user performs a right click on the list
 * @param item Item right cliked
 * @param pos Position of the mouse
 * @param  
 */
void NMGModuleListViewWidget::rightButtonPressed( QListViewItem * item, const QPoint & pos, int )
{
	if(item and !item->firstChild() and item->parent())
	{
		menu.setItemEnabled(idMenuCreateInstance,true);
		menu.setItemEnabled(idMenuUnloadModule,true);
	}
	else
	{
		menu.setItemEnabled(idMenuCreateInstance,false);
		menu.setItemEnabled(idMenuUnloadModule,false);	
	}
	menu.popup(pos);
}

/**
 * Slot executed whan the user clicks on a item of the list
 * @param  
 * @param item Item clicked
 * @param  
 * @param  
 */
void NMGModuleListViewWidget::mouseButtonClicked ( int /*button */, QListViewItem * item, const QPoint &, int)
{
	if(item and !item->firstChild() and item->parent()) moduleSelected(item->text(0));
}

/**
 * Slot executed when the user wants to create a new instance of the selected item
 */
void NMGModuleListViewWidget::menuCreateInstance()
{
	QListViewItem * item = selectedItem();
	if(item and !item->firstChild() and item->parent()) emit(createInstaceModule(item->text(0)));
}

/**
 * Slot executed whan the user wants to unload the selected module
 */
void NMGModuleListViewWidget::menuUnloadModule()
{
	QListViewItem * item = selectedItem();
	if(item and !item->firstChild() and item->parent())
	{
		emit(unloadModule(item->text(0)));
		item->parent()->takeItem(item);
		delete item;
	}
}

/**
 * Slot executed when the user wants to load a new module
 */
void NMGModuleListViewWidget::menuLoadModule()
{

	QString file = QFileDialog::getOpenFileName(QString::null,"Modules (*.so);;Libtool library (*.la);;All files (*.*)",0,0,"Select an existing module");
	if (file and file!=QString::null and file!="") emit loadNewModule(file);
}


#include "nmgmodulelistviewwidget.moc"

