/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgmoduletoolboxitem.h"

#include "../nmgimagemanager.h"


#define DEFAULT_ICON_NAME	"icon.png"

#include <qtooltip.h>
#include <qpushbutton.h>
#include <qwhatsthis.h>

#include <iostream>
using namespace std;

/**
 * Constructor of the class
 * @param parent 
 * @param name 
 * @return 
 */
NMGModuleToolBoxItem::NMGModuleToolBoxItem(QWidget *parent, QString modName, const QColor & backgroundColor)
 : QWidget(parent, modName)
{
	setMouseTracking(true);
	
	setPaletteBackgroundColor(backgroundColor);
	moduleName = modName;
	
	QVBoxLayout * layoutVertical = new QVBoxLayout(this, 0, 0, moduleName+"LayoutVertical"); 
	
	QHBoxLayout * layoutHoritzontal = new QHBoxLayout( 0, 0, 0, moduleName+"LayoutHoritzontal"); 
	QSpacerItem * leftSpacer = new QSpacerItem( 1, 21, QSizePolicy::Maximum, QSizePolicy::Minimum );
	layoutHoritzontal->addItem(leftSpacer);
	
	QString toolTip = ((NMGModuleParent *) NMGModuleManager::self()->getModule(moduleName))->getModuleToolTip().c_str();
	

/*	QPushButton * button = new QPushButton(this, moduleName+"Button");
	if(toolTip!="")
	{
		QToolTip::add(button, toolTip);
		QWhatsThis::add(button,toolTip);
	}
	button->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)4, (QSizePolicy::SizeType)5, 0, 0, button->sizePolicy().hasHeightForWidth() ) );
	button->setMaximumSize( QSize( 64, 64 ) );
	button->setMinimumSize( QSize( 64, 64 ) );
	button->setFlat(true);
	layoutHoritzontal->addWidget( button);
			
	QString iconFile = ((NMGModuleParent *) NMGModuleManager::self()->getModule(moduleName))->getModuleIcon().c_str();
	if(iconFile=="") iconFile = DEFAULT_ICON_NAME;
	button->setPixmap(*(NMGImageManager::loadPixmap(iconFile)));
	connect(button,SIGNAL(clicked()),this,SLOT(emitActivated()));*/
	
	
	QLabel * pixmapLabel1 = new QLabel(this, moduleName+"Label" );
	if(toolTip!="")
	{
		QToolTip::add(pixmapLabel1, toolTip);
		QWhatsThis::add(pixmapLabel1,toolTip);
	}
	pixmapLabel1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)4, (QSizePolicy::SizeType)5, 0, 0, pixmapLabel1->sizePolicy().hasHeightForWidth() ) );
	pixmapLabel1->setMaximumSize( QSize( 64, 64 ) );

	QString iconFile = ((NMGModuleParent *) NMGModuleManager::self()->getModule(moduleName))->getModuleIcon().c_str();
	if(iconFile=="") iconFile = DEFAULT_ICON_NAME;
	pixmapLabel1->setPixmap(*(NMGImageManager::loadPixmap(iconFile)));
	pixmapLabel1->setScaledContents( TRUE );
	layoutHoritzontal->addWidget( pixmapLabel1 );
	pixmapLabel1->setMouseTracking(true);
	
	
	
	QSpacerItem * rightSpacer = new QSpacerItem( 1, 21, QSizePolicy::Maximum, QSizePolicy::Minimum );
	layoutHoritzontal->addItem(rightSpacer);
	layoutVertical->addLayout( layoutHoritzontal );
	
	
	QLabel * textLabel1 = new QLabel("<p align=\"center\">"+moduleName+"</p>",this, moduleName+"Label");
	if(toolTip!="")
	{
		QToolTip::add(textLabel1, toolTip);
		QWhatsThis::add(textLabel1,toolTip);
	}
	textLabel1->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)3, (QSizePolicy::SizeType)4, 0, 0, textLabel1->sizePolicy().hasHeightForWidth() ) );	
	textLabel1->setMaximumSize( QSize( 84, 32767 ) );
	textLabel1->setPaletteBackgroundColor(backgroundColor);
	textLabel1->setAlignment( int( QLabel::WordBreak | QLabel::AlignVCenter ) );
	layoutVertical->addWidget( textLabel1 );
}


/**
 * Destructor of the class
 * @return 
 */
NMGModuleToolBoxItem::~NMGModuleToolBoxItem()
{
}

/**
 * Returns the module name
 * @return 
 */
QString NMGModuleToolBoxItem::getModuleName()
{
	return moduleName;
}

/**
 * Emits the activate signal of the module
 */
void NMGModuleToolBoxItem::emitActivated()
{
	emit moduleItemActivated(moduleName);
}

/**
 * Handles the mouse events
 * @param e 
 */
void NMGModuleToolBoxItem::mousePressEvent(QMouseEvent * e)
{
	if(e->button()==LeftButton) emitActivated();
	else QWidget::mousePressEvent(e);
}


#include "nmgmoduletoolboxitem.moc"
