/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgpreferencesmodulewidget.h"

#include "../../plugin/nmgmodulemanager.h"

#include "../nmgpreferencesmanager.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qtoolbox.h>
#include <qwidget.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmap.h>
#include <qobjectlist.h>

#define PRE_MOD			"["
#define POST_MOD		"]"

#include <iostream>
using namespace std;


/**
 * Contructor  of the class
 * @param parent 
 * @param name 
 * @param fl 
 * @return 
 */
NMGPreferencesModuleWidget::NMGPreferencesModuleWidget(QWidget *parent, const char *name, WFlags fl)
 : QDialog(parent, name, fl)
{
	if ( !name ) setName( "NMGPreferencesModuleBaseWidget" );
	setSizeGripEnabled( TRUE );
	NMGPreferencesModuleBaseWidgetLayout = new QVBoxLayout( this, 11, 6, "NMGPreferencesModuleBaseWidgetLayout"); 
	
	setCaption( "Select modules to load" );
	
	toolBox1 = new QToolBox( this, "toolBox1" );
	toolBox1->setCurrentIndex( 0 );

	loadExistingModules();
	NMGPreferencesModuleBaseWidgetLayout->addWidget( toolBox1 );

	Layout1 = new QHBoxLayout( 0, 0, 6, "Layout1"); 
	Horizontal_Spacing2 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	Layout1->addItem( Horizontal_Spacing2 );
	
	buttonOk = new QPushButton( this, "buttonOk" );
	buttonOk->setAutoDefault( TRUE );
	buttonOk->setDefault( TRUE );
	buttonOk->setText(  "&OK" ) ;
	buttonOk->setAccel( QKeySequence( QString::null ) );
	Layout1->addWidget( buttonOk );
	
	buttonCancel = new QPushButton( this, "buttonCancel" );
	buttonCancel->setAutoDefault( TRUE );
	buttonCancel->setText(  "&Cancel" ) ;
	buttonCancel->setAccel( QKeySequence( QString::null )) ;
	Layout1->addWidget( buttonCancel );
	NMGPreferencesModuleBaseWidgetLayout->addLayout( Layout1 );
    
	resize( QSize(229, 274).expandedTo(minimumSizeHint()) );
	clearWState( WState_Polished );
	
	// signals and slots connections
	connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

/**
 * Destroyer of the class
 * @return 
 */
NMGPreferencesModuleWidget::~NMGPreferencesModuleWidget()
{
}

/**
 * Loads the existing modules and creates the widgets (checkboxes) for them
 */
void NMGPreferencesModuleWidget::loadExistingModules()
{
	QString moduleList = NMGPreferencesManager::self()->getValue(ID_MODULES_TO_LOAD);
	QString loadAllModules = NMGPreferencesManager::self()->getValue(ID_LOAD_ALL_MODULES);

	QMap<QString,QWidget*> mapWidget; //key capability
	
	QValueList<NMModule *> l = NMGModuleManager::self()->getModuleList();
	for(QValueList<NMModule *>::iterator it = l.begin(); it!=l.end(); ++it) 
	{
		QString modCap, modName;
		NMGModuleParent * mod = (NMGModuleParent *) *it;
		modCap = transformCapabilities(mod->getCapabilities().c_str());
		modName = mod->getPluginNameGUI().c_str();
		
		QWidget * page;
		QMap<QString,QWidget*>::iterator it = mapWidget.find(modCap);
		if(it==mapWidget.end())
		{
			page = new QWidget( toolBox1, "page"+modCap);
			page->setBackgroundMode( QWidget::PaletteBackground );
			QVBoxLayout* pageLayout = new QVBoxLayout( page, 11, 6, "page1Layout"); 
			pageLayout->setAutoAdd(true);
			toolBox1->addItem(page, modCap);
			mapWidget.insert(modCap,page);
		}
		else page = *it;
		
		QCheckBox * checkBox = new QCheckBox( page, modName);
		checkBox->setText(modName);
		if(loadAllModules==ID_LOAD_ALL_MODULES_TRUE or moduleList=="" or moduleList.contains(PRE_MOD+modName+POST_MOD))
			checkBox->setChecked(true);
	}
}

/**
 * Slot executed whn the user clicks the ok button
 */
void NMGPreferencesModuleWidget::accept()
{
	QString moduleList = "";
	hide();
	
	QObjectList *l = topLevelWidget()->queryList("QCheckBox");
	QObjectListIt it( *l ); // iterate over the buttons
	QObject *obj;
	
	while ( (obj = it.current()) != 0 )
	{
		++it;
		QCheckBox * checkBox = (QCheckBox *) obj;
		QString moduleName = checkBox->name();
		if(checkBox->isChecked())
		{
			if(moduleList=="") moduleList = PRE_MOD + moduleName + POST_MOD;
			else moduleList += (QString(",")+QString(PRE_MOD)+moduleName+QString(POST_MOD));
		}
	}
	
	if(moduleList=="")
	{
		NMGPreferencesManager::self()->removeValue(ID_MODULES_TO_LOAD);
		NMGPreferencesManager::self()->addValue(ID_LOAD_ALL_MODULES,ID_LOAD_ALL_MODULES_TRUE);	
	}
	else 
	{
		NMGPreferencesManager::self()->addValue(ID_MODULES_TO_LOAD,moduleList);
		NMGPreferencesManager::self()->addValue(ID_LOAD_ALL_MODULES,ID_LOAD_ALL_MODULES_FALSE);
	}
	
	delete l; // delete the list, not the objects
	delete this;
}

/**
 * Slot executed when the user clicks the cancel button
 */
void NMGPreferencesModuleWidget::reject()
{
	hide();
	delete this;
}

/**
 * Transform the string capabilites to be shown correctly to the user
 * @param capabilities 
 * @return 
 */
QString NMGPreferencesModuleWidget::transformCapabilities(QString capabilities)
{
	return capabilities.lower().replace(0,1,capabilities.left(1).upper());
}

#include "nmgpreferencesmodulewidget.moc"
