/***************************************************************************
 *   Copyright (C) 2004 by David Faure <faure@kde.org>                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "overlaywidget.h"

OverlayWidget::OverlayWidget( QWidget* alignwidget, QWidget* parent, const char* name )
: QHBox( parent, name ), m_alignwidget( 0 )
{
	setAlignWidget( alignwidget );
}


OverlayWidget::~OverlayWidget()
{
}

void OverlayWidget::reposition()
{
  if ( !m_alignwidget )
    return;
  // p is in the alignWidget's coordinates
  QPoint p;
  // We are always above the alignWidget, right-aligned with it.
  p.setX( m_alignwidget->width() - width() );
  p.setY( -height() );
  // Position in the toplevelwidget's coordinates
  QPoint pTopLevel = m_alignwidget->mapTo( topLevelWidget(), p );
  // Position in the widget's parentWidget coordinates
  QPoint pParent = parentWidget()->mapFrom( topLevelWidget(), pTopLevel );
  // Move 'this' to that position.
  move( pParent );
}

void OverlayWidget::setAlignWidget( QWidget * w )
{
  if (w == m_alignwidget)
    return;

  if (m_alignwidget)
    m_alignwidget->removeEventFilter(this);

  m_alignwidget = w;

  if (m_alignwidget)
    m_alignwidget->installEventFilter(this);

  reposition();
}


bool OverlayWidget::eventFilter( QObject* o, QEvent* e)
{
  if ( o == m_alignwidget &&
       ( e->type() == QEvent::Move || e->type() == QEvent::Resize ) ) {
    reposition();
  }
  return QFrame::eventFilter(o,e);
}


void OverlayWidget::resizeEvent( QResizeEvent* ev )
{
  reposition();
  QFrame::resizeEvent( ev );
}

#include "overlaywidget.moc"


