/***************************************************************************
 *   Copyright (C) 2005 by Pau Capella and kmail authors                   *
 *   pau.capella@ac.upc.edu                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "statusbarprogresswidget.h"

#include <qtooltip.h>
#include <qlayout.h>
#include <qwidgetstack.h>
#include <qpixmap.h>
#include <nmgimagemanager.h>

StatusbarProgressWidget::StatusbarProgressWidget( ProgressDialog* progressdialog, QWidget* parent, bool button )
  : QFrame( parent ), m_currentitem( 0 ), m_progressdialog( progressdialog ),
    m_delaytimer( 0 ), m_busytimer( 0 )
{
  m_showbutton = button;
  int w = fontMetrics().width( " 999.9 kB/s 00:00:01 " ) + 8;
  m_box = new QHBoxLayout( this, 0, 0 );

  m_button = new QPushButton( this );
  m_button->setSizePolicy( QSizePolicy( QSizePolicy::Maximum,
                                         QSizePolicy::Maximum ) );
/*  m_button->setSizePolicy(QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0,
                          m_button->sizePolicy().hasHeightForWidth() ) );
  m_button->setMaximumSize( QSize( 24, 24 ) );*/
  // m_button->setPixmap( QPixmap( "up.png" ) );
  m_button->setPixmap( *NMGImageManager::loadPixmap("up.png" ) ); // rserral: works?
  m_box->addWidget( m_button  );
  m_stack = new QWidgetStack( this );
  m_stack->setMaximumHeight( fontMetrics().height() );
  m_box->addWidget( m_stack );

  QToolTip::add( m_button, tr("Open Tests detailed progress dialog") );

  m_progressbar = new QProgressBar( this );
  m_progressbar->setLineWidth( 1 );
  m_progressbar->setFrameStyle( QFrame::Box );
  m_progressbar->installEventFilter( this );
  m_progressbar->setMinimumWidth( w );
  m_stack->addWidget( m_progressbar, 1 );

  m_label = new QLabel( QString::null, this );
  m_label->setAlignment( AlignHCenter | AlignVCenter );
  m_label->installEventFilter( this );
  m_label->setMinimumWidth( w );
  m_stack->addWidget( m_label, 2 );
  m_button->setMaximumHeight( fontMetrics().height() );
  setMinimumWidth( minimumSizeHint().width() );

  m_mode = None;
  setMode();

  connect( m_button, SIGNAL( clicked() ),
           m_progressdialog, SLOT( slotToggleVisibility() ) );

  connect ( ProgressManager::self(), SIGNAL( progressItemAdded( ProgressItem * ) ),
            this, SLOT( slotProgressItemAdded( ProgressItem * ) ) );
  connect ( ProgressManager::self(), SIGNAL( progressItemCompleted( ProgressItem * ) ),
            this, SLOT( slotProgressItemCompleted( ProgressItem * ) ) );

  connect ( m_progressdialog, SIGNAL( visibilityChanged( bool )),
            this, SLOT( slotProgressDialogVisible( bool ) ) );

  m_delaytimer = new QTimer( this );
  connect ( m_delaytimer, SIGNAL( timeout() ),
            this, SLOT( slotShowItemDelayed() ) );
}

// There are three cases: no progressitem, one progressitem (connect to it directly),
// or many progressitems (display busy indicator). Let's call them 0,1,N.
// In slot..Added we can only end up in 1 or N.
// In slot..Removed we can end up in 0, 1, or we can stay in N if we were already.

void StatusbarProgressWidget::slotProgressItemAdded( ProgressItem *item )
{
  if ( item->parent() ) return; // we are only interested in top level items
  connectSingleItem(); // if going to 1 item
  if ( m_currentitem ) { // Exactly one item
    delete m_busytimer;
    m_busytimer = 0;
    m_delaytimer->start( 1000, true );
  }
  else { // N items
    if ( !m_busytimer ) {
      m_busytimer = new QTimer( this );
      connect( m_busytimer, SIGNAL( timeout() ),
               this, SLOT( slotBusyIndicator() ) );
      m_delaytimer->start( 1000, true );
    }
  }
}

void StatusbarProgressWidget::slotProgressItemCompleted( ProgressItem *item )
{
  if ( item->parent() ) return; // we are only interested in top level items
  connectSingleItem(); // if going back to 1 item
  if ( ProgressManager::self()->isEmpty() ) { // No item
    // Done. In 5s the progress-widget will close, then we can clean up the statusbar
    QTimer::singleShot( 5000, this, SLOT( slotClean() ) );
  } else if ( m_currentitem ) { // Exactly one item
    delete m_busytimer;
    m_busytimer = 0;
    activateSingleItemMode();
  }
}

void StatusbarProgressWidget::connectSingleItem()
{
  if ( m_currentitem ) {
    disconnect ( m_currentitem, SIGNAL( progressItemProgress( ProgressItem *, unsigned int ) ),
                 this, SLOT( slotProgressItemProgress( ProgressItem *, unsigned int ) ) );
    m_currentitem = 0;
  }
  m_currentitem = ProgressManager::self()->singleItem();
  if ( m_currentitem ) {
    connect ( m_currentitem, SIGNAL( progressItemProgress( ProgressItem *, unsigned int ) ),
              this, SLOT( slotProgressItemProgress( ProgressItem *, unsigned int ) ) );
  }
}

void StatusbarProgressWidget::activateSingleItemMode()
{
  m_progressbar->setTotalSteps( 100 );
  m_progressbar->setProgress( m_currentitem->progress() );
  m_progressbar->setPercentageVisible( true );
}

void StatusbarProgressWidget::slotShowItemDelayed()
{
  bool noItems = ProgressManager::self()->isEmpty();
  if ( m_currentitem ) {
    activateSingleItemMode();
  } else if ( !noItems ) { // N items
    m_progressbar->setTotalSteps( 0 );
    m_progressbar->setPercentageVisible( false );
    Q_ASSERT( m_busytimer );
    if ( m_busytimer )
      m_busytimer->start( 100 );
  }

  if ( !noItems && m_mode == None ) {
    m_mode = Progress;
    setMode();
  }
}

void StatusbarProgressWidget::slotBusyIndicator()
{
  int p = m_progressbar->progress();
  m_progressbar->setProgress( p + 10 );
}

void StatusbarProgressWidget::slotProgressItemProgress( ProgressItem *item, unsigned int value )
{
  Q_ASSERT( item == m_currentitem); // the only one we should be connected to
  m_progressbar->setProgress( value );
}

void StatusbarProgressWidget::setMode() {
  switch ( m_mode ) {
  case None:
    if ( m_showbutton ) {
      m_button->hide();
    }

    m_stack->show();
    m_stack->raiseWidget( m_label );
    break;

#if 0
  case Label:
    if ( m_showbutton ) {
      m_button->show();
    }
    m_sslLabel->setState( m_sslLabel->lastState() );
    m_stack->show();
    m_stack->raiseWidget( m_label );
    break;
#endif

  case Progress:
    m_stack->show();
    m_stack->raiseWidget( m_progressbar );
    if ( m_showbutton ) {
      m_button->show();
    }
    break;
  }
}

void StatusbarProgressWidget::slotClean()
{
  // check if a new item showed up since we started the timer. If not, clear
  if ( ProgressManager::self()->isEmpty() ) {
    m_progressbar->setProgress( 0 );
    //m_label->clear();
    m_mode = None;
    setMode();
  }
}

bool StatusbarProgressWidget::eventFilter( QObject *, QEvent *ev )
{
  if ( ev->type() == QEvent::MouseButtonPress ) {
    QMouseEvent *e = (QMouseEvent*)ev;

    if ( e->button() == LeftButton && m_mode != None ) {    // toggle view on left mouse button
      // Consensus seems to be that we should show/hide the fancy dialog when the user
      // clicks anywhere in the small one.
      m_progressdialog->slotToggleVisibility();
      return true;
    }
  }
  return false;
}

void StatusbarProgressWidget::slotProgressDialogVisible( bool b )
{
  // Update the hide/show button when the detailed one is shown/hidden
  if ( b ) {
    //m_button->setPixmap( QPixmap("down.png" ) );
    m_button->setPixmap( *NMGImageManager::loadPixmap("down.png" ) );
    QToolTip::remove( m_button );
    QToolTip::add( m_button, tr("Hide detailed progress window") );
    setMode();
  } else {
    // m_button->setPixmap( QPixmap( "up.png" ) );
    m_button->setPixmap( *NMGImageManager::loadPixmap("up.png" ) );
    QToolTip::remove( m_button );
    QToolTip::add( m_button, tr("Show detailed progress window") );
  }
}

#include "statusbarprogresswidget.moc"
