/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <linux/if_ether.h>

#include <time.h>

#include <iostream>
#include <fstream>
using namespace std;

#include <qregexp.h>
#include <messagebar.h>


#include <nmgutil.h>

#include <hash_map.h>

#include "dagthread.h"

#define ERF_HEADER_LEN          18
//#ifdef HAVE_LONG_LONG

#define pletohll(p) ((unsigned long long)*((const unsigned char *)(p)+7)<<56|  \
                     (unsigned long long)*((const unsigned char *)(p)+6)<<48|  \
                     (unsigned long long)*((const unsigned char *)(p)+5)<<40|  \
                     (unsigned long long)*((const unsigned char *)(p)+4)<<32|  \
                     (unsigned long long)*((const unsigned char *)(p)+3)<<24|  \
                     (unsigned long long)*((const unsigned char *)(p)+2)<<16|  \
                     (unsigned long long)*((const unsigned char *)(p)+1)<<8|   \
                     (unsigned long long)*((const unsigned char *)(p)+0)<<0)
/*#else
#define pletohl(p)  ((unsigned long)*((const unsigned char *)(p)+3)<<24|  \
                    (unsigned long)*((const unsigned char *)(p)+2)<<16|  \
                    (unsigned long)*((const unsigned char *)(p)+1)<<8|   \
                    (unsigned long)*((const unsigned char *)(p)+0)<<0)
#endif*/

NMGDagThread::NMGDagThread( const QString & iFile, const QString & oFile) {

        //progressItem = pBar;
        origFile = iFile;
        testOutput = oFile;
	conversionEnded = false;
}

NMGDagThread::~NMGDagThread() {

}

char *my_inet_ntoa2(__be32 theAddr){

	struct in_addr Addr;
	Addr.s_addr=theAddr;
	return (inet_ntoa(Addr));
}

void get_string_from_unumber( unsigned short port, char *unumberStr )
{
	short i,j;
	char number[100];

	i=0;
	while ( port )
	{
		number[i] = '0' + port % 10;
		port = port / 10;
		i++;
	}
	i--;
	j=0;
	while ( i != -1 )
	{
		*( unumberStr + j ) = number[i];
		i--;
		j++;
	}
	*( unumberStr + j ) = '\0';
}

void parseTCPpacket(char *data, NMGCapturedFlow *currPacket){

	const struct tcphdr *tcp_hdr;
	char sPort[10];
	char dPort[10];
	char size[10];
	char Seq[10];
	char ISeqN[10];
	int size_tcp;

	
	tcp_hdr  = (const struct tcphdr*)( data );
	size_tcp = sizeof(const struct tcphdr);//variable for each packet
	get_string_from_unumber( ntohs(tcp_hdr->source), sPort );
//MessageBar::longMessage(sPort);	
	currPacket->setSourcePort( sPort );
	get_string_from_unumber( ntohs(tcp_hdr->source), dPort );
	currPacket->setDestinationPort( dPort );
	unsigned short psize = sizeof(data)-size_tcp;
	get_string_from_unumber( psize, size );
	//sprintf(size,"%d", psize );
//MessageBar::longMessage(size);	
	currPacket->setSize( size );
}

void parseUDPpacket(char *data, NMGCapturedFlow *currPacket){

	
	const struct udphdr *udp_hdr;
	char sPort[10];
	char dPort[10];
	char size[10];
	
	
	udp_hdr = (const struct udphdr*)( data );
	get_string_from_unumber( ntohs(udp_hdr->source), sPort );
//MessageBar::longMessage(sPort);
//sprintf(sPort,"%d", ntohs(udp_hdr->source));
	currPacket->setSourcePort( sPort );
//sprintf(dPort,"%d", ntohs(udp_hdr->dest));
	get_string_from_unumber( ntohs(udp_hdr->dest), dPort );
	currPacket->setDestinationPort( dPort );
//sprintf(size,"%d", ntohs(udp_hdr->len));
	get_string_from_unumber( ntohs(udp_hdr->len), size );
	currPacket->setSize( size );
	
//sPort = QString("%1").arg(udp_hdr->source);	
//dPort = QString("%1").arg(udp_hdr->dest);
//MessageBar::longMessage(sPort);
//MessageBar::longMessage(dPort);
//MessageBar::longMessage(size);
}

int parseIpPacket(const struct iphdr *ip_hdr, char *data, NMGCapturedFlow *currPacket)
{
	int size_ip;
	QString sAddr;
	QString dAddr;
	sAddr = QString("%1").arg( my_inet_ntoa2(( ip_hdr->saddr )) );
	currPacket->setSourceAddress( sAddr );
	dAddr = QString("%1").arg( my_inet_ntoa2( ip_hdr->daddr ) );
	currPacket->setDestinationAddress( dAddr );	
	//MessageBar::longMessage( sAddr );
	//MessageBar::longMessage( dAddr );
	if ( (ip_hdr->protocol) == 0x6)//TCP
	{
		parseTCPpacket( data, currPacket );
		return 1;
	}
	else if ( (ip_hdr->protocol) == 0x11)//UDP
	{
		parseUDPpacket( data, currPacket );
		return 1;
	}	
	else
		return 0;
}

int NMGDagThread::parseEthPacket( char *data, NMGCapturedFlow *currPacket )
{
	

	int size_ip = sizeof ( struct iphdr );
	struct iphdr *ip_hdr = (struct iphdr *)data;
	data = (char *)(data + size_ip);
	parseIpPacket( ip_hdr, data, currPacket );
	return 1;
}

QString getDagPaquetSize( erf_record_t *dag_record )
{
	int dag_psize = ntohs ( dag_record->rlen ) - ERF_HEADER_LEN;
	return QString("%1").arg( dag_psize );
}
/*QString getRxtime( erf_record_t *dag_record ){
	
	QString rxDate = ctime(&(dag_record->(ts.tv_sec)));
	QRegExp rx("\\s+");
	QString rxTime =  rxDate.section( rx,3,3);
	rxTime = rxTime + "." + QString("%1").arg(dag_record->ts.tv_usec);
	return rxTime;
}*/

QString get_time_from_timeval( struct timeval ptime )
{
	char *rxTime = ctime(&(ptime.tv_sec));
	char *tok = strtok (rxTime," ");
	short i = 1;
	while (i<4)
	{
		
		tok = strtok (NULL, " ");
		i++;
	}
	return QString("%1").arg( tok ) + "." + QString("%1").arg( ptime.tv_usec);
}

QString getDagRxTime( erf_record_t *dag_record )
{
	
	struct timeval ptime;
//#ifdef HAVE_LONG_LONG
	unsigned long long ts = pletohll(&dag_record->ts);
	ptime.tv_sec = ts >> 32;
	ts = (( ts & 0xffffffffULL) * 1000 * 1000 );
	ts += ( ts & 0x80000000ULL ) << 1; 
	ptime.tv_usec = ts >> 32;
	if (ptime.tv_usec >= 1000000) {
		ptime.tv_usec -= 1000000;
		ptime.tv_sec += 1;
	}
/*#else
	ptime.tv_sec = pletohl(&dag_record->ts[1]);
	ptime.tv_usec = (unsigned long)((pletohl(&dag_record->ts[0])*1000000.0)/0xffffffffUL);*/
//#endif	
	/*QString rxDate = ctime(&(ptime.tv_sec));
	QRegExp rx("\\s+");
	//killer action
	QString rxTime =  rxDate.section( rx,3,3);
	rxTime = rxTime + "." + QString("%1").arg( ptime.tv_usec);*/

	return get_time_from_timeval( ptime );
}

bool NMGDagThread::pushBackPacket(NMGCapturedFlow *capturedPacket){
	

	list<NMGCapturedFlow *>::iterator currFlow;
	int flowID = 1;
	currFlow = flowsFromDAG.begin();
	bool foundFlow = false;
	while ( (currFlow !=flowsFromDAG.end()) && (!foundFlow) )
	{
		foundFlow = (*currFlow)->eqFlow( capturedPacket );
		if (!foundFlow)
		{
			currFlow++;
			flowID++;
		}
	}
	if (foundFlow)
	{
		//add packet seq id to the end of the list for The Flow
		capturedPacket->setSeqID(QString("%1").arg( (*currFlow)->getQty() ) );
		(*currFlow)->incQty();
		(*currFlow)->addPaquetInfo( capturedPacket );
		delete capturedPacket;
	}
	else
	{
		capturedPacket->setFlowID ( QString("%1").arg(flowID) );
		capturedPacket->setSeqID( QString("0") );
		flowsFromDAG.push_back( capturedPacket );
	}
}

void NMGDagThread::parseDagRecord( erf_record_t *dag_record )
{
	NMGCapturedFlow *currPacket;
	
//MessageBar::longMessage( getDagRxTime( dag_record ) );	
	if (dag_record->type == TYPE_ETH )
	{
		if ( ( ntohs(dag_record->rec.eth.etype)) == ETHERTYPE_VLAN ) 
		{
			currPacket = new NMGCapturedFlow(getDagRxTime( dag_record ) );

			if ( parseEthPacket( (char *)(dag_record->rec.eth_vlan.pload), currPacket ) )
			{
				pushBackPacket(currPacket);	
			}
		}
		else
		{
			//currPacket = new NMGCapturedFlow(getDagRxTime( dag_record ) );
			/*if ( parseEthPacket( (char *)(dag_record->rec.eth.pload), currPacket ) )
			{
				pushBackPacket(currPacket);	
			}*/
			
		}
	}
	else
		MessageBar::longMessage( QString("Frame subtype not supported %1").arg( dag_record->type) );
	
}

void NMGDagThread::parseDAG()
{
	
	char dag_record[10000];
	fstream dagFile;
	erf_record_t erf_header;
	erf_record_t *perf_header;
	//char erf_header[ERF_HEADER_LEN+1];
	const char *p_filename  = origFile.latin1();
	
	dagFile.open( p_filename, fstream::in | fstream::binary );
	//GET LENGTH OF THE FILE
	dagFile.seekg (0, fstream::end);
  	unsigned long long  length = dagFile.tellg();
//MessageBar::longMessage( QString("File Length %1").arg( length ) );
  	dagFile.seekg (0, fstream::beg);

	//READ LENGHT OF THE ERF RECORD
	unsigned short rlen =0; 
	unsigned long long read = rlen;
	statusText = QString("Converting DAG");
        statusPercent = 0;
	while ( read < length )
	{
		dagFile.read( (char *)(&erf_header), ERF_HEADER_LEN );
		rlen = ntohs( erf_header.rlen );
		read = read + (unsigned long long)rlen;
		dagFile.seekg( -ERF_HEADER_LEN, fstream::cur );
		dagFile.read( dag_record, rlen );
		parseDagRecord( (erf_record_t *)dag_record );
		statusPercent = (read*100)/length;
	}
	dagFile.close();

}

void NMGDagThread::buildXml(XmlWriter *p_xw){

//p_xw->writeOpenTag( TAG_FLOW );
//p_xw->writeTaggedString( TAG_RXTIME, getRxtime( ppkt_header ) );
//p_xw->writeCloseTag( TAG_FLOW );
	list<NMGCapturedFlow *>::iterator currFlow;
	list<NMGpaqInfo *> paquetList;
	list<NMGpaqInfo *>::iterator currPaquet;
	p_xw->writeOpenTag( TAG_FLOWS );
	for(currFlow = flowsFromDAG.begin(); currFlow != flowsFromDAG.end(); ++currFlow)
	{

		
		p_xw->writeOpenTag( TAG_FLOW, AttrMap( TAG_ID,((*currFlow)->getFlowID())));
		p_xw->writeOpenTag( TAG_SOURCE );
		p_xw->writeTaggedString( TAG_ADDRESS, ((*currFlow)->getSourceAddress()));
		p_xw->writeTaggedString( TAG_PORT, ((*currFlow)->getSourcePort()));
		p_xw->writeCloseTag( TAG_SOURCE );
		p_xw->writeOpenTag( TAG_DESTINATION );
		p_xw->writeTaggedString( TAG_ADDRESS, ((*currFlow)->getDestinationAddress()));
		p_xw->writeTaggedString( TAG_PORT, ((*currFlow)->getDestinationPort()));
		p_xw->writeCloseTag( TAG_DESTINATION );
	
		paquetList = ((*currFlow)->getListPaquet());		
		for(currPaquet = paquetList.begin(); currPaquet != paquetList.end(); ++currPaquet)
		{
			p_xw->writeOpenTag(TAG_SEQ, AttrMap(ATTR_ID, ((*currPaquet)->getSeqId() )));
			p_xw->writeTaggedString(TAG_RXTIME, ((*currPaquet)->getrxtime()));
			p_xw->writeTaggedString( ATTR_SIZE, ((*currPaquet)->getSize()));
			//p_xw->writeTaggedString(TAG_RX_TIME, );
			//p_xw->writeTaggedString(TAG_SIZE, );
			p_xw->writeCloseTag(TAG_SEQ);
		}
		p_xw->writeCloseTag( TAG_FLOW );
	}
	p_xw->writeCloseTag( TAG_FLOWS );
}

bool NMGDagThread::startConversion()
{
	
	QFile fOutput(testOutput);
	hash_map< Klau, NMGCapturedFlow, hash<Klau> > cflowsFromDAG;
	if (!fOutput.open(IO_WriteOnly)) {
		// FIXME: We have to report back
		return false;
	}
	conversionEnded = false;
	QTextCodec *codec = QTextCodec::codecForName("UTF-8");
	XmlWriter xw(&fOutput, codec);
	xw.setAutoNewLine( true );
	xw.writeOpenTag(TAG_TEST);	
	parseDAG();
	buildXml(&xw);
	//progressItem->setComplete();
	conversionEnded = true;
	fOutput.close();
	return true;

}
void NMGDagThread::stop() {
    isStopped = true;
}

bool NMGDagThread::isFinished() {
    return conversionEnded;
}

void NMGDagThread::getStatus(QString &message, int *percent, bool *ended) {
    message = statusText;
    if (percent)
        *percent = statusPercent;
    *ended = conversionEnded;
}
