/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <netinet/if_ether.h>  /* ether_type */
#include <netinet/in.h>     /* ntohs */   
#include <netinet/ip.h>     /* struct ip, struct iphdr */
#include <netinet/tcp.h>    /* struct tcphdr */
#include <netinet/udp.h>    /* struct udphdr */
#include <arpa/inet.h>     /* inet_ntoa */
#include <string.h>       /* memcpy */




#include <xmlwriter.h>
#include <progressmanager.h>
#include <qstring.h>
#include <qfile.h>
#include <qtextcodec.h>

#include "nmgdagxmltags.h"
#include "nmgcapturedflow.h"
#include <progressmanager.h>

#define ERF_HEADER_LEN	16
#define TYPE_ETH	2
#define ETHERTYPE_VLAN 0x8100

typedef struct eth_rec {
	unsigned char    dst[6];
	unsigned char    src[6];
	unsigned short    etype;
	unsigned char  pload[1];
} eth_rec_t;

typedef struct eth_vlan_rec {
	unsigned char	dst[6];
	unsigned char	src[6];
	unsigned short	tagtype;
	unsigned short	tagcontrol;
	unsigned short	etype;
	unsigned char	pload[1];
} eth_vlan_rec_t;


//#ifdef HAVE_LONG_LONG
typedef unsigned long long erf_timestamp_t;
/*#else
typedef unsigned long erf_timestamp_t[2];
#endif*/


typedef struct Klau{
	char *saddr;
	char *daddr;
	char *sport;
	char *dport;
};
typedef struct erf_record {
	erf_timestamp_t ts;
	unsigned char   type;
	unsigned char   flags;
	unsigned short  rlen;
	unsigned short  lctr;
	unsigned short  wlen;
	unsigned char	offset;
  	unsigned char   pad;
	union{
		eth_rec_t            eth;
		eth_vlan_rec_t	eth_vlan;
	}rec;
} erf_record_t;
/*
typedef struct erf_record {
	erf_timestamp_t ts;
	unsigned char   type;
	unsigned char   flags;
	unsigned short  rlen;
	unsigned short  lctr;
	unsigned short  wlen;
	unsigned char	offset;
  	unsigned char   pad;
	eth_rec_t       eth;
} erf_record_t;
*/
class NMGDagThread {

        ProgressItem *progressItem;
        QString origFile;
        QString testOutput;
	list<NMGCapturedFlow *> flowsFromDAG;
	bool isStopped;
	bool conversionEnded;
	QString statusText;	
	int statusPercent;

public:
	NMGDagThread(/*ProgressItem *pBar,*/ const QString & iFile, const QString & oFile);
	~NMGDagThread();
	bool pushBackPacket(NMGCapturedFlow *capturedPacket);
	bool startConversion();
	void parseDAG();
	void parseDagRecord( erf_record_t *dag_record );
	void buildXml(XmlWriter *p_xw);
	int parseEthPacket( char *data, NMGCapturedFlow *currPacket );
	void stop();
	bool isFinished();
	void getStatus(QString &message, int *percent, bool *ended);
};
