/***************************************************************************
 *   $Id: nmgmgenthread.cpp,v 1.7 2006-10-03 18:45:33 rserral Exp $
 *   Copyright (C) 2005 by Ren Serral-Graci                              *
 *   rserral@ac.upc.edu                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "nmmgeninterpret.h"
#include <nmfile.h>
#include "nmgmgenxmltags.h"
#include <nmgwidgetfromxmltags.h>
#include <nmgmgenthread.h>

#include <iostream>
#include <fstream>
using namespace std;

#include <string.h>
#include <libxml/xmlmemory.h>
#include <libxml/debugXML.h>
#include <libxml/HTMLtree.h>
#include <libxml/xmlIO.h>
#include <libxml/xinclude.h>
#include <libxml/catalog.h>

#include <qstring.h>
#include <qfile.h>
#include <qstringlist.h>
#include <qtextcodec.h>
#include <qdatetime.h>

#define BUFF_SIZE   2048

NMGMGenThread::NMGMGenThread(const QString & iFile, const QString & myMclFile, const QString & oFile) {
    mclFile = myMclFile;
	cleanInputFields();
	origFile = iFile;
	testOutput = oFile;
    isStopped = false;
    conversionEnded = false;
}

NMGMGenThread::~NMGMGenThread() {

}

/**
 * Parses the start XML document
 * @param xml
 * @return
 */
bool NMGMGenThread::parseMCL(QString MCLFile)
{
#define MCL_SEPARATOR ':'
	s_flowInformation flowInfo;
	cleanInputFields();

    ifstream inputFile;
    inputFile.open(MCLFile.latin1(), fstream::in);
    char line[BUFF_SIZE];

    while(inputFile.getline(line,BUFF_SIZE)) {
        QStringList myFields(QStringList::split(MCL_SEPARATOR, QString(line).simplifyWhiteSpace()));

        QStringList::Iterator it = myFields.begin();

        if ((*it).contains("Flow name")) {
            it++;
            inputFields.testName = (*it).simplifyWhiteSpace();
        } else if ((*it).contains("Flow IPv4 ID")) {
            flowInfo.flowID = (*it).section(' ', 3, 3);
        } else if ((*it).contains("Flow Relative Start (msec)")) {
            it++;
            flowInfo.relativeStart = (*it).toInt();
        } else if ((*it).contains("Flow Relative Stop (msec)")) {
            it++;
            flowInfo.relativeStop = (*it).toInt();
        } else if ((*it).contains("Pattern")) {
            flowInfo.pattern = (*it).simplifyWhiteSpace().section(' ', 1, 1);
        } else if ((*it).contains("-Send Rate (packets/sec)") ) {
            it++;
            flowInfo.rate = (*it).toInt();
        } else if ((*it).contains("-UDP Packet Size (bytes)") ) {
            it++;
            flowInfo.packetSize = (*it).toInt();
        } else if ((*it).contains("Flow Source")) {
            inputFields.sourceAddress = (*it).simplifyWhiteSpace().section(' ', 2, 2);
            it++;
            inputFields.sourceManagementAddressPort = (*it).toInt();
        } else if ((*it).contains("Flow Destination")) {
            inputFields.destAddress = (*it).simplifyWhiteSpace().section(' ', 2, 2);
            it++;
            inputFields.destManagementAddressPort = (*it).toInt();
        } else if ((*it).contains("Received packets")) {
            it++;
            flowInfo.receivedPackets = (*it).toInt();
            flowInfo.packetLosses = (flowInfo.relativeStop - flowInfo.relativeStart)/1000*flowInfo.rate - flowInfo.receivedPackets;
        }
    }

    return true;
}

/**
 * Converts the decode output file to XML
 * @param decodeOutputFile fileOutput
 * @return XML
 */
bool NMGMGenThread::startConversion()
{
	ifstream inputFile;

    inputFile.open(origFile.latin1(), fstream::in);
	char line[BUFF_SIZE];
	QString strLine, flowId, flowSeq, flowSrc, flowDest, flowTxTime, flowRxTime, flowSize;

	long fileSize;
    conversionEnded = false;

    inputFile.seekg(0, ios::end);
	fileSize = inputFile.tellg();
	inputFile.seekg(0, ios::beg);

	QFile fOutput(testOutput);
	if (!fOutput.open(IO_WriteOnly)) {
		// FIXME: We have to report back
		return false;
	}
	QTextCodec *codec = QTextCodec::codecForName("UTF-8");
	XmlWriter xw(&fOutput, codec);
	xw.setAutoNewLine( true );
	xw.writeOpenTag(TAG_TEST);
	xw.writeOpenTag(TAG_METADATA);
	parseMCL(mclFile);

	xw.writeTaggedString(TAG_NAME, inputFields.testName);
	xw.writeTaggedString(TAG_IDENTIFIER, inputFields.testIdentifier);

	/* Source station parameters */
	xw.writeOpenTag(TAG_SOURCE);
	xw.writeTaggedString(TAG_ADDRESS, inputFields.sourceAddress);
	xw.writeTaggedString(TAG_MGEN_PATH, inputFields.sourceMgenPath);
	xw.writeOpenTag(TAG_MANAGEMENT);
	xw.writeTaggedString(TAG_ADDRESS, inputFields.sourceManagementAddress);
	xw.writeTaggedString(TAG_PORT, QString("%1").arg(inputFields.sourceManagementAddressPort));
	xw.writeCloseTag(TAG_MANAGEMENT);
	xw.writeCloseTag(TAG_SOURCE);

	/* Destination station parameters */
	xw.writeOpenTag(TAG_DESTINATION);
	xw.writeTaggedString(TAG_ADDRESS, inputFields.destAddress);
	xw.writeTaggedString(TAG_DREC_PATH, inputFields.destDrecPath);
	xw.writeTaggedString(TAG_DECODE_PATH, inputFields.destDecodePath);
	xw.writeOpenTag(TAG_MANAGEMENT);
	xw.writeTaggedString(TAG_ADDRESS, inputFields.destManagementAddress);
	xw.writeTaggedString(TAG_PORT, QString("%1").arg(inputFields.destManagementAddressPort));
	xw.writeCloseTag(TAG_MANAGEMENT);
	xw.writeCloseTag(TAG_DESTINATION);

	xw.writeOpenTag(TAG_CONF_MGEN);
	xw.writeTaggedString(TAG_START, getStartTimeFromDLY(inputFile));
	xw.writeTaggedString(TAG_PORT, QString("%1").arg(inputFields.sourceManagementAddressPort));

	list<NMGCFlowInfo *>::iterator currFlow;
	for(currFlow = flowsFromMCL.begin(); currFlow != flowsFromMCL.end(); ++currFlow)
	{
		AttrMap flowProperties, flowOff;
		QString tmp;
		int iTmp;
		(*currFlow)->getFlowID(tmp);
		flowProperties.insert(ATTR_FLOW_ID, tmp);
		flowOff.insert(ATTR_FLOW_ID, tmp);
		(*currFlow)->getFlowName(tmp);
		flowProperties.insert(ATTR_NAME, tmp);
		tmp = inputFields.destAddress;
		tmp = tmp + ":" + QString("%1").arg(inputFields.destManagementAddressPort);
		flowProperties.insert(ATTR_DEST, tmp);
		(*currFlow)->getRate(&iTmp);
		tmp = QString("%1").arg(iTmp);
		flowProperties.insert(ATTR_RATE, tmp);
		(*currFlow)->getRelativeStart(&iTmp);
		tmp = QString("%1").arg(iTmp);
		flowProperties.insert(ATTR_TIME, tmp);
		(*currFlow)->getPacketSize(&iTmp);
		tmp = QString("%1").arg(iTmp);
		flowProperties.insert(ATTR_SIZE, tmp);
		(*currFlow)->getPattern(tmp);
		flowProperties.insert(ATTR_PATTERN, tmp);
		xw.writeAtomTag(TAG_ON, flowProperties);

		(*currFlow)->getRelativeStop(&iTmp);
		tmp = QString("%1").arg(iTmp);
		flowOff.insert(ATTR_TIME, tmp);
		xw.writeAtomTag(TAG_OFF, flowOff);
        }
	xw.writeCloseTag(TAG_CONF_MGEN);
	xw.writeOpenTag(TAG_CONF_DREC);
	xw.writeTaggedString(TAG_PORTS, QString("%1").arg(inputFields.destManagementAddressPort));
	xw.writeCloseTag(TAG_CONF_DREC);
// End of MetaData
	xw.writeCloseTag(TAG_METADATA);

	xw.writeOpenTag(TAG_RESULT);
	xw.writeOpenTag(TAG_FLOWS);
	int currentIdFlow = -1;
	long i = 1, totalLines, aux;
	QTime last, eta(0,0,0);
	int lineSize, passTime;
	last.start();
#define COUNT_STEP 1000
	while(inputFile.getline(line,BUFF_SIZE))
	{
		if (!(i % COUNT_STEP)) {
			passTime = last.elapsed();
 			lineSize = strlen(line);
			totalLines = fileSize / lineSize;
			aux = (fileSize - inputFile.tellg()) / lineSize;
			eta.setHMS(0,0,0);
			eta = eta.addMSecs(passTime * aux / COUNT_STEP);
            if (isStopped)
                return false;
            statusText = QString("Converting MGen (ETA %1)").arg(eta.toString());
            statusPercent = 100 - (aux * 100 / totalLines);
            /*
            // This doesn't work because of the threading problem, only one thread must update the GUI
			progressItem->setStatus(QString("Converting MGen (ETA %1)").arg(eta.toString()));
			progressItem->setCompletedItems(100 - (aux * 100 / totalLines));
			progressItem->updateProgress();
            */

            last.restart();
		}
		strLine = line;
		strLine = strLine.stripWhiteSpace();
		if(strLine.length()>0)
		{
			putValuesFromDecodeLine(strLine, flowId, flowSeq, flowSrc, flowDest, flowTxTime, flowRxTime, flowSize);
			if(currentIdFlow != flowId.toInt())
			{
				if (currentIdFlow != -1)
					xw.writeCloseTag(TAG_FLOW);
				currentIdFlow = flowId.toInt();
				xw.writeOpenTag(TAG_FLOW, AttrMap(ATTR_ID, QString("%1").arg(currentIdFlow)));

				int pos;
				xw.writeOpenTag(TAG_SOURCE);
				pos = flowSrc.find("/");
				xw.writeTaggedString(TAG_ADDRESS, flowSrc.mid(0,pos));
				xw.writeTaggedString(TAG_PORT, flowSrc.mid(pos+1, flowSrc.length()));
				xw.writeCloseTag(TAG_SOURCE);

				xw.writeOpenTag(TAG_DESTINATION);
				pos = flowDest.find("/");
				xw.writeTaggedString(TAG_ADDRESS, flowDest.mid(0,pos));
				xw.writeTaggedString(TAG_PORT, flowDest.mid(pos+1, flowDest.length()));
				xw.writeCloseTag(TAG_DESTINATION);

			}
			createSeqNode(&xw, PacketInfo(flowId, flowSeq, flowSrc, flowDest, flowTxTime, flowRxTime, flowSize));
		}
		i++;
	}
	xw.writeCloseTag(TAG_FLOW);
	xw.writeCloseTag(TAG_FLOWS);
	xw.writeCloseTag(TAG_RESULT);

	inputFile.close();
	xw.writeCloseTag(TAG_TEST);

	//progressItem->setComplete();
	fOutput.close();
    conversionEnded = true;
	return true;
}


/**
 * Creates the sed node of the document
 * @param pi
 * @return
 */
void NMGMGenThread::createSeqNode(XmlWriter *xw, const PacketInfo& pi)
{
	xw->writeOpenTag(TAG_SEQ, AttrMap(ATTR_ID, QString("%1").arg(pi.flowSeq)));

	xw->writeTaggedString(TAG_TX_TIME, pi.flowTxTime);
	xw->writeTaggedString(TAG_RX_TIME, pi.flowRxTime);
	xw->writeTaggedString(TAG_SIZE, QString("%1").arg(pi.flowSize));
	xw->writeCloseTag(TAG_SEQ);
}

/**
 * Reads the info of a decode file line
 * @param decodeLine
 * @param pi
 */
void NMGMGenThread::putValuesFromDecodeLine(const QString &decLine, PacketInfo &pi) {
    QString flowId, flowSeq, flowSize;
    putValuesFromDecodeLine(decLine, flowId, flowSeq, pi.flowSrc, pi.flowDest, pi.flowTxTime, pi.flowRxTime, flowSize);
    pi.flowId = flowId.toInt();
    pi.flowSeq = flowSeq.toInt();
    pi.flowSize = flowSize.toInt();
}

/**
 * Reads the info of a decode file line
 * @param decodeLine
 * @param strLine
 * @param flowId
 * @param flowSeq
 * @param flowSrc
 * @param flowDest
 * @param flowTxTime
 * @param flowRxTime
 * @param flowSize
 */
void NMGMGenThread::putValuesFromDecodeLine(const QString & decodeLine, QString &flowId, QString &flowSeq,
    						QString &flowSrc, QString &flowDest, QString &flowTxTime, QString &flowRxTime, QString &flowSize)
{
	#define DECODE_SEPARATOR ">"

	QStringList myFields(QStringList::split(' ', decodeLine.simplifyWhiteSpace()));

	QStringList::Iterator it = myFields.begin();
	int i = 0;
	for ( ; it != myFields.end(); ++it ) {
		switch (i) {
		case 0:
			flowId = (*it).section(DECODE_SEPARATOR, 1, 1);
			break;
		case 1:
			flowSeq = (*it).section(DECODE_SEPARATOR, 1, 1);
			break;
		case 2:
			break;
		case 3:
			flowSrc = (*it).stripWhiteSpace();
			break;
		case 4:
			break;
		case 5:
			flowDest = (*it).stripWhiteSpace();
			break;
		case 6:
			flowTxTime = (*it).section(DECODE_SEPARATOR, 1, 1);
			break;
		case 7:
			flowRxTime = (*it).section(DECODE_SEPARATOR, 1, 1);
			break;
		case 8:
			flowSize = (*it).section(DECODE_SEPARATOR, 1, 1);
			break;
		default:
			break;
		}
		i++;
	}
}

/**
 * Cleans the input fields
 */
void NMGMGenThread::cleanInputFields()
{
	inputFields.testName = "";
	inputFields.testIdentifier = "";
	inputFields.sourceAddress = "";
	inputFields.sourceMgenPath = "";
	inputFields.sourceManagementAddress = "";
	inputFields.sourceManagementAddressPort = 22;
	inputFields.sourceManagementUsername = "";
	inputFields.sourceManagementPassword = "";
	inputFields.sourceManagementProtocol = "";
	inputFields.destAddress = "";
	inputFields.destDrecPath = "";
	inputFields.destDecodePath = "";
	inputFields.destManagementAddress = "";
	inputFields.destManagementAddressPort = 22;
	inputFields.destManagementUsername = "";
	inputFields.destManagementPassword = "";
	inputFields.destManagementProtocol = "";
}


/*!
    \fn NMGMGenThread::getStartTimeFromDLY()
    Helper function for guessing the Start Time from the tests. It gets the Transmit time of the first packet
 */
QString NMGMGenThread::getStartTimeFromDLY(ifstream &inFile)
{
    char line[BUFF_SIZE];
    long pos = inFile.tellg();
    if (inFile.getline(line,BUFF_SIZE)) {
        PacketInfo pi;
        putValuesFromDecodeLine(QString(line), pi);
        inFile.seekg(pos, ios::beg);
        return pi.flowRxTime;
    }
    else
        return "00:00:00";
}

void NMGMGenThread::stop() {
    isStopped = true;
}

bool NMGMGenThread::isFinished() {
    return conversionEnded;
}

void NMGMGenThread::getStatus(QString &message, int *percent, bool *ended) {
    message = statusText;
    if (percent)
        *percent = statusPercent;
    *ended = conversionEnded;
}
