/***************************************************************************
 *   Copyright (C) 2005 by Ren Serral-Graci                              *
 *   rserral@ac.upc.edu                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMMGENTHREAD_H
#define NMMGENTHREAD_H

#define TAG_SOURCE	"source"
#define TAG_DESTINATION	"destination"
#define TAG_SEQ		"seq"
#define TAG_FLOW	"flow"
#define TAG_FLOWS	"flows"
#define TAG_TX_TIME	"tx_time"
#define TAG_RX_TIME	"rx_time"
#define TAG_SIZE	"size"
#define TAG_ADDRESS	"address"
#define TAG_PORT	"port"
#define TAG_RESULT	"result"
#define PATH_XML_RESULT	"/test/result"
#define PATH_XML_SOURCE_USERNAME	"/test/metadata/source/management/username"
#define PATH_XML_SOURCE_PASSWORD	"/test/metadata/source/management/password"
#define PATH_XML_DEST_USERNAME		"/test/metadata/destination/management/username"
#define PATH_XML_DEST_PASSWORD		"/test/metadata/destination/management/password"

#define TAG_	""

#define ATTR_ID		"id"

#include <xmlwriter.h>

#include "nmgcflowinfo.h"
#include <nmgconverterbase.h>
#include <nmthread.h>

#include <list>

class PacketInfo;

class NMMGenInputFields
{
public:
    QString testName;
    QString testIdentifier;
    QString sourceAddress;
    QString sourceMgenPath;
    QString sourceManagementAddress;
    int sourceManagementAddressPort;
    QString sourceManagementUsername;
    QString sourceManagementPassword;
    QString sourceManagementProtocol;
    QString destAddress;
    QString destDrecPath;
    QString destDecodePath;
    QString destManagementAddress;
    int destManagementAddressPort;
    QString destManagementUsername;
    QString destManagementPassword;
    QString destManagementProtocol;
    QString pathMgenScript;
    QString pathDrecScript;
    QString localDistrPath;
};

class NMGMGenThread {
    void cleanInputFields();
    void putValuesFromDecodeLine(const QString &decLine, PacketInfo &pi);
	void putValuesFromDecodeLine(const QString & decodeLine, QString & flowId, QString & flowSeq,
				QString & flowSrc, QString & flowDest, QString & flowTxTime,
				QString & flowRxTime, QString & flowSize);
	void createSeqNode(XmlWriter *xw, const PacketInfo& pi);
    QString getStartTimeFromDLY(ifstream &inFile);
	NMMGenInputFields inputFields;
	QString filesSuffix;
	QString origFile, mclFile;
	QString testOutput;
	list<NMGCFlowInfo *> flowsFromMCL;
	bool parseMCL(QString MCLFile);
    bool isStopped;
    bool conversionEnded;
    QString statusText;
    int statusPercent;
public:
	NMGMGenThread(const QString & iFile, const QString & myMclFile, const QString & oFile);
	~NMGMGenThread();
	bool startConversion();
    void stop();
    bool isFinished();
    void getStatus(QString &message, int *percent, bool *ended);
};

class PacketInfo
{
public:
	PacketInfo ()
	{
		flowId=-1;
		flowSeq=-1;
		flowSrc="";
		flowDest="";
		flowTxTime="";
		flowRxTime="";
		flowSize=0;
	}

	PacketInfo (QString pflowId, QString pflowSeq, QString pflowSrc, QString pflowDest, QString pflowTxTime,
		QString pflowRxTime, QString pflowSize)
	{
		flowId=getIntFromString(pflowId);
		flowSeq=getIntFromString(pflowSeq);
		flowSrc=pflowSrc.stripWhiteSpace();
		flowDest=pflowDest.stripWhiteSpace();
		flowTxTime=pflowTxTime.stripWhiteSpace();
		flowRxTime=pflowRxTime.stripWhiteSpace();
		flowSize=getIntFromString(pflowSize);
	}

	PacketInfo (const PacketInfo & second)
	{
		this->flowId = second.flowId;
		this->flowSeq = second.flowSeq;
		this->flowSrc = second.flowSrc;
		this->flowDest = second.flowDest;
		this->flowTxTime = second.flowTxTime;
		this->flowRxTime = second.flowRxTime;
		this->flowSize = second.flowSize;
	}

	int flowId;
	int flowSeq;
	QString flowSrc;
	QString flowDest;
	QString flowTxTime;
	QString flowRxTime;
	int flowSize;

	PacketInfo & operator= (const PacketInfo & second)
	{
		this->flowId = second.flowId;
		this->flowSeq = second.flowSeq;
		this->flowSrc = second.flowSrc;
		this->flowDest = second.flowDest;
		this->flowTxTime = second.flowTxTime;
		this->flowRxTime = second.flowRxTime;
		this->flowSize = second.flowSize;

		return *this;
	}

	int operator< (const PacketInfo & second) const
	{
		if(flowId<second.flowId) return 1;
		if(flowId>second.flowId) return 0;

		if(flowSeq<second.flowSeq) return 1;
		if(flowSeq>second.flowSeq) return 0;

		//sequence number must be unique
		return 1;
	}

	int operator== (const PacketInfo & second) const
	{
		if(flowId<second.flowId) return 0;
		if(flowId>second.flowId) return 0;

		if(flowSeq<second.flowSeq) return 0;
		if(flowSeq>second.flowSeq) return 0;

		//sequence number must be unique
		return 1;
	}

private:
	int getIntFromString(const QString & str)
	{
		QString aux = str.stripWhiteSpace();
		while(aux.length()>1 and aux.mid(0,1)=="0")
			aux = aux.mid(1,aux.length());
		return aux.toInt();
	}
};

#endif
