/*
 *   $Id: nmmgeninterpret.cpp,v 1.7 2006-07-26 19:20:29 rserral Exp $
 *
 *   Copyright (c) 2006 Ren Serral
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "nmmgeninterpret.h"
#include <nmfile.h>
#include "nmgmgenxmltags.h"
#include <nmgwidgetfromxmltags.h>

#include <iostream>
#include <fstream>
using namespace std;

#include <string.h>
#include <libxml/xmlmemory.h>
#include <libxml/debugXML.h>
#include <libxml/HTMLtree.h>
#include <libxml/xmlIO.h>
#include <libxml/xinclude.h>
#include <libxml/catalog.h>

#include <qstring.h>
#include <qfile.h>
#include <qstringlist.h>
#include <qtextcodec.h>

/**
 * Contructor of the class
 * @return nothing
 */
NMMGenInterpret::NMMGenInterpret(const QString conv_name) : NMGConverterBase(conv_name)
{
    dlyFilter = new QByteArray(4);
    mclFilter = new QByteArray(4);

    (*dlyFilter)[0] = 'F';
    (*dlyFilter)[1] = 'l';
    (*dlyFilter)[2] = 'o';
    (*dlyFilter)[3] = 'w';

    (*mclFilter)[0] = 0x20;
    (*mclFilter)[1] = 0x0a;
    (*mclFilter)[2] = 0x4d;
    (*mclFilter)[3] = 0x67;

	addMagic();
	addFilter();
	conversion = 0;
}

/**
 * Destructor of the class
 * @return nothing
 */
NMMGenInterpret::~NMMGenInterpret()
{
    delete dlyFilter;
    delete mclFilter;
}

bool NMMGenInterpret::startConversion(const QString & origFile, const QString & fileOutput)
{
    QString mclFile, dlyFile = origFile;
    chooseFilter(dlyFile, mclFile);
	conversion = new NMGMGenThread(dlyFile, mclFile, fileOutput);
	if (!conversion)
		return false;
	return true;
}

void NMMGenInterpret::run() {
	conversion->startConversion();
}

void NMMGenInterpret::addFilter() {
	QString mFilter("MGen Files (*.mcl *.dly)");
	NMGConverterBase::addFilter(mFilter);
}

void NMMGenInterpret::addMagic() {
	NMGConverterBase::addMagic(*dlyFilter);
    NMGConverterBase::addMagic(*mclFilter);
}

void NMMGenInterpret::kill() {
    conversion->stop();
}

void NMMGenInterpret::chooseFilter(QString &origFile, QString &mclFile) {
    int i;
    QByteArray qbaFile(MAX_MAGIC);
    QFile qInputFile(origFile);
    if ( ! qInputFile.open( IO_ReadOnly ) ) {
        return;
    }
    for (i = 0; i < MAX_MAGIC; i++) {
        qbaFile[i] = (char)(qInputFile.getch());
    }

    if (qbaFile == *dlyFilter) {
        mclFile = origFile.latin1();
        mclFile = mclFile.replace(".dly", ".mcl");
    } else {
        mclFile = origFile.latin1();
        origFile = origFile.replace(".mcl", ".dly");
    }
    qInputFile.close();
}

bool NMMGenInterpret::isFinished() {
    if (conversion)
        return conversion->isFinished();
    return false;
}

void NMMGenInterpret::getStatus(QString &mess, int *percent, bool *ended) {
    conversion->getStatus(mess, percent, ended);
}
