/***************************************************************************
 *   $Id: nmgconvertermgr.cpp,v 1.13 2007-04-16 22:31:19 rserral Exp $
 *   Copyright (C) 2005 by Ren Serral-Graci                              *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <nmgmoduleutils.h>
#include <nmthread.h>

#include <qlistbox.h>
#include <qpushbutton.h>

#include <messagebar.h>

#include "nmgconvertermgr.h"


NMGConverterMgr::NMGConverterMgr(NMGModTabWidgetManager *tabWidgetMan, NMGModulePreferencesManager *prefMan,
	QWidget *parent, const char *name) : NMGConverterWidget(parent, name)
{
	tabWidgetManager = tabWidgetMan;
	preferencesManager = prefMan;

	// For starters we load all the available format converters add all you want statically down here
	NMMGenInterpret *mgenInterpreter = new NMMGenInterpret("MGen v3 Conversor");
	converters.push_back(mgenInterpreter);

	NMGDagInterpret *dagInterpreter = new NMGDagInterpret("Dag Conversor");
	converters.push_back(dagInterpreter);
#ifdef PCAP_FOUND
	NMGPcapInterpret *pcapInterpreter = new NMGPcapInterpret("Pcap Conversor");
	converters.push_back(pcapInterpreter);
#endif
        // Here we should put all the available converters in the form shown above.

        initLastConversionPath();
        NMGConverterWidget::initDialogs(lastInputDir, lastOutputDir);

	initConverters();
	createConvertMenu();
	toolBar = 0;
        chooseDialog = 0;
        timer = new QTimer(this);
        connect( timer, SIGNAL(timeout()), this, SLOT(updateProgress()) );
        progressItem = 0;
}


NMGConverterMgr::~NMGConverterMgr()
{
	if (progressItem)
		progressItem->setComplete();
	progressItem = 0;
}

bool NMGConverterMgr::close( bool alsoDelete) {
    saveLastTestPathFromFilePath(lastInputDir, lastOutputDir);
    return true;
}

/**
 * Returns the toolbar of the Converter widget
 * @return Pointer to the toolbar
 */
QToolBar * NMGConverterMgr::getToolBar()
{
	if(!toolBar)
	{
		toolBar = new QToolBar(NMGModuleUtils::getMainWindow());
		tb_start = new QAction(QIconSet(*NMGModuleUtils::getPixmap("button_ok.png")), tr("Start Conversion"),0,this);
		connect(tb_start,SIGNAL(activated()),this,SLOT(startConversion()));
		tb_start->addTo(toolBar);
	}
	return toolBar;
}

/**
 * Returns the op up menu of the widget
 * @return
 */
QPopupMenu * NMGConverterMgr::getPopupMenu()
{
	if (!convertMenu)
		createConvertMenu();
	return convertMenu;
}

/**
 * Slot which Starts the conversion
 */
void NMGConverterMgr::startConversion()
{
	QString iFile = getiFile1Name();
	QString iFile2 = getiFile2Name();
	theConverter = getConverter(iFile);
	if (!theConverter) {
		return;
	}
        lastInputDir = fdInput->dirPath();
        lastOutputDir = fdOutput->dirPath();
        saveLastTestPathFromFilePath(lastInputDir, lastOutputDir);

	progressItem = ProgressManager::createProgressItem("File Conversion");
	progressItem->setTotalItems(100);
	progressItem->setStatus("Converting...");

	if (iFile2 != "") {
            theConverter->startDualConversion(iFile, iFile2, getoFileName());
        } else {
            theConverter->startConversion(iFile, getoFileName());
        }
        connect(progressItem, SIGNAL(progressItemCanceled( ProgressItem* )), this, SLOT( conversionCancelled( ProgressItem * ) ) );
        connect(progressItem, SIGNAL(progressItemCompleted( ProgressItem* )), this, SLOT( conversionFinished( ProgressItem * ) ) );

    // We must disable the menus for enabling them later
        if (!tb_start)
            return;

        tb_start->setEnabled(false);
        convertMenu->setItemEnabled(startConversionID, false);

    // Let's start
        timer->start(100);      // Well, this is because we can't have a thread updating the GUI, so fuck you and put a timer for updating the %
	theConverter->start();
}

/**
 * Slot which Cancels the conversion
 */
void NMGConverterMgr::conversionCancelled(ProgressItem *) {
    if (theConverter)
	   theConverter->kill();
    if (!tb_start)
        return;
    tb_start->setEnabled(true);
    convertMenu->setItemEnabled(startConversionID, true);
    progressItem->setComplete();
    progressItem = 0;
    MessageBar::longMessage("Converstion cancelled");
}

/**
 * Slot for reporting the finishing of the conversion
 */
void NMGConverterMgr::conversionFinished(ProgressItem *) {
    progressItem = 0;
    if (!tb_start) {
        return;
    }
    tb_start->setEnabled(true);
    convertMenu->setItemEnabled(startConversionID, true);
    if (theConverter->isFinished())
        MessageBar::longMessage("Conversion finished successfully");
}

/*!
    \fn NMGConverterMgr::createConvertMenu()
 */
void NMGConverterMgr::createConvertMenu()
{
	convertMenu = new QPopupMenu();
	startConversionID = convertMenu->insertItem("Start &Conversion", this, SLOT(startConversion()), CTRL+Key_C);
}

void NMGConverterMgr::initConverters()
{
	list<NMGConverterBase *>::iterator interpreter;
	for(interpreter = converters.begin(); interpreter != converters.end(); ++interpreter)
	{
		putFilter(*interpreter);
		putMagic(*interpreter);
	}
}

void NMGConverterMgr::putFilter(NMGConverterBase *converter) {
	QStringList::iterator it;
	if (!converter)
		return;
	for (it = converter->filters().begin(); it != converter->filters().end(); ++it) {
		addInputFilter(*it);
	}
}

void NMGConverterMgr::putMagic(NMGConverterBase *) {

}

/*!
    \fn NMGConverterMgr::getConverter(QString inFile)
 */
NMGConverterBase * NMGConverterMgr::getConverter(QString inFile)
{
	QFile qInputFile(inFile);
	QByteArray qbaFile(MAX_MAGIC);
	int i;

	if ( ! qInputFile.open( IO_ReadOnly ) ) {
		MessageBar::longMessage(QString("Sorry, file %1 not found").arg(inFile));
		return NULL;
	}
	for (i = 0; i < MAX_MAGIC; i++) {
		qbaFile[i] = (char)(qInputFile.getch());
	}
	list<NMGConverterBase *>::iterator interpreter;
	list<QByteArray>::iterator it;
    list<NMGConverterBase *> possibleInterpreter;
	for(interpreter = converters.begin(); interpreter != converters.end(); ++interpreter)
	{
		for (it = (*interpreter)->magics().begin(); it != (*interpreter)->magics().end(); ++it) {
            if ((*it).size() == 0) {//modified by jnunyez		
                // Generic filter, we should take it into account, if no other filters are set take this one :)
                possibleInterpreter.push_back(*interpreter);
            }
			if ((*it).size() != MAX_MAGIC) {
				MessageBar::longMessage("Internal Error, magic size is fixed, please report to the developer of the interpreter");
			}
			if (qbaFile == *it) {
				qInputFile.close();
                possibleInterpreter.push_back(*interpreter);
			}
		}
	}

    if (possibleInterpreter.size() == 0) {
        MessageBar::longMessage("Sorry, no plugin available for this file format");
        return NULL;
    }
    if (possibleInterpreter.size() == 1){
	return *(possibleInterpreter.begin());
	}
    chooseDialog = new NMGConvSelector(this);
    for (interpreter = possibleInterpreter.begin(); interpreter != possibleInterpreter.end(); ++interpreter) {
        chooseDialog->lb_convertOptions->insertItem((*interpreter)->getName());
    }
    connect(chooseDialog->pb_OK, SIGNAL(clicked( )), this, SLOT( okButton( ) ) );
    connect(chooseDialog->pb_cancel, SIGNAL(clicked( )), this, SLOT( cancelButton( ) ) );
    chooseDialog->exec();
    if (selectedID == -1) {
        MessageBar::longMessage("Conversion cancelled");
        return NULL;
    }
    interpreter = possibleInterpreter.begin();
    while (selectedID > 0) {
        interpreter++;
        selectedID--;
    }
    return *interpreter;
}

void NMGConverterMgr::okButton() {
    if (!chooseDialog) {
        selectedID = -1;
        return;
    }
    selectedID = chooseDialog->lb_convertOptions->currentItem();
    chooseDialog->close();
    timer->stop();
    delete chooseDialog;
}

void NMGConverterMgr::cancelButton() {
    selectedID = -1;
    chooseDialog->close();
    timer->stop();
    delete chooseDialog;
}

void NMGConverterMgr::updateProgress() {
    QString mess;
    int percent;
    bool ended;
    theConverter->getStatus(mess, &percent, &ended);
    if (ended) {
        timer->stop();
        progressItem->setComplete();
        progressItem = 0;
        return;
    }
    if (progressItem) {
        progressItem->setStatus(mess);
        progressItem->setCompletedItems(percent);
        progressItem->updateProgress();
    }
}

/**
 * Returns the last test path (or "" if undefined)
 * @return 
 */
void NMGConverterMgr::initLastConversionPath()
{
    lastInputDir = preferencesManager->getValue(ID_PREFERENCE_LAST_CONVERT_INPUT_PATH);
    if(!lastInputDir || lastInputDir == "")
        lastInputDir = ".";
    lastOutputDir = preferencesManager->getValue(ID_PREFERENCE_LAST_CONVERT_OUTPUT_PATH);
    if(!lastOutputDir || lastOutputDir == "")
        lastOutputDir = ".";
}

/**
 * Saves the last test path from the filename path of the test
 * @param filePath 
 */
void NMGConverterMgr::saveLastTestPathFromFilePath(const QString &iFilePath, const QString &oFilePath)
{
    preferencesManager->addValue(ID_PREFERENCE_LAST_CONVERT_INPUT_PATH, iFilePath);
    preferencesManager->addValue(ID_PREFERENCE_LAST_CONVERT_OUTPUT_PATH, oFilePath);
}

#include "nmgconvertermgr.moc"
