/***************************************************************************
 *   Copyright (C) 2005 by Ren Serral-Gracia                              *
 *   rserral@ac.upc.edu                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGCONVERTERMGR_H
#define NMGCONVERTERMGR_H

#include <config.h>

#ifdef CMAKE_BS
#include <config-converter.h>
#endif

#include <qwidget.h>
#include <qpopupmenu.h>
#include <qfiledialog.h>
#include <qtoolbar.h>
#include <qtimer.h>

#include <nmgconverterwidget.h>
#include "nmgconvselector.h"

#include <nmgconverterbase.h>
#include <nmmgeninterpret.h>
#include <daginterpret.h>

#ifdef PCAP_FOUND
#include <pcapinterpret.h>
#endif

#include <progressmanager.h>
#include <nmgmodulepreferencesmanager.h>
#include <nmgimodulecommunication.h>
#include <nmgisender.h>
#include <nmgxmlconfigurationfile.h>
#include <nmgimagemanager.h>
#include <nmgmodtabwidgetmanager.h>
#include <qaction.h>

#define MAX_MAGIC 4

#define ID_PREFERENCE_LAST_CONVERT_INPUT_PATH	"ConvertLastInputPath"
#define ID_PREFERENCE_LAST_CONVERT_OUTPUT_PATH	"ConvertLastOutputPath"

/**
Main class to handle the user interface for file conversion

@author Rene Serral-Gracia
*/

class NMGConverterMgr : public NMGConverterWidget
{
Q_OBJECT
public:
    NMGConverterMgr(NMGModTabWidgetManager * tabWidgetManager, NMGModulePreferencesManager * prefMan, QWidget *parent = 0, const char *name = 0);

    ~NMGConverterMgr();
    
    virtual bool close( bool alsoDelete  = false );

public slots:
    void startConversion();
    QPopupMenu * getPopupMenu();
    QToolBar *getToolBar();
    void conversionCancelled(ProgressItem *progressItem);
    void conversionFinished(ProgressItem *progressItem);
    void okButton();
    void cancelButton();
    void updateProgress();

private:
    int startConversionID;
    int selectedID;
    QString lastInputDir, lastOutputDir;
    NMGConvSelector *chooseDialog;
    QToolBar * toolBar;
    QPopupMenu * convertMenu;
    QAction * tb_start;
    NMGModTabWidgetManager * tabWidgetManager;
    NMGIModuleCommunication * moduleCommunication;
    NMGModulePreferencesManager * preferencesManager;
    ProgressItem * progressItem;
    list<NMGConverterBase *> converters;
    void createConvertMenu();
    void initConverters();
    void putFilter(NMGConverterBase *converter);
    void putMagic(NMGConverterBase *converter);
    NMGConverterBase * getConverter(QString inFile);
    NMGConverterBase *theConverter;
    QTimer *timer;
    void saveLastTestPathFromFilePath(const QString &, const QString &);
    void initLastConversionPath();
};

#endif
