/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <time.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <linux/if_ether.h>

#include <messagebar.h>

#include <pcap.h>

#include <qregexp.h>
#include <qtextcodec.h>
#include <nmgutil.h>

#include <nmgwidgetfromxmltags.h>

#include "nmgpcapxmltags.h"
#include "pcapthread.h"
#include "nmghashflows.h"



NMGPcapThread::NMGPcapThread(/*ProgressItem *pBar, */ const QString & iFile, const QString & oFile) {

	//progressItem = pBar;
	origFile     = iFile;
	testOutput   = oFile;
	flowsFromPCAP = new NMGHashFlows();
	
}
/*
void parseUDPpacket(char *ip_packet, int size_ip, bpf_u_int32  pkt_len, NMGCapturedFlow *currPacket){

	
	const struct udphdr *udp_hdr;
	char sPort[10];
	char dPort[10];
	char size[10];
	
	
	udp_hdr = (const struct udphdr*)( ip_packet + size_ip );
	sprintf(sPort,"%d", ntohs(udp_hdr->source));
	currPacket->setSourcePort( sPort );
	sprintf(dPort,"%d", ntohs(udp_hdr->dest));
	currPacket->setDestinationPort( dPort );
	sprintf(size,"%d", ntohs(udp_hdr->len));
	currPacket->setSize( size );
	//sPort = QString("%1").arg(udp_hdr->source);	
	//dPort = QString("%1").arg(udp_hdr->dest);
	//MessageBar::longMessage(sPort);
	//MessageBar::longMessage(dPort);
	//MessageBar::longMessage(size);
}

void parseTCPpacket(char *ip_packet,  int size_ip, int size_ethernet, bpf_u_int32  pkt_len, NMGCapturedFlow *currPacket){

	const struct tcphdr *tcp_hdr;
	char sPort[10];
	char dPort[10];
	char size[10];
	char Seq[10];
	char ISeqN[10];
	int size_tcp;
	
	tcp_hdr  = (const struct tcphdr*)( ip_packet + size_ip );
	size_tcp = sizeof(const struct tcphdr);//variable for each packet
	sprintf(sPort,"%d", ntohs(tcp_hdr->source));	
	currPacket->setSourcePort( sPort );
	sprintf(dPort,"%d", ntohs(tcp_hdr->dest));
	currPacket->setDestinationPort( dPort );
	sprintf(size,"%d", pkt_len-size_ethernet-size_ip-size_tcp  );
	currPacket->setSize( size );
	if (ntohs(tcp_hdr->syn)){
		sprintf(ISeqN,"%d",ntohs(tcp_hdr->seq));
		//MessageBar::longMessage("0");
	}
	//sPort = QString("%1").arg(udp_hdr->source);	
	//dPort = QString("%1").arg(udp_hdr->dest);
	//MessageBar::longMessage(sPort);
	//MessageBar::longMessage(dPort);
	//MessageBar::longMessage(size);
	//sprintf(Seq,"%d",ntohs(tcp_hdr->seq));
	//MessageBar::longMessage(Seq);
	
}
*/
char *my_inet_ntoa(__be32 theAddr){

	struct in_addr Addr;
	Addr.s_addr=theAddr;
	return (inet_ntoa(Addr));
}

QString getRxtime( pcap_pkthdr *ppkt_header ){
	
	QString rxDate = ctime(&(ppkt_header->ts.tv_sec));
	QRegExp rx("\\s+");
	QString rxTime =  rxDate.section( rx,3,3);
	rxTime = rxTime + "." + QString("%1").arg(ppkt_header->ts.tv_usec);
	return rxTime;
}


unsigned long  NMGPcapThread::crc_append(unsigned char *buf, int len){
	
	int i;
	for (i = 0; i < len; i++)
	{
		crc = UPDC32 (buf[i], crc);
	}
	return (crc);
}

unsigned long NMGPcapThread::hash_flow(const struct iphdr *ip_header,const struct tcphdr* transport_header){


	crc_flow = crc_append ((unsigned char *)&(ip_header->saddr),4);
	crc_flow = crc_append ((unsigned char *)&(ip_header->daddr),4);
	crc_flow = crc_append ((unsigned char *)&(transport_header->source),2);
	crc_flow = crc_append ((unsigned char *)&(transport_header->dest),2);
	crc_flow = crc_append ((unsigned char *)(ip_header->protocol),1);
	return (crc_flow);
}

int NMGPcapThread::parseIPpacket( const u_char *pkt_data, int size_ethernet, pcap_pkthdr *ppkt_header){


	/* pointers to different parts of the packet */

	const struct ethhdr *ethernet_hdr;
	const struct iphdr *ip_hdr;
	const struct tcphdr *tcp_hdr;
	const struct udphdr *udp_hdr;


	/* fields pcap header */
	QString qsrx_time;

	/* fields network header */
	int size_ip;
	
	unsigned char proto;
	QString qstprot;
	QString qssAddr;
	QString qsdAddr;
	
	/* fields transport header */
	QString qssPort;
	QString qsdPort;
	QString qsSeq;
	QString qssize_payload;
	char sPort[10];
	char dPort[10];
	char size[10];
	char seq[10];
	char size_payload[10];
	int size_tcp;
	int size_udp;
	
	/* hashes */
	unsigned long iflow;

	/* length of the packet */
	unsigned long packet_len;
	unsigned long payload;

	qsrx_time = getRxtime(ppkt_header); // Reception Time of the Packet
	ip_hdr = (struct iphdr*)( pkt_data + size_ethernet ); // IP header of the packet
	size_ip = (ip_hdr->ihl)*4; // Size IP Header
	qssAddr = QString("%1").arg( my_inet_ntoa(( ip_hdr->saddr )) ); // Source address
	qsdAddr = QString("%1").arg( my_inet_ntoa( ip_hdr->daddr ) );// Destination address	
	unsigned char tprot = ip_hdr->protocol;
	const u_char *ip_packet = (const u_char *)(ip_hdr);
	const u_char *tcp_packet;
	
	
	//sport dport protocol
	if  ( tprot == 0x6 ){
		//calculation function hash
		tcp_hdr  = (const struct tcphdr*)( ip_packet + size_ip );
		tcp_packet = (const u_char*)( ip_packet + size_ip );
		iflow = hash_flow(ip_hdr,tcp_hdr);

		size_tcp = sizeof(const struct tcphdr);
		sprintf(sPort,"%d", ntohs(tcp_hdr->source));	
		sprintf(dPort,"%d", ntohs(tcp_hdr->dest));
		sprintf(seq,"%d", ntohs(tcp_hdr->seq));
		size_tcp = sizeof(const struct tcphdr);
		packet_len = ppkt_header->len;	
		payload = packet_len - (size_ethernet + size_ip + size_tcp);
		sprintf(size_payload, "%d", payload);
	}
	else if ( tprot == 0x11 ){
		//calculation function hash
		udp_hdr = (const struct udphdr*)( ip_packet + size_ip );
		//udp_packet = (const u_char*)( ip_packet + size_ip );
		//iflow = hash_flow(ip_hdr,udp_hdr);

		sprintf(sPort, "%d", ntohs(udp_hdr->source));
		sprintf(dPort, "%d", ntohs(udp_hdr->dest));
		sprintf(size_payload, "%d", ntohs(udp_hdr->len));
	}
	else
		return 0; 
	//currPacket->setSourceAddress( sAddr );
	//currPacket->setDestinationAddress( dAddr );
	//check if flow exists in the hash
	//if flow not exists create new flow
	
	// if flow exists add the packet
	
	qstprot = QString("%1").arg(tprot);
	qssPort = QString("%1").arg(sPort);	
	qsdPort = QString("%1").arg(dPort);
	qsSeq = QString("%1").arg(seq);
	qssize_payload = QString("%1").arg(size_payload);
	MessageBar::longMessage(QString("%1").arg(iflow));
	if ( flowsFromPCAP->exists(iflow) ){
		MessageBar::longMessage("*** Existe***");
		//add a paquet with qspayload, qsrxtime
		flowsFromPCAP->add_packet_to_flow(iflow, qsSeq, qssize_payload, qsrx_time );
	}
	else {
		MessageBar::longMessage("No Existe. Indice Hash de flujos: "+QString("%1").arg(iflow));
		
		//add a paquet with qspayload, qsrxtime
		flowsFromPCAP->add_flow(iflow, qssAddr, qsdAddr, qssPort, qsdPort, qstprot, qsSeq, qssize_payload, qsrx_time );
	}
	/*MessageBar::longMessage(qssize_payload);
	MessageBar::longMessage(rx_time);
	MessageBar::longMessage(qssAddr);
	MessageBar::longMessage(qsdAddr);
	MessageBar::longMessage(qssPort);
	MessageBar::longMessage(qsdPort);*/
	

	return 0;
}
/*
bool NMGPcapThread::pushBackPacket(NMGCapturedFlow *capturedPacket){
	

	list<NMGCapturedFlow *>::iterator currFlow;
	int flowID = 1;
	currFlow = flowsFromPCAP.begin();
	bool foundFlow = false;
	while ( (currFlow !=flowsFromPCAP.end()) && (!foundFlow) )
	{
		foundFlow = (*currFlow)->eqFlow( capturedPacket );
		if (!foundFlow)
		{
			currFlow++;
			flowID++;
		}
	}
	if (foundFlow)
	{
		//add packet seq id to the end of the list for The Flow
		capturedPacket->setSeqID(QString("%1").arg( (*currFlow)->getQty() ) );
		(*currFlow)->incQty();
		(*currFlow)->addPaquetInfo( capturedPacket );
	}
	else
	{
		capturedPacket->setFlowID ( QString("%1").arg(flowID) );
		capturedPacket->setSeqID( QString("0") );
		flowsFromPCAP.push_back( capturedPacket );
	}
}
*/

int NMGPcapThread::parsePaket( pcap_pkthdr *ppkt_header, const u_char *pkt_data )
{

	const struct ethhdr *ethernet_hdr;
	int size_ethernet;
	//NMGCapturedFlow *currPacket;
	NMGpaqInfo *currInfoPaquet;
	ethernet_hdr = (struct ethhdr*)( pkt_data );
	if ( ntohs( ethernet_hdr->h_proto )== 0x800 ){
		size_ethernet = sizeof(const struct ethhdr);//FIXME 
		
		parseIPpacket(pkt_data, size_ethernet, ppkt_header);

		//FIXED SIZE GLOBAL
		//currPacket = new NMGCapturedFlow(getRxtime(ppkt_header));
		/*if ( parseIPpacket( pkt_data, size_ethernet, ppkt_header*, currPacket*) )
		{
			//pushBackPacket(currPacket);
			//inserto el paquet al hash
			return 1;
		}
		else 
			return 0;*/
	}
	else//ERROR IF NOT ETHERNET
		return 0;
}

void NMGPcapThread::parsePCAP()
{
	const char *p_filename;
	char *err;
	pcap_t *p;
	pcap_pkthdr pkt_header;
	const u_char *pkt_data;
	QString txtime,rxtime;

	
	p_filename  = origFile.latin1();
	unsigned long long length = strlen( p_filename );  
	p           = pcap_open_offline(p_filename, err);
	if (p == NULL)
		pcap_perror(p, err);
	/* Read first packet from const char * ascii () constthe tracefile */
	
	pkt_data = pcap_next(p, &pkt_header);
	unsigned long long  read = (unsigned long long)pkt_header.caplen; 
	statusText = QString("Converting PCAP");
	while (pkt_data != NULL)
	{
		if ( !parsePaket( &pkt_header, pkt_data ) ){
			//MessageBar::longMessage(QString("Type of Packet not valid or supported"));		
		}
		pkt_data = pcap_next(p, &pkt_header);
		statusPercent = (read*100)/length;
		read = read + (unsigned long long)pkt_header.caplen;
	}
//MessageBar::longMessage(QString("no mas paquetes"));		
	pcap_close(p);
	
}

void NMGPcapThread::buildXml(XmlWriter *p_xw)
{

	//hmap lFlows;
	p_xw->writeOpenTag( TAG_FLOWS );


	
	p_xw->writeCloseTag( TAG_FLOWS );
}
/*
void NMGPcapThread::buildXml(XmlWriter *p_xw){

//p_xw->writeOpenTag( TAG_FLOW );
//p_xw->writeTaggedString( TAG_RXTIME, getRxtime( ppkt_header ) );
//p_xw->writeCloseTag( TAG_FLOW );
	list<NMGCapturedFlow *>::iterator currFlow;
	list<NMGpaqInfo *> paquetList;
	list<NMGpaqInfo *>::iterator currPaquet;
	p_xw->writeOpenTag( TAG_FLOWS );
	for(currFlow = flowsFromPCAP.begin(); currFlow != flowsFromPCAP.end(); ++currFlow)
	{

		
		p_xw->writeOpenTag( TAG_FLOW, AttrMap( TAG_ID,((*currFlow)->getFlowID())));
		p_xw->writeOpenTag( TAG_SOURCE );
		p_xw->writeTaggedString( TAG_ADDRESS, ((*currFlow)->getSourceAddress()));
		p_xw->writeTaggedString( TAG_PORT, ((*currFlow)->getSourcePort()));
		p_xw->writeCloseTag( TAG_SOURCE );
		p_xw->writeOpenTag( TAG_DESTINATION );
		p_xw->writeTaggedString( TAG_ADDRESS, ((*currFlow)->getDestinationAddress()));
		p_xw->writeTaggedString( TAG_PORT, ((*currFlow)->getDestinationPort()));
		p_xw->writeCloseTag( TAG_DESTINATION );
//MessageBar::longMessage(QString("Length %1").arg(((*currFlow)->getListPaquet()).size()));	
		paquetList = ((*currFlow)->getListPaquet());		
		for(currPaquet = paquetList.begin(); currPaquet != paquetList.end(); ++currPaquet)
		{
			p_xw->writeOpenTag(TAG_SEQ, AttrMap(ATTR_ID, ((*currPaquet)->getSeqId() )));
			p_xw->writeTaggedString(TAG_RXTIME, ((*currPaquet)->getrxtime()));
			p_xw->writeTaggedString( ATTR_SIZE, ((*currPaquet)->getSize()));
			//p_xw->writeTaggedString(TAG_RX_TIME, );
			//p_xw->writeTaggedString(TAG_SIZE, );
			p_xw->writeCloseTag(TAG_SEQ);
		}
		***taba comentado for( ( currPaquet = ( ( ( *currFlow )->getListPaquet() ).begin() ) ); ( currPaquet != ( ( ( *currFlow )->getListPaquet() ).end() ) ); ++currPaquet)
		{
			MessageBar::longMessage(QString("VUELTA"));		
			p_xw->writeOpenTag(TAG_SEQ, AttrMap(ATTR_ID, ((*currPaquet)->getSeqId() )));
			//xw->writeTaggedString(TAG_TX_TIME, );
			//p_xw->writeTaggedString(TAG_RX_TIME, );
			//xw->writeTaggedString(TAG_SIZE, );
			p_xw->writeCloseTag(TAG_SEQ);
		}***taba comentado
		p_xw->writeCloseTag( TAG_FLOW );
	}
	p_xw->writeCloseTag( TAG_FLOWS );
}
*/

bool NMGPcapThread::startConversion()
{
	QFile fOutput(testOutput);
	if (!fOutput.open(IO_WriteOnly)) {
		// FIXME: We have to report back
		return false;
	}
	
	
	QTextCodec *codec = QTextCodec::codecForName("UTF-8");
	XmlWriter xw(&fOutput, codec);
	xw.setAutoNewLine( true );
	xw.writeOpenTag(TAG_TEST);	
	parsePCAP();
	buildXml(&xw);
	xw.writeCloseTag(TAG_TEST);
	//progressItem->setComplete();
	conversionEnded = true;
	fOutput.close();
	return true;
}

void NMGPcapThread::stop() {
    isStopped = true;
}

bool NMGPcapThread::isFinished() {
    return conversionEnded;
}

void NMGPcapThread::getStatus(QString &message, int *percent, bool *ended) {
    message = statusText;
    if (percent)
        *percent = statusPercent;
    *ended = conversionEnded;
}

NMGPcapThread::~NMGPcapThread() {

}
