/***************************************************************************
 *   Copyright (C) 2005 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgflowlistitem.h"

NMGFlowListItem::NMGFlowListItem ( QListView * parent, QString label1, QString label2, QString label3, QString label4, QString label5, QString label6, QString label7, QString label8)
	: QListViewItem( parent, label1, label2, label3, label4, label5, label6, label7, label8 ) 
{
	m_col_textColor = Qt::black;
	m_col_baseColor = Qt::white;
	m_b_isHidden = false;
	m_b_isAveraged = true;
	m_b_isAccumulated = false;
	m_b_polygonSquare = false;
	m_b_polygonTriangle = false;
	m_b_polygonRhomb = false;
	m_b_polygonInvertedTriangle = false;
	m_i_dotSize = 1;
}

void NMGFlowListItem::paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int alignment )
{
    QColorGroup _cg( cg );
    QColor t = _cg.text();
    QColor b = _cg.base();

	// When not selected: white background with colored letters
    _cg.setColor( QColorGroup::Text, m_col_textColor );
	// When selected: colored background with black letters
    _cg.setColor( QColorGroup::Highlight, m_col_baseColor );

    QListViewItem::paintCell( p, _cg, column, width, alignment );

    _cg.setColor( QColorGroup::Text, t );
    _cg.setColor( QColorGroup::Highlight, b );
}

void NMGFlowListItem::setColors(QColor col_textColor, QColor col_baseColor)
{
	m_col_textColor = col_textColor;
	m_col_baseColor = col_baseColor;
}

void NMGFlowListItem::getColors(QColor &col_textColor, QColor &col_baseColor)
{
	col_textColor = m_col_textColor;
	col_baseColor = m_col_baseColor;
}

bool NMGFlowListItem::isHidden()
{
	return m_b_isHidden;
}

void NMGFlowListItem::setHidden(bool b_hidden)
{
	m_b_isHidden = b_hidden;
}

bool NMGFlowListItem::isAveraged()
{
	return m_b_isAveraged;
}

void NMGFlowListItem::setAveraged(bool b_averaged)
{
	m_b_isAveraged = b_averaged;
	if (b_averaged)
		m_b_isAccumulated = false;
}

bool NMGFlowListItem::isAccumulated()
{
	return m_b_isAccumulated;
}

void NMGFlowListItem::setAccumulated(bool b_accumulated)
{
	m_b_isAccumulated = b_accumulated;
	if (b_accumulated)
		m_b_isAveraged = false;
}

bool NMGFlowListItem::polygonsShowed()
{
	return m_b_polygonsShowed;
}

void NMGFlowListItem::setPolygonsShowed(bool b_showed)
{
	m_b_polygonsShowed = b_showed;
}
bool NMGFlowListItem::linesShowed()
{
	return m_b_linesShowed;
}

void NMGFlowListItem::setLinesShowed(bool b_showed)
{
	m_b_linesShowed = b_showed;
}

bool NMGFlowListItem::areaShowed()
{
	return m_b_areaShowed;
}

void NMGFlowListItem::setAreaShowed(bool b_showed)
{
	m_b_areaShowed = b_showed;
}

bool NMGFlowListItem::polygonSquare()
{
	return m_b_polygonSquare;
}

bool NMGFlowListItem::polygonTriangle()
{
	return m_b_polygonTriangle;
}

bool NMGFlowListItem::polygonRhomb()
{
	return m_b_polygonRhomb;
}

bool NMGFlowListItem::polygonInvertedTriangle()
{
	return m_b_polygonInvertedTriangle;
}

void NMGFlowListItem::setPolygonSquare()
{
	m_b_polygonSquare = true;
	m_b_polygonTriangle = false;
	m_b_polygonRhomb = false;
	m_b_polygonInvertedTriangle = false;
}

void NMGFlowListItem::setPolygonTriangle()
{
	m_b_polygonSquare = false;
	m_b_polygonTriangle = true;
	m_b_polygonRhomb = false;
	m_b_polygonInvertedTriangle = false;
}

void NMGFlowListItem::setPolygonRhomb()
{
	m_b_polygonSquare = false;
	m_b_polygonTriangle = false;
	m_b_polygonRhomb = true;
	m_b_polygonInvertedTriangle = false;
}

void NMGFlowListItem::setPolygonInvertedTriangle()
{
	m_b_polygonSquare = false;
	m_b_polygonTriangle = false;
	m_b_polygonRhomb = false;
	m_b_polygonInvertedTriangle = true;
}


int NMGFlowListItem::getDotSize()
{
	return m_i_dotSize;
}

void NMGFlowListItem::setDotSize(int i_dotSize)
{
	m_i_dotSize = i_dotSize;
}


