/***************************************************************************
 *   Copyright (C) 2006 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgflowlistview.h"
#include <qdragobject.h>
#include <qapplication.h>
#include <qheader.h>

#include <iostream>
using namespace std;


NMGFlowListView::NMGFlowListView(QWidget *parent, const char *name)
	: QListView( parent, name), m_pm_flowMenu(this,"flowMenu") 
//		,oldCurrent( 0 ), dropItem( 0 ), mousePressed( FALSE )
{
	setSorting(-1);
	
	// arrangeOptions
	m_i_idMenuMoveUp = m_pm_arrangeOptions.insertItem("Move up",this,SLOT(menuMoveUp()));
	m_i_idMenuMoveDown = m_pm_arrangeOptions.insertItem("Move down",this,SLOT(menuMoveDown()));
	m_pm_arrangeOptions.insertSeparator();
	m_i_idMenuMoveTop = m_pm_arrangeOptions.insertItem("Bring to top",this,SLOT(menuMoveTop()));
	m_i_idMenuMoveBottom = m_pm_arrangeOptions.insertItem("Send to bottom",this,SLOT(menuMoveBottom()));

	// polygons	
	m_i_idSquarePolygon = m_pm_polygonOptions.insertItem("Squares",this,SLOT(menuSquares()));
	m_i_idTrianglePolygon = m_pm_polygonOptions.insertItem("Triangles",this,SLOT(menuTriangles()));
	m_i_idRhombPolygon = m_pm_polygonOptions.insertItem("Rhombs",this,SLOT(menuRhombs()));
	m_i_idInvertedTrianglePolygon = m_pm_polygonOptions.insertItem("Inverted triangles",this,SLOT(menuInvertedTriangles()));

	// dot size
	int id = m_pm_dotSizeOptions.insertItem("1 pixel",this,SLOT(menuDotSize(int)));
	m_pm_dotSizeOptions.setItemParameter( id, 1 );
	id = m_pm_dotSizeOptions.insertItem("2 pixels",this,SLOT(menuDotSize(int)));
	m_pm_dotSizeOptions.setItemParameter( id, 2 );
	id = m_pm_dotSizeOptions.insertItem("3 pixels",this,SLOT(menuDotSize(int)));
	m_pm_dotSizeOptions.setItemParameter( id, 3 );
	
	// appearanceOptions
	m_i_idLineColor = m_pm_appearanceOptions.insertItem("Line color",this,SLOT(menuLineColor()));
	m_i_idAreaColor = m_pm_appearanceOptions.insertItem("Area color",this,SLOT(menuAreaColor()));
	m_pm_appearanceOptions.insertSeparator();
	m_i_idAverage = m_pm_appearanceOptions.insertItem("Average",this,SLOT(menuAverage()));
	m_i_idAccumulate = m_pm_appearanceOptions.insertItem("Accumulate",this,SLOT(menuAccumulate()));
	m_pm_appearanceOptions.insertSeparator();
	m_i_idPolygons = m_pm_appearanceOptions.insertItem("Draw polygons",this,SLOT(menuShowPolygons()));
	m_i_idLines = m_pm_appearanceOptions.insertItem("Draw lines",this,SLOT(menuShowLines()));
	m_i_idArea = m_pm_appearanceOptions.insertItem("Paint area",this,SLOT(menuShowArea()));
	m_pm_appearanceOptions.insertSeparator();
	m_i_idPolygonType = m_pm_appearanceOptions.insertItem("Polygon type",&m_pm_polygonOptions);
	m_i_idDotSize = m_pm_appearanceOptions.insertItem("Dot Size",&m_pm_dotSizeOptions);
	
	m_i_idArrange = m_pm_flowMenu.insertItem("Arrange",&m_pm_arrangeOptions);
	m_pm_flowMenu.insertSeparator();
	m_i_idHideGraph = m_pm_flowMenu.insertItem("Hide flow",this,SLOT(menuHideGraph()));
	m_i_idDeleteGraph = m_pm_flowMenu.insertItem("Delete flow",this,SLOT(menuDeleteGraph()));
	m_pm_flowMenu.insertSeparator();
	m_i_idAppearance = m_pm_flowMenu.insertItem("Appearance",&m_pm_appearanceOptions);
	
	connect(this,SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int )),this,SLOT(rightButtonPressed(QListViewItem*, const QPoint&, int )));
	
	setAcceptDrops(TRUE);
	viewport()->setAcceptDrops( TRUE);
}

NMGFlowListView::~NMGFlowListView()
{
}


int NMGFlowListView::selectedItemIndex()
{
	if (currentItem()!=0)
		return (currentItem()->itemPos()/currentItem()->height());
	else
		return -1;
}

void NMGFlowListView::reportGraphDataLoaded(int i_numLoadedGraphs)
{
	emit graphDataLoaded( i_numLoadedGraphs );
}


/**
 * Slot executed when the user performs a right click on the list
 * @param item Item right cliked
 * @param pos Position of the mouse
 * @param  
 */
void NMGFlowListView::rightButtonPressed( QListViewItem * lvi_flow, const QPoint & pos, int )
{
	if(lvi_flow)
	{
		if (lvi_flow->itemBelow())
		{
			m_pm_arrangeOptions.setItemEnabled(m_i_idMenuMoveDown,true);
			m_pm_arrangeOptions.setItemEnabled(m_i_idMenuMoveBottom,true);	
		}
		else
		{
			m_pm_arrangeOptions.setItemEnabled(m_i_idMenuMoveDown,false);
			m_pm_arrangeOptions.setItemEnabled(m_i_idMenuMoveBottom,false);	
		}
		
		if (lvi_flow->itemAbove())
		{
			m_pm_arrangeOptions.setItemEnabled(m_i_idMenuMoveUp,true);
			m_pm_arrangeOptions.setItemEnabled(m_i_idMenuMoveTop,true);	
		}
		else
		{
			m_pm_arrangeOptions.setItemEnabled(m_i_idMenuMoveUp,false);
			m_pm_arrangeOptions.setItemEnabled(m_i_idMenuMoveTop,false);	
		}
		m_pm_flowMenu.setItemChecked(m_i_idHideGraph,((NMGFlowListItem*)lvi_flow)->isHidden());

		m_pm_appearanceOptions.setItemChecked(m_i_idAverage,((NMGFlowListItem*)lvi_flow)->isAveraged());
		m_pm_appearanceOptions.setItemChecked(m_i_idAccumulate,((NMGFlowListItem*)lvi_flow)->isAccumulated());

		m_pm_appearanceOptions.setItemChecked(m_i_idPolygons,((NMGFlowListItem*)lvi_flow)->polygonsShowed());
		m_pm_appearanceOptions.setItemChecked(m_i_idLines,((NMGFlowListItem*)lvi_flow)->linesShowed());
		m_pm_appearanceOptions.setItemChecked(m_i_idArea,((NMGFlowListItem*)lvi_flow)->areaShowed());
		
		m_pm_appearanceOptions.setItemEnabled(m_i_idPolygonType,((NMGFlowListItem*)lvi_flow)->polygonsShowed());
		m_pm_polygonOptions.setItemChecked(m_i_idSquarePolygon,((NMGFlowListItem*)lvi_flow)->polygonSquare());
		m_pm_polygonOptions.setItemChecked(m_i_idTrianglePolygon,((NMGFlowListItem*)lvi_flow)->polygonTriangle());
		m_pm_polygonOptions.setItemChecked(m_i_idRhombPolygon,((NMGFlowListItem*)lvi_flow)->polygonRhomb());
		m_pm_polygonOptions.setItemChecked(m_i_idInvertedTrianglePolygon,((NMGFlowListItem*)lvi_flow)->polygonInvertedTriangle());

		if (!((NMGFlowListItem*)lvi_flow)->polygonsShowed() && !((NMGFlowListItem*)lvi_flow)->areaShowed() &&
			!((NMGFlowListItem*)lvi_flow)->linesShowed())
		{

			m_pm_appearanceOptions.setItemEnabled(m_i_idDotSize,true);
			// uncheck & enable all items
			for(int i=0;i<3;i++)
			{
				m_pm_dotSizeOptions.setItemChecked(m_pm_dotSizeOptions.idAt(i),false);
				m_pm_dotSizeOptions.setItemEnabled(m_pm_dotSizeOptions.idAt(i),true);
			}
			// check & disable selected dot size
			m_pm_dotSizeOptions.setItemChecked(m_pm_dotSizeOptions.idAt(((NMGFlowListItem*)lvi_flow)->getDotSize()-1),true);
			m_pm_dotSizeOptions.setItemEnabled(m_pm_dotSizeOptions.idAt(((NMGFlowListItem*)lvi_flow)->getDotSize()-1),false);
		}
		else
		{
			m_pm_appearanceOptions.setItemEnabled(m_i_idDotSize,false);
		}
		
		m_pm_flowMenu.popup(pos);
	}
}


void NMGFlowListView::menuMoveUp()
{
	if (currentItem()!=0)
	{
		QListViewItem * lvi_selectedFlow = currentItem();
		int i_itemPos = lvi_selectedFlow->itemPos();
		
		// if the item above the current one has another item above it move the current item between them
		if (lvi_selectedFlow->itemAbove()->itemAbove())
		{
			lvi_selectedFlow->moveItem(lvi_selectedFlow->itemAbove()->itemAbove());
		}
		else
		{	
			takeItem(lvi_selectedFlow);
			insertItem(lvi_selectedFlow);
			setSelected(lvi_selectedFlow,true);
		}
		
		emit itemMove(i_itemPos/lvi_selectedFlow->height(),(i_itemPos/lvi_selectedFlow->height())-1);
	}
}

void NMGFlowListView::menuMoveDown()
{
	if (currentItem()!=0)
	{
		QListViewItem * lvi_selectedFlow = currentItem();
		int i_itemPos = lvi_selectedFlow->itemPos();
		
		lvi_selectedFlow->moveItem(lvi_selectedFlow->itemBelow());
		
		emit itemMove(i_itemPos/lvi_selectedFlow->height(),(i_itemPos/lvi_selectedFlow->height())+1);
	}
}

void NMGFlowListView::menuMoveTop()
{
	if (currentItem()!=0)
	{	
		QListViewItem * lvi_selectedFlow = currentItem();
		int i_itemPos = lvi_selectedFlow->itemPos();
		
		takeItem(lvi_selectedFlow);
		insertItem(lvi_selectedFlow);
		setSelected(lvi_selectedFlow,true);
		
		emit itemMove(i_itemPos/lvi_selectedFlow->height(),0);
	}
}

void NMGFlowListView::menuMoveBottom()
{
	if (currentItem()!=0)
	{
		QListViewItem * lvi_selectedFlow = currentItem();
		int i_itemPos = lvi_selectedFlow->itemPos();
		
		lvi_selectedFlow->moveItem(lastItem());
		
		emit itemMove(i_itemPos/lvi_selectedFlow->height(),childCount()-1);
	}
}

void NMGFlowListView::menuHideGraph()
{
	if (currentItem()!=0)
	{
		bool b_HideItem = !((NMGFlowListItem*)currentItem())->isHidden();
		((NMGFlowListItem*)currentItem())->setHidden(b_HideItem);
		
		emit itemHide(selectedItemIndex(),b_HideItem);
	}
}

void NMGFlowListView::menuDeleteGraph()
{
	if (currentItem()!=0)
		emit itemDelete(selectedItemIndex());
}

void NMGFlowListView::menuLineColor()
{
	if (currentItem()!=0)
		emit itemChangeLineColor(selectedItemIndex());
}

void NMGFlowListView::menuAreaColor()
{
	if (currentItem()!=0)
		emit itemChangeAreaColor(selectedItemIndex());
}

void NMGFlowListView::menuAverage()
{
	if (currentItem()!=0)
	{
		bool b_Average = !((NMGFlowListItem*)currentItem())->isAveraged();
		((NMGFlowListItem*)currentItem())->setAveraged(b_Average);
	
		emit itemAverage(selectedItemIndex(),b_Average);
	}
}

void NMGFlowListView::menuAccumulate()
{
	if (currentItem()!=0)
	{
		bool b_Accumulate = !((NMGFlowListItem*)currentItem())->isAccumulated();
		((NMGFlowListItem*)currentItem())->setAccumulated(b_Accumulate);
	
		emit itemAccumulate(selectedItemIndex(),b_Accumulate);
	}
}

void NMGFlowListView::menuShowPolygons()
{
	if (currentItem()!=0)
	{
		bool b_showPolygons = !((NMGFlowListItem*)currentItem())->polygonsShowed();
		((NMGFlowListItem*)currentItem())->setPolygonsShowed(b_showPolygons);
		
		emit itemShowPolygons(selectedItemIndex(),b_showPolygons);
	}
}

void NMGFlowListView::menuShowLines()
{
	if (currentItem()!=0)
	{
		bool b_showLines = !((NMGFlowListItem*)currentItem())->linesShowed();
		((NMGFlowListItem*)currentItem())->setLinesShowed(b_showLines);
		
		emit itemShowLines(selectedItemIndex(),b_showLines);
		if (!b_showLines && ((NMGFlowListItem*)currentItem())->areaShowed())
		{		// it has not sense, so disable the area
			((NMGFlowListItem*)currentItem())->setAreaShowed(false);
	
			emit itemShowArea(selectedItemIndex(),false);
		}
	}
}

void NMGFlowListView::menuShowArea()
{
	if (currentItem()!=0)
	{
		bool b_showArea = !((NMGFlowListItem*)currentItem())->areaShowed();
		((NMGFlowListItem*)currentItem())->setAreaShowed(b_showArea);
		
		emit itemShowArea(selectedItemIndex(),b_showArea);
	
		if (b_showArea && !((NMGFlowListItem*)currentItem())->linesShowed())
		{		// it has not sense, so disable the area
			((NMGFlowListItem*)currentItem())->setLinesShowed(true);
	
			emit itemShowLines(selectedItemIndex(),true);
		}
	}
}

void NMGFlowListView::menuSquares()
{
	if (currentItem()!=0)
	{
		m_pm_polygonOptions.setItemChecked(m_i_idSquarePolygon,true);
		m_pm_polygonOptions.setItemChecked(m_i_idTrianglePolygon,false);
		m_pm_polygonOptions.setItemChecked(m_i_idRhombPolygon,false);
		m_pm_polygonOptions.setItemChecked(m_i_idInvertedTrianglePolygon,false);
		
		((NMGFlowListItem*)currentItem())->setPolygonSquare();
		
		emit itemPolygonSquare(selectedItemIndex());
	}
}

void NMGFlowListView::menuTriangles()
{
	if (currentItem()!=0)
	{
		m_pm_polygonOptions.setItemChecked(m_i_idSquarePolygon,false);
		m_pm_polygonOptions.setItemChecked(m_i_idTrianglePolygon,true);
		m_pm_polygonOptions.setItemChecked(m_i_idRhombPolygon,false);
		m_pm_polygonOptions.setItemChecked(m_i_idInvertedTrianglePolygon,false);
		
		((NMGFlowListItem*)currentItem())->setPolygonTriangle();
		
		emit itemPolygonTriangle(selectedItemIndex());
	}
}

void NMGFlowListView::menuRhombs()
{
	if (currentItem()!=0)
	{
		m_pm_polygonOptions.setItemChecked(m_i_idSquarePolygon,false);
		m_pm_polygonOptions.setItemChecked(m_i_idTrianglePolygon,false);
		m_pm_polygonOptions.setItemChecked(m_i_idRhombPolygon,true);
		m_pm_polygonOptions.setItemChecked(m_i_idInvertedTrianglePolygon,false);
		
		((NMGFlowListItem*)currentItem())->setPolygonRhomb();
		
		emit itemPolygonRhomb(selectedItemIndex());
	}
}

void NMGFlowListView::menuInvertedTriangles()
{
	if (currentItem()!=0)
	{
		m_pm_polygonOptions.setItemChecked(m_i_idSquarePolygon,false);
		m_pm_polygonOptions.setItemChecked(m_i_idTrianglePolygon,false);
		m_pm_polygonOptions.setItemChecked(m_i_idRhombPolygon,false);
		m_pm_polygonOptions.setItemChecked(m_i_idInvertedTrianglePolygon,true);
		
		((NMGFlowListItem*)currentItem())->setPolygonInvertedTriangle();
		
		emit itemPolygonInvertedTriangle(selectedItemIndex());
	}
}


void NMGFlowListView::menuDotSize(int i_size)
{
	if (currentItem()!=0)
	{
		((NMGFlowListItem*)currentItem())->setDotSize(i_size);
		
		emit itemDotSize(selectedItemIndex(),i_size);
	}
}


void NMGFlowListView::enableAverage(bool b_enabled)
{
	m_pm_appearanceOptions.setItemEnabled(m_i_idAverage,b_enabled);
}

void NMGFlowListView::enableAccumulate(bool b_enabled)
{
	m_pm_appearanceOptions.setItemEnabled(m_i_idAccumulate,b_enabled);
}

void NMGFlowListView::viewAverage(bool b_visible)
{
	m_pm_appearanceOptions.setItemVisible(m_i_idAverage,b_visible);
}

void NMGFlowListView::viewAccumulate(bool b_visible)
{
	m_pm_appearanceOptions.setItemVisible(m_i_idAccumulate,b_visible);
}

// Drag & drop
void NMGFlowListView::contentsDragEnterEvent( QDragEnterEvent *e )
{
	if ( !QTextDrag::canDecode(e) ) 
	{
		e->ignore();
		return;
    }

    m_lvi_oldCurrent = currentItem();

    QListViewItem *i = itemAt( contentsToViewport(e->pos()) );
    if ( i ) 
		m_lvi_dropItem = i;
}

void NMGFlowListView::contentsDragMoveEvent( QDragMoveEvent *e )
{
	if ( !QTextDrag::canDecode(e) ) 
	{
		e->ignore();
		return;
    }

    QPoint vp = contentsToViewport( ( (QDragMoveEvent*)e )->pos() );
    QListViewItem *i = itemAt( vp );
	if ( i ) 
	{
		setSelected( m_lvi_dropItem, FALSE );
		setSelected( i, TRUE );
		e->accept();
		if ( i != m_lvi_dropItem ) 
		{
		    m_lvi_dropItem = i;
		}
		
		switch ( e->action() ) 
		{
			case QDropEvent::Copy:
	    		break;
			case QDropEvent::Move:
	    		e->acceptAction();
	    		break;
			case QDropEvent::Link:
	    		break;
			default:
			    ;
		}
    } 
	else 
	{
		e->ignore();
		m_lvi_dropItem = 0;
    }
}

void NMGFlowListView::contentsDragLeaveEvent( QDragLeaveEvent * )
{
	m_lvi_dropItem = 0;
    setCurrentItem( m_lvi_oldCurrent );
}

void NMGFlowListView::contentsDropEvent( QDropEvent *e )
{
	if ( !QTextDrag::canDecode(e) ) 
	{
		e->ignore();
		return;
    }

    QListViewItem *item = itemAt( contentsToViewport(e->pos()) );
    if ( item ) 
	{
		if ( e->action() == QDropEvent::Move ) 
		{
			int i_oldItemPos, i_newItemPos;
			i_oldItemPos = m_lvi_oldCurrent->itemPos()/m_lvi_oldCurrent->height();
			i_newItemPos = item->itemPos()/item->height();
			
			e->acceptAction();
			if( item->itemPos() < m_lvi_oldCurrent->itemPos()) {
				QListViewItem *aux = item->itemAbove();
				if( aux)
					m_lvi_oldCurrent->moveItem( aux);
				else {//Canvi amb el primer de la llista...
					takeItem( m_lvi_oldCurrent);
					insertItem( m_lvi_oldCurrent);
				}	
			}
			else 
			{
				m_lvi_oldCurrent->moveItem( item);
			}
			emit itemMove(i_oldItemPos,i_newItemPos);
			setSelected( item, FALSE);
			setSelected( m_lvi_oldCurrent, TRUE);
		}
		e->accept();
    } 
	else
		e->ignore();

}

void NMGFlowListView::contentsMousePressEvent( QMouseEvent* e )
{
	QListView::contentsMousePressEvent( e);
	if (e->button()==LeftButton)
	{
		QPoint p( contentsToViewport( e->pos()));
		QListViewItem *i = itemAt( p);
		if( i) 
		{
			m_p_presspos = e->pos();
			m_b_mousePressed = TRUE;
		}
	}
}

void NMGFlowListView::contentsMouseMoveEvent( QMouseEvent* e )
{
 	if( m_b_mousePressed && ( m_p_presspos - e->pos()).manhattanLength() > QApplication::startDragDistance()) 
	{
 		m_b_mousePressed = FALSE;
 		QListViewItem *item = itemAt( contentsToViewport( m_p_presspos));
 		if( item) 
		{
 			QUriDrag *ud = new QUriDrag( viewport());
 			ud->drag();
 		}
 	}
}

void NMGFlowListView::contentsMouseReleaseEvent( QMouseEvent * )
{
	m_b_mousePressed = FALSE;
}

#include "nmgflowlistview.moc"
