/***************************************************************************
 *   Copyright (C) 2006 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGFLOWLISTVIEW_H
#define NMGFLOWLISTVIEW_H

#include <qlistview.h>
#include <qpopupmenu.h>
#include <qpainter.h>
#include "nmgflowlistitem.h"

/**
	@author David Lacort <david.lacort@gmail.com>
*/

class NMGFlowListView : public QListView
{
    Q_OBJECT

public:
    NMGFlowListView(QWidget *parent = 0, const char *name = 0);
    ~NMGFlowListView();

	int selectedItemIndex();
	void reportGraphDataLoaded(int i_numLoadedGraphs);

	void enableAverage(bool b_enabled);
	void enableAccumulate(bool b_enabled);
	void viewAverage(bool b_enabled);
	void viewAccumulate(bool b_enabled);

public slots:
	void rightButtonPressed( QListViewItem * item, const QPoint & pos, int );
	void menuMoveUp();
	void menuMoveDown();
	void menuMoveTop();
	void menuMoveBottom();
	void menuHideGraph();
	void menuDeleteGraph();
	void menuLineColor();
	void menuAreaColor();
	void menuAverage();
	void menuAccumulate();
	void menuShowPolygons();
	void menuShowLines();
	void menuShowArea();
	void menuSquares();
	void menuTriangles();
	void menuRhombs();
	void menuInvertedTriangles();
	void menuDotSize(int i_size);

signals:
	void itemMove(uint,uint);
	void itemHide(uint,bool);
	void itemDelete(uint);
	void itemChangeLineColor( uint );
	void itemChangeAreaColor( uint );
	void itemAverage(uint,bool);
	void itemAccumulate(uint,bool);
	void itemShowPolygons(uint,bool);
	void itemShowLines(uint,bool);
	void itemShowArea(uint,bool);
	void itemPolygonSquare(uint);
	void itemPolygonTriangle(uint);
	void itemPolygonRhomb(uint);
	void itemPolygonInvertedTriangle(uint);
	void itemDotSize(uint,int);

	void graphDataLoaded(int i_numLoadedGraphs);

protected:

    void contentsDragEnterEvent( QDragEnterEvent *e );
    void contentsDragMoveEvent( QDragMoveEvent *e );
    void contentsDragLeaveEvent( QDragLeaveEvent *e );
    void contentsDropEvent( QDropEvent *e );
    void contentsMouseMoveEvent( QMouseEvent *e );
    void contentsMousePressEvent( QMouseEvent *e );
    void contentsMouseReleaseEvent( QMouseEvent *e );

private:
    QPopupMenu m_pm_flowMenu;
    QPopupMenu m_pm_arrangeOptions;
    QPopupMenu m_pm_appearanceOptions;
	QPopupMenu m_pm_polygonOptions;
	QPopupMenu m_pm_dotSizeOptions;

	int m_i_idMenuMoveUp;
	int m_i_idMenuMoveDown;
	int m_i_idMenuMoveTop;
	int m_i_idMenuMoveBottom;
	int m_i_idArrange;
	int m_i_idHideGraph;
	int m_i_idDeleteGraph;
	int m_i_idAppearance;
	int m_i_idLineColor;
	int m_i_idAreaColor;
	int m_i_idAverage;
	int m_i_idAccumulate;
	int m_i_idPolygons;
	int m_i_idLines;
	int m_i_idArea;
	int m_i_idSquarePolygon;
	int m_i_idTrianglePolygon;
	int m_i_idRhombPolygon;
	int m_i_idInvertedTrianglePolygon;
	int m_i_idPolygonType;
	int m_i_idDotSize;

    QListViewItem *m_lvi_oldCurrent;
    QListViewItem *m_lvi_dropItem;
    QPoint m_p_presspos;
    bool m_b_mousePressed;
};

#endif
