/***************************************************************************
 *   Copyright (C) 2005 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmggloaderthread.h"

NMGGLoaderThread::NMGGLoaderThread()
 : NMThread()
{
	m_xffp_parser = 0;
	m_b_acceptInvalidData = false;
}


NMGGLoaderThread::~NMGGLoaderThread()
{
	if (m_xffp_parser)
		delete m_xffp_parser;
	m_xffp_parser = 0;
}

void NMGGLoaderThread::loadGraphData(QString str_fileName)
{
	m_str_fileName = str_fileName;
}

void NMGGLoaderThread::acceptInvalidData(bool b_accept)
{
	m_b_acceptInvalidData = b_accept;
}

void NMGGLoaderThread::run()
{
	QFile xmlFile( m_str_fileName );
	QXmlInputSource source( &xmlFile );

	QXmlSimpleReader reader;
	m_xffp_parser = new NMGMGENXMLFlowFormatParser();
	m_xffp_parser->acceptInvalidData( m_b_acceptInvalidData );
	reader.setContentHandler( m_xffp_parser );
	reader.parse( source );
	
}

void NMGGLoaderThread::kill()
{
	m_xffp_parser->stop();
}

bool NMGGLoaderThread::isFinished()
{
	if (m_xffp_parser)
		return m_xffp_parser->isFinished();	
	else
		return false;
}

int NMGGLoaderThread::getError()
{
	if (m_xffp_parser)
		return m_xffp_parser->getError();
	else
		return 0;
}

QString NMGGLoaderThread::getErrorString()
{
	if (m_xffp_parser)
		return m_xffp_parser->getErrorString();
	else
		return "";
}

QList<NMGMGENReadFlowData>* NMGGLoaderThread::getReadData()
{
	if (m_xffp_parser)
		return m_xffp_parser->getReadData();
	else
		return 0;
}

int NMGGLoaderThread::getPercentCompleted()
{
	if (m_xffp_parser)
		return m_xffp_parser->getPercentCompleted();
	else
		return 0;
}

QString NMGGLoaderThread::getLoadStatus()
{
	if (!m_xffp_parser)
		return "Initializing parser";
	else
		return m_xffp_parser->getLoadStatus();
}
