/***************************************************************************
 *   Copyright (C) 2006 by David Lacort Ankli  *
 *   david.lacort@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGGRAPHICOBJECT_H
#define NMGGRAPHICOBJECT_H

#include <qcolor.h>
#include <qgl.h>

#define NM_DOTS 				0x00
#define NM_POLYGONS 			0x01
#define NM_LINES 				0x02
#define NM_LINES_POLYGONS		0x03
#define NM_AREA					0x04
#define NM_LINES_AREA			0x06
#define NM_LINES_POLYGONS_AREA	0x07

/**
  * @author David Lacort Ankli
  * @brief Class to contain the data of a graphic object
*/

class NMGGraphicObject{
public:
	// The graphic modes
	enum NMVertexPolygon{VPSQUARE,VPTRIANGLE,VPRHOMB,VPINVTRIANGLE,VPNONE};

	// Constructor & destructor
    NMGGraphicObject(QString qstr_name);
    ~NMGGraphicObject();

	// Set attributes values
	void setVertexList(QPtrList<double> &da_xData, QPtrList<double> &da_yData);
	void setRepresentation(char c_representation);
	void setLineColor(QColor col_newColor);
	void setAreaColor(QColor col_newColor);
	void setHidden(bool b_hidden);
	void setAverage(bool b_average);
	void setAccumulate(bool b_accumulate);
	void setDrawZeroIfNoData(bool b_drawZero);
	void setShowLines(bool b_show);
	void setShowArea(bool b_show);
	void setShowPolygons(bool b_show);
	void setVertexPolygon(NMVertexPolygon nmvp_polygon);
	void setDotsSize(int i_dotSize);

	// Query attributes
	uint getNumVertex();
	bool first();
	bool last();
	bool next();
	bool prev();
	QString getName();
	double getCurrentX();
	double getCurrentY();
	double getMaxXValue();
	double getMaxYValue();
	double getMinXValue();
	double getMinYValue();
	QColor getLineColor();
	QColor getAreaColor();
	bool getHidden();
	bool getAverage();
	bool getAccumulate();
	bool getDrawZeroIfNoData();
	bool getShowLines();
	bool getShowArea();
	bool getShowPolygons();
	NMVertexPolygon getVertexPolygon();
	int getDotsSize();

private:
	// Definition of a dot in the space (its position)
	typedef struct NMPosition{
		double x;
		double y;
	};

	QString m_qstr_name;
	QPtrList<NMPosition> m_posl_vertexList;
	QColor m_col_lineColor;
	QColor m_col_areaColor;
	NMVertexPolygon m_nmvp_polygon;

	NMPosition m_pos_maxValues;
	NMPosition m_pos_minValues;

	bool m_b_hidden;
	bool m_b_average;
	bool m_b_accumulate;
	bool m_b_drawZeroIfNoData;
	char m_c_representation;
	int m_i_dotSize;
};

#endif
