/***************************************************************************
 *   Copyright (C) 2006 by David Lacort Ankli   *
 *   david.lacort@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmggraphicwidget.h"
#include <iostream>
#include <math.h>
#include <qfiledialog.h>
#include <qimage.h>
//#include <GL/glx.h>     // Header file fot the glx libraries.
//#include <GL/freeglut.h>     // Header file fot the glut libraries.


using namespace std;

NMGGraphicWidget::NMGGraphicWidget(QWidget *parent, char *name, QString str_fontPath)
    : QGLWidget( parent, name )
{
	m_ui_reference = 0;
	m_ui_object = 0;
	m_ui_axis = 0;
	m_ui_labels = 0;
	m_ui_legend = 0;
	m_nmggol_graphicObjects.setAutoDelete(true);

	m_d_maxCoordsRight = 0.0;
	m_d_maxCoordsTop = 0.0;
	m_d_minCoordsLeft = 0.0;
	m_d_minCoordsBottom = 0.0;

	m_d_minCoordsWidth = 0.000001;
	m_d_minCoordsHeight = 0.000001;

	m_d_topMargin = 0.0;
	m_d_rightMargin = 0.0;
	
	m_f_zoomFactor = 0.9;
	m_f_xZoom = 1.0;
	m_f_yZoom = 1.0;

	m_b_showZoomOptionsPopup = true;
	
	m_i_lastDefaultColor = 0;
	m_i_lastDefaultPolygon = 0;
	m_i_defaultDotSize = 2;
	m_col_referenceLinesColor = lightGray;
	m_i_xReferenceLinesMode = NOREFLINE;
	m_i_yReferenceLinesMode = NOREFLINE;
	m_i_numXReferenceLines = 10;
	m_i_numYReferenceLines = 10;
	m_b_showXReferenceLines = false;
	m_b_showYReferenceLines = false;
	
	m_i_legendXPos = 0;
	m_i_legendYPos = 0;
	
	m_fnt_font = QFont("Luxi Sans",7);
	m_str_fontPath = str_fontPath;

	m_ftfnt_font = 0;
	
	m_i_legendPixelsLeft = -1;
	m_i_legendPixelsTop = -1;
	m_i_legendPixelsWidth = 0;
	m_i_legendPixelsHeight = 2;
		
	m_b_showReferenceLinesOptionsPopup = true;

	m_i_idShowYLines = -1;
	m_i_idHideYLines = -1;
	m_i_idShowXLines = -1;
	m_i_idHideXLines = -1;

	m_b_shiftPressed = false;
	m_b_ctrlPressed	= false;

	setFocusPolicy(QWidget::WheelFocus);
	
	m_b_needToResize = false;

	setAcceptDrops(TRUE);
	
	m_i_currentWidth = width();
	m_i_currentHeight = height();
	m_b_resizing = false;
}

NMGGraphicWidget::~NMGGraphicWidget()
{
    makeCurrent();
    glDeleteLists( m_ui_reference , 1 );
    glDeleteLists( m_ui_object , 1 );
    glDeleteLists( m_ui_axis , 1 );
	glDeleteLists( m_ui_labels , 1 );
	glDeleteLists( m_ui_legend , 1 );
}

void NMGGraphicWidget::addGraph(QString qstr_name, QPtrList<double> &da_xData, QPtrList<double> &da_yData)
{
	// Create and append the new object in the list
	NMGGraphicObject *nmggo_graphicObject = new NMGGraphicObject(qstr_name);
	nmggo_graphicObject->setVertexList(da_xData,da_yData);
		// default values
	setDefaultColors(nmggo_graphicObject);
	nmggo_graphicObject->setAverage(false);
	nmggo_graphicObject->setAccumulate(false);
	nmggo_graphicObject->setVertexPolygon(getDefaultPolygon());
	nmggo_graphicObject->setDotsSize(m_i_defaultDotSize);

	m_nmggol_graphicObjects.prepend(nmggo_graphicObject);

	m_d_maxCoordsRight = fmax(m_d_maxCoordsRight,nmggo_graphicObject->getMaxXValue());
	m_d_maxCoordsTop = fmax(m_d_maxCoordsTop,nmggo_graphicObject->getMaxYValue());
	m_d_minCoordsLeft = (m_d_minCoordsLeft==0?nmggo_graphicObject->getMinXValue():fmin(m_d_minCoordsLeft,nmggo_graphicObject->getMinXValue()));
	m_d_minCoordsBottom = fmin(m_d_minCoordsBottom,nmggo_graphicObject->getMinYValue());
	// Some height and some width to be painted...
	if (m_d_minCoordsLeft == m_d_maxCoordsRight)
		m_d_maxCoordsRight = m_d_minCoordsLeft + m_d_minCoordsWidth;
	if (m_d_minCoordsBottom == m_d_maxCoordsTop)
		m_d_maxCoordsTop = m_d_minCoordsBottom + m_d_minCoordsHeight;

	m_d_currentCoordsWidth = m_d_maxCoordsRight - m_d_minCoordsLeft;
	m_d_currentCoordsHeight = m_d_maxCoordsTop - m_d_minCoordsBottom;
	m_d_currentCoordsLeft = m_d_minCoordsLeft;
	m_d_currentCoordsBottom = m_d_minCoordsBottom;
	m_f_xZoom = 1.0;
	m_f_yZoom = 1.0;
	
	for(int i=0; i<=qstr_name.contains('\n'); i++)
	{
		QString str_section = qstr_name.section('\n',i,i);
		m_i_legendPixelsWidth = fmaxf(m_i_legendPixelsWidth, (m_ftfnt_font->Advance(str_section)*0.8)+14+3+2+2);
		m_i_legendPixelsHeight += 8;
	}
	m_i_legendPixelsHeight += 4;
}

void NMGGraphicWidget::removeGraph(uint i_pos)
{
	QString qstr_name;
	qstr_name = m_nmggol_graphicObjects.at(i_pos)->getName();
	m_nmggol_graphicObjects.remove(i_pos);
	// recalculate max values
	if (!m_nmggol_graphicObjects.isEmpty())
	{
		calculateNewSize(false);
		recalculateAttributes();
	}
	// resize legend's box
	for(int i=0; i<=qstr_name.contains('\n'); i++)
		m_i_legendPixelsHeight -= 8;
	m_i_legendPixelsHeight -= 4;
}

void NMGGraphicWidget::setData(uint i_pos, QPtrList<double> &da_xData, QPtrList<double> &da_yData)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setVertexList(da_xData,da_yData);
//		calculateNewSize(false);
		recalculateAttributes();
	}
}

void NMGGraphicWidget::recalculateAttributes()
{
	NMGGraphicObject *nmggo_currentGraph;

	m_d_maxCoordsRight = 0.0;
	m_d_maxCoordsTop = 0.0;
	m_d_minCoordsLeft = 0.0;
	m_d_minCoordsBottom = 0.0;
	nmggo_currentGraph = m_nmggol_graphicObjects.first();
	while(nmggo_currentGraph!=0)
	{
		m_d_maxCoordsRight = fmax(m_d_maxCoordsRight,nmggo_currentGraph->getMaxXValue());
		m_d_maxCoordsTop = fmax(m_d_maxCoordsTop,nmggo_currentGraph->getMaxYValue());
		m_d_minCoordsLeft = (m_d_minCoordsLeft==0.0?nmggo_currentGraph->getMinXValue():fmin(m_d_minCoordsLeft,nmggo_currentGraph->getMinXValue()));
		m_d_minCoordsBottom = fmin(m_d_minCoordsBottom,nmggo_currentGraph->getMinYValue());
		nmggo_currentGraph = m_nmggol_graphicObjects.next();
	}
		// Some height and some width to be painted...
	if (m_d_minCoordsLeft == m_d_maxCoordsRight)
		m_d_maxCoordsRight = m_d_minCoordsLeft + m_d_minCoordsWidth;
	if (m_d_minCoordsBottom == m_d_maxCoordsTop)
		m_d_maxCoordsTop = m_d_minCoordsBottom + m_d_minCoordsHeight;
	m_d_currentCoordsWidth = m_d_maxCoordsRight - m_d_minCoordsLeft;
	m_d_currentCoordsHeight = m_d_maxCoordsTop - m_d_minCoordsBottom;
	m_d_currentCoordsLeft = m_d_minCoordsLeft;
	m_d_currentCoordsBottom = m_d_minCoordsBottom;
	m_f_xZoom = 1.0;
	m_f_yZoom = 1.0;
}


void NMGGraphicWidget::repaintScene(void)
{
	// repaint the scene
	// sometimes (if average or accumulate are set/unset) we have to recalculate max's & min's in X axis
	if (m_b_needToResize)
		calculateNewSize();
	
	m_ui_object = makeObject();		// Generate an OpenGL display list
	m_ui_reference = makeReferenceLines();
	m_ui_legend = makeLegend();
	m_ui_axis = makeAxis();
	m_ui_labels = makeLabels();
	repaintObject();
}

void NMGGraphicWidget::calculateNewSize(bool b_keepZoom)
{
	NMGGraphicObject *nmggo_currentGraph = 0;
	
	float f_old_yZoom = m_f_yZoom;
	double d_old_maxHeight = m_d_maxCoordsTop - m_d_minCoordsBottom;
	
	m_d_maxCoordsTop = 0.0;
	m_d_minCoordsBottom = 0.0;
	
	nmggo_currentGraph = m_nmggol_graphicObjects.last();
	while(nmggo_currentGraph!=0)
	{
		// if the current graph is flagged as hidden it's ignored
		if (!nmggo_currentGraph->getHidden())
		{
			if ((!nmggo_currentGraph->getAverage()) && (!nmggo_currentGraph->getAccumulate()))
			{
				m_d_maxCoordsTop = fmax(m_d_maxCoordsTop,nmggo_currentGraph->getMaxYValue());
				m_d_minCoordsBottom = fmin(m_d_minCoordsBottom,nmggo_currentGraph->getMinYValue());
			}
			else
			{
				bool b_remainingDots = nmggo_currentGraph->first();
				int i_currentPixel;
				
				if (b_remainingDots)
				{
					i_currentPixel = xCoordsToPixels(nmggo_currentGraph->getCurrentX());
					while (b_remainingDots)
					{
						double d_accumY = 0.0;
						int i_numElemsAvrg = 0;
						
						while (b_remainingDots && nmggo_currentGraph->getCurrentX()<xPixelsToCoords(i_currentPixel+1))
						{
							d_accumY += nmggo_currentGraph->getCurrentY();
							i_numElemsAvrg++;
							b_remainingDots = nmggo_currentGraph->next();
						}
						if (i_numElemsAvrg>0)
						{
							double d_pixelY;
							if (nmggo_currentGraph->getAverage())
								d_pixelY = d_accumY/(double)i_numElemsAvrg;
							else	// if (nmggo_graphicObject->getAccumalative())
								d_pixelY = d_accumY;
							
							m_d_maxCoordsTop = fmax(m_d_maxCoordsTop,d_pixelY);
							m_d_minCoordsBottom = fmin(m_d_minCoordsBottom,d_pixelY);
						}
						
						i_currentPixel++;
					}
				}
			}
		}
		
		nmggo_currentGraph = m_nmggol_graphicObjects.prev();
	}
	
	if (m_d_minCoordsBottom == m_d_maxCoordsTop)
		m_d_maxCoordsTop = m_d_minCoordsBottom + m_d_minCoordsHeight;
		
	if (b_keepZoom)
		m_f_yZoom = (float)((d_old_maxHeight * (double) f_old_yZoom) / (m_d_maxCoordsTop - m_d_minCoordsBottom));
		
	if (m_f_yZoom>1.0)
		m_f_yZoom = 1.0;
	
	if (m_d_currentCoordsHeight > (m_d_maxCoordsTop - m_d_minCoordsBottom))
		m_d_currentCoordsHeight = m_d_maxCoordsTop - m_d_minCoordsBottom;
	if ((m_d_currentCoordsBottom + m_d_currentCoordsHeight) > m_d_maxCoordsTop)
		m_d_currentCoordsBottom = m_d_maxCoordsTop - m_d_currentCoordsHeight;
	
	m_b_needToResize = false;
}

void NMGGraphicWidget::repaintObject()
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluOrtho2D(m_d_currentCoordsLeft,m_d_currentCoordsLeft+(m_d_currentCoordsWidth*(1+m_d_rightMargin)),
			   m_d_currentCoordsBottom,m_d_currentCoordsBottom+(m_d_currentCoordsHeight*(1+m_d_topMargin)));
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	repaint();
}

void NMGGraphicWidget::setZoomFactor(float f_zoomFactor)
{
	m_f_zoomFactor = f_zoomFactor;
}

void NMGGraphicWidget::enableZoomOptionsPopup(bool b_Enable)
{
	m_b_showZoomOptionsPopup = b_Enable;
}

void NMGGraphicWidget::enableReferenceLinesOptionsPopup(bool b_Enable)
{
	m_b_showReferenceLinesOptionsPopup = b_Enable;
}

void NMGGraphicWidget::zoomX(bool b_ZoomIn)
{
	float f_relativePosX, f_relativePosY;
	bool b_NeedRemakeObject = false;

	if (b_ZoomIn)
	{
		b_NeedRemakeObject = needMakeObject(false);
		
		m_f_xZoom *= m_f_zoomFactor;
		
		if (m_b_ctrlPressed || m_pm_graphOptions.isItemChecked(m_i_idKeepAspectRatio))
		{
			m_f_yZoom = m_f_xZoom;
		}
	}
	if (!b_ZoomIn && m_d_currentCoordsWidth < (m_d_maxCoordsRight - m_d_minCoordsLeft))
	{
		m_f_xZoom /= m_f_zoomFactor;
		
		if (m_f_xZoom>1.0)
			m_f_xZoom = 1.0;
		
		if (m_b_ctrlPressed || m_pm_graphOptions.isItemChecked(m_i_idKeepAspectRatio))
		{
			m_f_yZoom = m_f_xZoom;
		}
		
		b_NeedRemakeObject = needMakeObject(false);
	}

	m_d_currentCoordsWidth = (m_d_maxCoordsRight - m_d_minCoordsLeft) * m_f_xZoom;
	m_d_currentCoordsHeight = (m_d_maxCoordsTop - m_d_minCoordsBottom) * m_f_yZoom;

	if (m_d_currentCoordsWidth > (m_d_maxCoordsRight - m_d_minCoordsLeft))
		m_d_currentCoordsWidth = (m_d_maxCoordsRight - m_d_minCoordsLeft);
	if ((m_d_currentCoordsLeft + m_d_currentCoordsWidth) > m_d_maxCoordsRight)
		m_d_currentCoordsLeft = m_d_maxCoordsRight - m_d_currentCoordsWidth;
	
	if (m_d_currentCoordsHeight > (m_d_maxCoordsTop - m_d_minCoordsBottom))
		m_d_currentCoordsHeight = m_d_maxCoordsTop - m_d_minCoordsBottom;
	if ((m_d_currentCoordsBottom + m_d_currentCoordsHeight) > m_d_maxCoordsTop)
		m_d_currentCoordsBottom = m_d_maxCoordsTop - m_d_currentCoordsHeight;
	
	if (b_NeedRemakeObject)
		repaintScene();
	else
	{
		m_ui_reference = makeReferenceLines();
		m_ui_legend = makeLegend();
		m_ui_axis = makeAxis();
		m_ui_labels = makeLabels();
		repaintObject();
	}
	
	if ((m_d_maxCoordsRight - m_d_minCoordsLeft) == m_d_currentCoordsWidth)
		f_relativePosX = 0.0;
	else
		f_relativePosX = (m_d_currentCoordsLeft - m_d_minCoordsLeft) / ((m_d_maxCoordsRight - m_d_minCoordsLeft) - m_d_currentCoordsWidth);

	if ((m_d_maxCoordsTop - m_d_minCoordsBottom) == m_d_currentCoordsHeight)
		f_relativePosY = 0.0;
	else
		f_relativePosY = (m_d_currentCoordsBottom - m_d_minCoordsBottom) / ((m_d_maxCoordsTop - m_d_minCoordsBottom) - m_d_currentCoordsHeight);

	emit viewMoved(f_relativePosX,f_relativePosY);
}

void NMGGraphicWidget::zoomY(bool b_ZoomIn)
{
	float f_relativePosX, f_relativePosY;
	bool b_NeedRemakeObject = false;

	if (b_ZoomIn)
	{
		b_NeedRemakeObject = needMakeObject(true);
		
		m_f_yZoom *= m_f_zoomFactor;
		
		if (m_b_ctrlPressed || m_pm_graphOptions.isItemChecked(m_i_idKeepAspectRatio))
		{
			b_NeedRemakeObject = needMakeObject(false);
			m_f_xZoom = m_f_yZoom;
		}
	}
	if (!b_ZoomIn && m_d_currentCoordsHeight < (m_d_maxCoordsTop - m_d_minCoordsBottom))
	{
		m_f_yZoom /= m_f_zoomFactor;
		
		if (m_f_yZoom>1.0)
			m_f_yZoom = 1.0;
		
		if (m_b_ctrlPressed || m_pm_graphOptions.isItemChecked(m_i_idKeepAspectRatio))
		{
			m_f_xZoom = m_f_yZoom;
			b_NeedRemakeObject = needMakeObject(false);
		}
				
		b_NeedRemakeObject = needMakeObject(true);
	}

	m_d_currentCoordsHeight = (m_d_maxCoordsTop - m_d_minCoordsBottom) * m_f_yZoom;
	m_d_currentCoordsWidth = (m_d_maxCoordsRight - m_d_minCoordsLeft) * m_f_xZoom;
	
	if (m_d_currentCoordsHeight > (m_d_maxCoordsTop - m_d_minCoordsBottom))
		m_d_currentCoordsHeight = (m_d_maxCoordsTop - m_d_minCoordsBottom);
	if ((m_d_currentCoordsBottom + m_d_currentCoordsHeight) > m_d_maxCoordsTop)
		m_d_currentCoordsBottom = m_d_maxCoordsTop - m_d_currentCoordsHeight;
	
	if (m_d_currentCoordsWidth > (m_d_maxCoordsRight - m_d_minCoordsLeft))
		m_d_currentCoordsWidth = (m_d_maxCoordsRight - m_d_minCoordsLeft);
	if ((m_d_currentCoordsLeft + m_d_currentCoordsWidth) > m_d_maxCoordsRight)
		m_d_currentCoordsLeft = m_d_maxCoordsRight - m_d_currentCoordsWidth;
	
//	m_b_needToResize = true;
	if (b_NeedRemakeObject)
		repaintScene();
	else
	{
		m_ui_reference = makeReferenceLines();
		m_ui_legend = makeLegend();
		m_ui_axis = makeAxis();
		m_ui_labels = makeLabels();
		repaintObject();
	}
	
	if ((m_d_maxCoordsRight - m_d_minCoordsLeft) == m_d_currentCoordsWidth)
		f_relativePosX = 0.0;
	else
		f_relativePosX = (m_d_currentCoordsLeft - m_d_minCoordsLeft) / ((m_d_maxCoordsRight - m_d_minCoordsLeft) - m_d_currentCoordsWidth);

	if ((m_d_maxCoordsTop - m_d_minCoordsBottom) == m_d_currentCoordsHeight)
		f_relativePosY = 0.0;
	else
		f_relativePosY = (m_d_currentCoordsBottom - m_d_minCoordsBottom) / ((m_d_maxCoordsTop - m_d_minCoordsBottom) - m_d_currentCoordsHeight);

	emit viewMoved(f_relativePosX,f_relativePosY);
}

void NMGGraphicWidget::fitToX(void)
{
	if (m_pm_graphOptions.isItemChecked(m_i_idKeepAspectRatio))
		restore();
	else
	{
		m_f_xZoom = 1.0;
		m_d_currentCoordsWidth = m_d_maxCoordsRight - m_d_minCoordsLeft;
		m_d_currentCoordsLeft = m_d_minCoordsLeft;

		m_b_needToResize = true;

		repaintScene();
	}
}

void NMGGraphicWidget::fitToY(void)
{
	if (m_pm_graphOptions.isItemChecked(m_i_idKeepAspectRatio))
		restore();
	else
	{
		m_f_yZoom = 1.0;
		m_d_currentCoordsHeight = m_d_maxCoordsTop - m_d_minCoordsBottom;
		m_d_currentCoordsBottom = m_d_minCoordsBottom;

		m_b_needToResize = true;

		repaintScene();
	}
}

void NMGGraphicWidget::keepAspectRatio(void)
{
	m_pm_graphOptions.setItemChecked ( m_i_idKeepAspectRatio, !m_pm_graphOptions.isItemChecked ( m_i_idKeepAspectRatio) );
	// if set on, zoom out width or height to keep the aspect ratio
	if (m_pm_graphOptions.isItemChecked(m_i_idKeepAspectRatio))
	{
		float f_relativePosX, f_relativePosY;

		m_f_xZoom = fmaxf(m_f_xZoom, m_f_yZoom);
		m_f_yZoom = m_f_xZoom;
		m_d_currentCoordsWidth = (m_d_maxCoordsRight - m_d_minCoordsLeft) * m_f_xZoom;
		m_d_currentCoordsHeight = (m_d_maxCoordsTop - m_d_minCoordsBottom) * m_f_yZoom;
		if ((m_d_currentCoordsLeft + m_d_currentCoordsWidth) > m_d_maxCoordsRight)
			m_d_currentCoordsLeft = m_d_maxCoordsRight - m_d_currentCoordsWidth;
		if ((m_d_currentCoordsBottom + m_d_currentCoordsHeight) > m_d_maxCoordsTop)
			m_d_currentCoordsBottom = m_d_maxCoordsTop - m_d_currentCoordsHeight;
		
		repaintScene();

		if ((m_d_maxCoordsRight - m_d_minCoordsLeft) == m_d_currentCoordsWidth)
			f_relativePosX = 0.0;
		else
			f_relativePosX = (m_d_currentCoordsLeft - m_d_minCoordsLeft) / ((m_d_maxCoordsRight - m_d_minCoordsLeft) - m_d_currentCoordsWidth);

		if ((m_d_maxCoordsTop - m_d_minCoordsBottom) == m_d_currentCoordsHeight)
			f_relativePosY = 0.0;
		else
			f_relativePosY = (m_d_currentCoordsBottom - m_d_minCoordsBottom) / ((m_d_maxCoordsTop - m_d_minCoordsBottom) - m_d_currentCoordsHeight);

		emit viewMoved(f_relativePosX,f_relativePosY);
	}
}

void NMGGraphicWidget::restore(void)
{
	m_f_xZoom = 1.0;
	m_f_yZoom = 1.0;
	m_d_currentCoordsWidth = m_d_maxCoordsRight - m_d_minCoordsLeft;
	m_d_currentCoordsHeight = m_d_maxCoordsTop - m_d_minCoordsBottom;
	m_d_currentCoordsLeft = m_d_minCoordsLeft;
	m_d_currentCoordsBottom = m_d_minCoordsBottom;

	m_b_needToResize = true;

	repaintScene();

	emit viewMoved(0.0,0.0);
}

void NMGGraphicWidget::moveViewX(float f_relativePosition)
{
	// Calculate de real position with the relative one
	m_d_currentCoordsLeft = m_d_minCoordsLeft + (((m_d_maxCoordsRight - m_d_minCoordsLeft) - m_d_currentCoordsWidth) * f_relativePosition);

	m_ui_legend = makeLegend();
	m_ui_axis = makeAxis();
	m_ui_labels = makeLabels();
	repaintObject();
}

void NMGGraphicWidget::moveViewY(float f_relativePosition)
{
	// Calculate the real position with the relative one
	m_d_currentCoordsBottom = m_d_minCoordsBottom + (((m_d_maxCoordsTop - m_d_minCoordsBottom) - m_d_currentCoordsHeight) * f_relativePosition);

	m_ui_legend = makeLegend();
	m_ui_axis = makeAxis();
	m_ui_labels = makeLabels();
	repaintObject();
}

float NMGGraphicWidget::getRelativePosX()
/**
  @brief Calculate the window relative position
*/
{
	if ((m_d_maxCoordsRight - m_d_minCoordsLeft) == m_d_currentCoordsWidth)
		return 0;
	else
		return ((m_d_currentCoordsLeft - m_d_minCoordsLeft)/((m_d_maxCoordsRight - m_d_minCoordsLeft) - m_d_currentCoordsWidth));
}

float NMGGraphicWidget::getRelativePosY()
/**
  @brief Calculate the window relative position
*/
{
	if ((m_d_maxCoordsTop - m_d_minCoordsBottom) == m_d_currentCoordsHeight)
		return 0;
	else
		return ((m_d_currentCoordsBottom - m_d_minCoordsBottom)/((m_d_maxCoordsTop - m_d_minCoordsBottom) - m_d_currentCoordsHeight));
}

void NMGGraphicWidget::moveGraph(uint i_from, uint i_to)
{
	m_nmggol_graphicObjects.insert(i_to,m_nmggol_graphicObjects.take(i_from));

	m_ui_object = makeObject();		// Generate an OpenGL display list
	m_ui_legend = makeLegend();
	repaintObject();
}

void NMGGraphicWidget::hideGraph(uint i_pos, bool b_hidden)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setHidden(b_hidden);
		// resize legend's box
		for(int i=0; i<=m_nmggol_graphicObjects.at(i_pos)->getName().contains('\n'); i++)
			m_i_legendPixelsHeight += (b_hidden?-8:8);
		m_i_legendPixelsHeight += (b_hidden?-4:4);

		m_ui_object = makeObject();		// Generate an OpenGL display list
		m_ui_legend = makeLegend();
		repaintObject();
	}	
}

NMGGraphicObject::NMVertexPolygon NMGGraphicWidget::getDefaultPolygon()
{
	NMGGraphicObject::NMVertexPolygon vp_return;

	switch((m_i_lastDefaultPolygon++)%4)
	{
		case 0:
			vp_return = NMGGraphicObject::VPSQUARE;
			break;
		case 1:
			vp_return = NMGGraphicObject::VPTRIANGLE;
			break;
		case 2:
			vp_return = NMGGraphicObject::VPRHOMB;
			break;
		case 3:
			vp_return = NMGGraphicObject::VPINVTRIANGLE;
			break;
	}
	return vp_return;
}

void NMGGraphicWidget::setRepresentation(uint i_pos, char c_representation)
{
	if (i_pos < m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setRepresentation(c_representation);
	}
}

void NMGGraphicWidget::setDefaultColors(NMGGraphicObject *nmggo_graphicObject)
{
	QColor col_lineColor, col_areaColor;
	
	switch((m_i_lastDefaultColor++)%12)
	{
		case 0:
			col_lineColor = QColor(127,0,0);
			col_areaColor = QColor(255,0,0);
			break;
		case 1:
			col_lineColor = QColor(0,127,0);
			col_areaColor = QColor(0,255,0);
			break;
		case 2:
			col_lineColor = QColor(0,0,127);
			col_areaColor = QColor(0,0,255);
			break;
		case 3:
			col_lineColor = QColor(127,127,0);
			col_areaColor = QColor(255,255,0);
			break;
		case 4:
			col_lineColor = QColor(127,0,127);
			col_areaColor = QColor(255,0,255);
			break;
		case 5:
			col_lineColor = QColor(0,127,127);
			col_areaColor = QColor(0,255,255);
			break;
		case 6:
			col_lineColor = QColor(127,63,0);
			col_areaColor = QColor(255,127,0);
			break;
		case 7:
			col_lineColor = QColor(63,127,0);
			col_areaColor = QColor(127,255,0);
			break;
		case 8:
			col_lineColor = QColor(63,0,127);
			col_areaColor = QColor(127,0,255);
			break;
		case 9:
			col_lineColor = QColor(127,0,63);
			col_areaColor = QColor(255,0,127);
			break;
		case 10:
			col_lineColor = QColor(0,127,63);
			col_areaColor = QColor(0,255,127);
			break;
		case 11:
			col_lineColor = QColor(0,63,127);
			col_areaColor = QColor(0,127,255);
			break;
	}
	
	nmggo_graphicObject->setLineColor(col_lineColor);
	nmggo_graphicObject->setAreaColor(col_areaColor);
}

void NMGGraphicWidget::setColors(uint i_pos, QColor col_lineColor, QColor col_areaColor)
{
	if (i_pos < m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setLineColor(col_lineColor);
		m_nmggol_graphicObjects.at(i_pos)->setAreaColor(col_areaColor);
	}
}

bool NMGGraphicWidget::getColors(uint i_pos, QColor *col_lineColor, QColor *col_areaColor)
{
	if (i_pos < m_nmggol_graphicObjects.count())
	{
		*col_lineColor = m_nmggol_graphicObjects.at(i_pos)->getLineColor();
		*col_areaColor = m_nmggol_graphicObjects.at(i_pos)->getAreaColor();
		return true;
	}
	else
	{
		return false;
	}
}

void NMGGraphicWidget::setAverage(uint i_pos, bool b_averaged)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setAverage(b_averaged);
		m_b_needToResize = true;
	}	
}

void NMGGraphicWidget::setAccumulate(uint i_pos, bool b_accumulate)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setAccumulate(b_accumulate);
		m_b_needToResize = true;
	}	
}

void NMGGraphicWidget::setDrawZeroIfNoData(uint i_pos, bool b_drawZeroIfNoData)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setDrawZeroIfNoData(b_drawZeroIfNoData);
	}	
}

void NMGGraphicWidget::setShowPolygons(uint i_pos, bool b_show)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setShowPolygons(b_show);
	}	
}

void NMGGraphicWidget::setShowLines(uint i_pos, bool b_show)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setShowLines(b_show);
	}	
}

void NMGGraphicWidget::setShowAreas(uint i_pos, bool b_show)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setShowArea(b_show);
	}	
}

void NMGGraphicWidget::setPolygons(uint i_pos, NMGGraphicObject::NMVertexPolygon vp_polygon)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setVertexPolygon(vp_polygon);
	}	
}

void NMGGraphicWidget::setDefaultDotsSize(int i_pixels)
{
	m_i_defaultDotSize = i_pixels;
}

void NMGGraphicWidget::setDotsSize(uint i_pos, int i_pixels)
{
	if (i_pos<m_nmggol_graphicObjects.count())
	{
		m_nmggol_graphicObjects.at(i_pos)->setDotsSize(i_pixels);
	}	
}

void NMGGraphicWidget::paintGL()
{
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glLoadIdentity();

	glCallList( m_ui_reference );
	glCallList( m_ui_object );
	glCallList( m_ui_legend );
	paintReferenceLabels();
}

void NMGGraphicWidget::paintReferenceLabels()
{	
	// and now paint the reference values
	float f_interval;

	// the Y labels
	glViewport( 0, 0, (GLint)width(), (GLint)height() );

	if (m_i_yReferenceLinesMode==FIXEDREFLINE)
		f_interval = (m_d_maxCoordsTop - m_d_minCoordsBottom)/(m_i_numYReferenceLines);
	else
		f_interval = m_d_currentCoordsHeight/(m_i_numYReferenceLines); 
	
	double d_maxTopWithMargin = m_d_maxCoordsTop + ((m_d_maxCoordsTop - m_d_minCoordsBottom) * m_d_topMargin);
	double d_maxRightWithMargin = m_d_maxCoordsRight + ((m_d_maxCoordsRight - m_d_minCoordsLeft) * m_d_rightMargin);
	
	int i_numDecimalDigits = decimalDigits(f_interval);
	qglColor( "black" );
	QString str;
	for(float f_i=ceilf(m_d_currentCoordsBottom/f_interval); (f_i*f_interval)<d_maxTopWithMargin; f_i+=1.0)
	{
		str = QString( "%1" ).arg( f_i*f_interval, 0, 'f', i_numDecimalDigits );
		renderText ( m_ftfnt_font->FaceSize(), height()-m_i_xLabelsMargin-yCoordsToPixels(f_i*f_interval-m_d_currentCoordsBottom) , str, m_fnt_font);
	}

	// and the X labels
	if (m_i_xReferenceLinesMode==FIXEDREFLINE)
		f_interval = (m_d_maxCoordsRight - m_d_minCoordsLeft)/(m_i_numXReferenceLines);
	else
		f_interval = m_d_currentCoordsWidth/(m_i_numXReferenceLines); 

	i_numDecimalDigits = decimalDigits(f_interval);
	for(float f_i=ceilf(m_d_currentCoordsLeft/f_interval); (f_i*f_interval)<d_maxRightWithMargin; f_i+=1.0)
	{
		str = QString( "%1" ).arg( f_i*f_interval, 0, 'f', i_numDecimalDigits );
		renderText ( m_i_yLabelsMargin+xCoordsToPixels(f_i*f_interval-m_d_currentCoordsLeft), height()-(m_i_xLabelsMargin-1)+(m_fnt_font.pointSize()+1) , str, m_fnt_font);
	}

	// the x and y axis
	glCallList(m_ui_axis);	

	// the labels
/*
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective( 90, (float)width() / (float)height(), 1, 1000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
//	gluLookAt( 0.0, 0.0, (float)height() / 2.0f, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);
*/
	glCallList( m_ui_labels);

	glViewport( m_i_yLabelsMargin, m_i_xLabelsMargin, (GLint)width()-m_i_yLabelsMargin, (GLint)height()-m_i_xLabelsMargin );

}

int NMGGraphicWidget::decimalDigits(float f_interval)
{
	int i_result = (int)floorf(log10f(f_interval));
	if (i_result >= 0)
		return 0;
	else
		return abs(i_result);
}

void NMGGraphicWidget::setXAxisLabel(QString str_label)
{
	m_str_XAxisLabel = str_label;
}

void NMGGraphicWidget::setYAxisLabel(QString str_label)
{
	m_str_YAxisLabel = str_label;
}

void NMGGraphicWidget::setGraphLabel(QString str_label)
{
	m_str_GraphLabel = str_label;
}

GLvoid NMGGraphicWidget::buildFont(GLvoid)		// Build Our Texture Font
{
	if (!m_str_fontPath.isEmpty())
	{
		if (m_ftfnt_font!=0)
			delete m_ftfnt_font;
		m_ftfnt_font = new FTGLOutlineFont( m_str_fontPath);
//		m_ftfnt_font = new FTGLPolygonFont( m_str_fontPath);
//		m_ftfnt_font = new FTGLExtrdFont( m_str_fontPath);
		if( m_ftfnt_font->Error())
		{	
			cerr << "Failed to open font " << m_str_fontPath << endl;
			return;
		}
			
		if( !m_ftfnt_font->FaceSize(12))
		{
			cerr << "Failed to set size" << endl;;
			return;
		}
		
//		m_ftfnt_font->Depth(1);
			
		m_ftfnt_font->CharMap(ft_encoding_unicode);
		
		// Load chars to optimize graph drawing
		GLuint glui_list;
		glui_list = glGenLists( 1 );
		glNewList( glui_list, GL_COMPILE );
		glPushMatrix();
		glColor3f( 0.0, 0.0, 0.0);
		m_ftfnt_font->Render( "\\!\"#$%&/()='^*+[]{},.;:-_<>1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZçñÇÑ" );
		glPopMatrix();
		glEndList();
	}
}

void NMGGraphicWidget::setFontFile(QString str_fontPath)
{
	m_str_fontPath = str_fontPath;
	buildFont();

	// calculate new legend's box width
	m_i_legendPixelsWidth = 0;
	NMGGraphicObject *nmggo_currentGraph = m_nmggol_graphicObjects.first();
	while(nmggo_currentGraph!=0)
	{
		QString qstr_name = nmggo_currentGraph->getName();

		for(int i=0; i<=qstr_name.contains('\n'); i++)
		{
			QString str_section = qstr_name.section('\n',i,i);
			m_i_legendPixelsWidth = fmaxf(m_i_legendPixelsWidth, (m_ftfnt_font->Advance(str_section)*0.8)+14+3+2+2);
		}
		nmggo_currentGraph = m_nmggol_graphicObjects.next();
	}

	m_ui_legend = makeLegend();
	m_ui_labels = makeLabels();
	repaintObject();
}

void NMGGraphicWidget::initializeGL()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    qglClearColor( white ); 		// Let OpenGL clear to white
	qglColor( black );

	buildFont();
    
	glShadeModel( GL_FLAT );
}

//  Set up the OpenGL view port, matrix mode, etc.
void NMGGraphicWidget::resizeGL( int i_width, int i_height )
{
	m_b_resizing = true;
	glViewport( m_i_yLabelsMargin, m_i_xLabelsMargin, (GLint)i_width-m_i_yLabelsMargin, (GLint)i_height-m_i_xLabelsMargin );
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();
	gluOrtho2D(m_d_currentCoordsLeft,m_d_currentCoordsLeft+(m_d_currentCoordsWidth*(1+m_d_rightMargin)),
			   m_d_currentCoordsBottom,m_d_currentCoordsBottom+(m_d_currentCoordsHeight*(1+m_d_topMargin)));
    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    m_ui_object = makeObject();		// Generate an OpenGL display list
	m_ui_reference = makeReferenceLines();
	m_ui_legend = makeLegend();
	m_ui_axis = makeAxis();
	m_ui_labels = makeLabels();
	
	m_b_resizing = false;
	m_i_currentWidth = i_width;
	m_i_currentHeight = i_height;
}


bool NMGGraphicWidget::needMakeObject(bool b_onlyPolygons)
{
	NMGGraphicObject *nmggo_currentGraph;
	bool bNeeded = false;

	nmggo_currentGraph = m_nmggol_graphicObjects.last();
	while(nmggo_currentGraph!=0 && !bNeeded)
	{
		if (nmggo_currentGraph->getShowPolygons()) // else polygons will be drawn distorted
			bNeeded = true;

		if (!b_onlyPolygons)
		{
			if (nmggo_currentGraph->getShowLines() || nmggo_currentGraph->getShowArea())
			{
				// only if we're drawing an average or accumulation line we have to make again the object.
				if (nmggo_currentGraph->getAverage() || nmggo_currentGraph->getAccumulate())
				{
					bNeeded = true;
					m_b_needToResize = true;
				}
			}
		}
		nmggo_currentGraph = m_nmggol_graphicObjects.prev();
	}
	return bNeeded;
}


void NMGGraphicWidget::setReferenceLines(QColor col_color, NMRefrenceLinesMode rlm_yMode, int i_numYLines, NMRefrenceLinesMode rlm_xMode, int i_numXLines, int i_yLabelsMargin, int i_xLabelsMargin, float f_graphRightMargin, float f_graphTopMargin)
{
	m_col_referenceLinesColor = col_color;
	m_i_yReferenceLinesMode = rlm_yMode;
	m_i_numYReferenceLines = i_numYLines;
	m_i_xReferenceLinesMode = rlm_xMode;
	m_i_numXReferenceLines = i_numXLines;
	m_i_yLabelsMargin = i_yLabelsMargin;
	m_i_xLabelsMargin = i_xLabelsMargin;
	m_d_rightMargin = f_graphRightMargin;
	m_d_topMargin = f_graphTopMargin;

	// Create the popup options menu
	createPopupOptions();
	enableDisablePopupOptions();
}

void NMGGraphicWidget::createPopupOptions()
{
	m_pm_graphOptions.clear();
	m_pm_zoomOptions.clear();

	if (m_b_showReferenceLinesOptionsPopup)
	{
		if (m_i_yReferenceLinesMode!=NOREFLINE)
		{
			m_i_idShowYLines = m_pm_graphOptions.insertItem("Show Y lines",this,SLOT(showYLines()),0,1);
			m_i_idHideYLines = m_pm_graphOptions.insertItem("Hide Y lines",this,SLOT(hideYLines()),0,2);
			m_b_showYReferenceLines = true;
		}
		if (m_i_yReferenceLinesMode!=NOREFLINE && m_i_xReferenceLinesMode!=NOREFLINE)
			m_pm_graphOptions.insertSeparator();
		if (m_i_xReferenceLinesMode!=NOREFLINE)
		{
			m_i_idShowXLines = m_pm_graphOptions.insertItem("Show X lines",this,SLOT(showXLines()),0,4);
			m_i_idHideXLines = m_pm_graphOptions.insertItem("Hide X lines",this,SLOT(hideXLines()),0,5);
			m_b_showXReferenceLines = true;
		}
	}
	if (m_b_showReferenceLinesOptionsPopup && m_b_showZoomOptionsPopup)
		m_pm_graphOptions.insertSeparator();
	if (m_b_showZoomOptionsPopup)
	{
		m_i_idFitToX = m_pm_zoomOptions.insertItem("Fit to X",this,SLOT(fitToX()),0,1);
		m_i_idFitToY = m_pm_zoomOptions.insertItem("Fit to Y",this,SLOT(fitToY()),0,2);
		m_i_idKeepAspectRatio = m_pm_zoomOptions.insertItem("Keep aspect ratio",this,SLOT(keepAspectRatio()),0,3);
		m_i_idRestore = m_pm_zoomOptions.insertItem("Restore",this,SLOT(restore()),0,4);
		
		m_i_idZoom = m_pm_graphOptions.insertItem("Zoom",&m_pm_zoomOptions,0,7);
	}
	if (m_b_showReferenceLinesOptionsPopup || m_b_showZoomOptionsPopup)
		m_pm_graphOptions.insertSeparator();
	m_pm_graphOptions.insertItem("Select graphic font",this,SLOT(setGraphicFont()),0,8);
	m_pm_graphOptions.insertItem("Save to PNG",this,SLOT(saveToPNG()),0,9);
	m_pm_graphOptions.insertItem("Save to JPEG",this,SLOT(saveToJPEG()),0,10);
}


void NMGGraphicWidget::mouseMoveEvent( QMouseEvent* e )
{
	if( m_b_mousePressed)
	{
		int i_newLeft = m_i_legendOldPixelsLeft+(e->pos().x()-m_p_presspos.x());
		int i_newTop = m_i_legendOldPixelsTop + (m_p_presspos.y() - e->pos().y());
		
		if (i_newLeft < 2)
			m_i_legendPixelsLeft = 2;
		else if ((i_newLeft+m_i_legendPixelsWidth) > (width()-m_i_yLabelsMargin-2))
			m_i_legendPixelsLeft = width()-m_i_yLabelsMargin-m_i_legendPixelsWidth-2;
		else
			m_i_legendPixelsLeft = i_newLeft;
		
		if ((i_newTop-m_i_legendPixelsHeight) < 2)
			m_i_legendPixelsTop = m_i_legendPixelsHeight+2;
		else if (i_newTop > (height()-m_i_xLabelsMargin-2))
			m_i_legendPixelsTop = height()-m_i_xLabelsMargin-2;
		else
			m_i_legendPixelsTop = i_newTop;
		
		m_ui_legend = makeLegend();
		repaintObject();
	}
}

void NMGGraphicWidget::mouseReleaseEvent( QMouseEvent * )
{
	m_b_mousePressed = FALSE;
}

void NMGGraphicWidget::mousePressEvent ( QMouseEvent * e )
{
	if (e->button() == LeftButton)
	{
		if (m_i_legendPixelsLeft==-1)
		{
			m_i_legendPixelsLeft = xCoordsToPixels((m_d_currentCoordsWidth)-xPixelsToCoords(m_i_legendPixelsWidth));
			m_i_legendPixelsTop = yCoordsToPixels(m_d_currentCoordsHeight);
		}
		m_i_legendOldPixelsLeft = m_i_legendPixelsLeft;
		m_i_legendOldPixelsTop = m_i_legendPixelsTop;
		QPoint p = e->pos();
		
		if(p.x()>=(m_i_yLabelsMargin+m_i_legendPixelsLeft) && p.x()<=(m_i_yLabelsMargin+m_i_legendPixelsLeft+m_i_legendPixelsWidth) && p.y()>= (height()-m_i_xLabelsMargin-m_i_legendPixelsTop )&& p.y()<=(height()-m_i_xLabelsMargin-m_i_legendPixelsTop+m_i_legendPixelsHeight))
		{
			m_p_presspos = e->pos();
			m_b_mousePressed = TRUE;
		}
	}
	else if (e->button() == RightButton)
		m_pm_graphOptions.popup(e->globalPos());
	e->accept();
}

void NMGGraphicWidget::keyPressEvent ( QKeyEvent * e )
{
	if (e->key()==Qt::Key_Shift)
		m_b_shiftPressed = true;
	else if (e->key()==Qt::Key_Control)
		m_b_ctrlPressed = true;
	e->accept();
}
		
void NMGGraphicWidget::keyReleaseEvent ( QKeyEvent * e )
{
	if (e->key()==Qt::Key_Shift)
		m_b_shiftPressed = false;
	else if (e->key()==Qt::Key_Control)
		m_b_ctrlPressed = false;
	e->accept();
}

void NMGGraphicWidget::wheelEvent ( QWheelEvent * e )
{
	float f_relativePosX, f_relativePosY;
	// 10 wheel notches per current width
	if (!m_b_shiftPressed)
	{
		if (!m_b_ctrlPressed)
		{
			m_d_currentCoordsLeft += (m_d_currentCoordsWidth/10) * (e->delta()/120);
			if (m_d_currentCoordsLeft < m_d_minCoordsLeft)
				m_d_currentCoordsLeft = m_d_minCoordsLeft;
			if (m_d_currentCoordsLeft > (m_d_maxCoordsRight - m_d_currentCoordsWidth))
				m_d_currentCoordsLeft = (m_d_maxCoordsRight - m_d_currentCoordsWidth);
		}
		else	// Zoom with aspect ratio
		{
			zoomX(e->delta()>0);
		}
	}
	else
	{
		m_d_currentCoordsBottom += (m_d_currentCoordsHeight/10) * (e->delta()/120);
		if (m_d_currentCoordsBottom < m_d_minCoordsBottom)
			m_d_currentCoordsBottom = m_d_minCoordsBottom;
		if (m_d_currentCoordsBottom > (m_d_maxCoordsTop - m_d_currentCoordsHeight))
			m_d_currentCoordsBottom = (m_d_maxCoordsTop - m_d_currentCoordsHeight);
	}
	e->accept();
	m_ui_legend = makeLegend();
	m_ui_axis = makeAxis();
	m_ui_labels = makeLabels();
	repaintObject();

	if ((m_d_maxCoordsRight - m_d_minCoordsLeft) == m_d_currentCoordsWidth)
		f_relativePosX = 0.0;
	else
		f_relativePosX = (m_d_currentCoordsLeft - m_d_minCoordsLeft) / ((m_d_maxCoordsRight - m_d_minCoordsLeft) - m_d_currentCoordsWidth);

	if ((m_d_maxCoordsTop - m_d_minCoordsBottom) == m_d_currentCoordsHeight)
		f_relativePosY = 0.0;
	else
		f_relativePosY = (m_d_currentCoordsBottom - m_d_minCoordsBottom) / ((m_d_maxCoordsTop - m_d_minCoordsBottom) - m_d_currentCoordsHeight);

	emit viewMoved(f_relativePosX,f_relativePosY);
}

void NMGGraphicWidget::showYLines(void)
{
	m_b_showYReferenceLines = true;
	enableDisablePopupOptions();
	m_ui_reference = makeReferenceLines();
	repaintObject();
}
void NMGGraphicWidget::hideYLines(void)
{
	m_b_showYReferenceLines = false;
	enableDisablePopupOptions();
	m_ui_reference = makeReferenceLines();
	repaintObject();
}
void NMGGraphicWidget::showXLines(void)
{
	m_b_showXReferenceLines = true;
	enableDisablePopupOptions();
	m_ui_reference = makeReferenceLines();
	repaintObject();
}
void NMGGraphicWidget::hideXLines(void)
{
	m_b_showXReferenceLines = false;
	enableDisablePopupOptions();
	m_ui_reference = makeReferenceLines();
	repaintObject();
}

void NMGGraphicWidget::enableDisablePopupOptions(void)
{
	if (m_i_yReferenceLinesMode!=NOREFLINE)
	{	
		m_pm_graphOptions.setItemEnabled(m_i_idShowYLines,!m_b_showYReferenceLines);
		m_pm_graphOptions.setItemEnabled(m_i_idHideYLines,m_b_showYReferenceLines);
	}
	if (m_i_xReferenceLinesMode!=NOREFLINE)
	{	
		m_pm_graphOptions.setItemEnabled(m_i_idShowXLines,!m_b_showXReferenceLines);
		m_pm_graphOptions.setItemEnabled(m_i_idHideXLines,m_b_showXReferenceLines);
	}
}

GLuint NMGGraphicWidget::makeLabels()
{
    GLuint glui_list;

    glui_list = glGenLists( 1 );

    glNewList( glui_list, GL_COMPILE );

	glEnable( GL_TEXTURE_2D);

	glPushMatrix();

    qglColor( black );		      // Shorthand for glColor3f or glIndex
//glColor3f( 0.0, 0.0, 0.0);

	// graph label
	if (m_ftfnt_font!=0 && m_str_GraphLabel && m_str_GraphLabel!="")
	{
		glTranslatef(m_d_currentCoordsLeft+xPixelsToCoords(m_i_yLabelsMargin)+(m_d_currentCoordsWidth/2),m_d_currentCoordsBottom+m_d_currentCoordsHeight,0.0);
	    glScalef(xPixelsToCoords(1), yPixelsToCoords(1), 1.0f);
		QString str_halfString = m_str_GraphLabel.left(m_str_GraphLabel.length()/2);
		glTranslatef(m_ftfnt_font->Advance((char*)str_halfString.latin1())*(-1.0),0.0,0.0);
		m_ftfnt_font->Render( (char*)m_str_GraphLabel.latin1() );
	}
	glPopMatrix();

	// X axis label
	glPushMatrix();
	if (m_ftfnt_font!=0 && m_str_XAxisLabel && m_str_XAxisLabel!="")
	{
		glTranslatef(m_d_currentCoordsLeft+xPixelsToCoords(m_i_yLabelsMargin)+(m_d_currentCoordsWidth/2),m_d_currentCoordsBottom+yPixelsToCoords(2),0.0);
	    glScalef(xPixelsToCoords(1)*0.8f, yPixelsToCoords(1)*0.8f, 1.0f);
		QString str_halfString = m_str_XAxisLabel.left(m_str_XAxisLabel.length()/2);
		glTranslatef(m_ftfnt_font->Advance((char*)str_halfString.latin1())*(-1.0),0.0,0.0);
		m_ftfnt_font->Render( (char*)m_str_XAxisLabel.latin1() );
	}
	glPopMatrix();

	// Y axis label
	glPushMatrix();
			glEnable( GL_TEXTURE_2D);

	if (m_ftfnt_font!=0 && m_str_YAxisLabel && m_str_YAxisLabel!="")
	{
		glTranslatef(m_d_currentCoordsLeft+xPixelsToCoords(m_ftfnt_font->FaceSize()*0.8f-1), m_d_currentCoordsBottom+yPixelsToCoords(m_i_xLabelsMargin)+(m_d_currentCoordsHeight/2),0.0);
	    glScalef(xPixelsToCoords(1)*0.8f, yPixelsToCoords(1)*0.8f, 1.0f);
		QString str_halfString = m_str_YAxisLabel.left(m_str_YAxisLabel.length()/2);
		glTranslatef(0.0,m_ftfnt_font->Advance((char*)str_halfString.latin1())*(-1.0),0.0);
		glRotatef(90.0,0.0,0.0,1.0);
		m_ftfnt_font->Render( (char*)m_str_YAxisLabel.latin1() );
	}
	glPopMatrix();

    glEndList();

    return glui_list;
}

GLvoid NMGGraphicWidget::makeSquare(GLenum mode, double d_left, double d_top, double d_width, double d_height)
{
	glBegin(mode);
	
	glVertex2d(d_left,d_top);
	glVertex2d(d_left+d_width,d_top);
	glVertex2d(d_left+d_width,d_top-d_height);
	glVertex2d(d_left,d_top-d_height);
	if (mode==GL_LINE_STRIP)
		glVertex2d(d_left,d_top);
	
	glEnd();
}

GLuint NMGGraphicWidget::makeLegend()
{
    GLuint glui_list;
	NMGGraphicObject *nmggo_currentGraph;

	glui_list = glGenLists( 1 );

    glNewList( glui_list, GL_COMPILE );

	glPushMatrix();

	if (m_i_legendPixelsLeft==-1)
	{
		glTranslatef(m_d_currentCoordsLeft+(m_d_currentCoordsWidth)-xPixelsToCoords(m_i_legendPixelsWidth),m_d_currentCoordsBottom+m_d_currentCoordsHeight,0.0);
	}
	else
	{
		if (m_b_resizing)
		{
			// reposition the legend
			double d_oldWidth = m_i_currentWidth-m_i_yLabelsMargin;
			double d_newWidth = width()-m_i_yLabelsMargin;
			int i_newLeft = (int)round(((double)m_i_legendPixelsLeft+m_i_legendPixelsWidth)*(d_newWidth/d_oldWidth))-m_i_legendPixelsWidth;
			
			double d_oldHeight = m_i_currentHeight-m_i_xLabelsMargin;
			double d_newHeight = height()-m_i_xLabelsMargin;
			int i_newTop = (int)round(((double)m_i_legendPixelsTop)*(d_newHeight/d_oldHeight));
		
			if (i_newLeft < 2)
				m_i_legendPixelsLeft = 2;
			else if ((i_newLeft+m_i_legendPixelsWidth) > (width()-m_i_yLabelsMargin-2))
				m_i_legendPixelsLeft = width()-m_i_yLabelsMargin-m_i_legendPixelsWidth-2;
			else
				m_i_legendPixelsLeft = i_newLeft;
		
			if ((i_newTop-m_i_legendPixelsHeight) < 2)
				m_i_legendPixelsTop = m_i_legendPixelsHeight+2;
			else if (i_newTop > (height()-m_i_xLabelsMargin-2))
				m_i_legendPixelsTop = height()-m_i_xLabelsMargin-2;
			else
				m_i_legendPixelsTop = i_newTop;
		}
		
		glTranslatef(m_d_currentCoordsLeft+xPixelsToCoords(m_i_legendPixelsLeft),m_d_currentCoordsBottom+yPixelsToCoords(m_i_legendPixelsTop),0.0);
	}
	
	// first of all, the square
	qglColor(white);
	makeSquare(GL_QUADS, 0.0, 0.0, xPixelsToCoords(m_i_legendPixelsWidth), yPixelsToCoords(m_i_legendPixelsHeight));
	
	qglColor(black);
	makeSquare(GL_LINE_STRIP, 0.0, 0.0, xPixelsToCoords(m_i_legendPixelsWidth), yPixelsToCoords(m_i_legendPixelsHeight));
	
	// Draw the legend's content
	nmggo_currentGraph = m_nmggol_graphicObjects.first();
	int i = 0;
	int i_currentPixelHeight = 2;
	while(nmggo_currentGraph!=0)
	{
		if (!nmggo_currentGraph->getHidden())
		{
			// if area drawn
			if (nmggo_currentGraph->getShowArea())
			{
				qglColor(nmggo_currentGraph->getAreaColor());
				makeSquare(GL_QUADS, xPixelsToCoords(2), -yPixelsToCoords(i_currentPixelHeight+5),
						xPixelsToCoords(14), yPixelsToCoords(5));
			}
			// if lines drawn
			if (nmggo_currentGraph->getShowLines())
			{
				glLineWidth(2.0);
				qglColor(nmggo_currentGraph->getLineColor());
				glBegin(GL_LINES);
				glVertex2d(xPixelsToCoords(2),-yPixelsToCoords(i_currentPixelHeight+5));
				glVertex2d(xPixelsToCoords(2+14),-yPixelsToCoords(i_currentPixelHeight+5));
				glEnd();
				glLineWidth(1.0);
			}
			if (nmggo_currentGraph->getShowPolygons())
				makePolygon(xPixelsToCoords(2+7), -yPixelsToCoords(i_currentPixelHeight+5), nmggo_currentGraph);
			if (!nmggo_currentGraph->getShowArea() && !nmggo_currentGraph->getShowLines() && !nmggo_currentGraph->getShowPolygons())
			{
				glPointSize(nmggo_currentGraph->getDotsSize());
				qglColor(nmggo_currentGraph->getLineColor());
				glBegin(GL_POINTS);						// Drawing Using Lines
				glVertex2d(xPixelsToCoords(2+7),-yPixelsToCoords(i_currentPixelHeight+5));
				glEnd();
			}
			qglColor(black);
/*			
			makeSquare(GL_LINE_STRIP, xPixelsToCoords(2), -yPixelsToCoords(i_currentPixelHeight),
					xPixelsToCoords(14), yPixelsToCoords(10));
*/			
			for(int i_section=0; i_section<=nmggo_currentGraph->getName().contains('\n'); i_section++)
			{
				i_currentPixelHeight+=8;
				glPushMatrix();
				QString str_section = nmggo_currentGraph->getName().section('\n',i_section,i_section);
				glTranslatef(xPixelsToCoords(2+14+2),-yPixelsToCoords(i_currentPixelHeight),0.0);
				glScalef(xPixelsToCoords(1)*0.8f, yPixelsToCoords(1)*0.8f, 1.0f);
				m_ftfnt_font->Render( (char*)str_section.latin1() );
				glPopMatrix();
			}
			i_currentPixelHeight+=4;
		}	// end if (!nmggo_currentGraph->getHidden())
		
		nmggo_currentGraph = m_nmggol_graphicObjects.next();
		i++;
	}
	
	glPopMatrix();
	
	glEndList();
	
	return glui_list;
}

GLuint NMGGraphicWidget::makeAxis()
{
    GLuint glui_list;

    glui_list = glGenLists( 1 );

    glNewList( glui_list, GL_COMPILE );

    qglColor( black );		      // Shorthand for glColor3f or glIndex
	glBegin(GL_LINES);

	// the axis will be drawn in the whole printable area
	double d_maxTopWithMargin = m_d_maxCoordsTop + ((m_d_maxCoordsTop - m_d_minCoordsBottom) * m_d_topMargin);
	double d_maxRightWithMargin = m_d_maxCoordsRight + ((m_d_maxCoordsRight - m_d_minCoordsLeft) * m_d_rightMargin);
	
	// x axis
	glVertex2d(xPixelsToCoords(m_i_yLabelsMargin-1)+m_d_currentCoordsLeft,yPixelsToCoords(m_i_xLabelsMargin-1)+m_d_currentCoordsBottom);
	glVertex2d(d_maxRightWithMargin,yPixelsToCoords(m_i_xLabelsMargin-1)+m_d_currentCoordsBottom);
	// y axis
	glVertex2d(xPixelsToCoords(m_i_yLabelsMargin-1)+m_d_currentCoordsLeft,yPixelsToCoords(m_i_xLabelsMargin-1)+m_d_currentCoordsBottom);
	glVertex2d(xPixelsToCoords(m_i_yLabelsMargin-1)+m_d_currentCoordsLeft,d_maxTopWithMargin);

	glEnd();

    glEndList();

    return glui_list;
}

GLuint NMGGraphicWidget::makeReferenceLines()
{
    GLuint glui_list;

    glui_list = glGenLists( 1 );

    glNewList( glui_list, GL_COMPILE );

    qglColor( m_col_referenceLinesColor );		      // Shorthand for glColor3f or glIndex
	glBegin(GL_LINES);

	// Y reference lines
	float f_interval;

	if (m_i_yReferenceLinesMode==VARIABLEREFLINE)
		f_interval = m_d_currentCoordsHeight/(m_i_numYReferenceLines); 
	else if (m_i_yReferenceLinesMode==FIXEDREFLINE)
		f_interval = (m_d_maxCoordsTop-m_d_minCoordsBottom)/(m_i_numYReferenceLines);
	
	double d_maxTopWithMargin = m_d_maxCoordsTop + ((m_d_maxCoordsTop - m_d_minCoordsBottom) * m_d_topMargin);
	double d_maxRightWithMargin = m_d_maxCoordsRight + ((m_d_maxCoordsRight - m_d_minCoordsLeft) * m_d_rightMargin);

	// Draw the lines to the top margin
	for(float f_i=ceilf(m_d_minCoordsBottom/f_interval) ; (f_i*f_interval)<d_maxTopWithMargin ; f_i+=1.0)
	{
		if (m_i_yReferenceLinesMode!=NOREFLINE && m_b_showYReferenceLines)
		{
			if (f_i==0)
			    qglColor( black );		      // Shorthand for glColor3f or glIndex

			glVertex2d(m_d_minCoordsLeft,f_i*f_interval);
			glVertex2d(d_maxRightWithMargin,f_i*f_interval);

			if (f_i==0)
			    qglColor( m_col_referenceLinesColor );		      // Shorthand for glColor3f or glIndex
		}
	}

	if (m_i_xReferenceLinesMode==VARIABLEREFLINE)
		f_interval = m_d_currentCoordsWidth/(m_i_numXReferenceLines); 
	else if (m_i_xReferenceLinesMode==FIXEDREFLINE)
		f_interval = (m_d_maxCoordsRight-m_d_minCoordsLeft)/(m_i_numXReferenceLines);
	
	// Draw the lines from 0 to the right margin
	for(float f_i=ceilf(m_d_minCoordsLeft/f_interval) ; (f_i*f_interval)<d_maxRightWithMargin ; f_i+=1.0)
	{
		if (m_i_xReferenceLinesMode!=NOREFLINE && m_b_showXReferenceLines)
		{
			if (f_i==0)
			    qglColor( black );		      // Shorthand for glColor3f or glIndex

			glVertex2d(f_i*f_interval, m_d_minCoordsBottom);
			glVertex2d(f_i*f_interval, d_maxTopWithMargin);

			if (f_i==0)
			    qglColor( m_col_referenceLinesColor );		      // Shorthand for glColor3f or glIndex
		}
	}
	
	glEnd();

    glEndList();

    return glui_list;
}

/*!
  Generate an OpenGL display list for the object to be shown
*/
GLuint NMGGraphicWidget::makeObject()
{	
    GLuint glui_list;
	NMGGraphicObject *nmggo_currentGraph;

    glui_list = glGenLists( 1 );

    glNewList( glui_list, GL_COMPILE );

	// Draw the graphs
	nmggo_currentGraph = m_nmggol_graphicObjects.last();
	while(nmggo_currentGraph!=0)
	{
		// if the current graph is flagged as hidden it's ignored
		if (!nmggo_currentGraph->getHidden())
			makeGraphicObject(nmggo_currentGraph);
		nmggo_currentGraph = m_nmggol_graphicObjects.prev();
	}
	glEndList();
	
	return glui_list;
}

int NMGGraphicWidget::xCoordsToPixels(double d_coord)
{
/*
	coordsToPixels(f_coord) == f_coord*f_dotsPerCoord(i)/f_dotsPerPixel(i)
							== f_coord*(f_dotsPerCoord(i)/(f_drawedDots(i)/(width()-m_i_yLabelsMargin)))
							== f_coord*(f_dotsPerCoord(i)/(f_dotsPerCoord(i)*m_d_currentCoordsWidth/(width()-m_i_yLabelsMargin)))
							== f_coord*((f_dotsPerCoord(i)*(width()-m_i_yLabelsMargin))/(f_dotsPerCoord(i)*m_d_currentCoordsWidth))
							== f_coord*((width()-m_i_yLabelsMargin)/m_d_currentCoordsWidth))
*/
	return (int)(d_coord*(width()-m_i_yLabelsMargin)/(m_d_currentCoordsWidth*(1+m_d_rightMargin)));
}

int NMGGraphicWidget::yCoordsToPixels(double d_coord)
{
	return (int)(d_coord*(height()-m_i_xLabelsMargin)/(m_d_currentCoordsHeight*(1+m_d_topMargin)));
}

double NMGGraphicWidget::xPixelsToCoords(int i_pixel)
{
/*
	pixelsToCoords(i_pixel) == i_pixel*f_dotsPerPixel(i)/f_dotsPerCoord(i)
							== i_pixel*((f_drawedDots(i)/(width()-m_i_yLabelsMargin))/f_dotsPerCoord(i))
							== i_pixel*((f_dotsPerCoord(i)*m_d_currentCoordsWidth/(width()-m_i_yLabelsMargin))/f_dotsPerCoord(i))
							== i_pixel*((f_dotsPerCoord(i)*m_d_currentCoordsWidth)/(f_dotsPerCoord(i)*(width()-m_i_yLabelsMargin)))
							== i_pixel*(m_d_currentCoordsWidth/(width()-m_i_yLabelsMargin)))
*/
	return (double)(((double)i_pixel)*(m_d_currentCoordsWidth*(1+m_d_rightMargin))/(double)(width()-m_i_yLabelsMargin));
}

double NMGGraphicWidget::yPixelsToCoords(int i_pixel)
{
	return (double)(((double)i_pixel)*(m_d_currentCoordsHeight*(1+m_d_topMargin))/(double)(height()-m_i_xLabelsMargin));
}

void NMGGraphicWidget::makeLineObject(NMGGraphicObject *nmggo_graphicObject)
{
	qglColor(nmggo_graphicObject->getLineColor());

	glBegin(GL_LINE_STRIP);						// drawing using lines
		// if avrg_line flag is not setted draw the line directly or window width is not setted yet
		if ((!nmggo_graphicObject->getAverage()) && (!nmggo_graphicObject->getAccumulate()))
		{
			if (nmggo_graphicObject->first())
			{
				int i_lastDrawnPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
				glVertex2d(nmggo_graphicObject->getCurrentX(),0.0);
				do 
				{
					if (nmggo_graphicObject->getDrawZeroIfNoData())
					{
						if (i_lastDrawnPixel < (xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1) )
						{
							glVertex2d(xPixelsToCoords(i_lastDrawnPixel+1),0.0);
						}
						
						if (i_lastDrawnPixel < (xCoordsToPixels(nmggo_graphicObject->getCurrentX())-2) )
						{
							glVertex2d(xPixelsToCoords(xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1),0.0);
						}
					
						i_lastDrawnPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
					}
					
					glVertex2d(nmggo_graphicObject->getCurrentX(),nmggo_graphicObject->getCurrentY());
				}
				while(nmggo_graphicObject->next());
				
				nmggo_graphicObject->last();
				glVertex2d(nmggo_graphicObject->getCurrentX(),0.0);
			}
		}
		else  // calculate the average or accumulated position of each interval represented in one x axis' pixel
		{
			int i_currentPixel;
			bool b_remainingDots = nmggo_graphicObject->first();
			if (b_remainingDots)
			{
				glVertex2d( nmggo_graphicObject->getCurrentX() , 0.0 );
				i_currentPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
				int i_lastDrawnPixel = i_currentPixel;
				while (b_remainingDots)
				{
					double d_accumX = 0.0;
					double d_accumY = 0.0;
					int i_numElemsAvrg = 0;
					
					if (nmggo_graphicObject->getDrawZeroIfNoData())
					{
						if (i_lastDrawnPixel < (xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1) )
						{
							glVertex2d(xPixelsToCoords(i_lastDrawnPixel+1),0.0);
						}
						
						if (i_lastDrawnPixel < (xCoordsToPixels(nmggo_graphicObject->getCurrentX())-2) )
						{
							glVertex2d(xPixelsToCoords(xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1),0.0);
						}
					
						i_lastDrawnPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
					}
					
					while (b_remainingDots && nmggo_graphicObject->getCurrentX()<xPixelsToCoords(i_currentPixel+1))
					{
						d_accumX += nmggo_graphicObject->getCurrentX();
						d_accumY += nmggo_graphicObject->getCurrentY();
						i_numElemsAvrg++;
						b_remainingDots = nmggo_graphicObject->next();
					}
					if (i_numElemsAvrg>0)
					{
						double d_pixelX;
						double d_pixelY;
						if (nmggo_graphicObject->getAverage())
						{
							d_pixelX = d_accumX/(float)i_numElemsAvrg;
							d_pixelY = d_accumY/(float)i_numElemsAvrg;
						}
						else	// if (nmggo_graphicObject->getAccumalative())
						{
							d_pixelX = d_accumX/(float)i_numElemsAvrg;
							d_pixelY = d_accumY;
						}
						
						glVertex2d( d_pixelX , d_pixelY );
					}
					
					i_currentPixel++;
				}
				nmggo_graphicObject->last();
				glVertex2d( nmggo_graphicObject->getCurrentX() , 0.0 );
			}
		}
	glEnd();							// finished drawing the line
	
}

void NMGGraphicWidget::makeAreaObject(NMGGraphicObject *nmggo_graphicObject)
{
	// ------------------------- Areas representation --------------------------------
	qglColor(nmggo_graphicObject->getAreaColor());
	glBegin(GL_QUAD_STRIP);						// Drawing Using Lines
	
		if ((!nmggo_graphicObject->getAverage()) && (!nmggo_graphicObject->getAccumulate()))
		{
			if (nmggo_graphicObject->first())
			{
				int i_lastDrawnPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
				do
				{
					
					if (nmggo_graphicObject->getDrawZeroIfNoData())
					{
						if (i_lastDrawnPixel < (xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1) )
						{
							glVertex2d(xPixelsToCoords(i_lastDrawnPixel+1),0.0);
							glVertex2d(xPixelsToCoords(i_lastDrawnPixel+1),0.0);
						}
						
						if (i_lastDrawnPixel < (xCoordsToPixels(nmggo_graphicObject->getCurrentX())-2) )
						{
							glVertex2d(xPixelsToCoords(xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1),0.0);
							glVertex2d(xPixelsToCoords(xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1),0.0);
						}
						
						i_lastDrawnPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
					}
					
					glVertex2d(nmggo_graphicObject->getCurrentX(),0.0);
					glVertex2d(nmggo_graphicObject->getCurrentX(),nmggo_graphicObject->getCurrentY());
				}
				while(nmggo_graphicObject->next());
				if (nmggo_graphicObject->getNumVertex()<2)	// need at least 2 vertexs to paint the area
				{
					nmggo_graphicObject->last();
					glVertex2d(nmggo_graphicObject->getCurrentX(),0.0);
					glVertex2d(nmggo_graphicObject->getCurrentX(),nmggo_graphicObject->getCurrentY());
				}
			}
		}
		else  // Calculate the average or accumulated position of each interval represented in one X axis' pixel
		{
			int i_currentPixel;
			bool bRemainingDots = nmggo_graphicObject->first();
			if (bRemainingDots)
			{
				glVertex2d(nmggo_graphicObject->getCurrentX(),0.0);
//				glVertex2d(nmggo_graphicObject->getCurrentX(),nmggo_graphicObject->getCurrentY());
				glVertex2d(nmggo_graphicObject->getCurrentX(),0.0);
				i_currentPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
				int i_lastDrawnPixel = i_currentPixel;
				while (bRemainingDots)
				{
					double d_accumX = 0.0;
					double d_accumY = 0.0;
					int i_numElemsAvrg = 0;
					
					if (nmggo_graphicObject->getDrawZeroIfNoData())
					{
						if (i_lastDrawnPixel < (xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1) )
						{
							glVertex2d(xPixelsToCoords(i_lastDrawnPixel+1),0.0);
							glVertex2d(xPixelsToCoords(i_lastDrawnPixel+1),0.0);
						}
						
						if (i_lastDrawnPixel < (xCoordsToPixels(nmggo_graphicObject->getCurrentX())-2) )
						{
							glVertex2d(xPixelsToCoords(xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1),0.0);
							glVertex2d(xPixelsToCoords(xCoordsToPixels(nmggo_graphicObject->getCurrentX())-1),0.0);
						}
						
						i_lastDrawnPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
					}
					
					while (bRemainingDots && nmggo_graphicObject->getCurrentX()<xPixelsToCoords(i_currentPixel+1))
					{
						d_accumX += nmggo_graphicObject->getCurrentX();
						d_accumY += nmggo_graphicObject->getCurrentY();
						i_numElemsAvrg++;
						bRemainingDots = nmggo_graphicObject->next();
					}
					if (i_numElemsAvrg>0)
					{
						double d_pixelX;
						double d_pixelY;
						if (nmggo_graphicObject->getAverage())
						{
							d_pixelX = d_accumX/(float)i_numElemsAvrg;
							d_pixelY = d_accumY/(float)i_numElemsAvrg;
						}
						else	// if (nmggo_graphicObject->getAccumalative())
						{
							d_pixelX = d_accumX/(float)i_numElemsAvrg;
							d_pixelY = d_accumY;
						}
						
						glVertex2d(d_pixelX,0.0);
						glVertex2d(d_pixelX,d_pixelY);
					}
					
					i_currentPixel++;
				}
				nmggo_graphicObject->last();
				glVertex2d(nmggo_graphicObject->getCurrentX(),0.0);
				glVertex2d(nmggo_graphicObject->getCurrentX(),0.0);
//				glVertex2d(nmggo_graphicObject->getCurrentX(),nmggo_graphicObject->getCurrentY());
			}
		}
	
	glEnd();							// Finished Drawing The Line
}

void NMGGraphicWidget::makePolygon(double d_coordX, double d_coordY, NMGGraphicObject *nmggo_graphicObject)
{
	switch(nmggo_graphicObject->getVertexPolygon())
	{
		case NMGGraphicObject::VPSQUARE:
			qglColor(nmggo_graphicObject->getAreaColor());
			glBegin(GL_QUADS);						
			glVertex2d(d_coordX+xPixelsToCoords(3),d_coordY+yPixelsToCoords(3));
			glVertex2d(d_coordX+xPixelsToCoords(3),d_coordY-yPixelsToCoords(3));
			glVertex2d(d_coordX-xPixelsToCoords(3),d_coordY-yPixelsToCoords(3));
			glVertex2d(d_coordX-xPixelsToCoords(3),d_coordY+yPixelsToCoords(3));
			glEnd();
			qglColor(nmggo_graphicObject->getLineColor());
			glBegin(GL_LINE_LOOP);
			glVertex2d(d_coordX+xPixelsToCoords(3),d_coordY+yPixelsToCoords(3));
			glVertex2d(d_coordX+xPixelsToCoords(3),d_coordY-yPixelsToCoords(3));
			glVertex2d(d_coordX-xPixelsToCoords(3),d_coordY-yPixelsToCoords(3));
			glVertex2d(d_coordX-xPixelsToCoords(3),d_coordY+yPixelsToCoords(3));
			glEnd();
			break;
		case NMGGraphicObject::VPTRIANGLE:
			qglColor(nmggo_graphicObject->getAreaColor());
			glBegin(GL_TRIANGLES);
			glVertex2d(d_coordX,d_coordY+yPixelsToCoords(4));
			glVertex2d(d_coordX+xPixelsToCoords(3),d_coordY-yPixelsToCoords(2));
			glVertex2d(d_coordX-xPixelsToCoords(3),d_coordY-yPixelsToCoords(2));
			glEnd();
			qglColor(nmggo_graphicObject->getLineColor());
			glBegin(GL_LINE_LOOP);
			glVertex2d(d_coordX,d_coordY+yPixelsToCoords(4));
			glVertex2d(d_coordX+xPixelsToCoords(3),d_coordY-yPixelsToCoords(2));
			glVertex2d(d_coordX-xPixelsToCoords(3),d_coordY-yPixelsToCoords(2));
			glEnd();
			break;
		case NMGGraphicObject::VPRHOMB:
			qglColor(nmggo_graphicObject->getAreaColor());
			glBegin(GL_QUADS);						
			glVertex2d(d_coordX,d_coordY+yPixelsToCoords(4));
			glVertex2d(d_coordX+xPixelsToCoords(4),d_coordY);
			glVertex2d(d_coordX,d_coordY-yPixelsToCoords(4));
			glVertex2d(d_coordX-xPixelsToCoords(4),d_coordY);
			glEnd();
			qglColor(nmggo_graphicObject->getLineColor());
			glBegin(GL_LINE_LOOP);
			glVertex2d(d_coordX,d_coordY+yPixelsToCoords(4));
			glVertex2d(d_coordX+xPixelsToCoords(4),d_coordY);
			glVertex2d(d_coordX,d_coordY-yPixelsToCoords(4));
			glVertex2d(d_coordX-xPixelsToCoords(4),d_coordY);
			glEnd();
			break;
		case NMGGraphicObject::VPINVTRIANGLE:
			qglColor(nmggo_graphicObject->getAreaColor());
			glBegin(GL_TRIANGLES);
			glVertex2d(d_coordX+xPixelsToCoords(3),d_coordY+yPixelsToCoords(2));
			glVertex2d(d_coordX,d_coordY-yPixelsToCoords(4));
			glVertex2d(d_coordX-xPixelsToCoords(3),d_coordY+yPixelsToCoords(2));
			glEnd();
			qglColor(nmggo_graphicObject->getLineColor());
			glBegin(GL_LINE_LOOP);
			glVertex2d(d_coordX+xPixelsToCoords(3),d_coordY+yPixelsToCoords(2));
			glVertex2d(d_coordX,d_coordY-yPixelsToCoords(4));
			glVertex2d(d_coordX-xPixelsToCoords(3),d_coordY+yPixelsToCoords(2));
			glEnd();
			break;
		default:
			// don't make any polygon
			break;
	}
}

void NMGGraphicWidget::makePolygonsObject(NMGGraphicObject *nmggo_graphicObject)
{
	if ((!nmggo_graphicObject->getAverage()) && (!nmggo_graphicObject->getAccumulate()))
	{
		if (nmggo_graphicObject->first())
		{
			do
			{
				makePolygon(nmggo_graphicObject->getCurrentX(), nmggo_graphicObject->getCurrentY(), nmggo_graphicObject);
			}
			while(nmggo_graphicObject->next());
		}
	}
	else   // calculate the average or accumulated position of each interval represented in one x axis' pixel 
	{   
		int i_currentPixel;
		bool b_remainingDots = nmggo_graphicObject->first();
		if (b_remainingDots)
		{
			double d_maxCoordsTop = 0.0;
			i_currentPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
			while (b_remainingDots)
			{
				double d_accumX = 0.0;
				double d_accumY = 0.0;
				int i_numElemsAvrg = 0;
				while (b_remainingDots && nmggo_graphicObject->getCurrentX()<xPixelsToCoords(i_currentPixel+1))
				{
					d_accumX += nmggo_graphicObject->getCurrentX();
					d_accumY += nmggo_graphicObject->getCurrentY();
					i_numElemsAvrg++;
					b_remainingDots = nmggo_graphicObject->next();
				}
				if (i_numElemsAvrg>0)
				{
					double d_pixelX;
					double d_pixelY;
					if (nmggo_graphicObject->getAverage())
					{
						d_pixelX = d_accumX/(double)i_numElemsAvrg;
						d_pixelY = d_accumY/(double)i_numElemsAvrg;
					}
					else	// if (nmggo_graphicObject->getAccumalative())
					{
						d_pixelX = d_accumX/(double)i_numElemsAvrg;
						d_pixelY = d_accumY;
						d_maxCoordsTop = fmax(d_maxCoordsTop,d_pixelY);
					}
					
					makePolygon(d_pixelX , d_pixelY, nmggo_graphicObject);
				}
				i_currentPixel++;
			}
		}
	}
}

void NMGGraphicWidget::makeDotsObject(NMGGraphicObject *nmggo_graphicObject)
{
	glPointSize(nmggo_graphicObject->getDotsSize());
	qglColor(nmggo_graphicObject->getLineColor());
	glBegin(GL_POINTS);						// Drawing Using Lines
	// if avrg flag is not setted draw the dots directly
	if ((!nmggo_graphicObject->getAverage()) && (!nmggo_graphicObject->getAccumulate()))
	{
		if (nmggo_graphicObject->first())
		{
			do glVertex2d(nmggo_graphicObject->getCurrentX(),nmggo_graphicObject->getCurrentY());
			while(nmggo_graphicObject->next());
		}
	}
	else  // calculate the average or accumulated position of each interval represented in one x axis' pixel
	{
		int i_currentPixel;
		bool b_remainingDots = nmggo_graphicObject->first();
		if (b_remainingDots)
		{
			i_currentPixel = xCoordsToPixels(nmggo_graphicObject->getCurrentX());
			while (b_remainingDots)
			{
				double d_accumX = 0.0;
				double d_accumY = 0.0;
				int i_numElemsAvrg = 0;
				while (b_remainingDots && nmggo_graphicObject->getCurrentX()<xPixelsToCoords(i_currentPixel+1))
				{
					d_accumX += nmggo_graphicObject->getCurrentX();
					d_accumY += nmggo_graphicObject->getCurrentY();
					i_numElemsAvrg++;
					b_remainingDots = nmggo_graphicObject->next();
				}
				if (i_numElemsAvrg>0)
				{
					double d_pixelX;
					double d_pixelY;
					if (nmggo_graphicObject->getAverage())
					{
						d_pixelX = d_accumX/(float)i_numElemsAvrg;
						d_pixelY = d_accumY/(float)i_numElemsAvrg;
					}
					else	// if (nmggo_graphicObject->getAccumalative())
					{
						d_pixelX = d_accumX/(float)i_numElemsAvrg;
						d_pixelY = d_accumY;
					}
					
					glVertex2d( d_pixelX , d_pixelY );
				}
				i_currentPixel++;
			}
		}
	}
	glEnd();							// finished drawing the line
}

void NMGGraphicWidget::makeGraphicObject(NMGGraphicObject *nmggo_graphicObject)
{
	if (nmggo_graphicObject->getShowArea())
		makeAreaObject(nmggo_graphicObject);
	if (nmggo_graphicObject->getShowLines())
		makeLineObject(nmggo_graphicObject);
	if (nmggo_graphicObject->getShowPolygons())
		makePolygonsObject(nmggo_graphicObject);

	if (!nmggo_graphicObject->getShowArea() && !nmggo_graphicObject->getShowLines() && !nmggo_graphicObject->getShowPolygons())
		makeDotsObject(nmggo_graphicObject);
}

void NMGGraphicWidget::saveToPNG()
{
	QImage img_screenshot;
	QString str_fileName;

 	str_fileName = QFileDialog::getSaveFileName(
                    QString::null,
                    "PNG Image (*.png)",
                    this,
                    "save image dialog",
                    "Save graph as PNG");
  
	img_screenshot = grabFrameBuffer();
	img_screenshot.save(str_fileName,"PNG");
}

void NMGGraphicWidget::saveToJPEG()
{
	QImage img_screenshot;
	QString str_fileName;

 	str_fileName = QFileDialog::getSaveFileName(
                    QString::null,
                    "JPEG Image (*.jpg *.jpeg)",
                    this,
                    "save image dialog",
                    "Save graph as JPEG");
  
	img_screenshot = grabFrameBuffer();
	img_screenshot.save(str_fileName,"JPEG");
}

void NMGGraphicWidget::setGraphicFont()
{
	emit selectGraphicFont();
}

#include "nmggraphicwidget.moc"
