/***************************************************************************
 *   Copyright (C) 2006 by David Lacort Ankli   *
 *   david.lacort@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGGRAPHICWIDGET_H
#define NMGGRAPHICWIDGET_H

#include "nmggraphicobject.h"
#include <qgl.h>
#include <qptrlist.h>
#include <qpopupmenu.h>
#include <FTGL/FTGLTextureFont.h>
#include <FTGL/FTGLPolygonFont.h>
#include <FTGL/FTGLPixmapFont.h>
#include <FTGL/FTGLOutlineFont.h>
#include <FTGL/FTGLExtrdFont.h>
//#include <freetype/ft2build.h>

#define MAX_OBJECTS 10

class NMGGraphicWidget: public QGLWidget
{
    Q_OBJECT

public:
    NMGGraphicWidget( QWidget *parent = 0 , char *name = 0 , QString str_fontPath = "");
    ~NMGGraphicWidget();

	void setFontFile(QString str_fontPath);

	void addGraph(QString qstr_name, QPtrList<double> &da_xData, QPtrList<double> &da_yData);
	void removeGraph(uint i_pos);

	void setData(uint i_pos, QPtrList<double> &da_xData, QPtrList<double> &da_yData);

	void setRepresentation(uint i_pos, char c_representation);
	void setColors(uint i_pos, QColor col_lineColor, QColor col_areaColor);
	bool getColors(uint i_pos, QColor *col_lineColor, QColor *col_areaColor);
	void setAverage(uint i_pos, bool b_averaged);
	void setAccumulate(uint i_pos, bool b_accumulate);
	void setDrawZeroIfNoData(uint i_pos, bool b_drawZeroIfNoData);
	void setShowPolygons(uint i_pos, bool b_show);
	void setShowLines(uint i_pos, bool b_show);
	void setShowAreas(uint i_pos, bool b_show);
	void setPolygons(uint i_pos, NMGGraphicObject::NMVertexPolygon vp_polygon);
	void setDefaultDotsSize(int i_pixels);
	void setDotsSize(uint i_pos, int i_pixels);
	void setXAxisLabel(QString str_label);
	void setYAxisLabel(QString str_label);
	void setGraphLabel(QString str_label);

	void setZoomFactor(float f_zoomFactor);
	void enableZoomOptionsPopup(bool b_enable);
	void enableReferenceLinesOptionsPopup(bool b_enable);
	void zoomX(bool b_zoomIn);
	void zoomY(bool b_zoomIn);
	void moveViewX(float f_relativePosition);
	void moveViewY(float f_relativePosition);

	void repaintScene();

	float getRelativePosX();
	float getRelativePosY();

	void moveGraph(uint i_from, uint i_to);

	void hideGraph(uint i_pos, bool b_hidden);

	enum NMRefrenceLinesMode{NOREFLINE,FIXEDREFLINE,VARIABLEREFLINE};
	void setReferenceLines(QColor col_color=lightGray, NMRefrenceLinesMode rlm_yMode=NOREFLINE, int i_numYLines=10, NMRefrenceLinesMode rlm_xMode=NOREFLINE, int i_numXLines=10, int i_yLabelsMargin=20, int i_xLabelsMargin=20, float f_graphRightMargin=0.05, float f_graphTopMargin=0.05);

	QPixmap getSample(int i_itemPos);

public slots:
	void showYLines(void);
	void hideYLines(void);
	void showXLines(void);
	void hideXLines(void);
	void fitToX(void);
	void fitToY(void);
	void keepAspectRatio(void);
	void restore(void);
	void saveToPNG(void);
	void saveToJPEG(void);
	void setGraphicFont(void);

signals:
	void viewMoved(float,float);
	void selectGraphicFont(void);

protected:

	void initializeGL();
	void resizeGL(int i_width, int i_height);
	void paintGL();

	void mousePressEvent ( QMouseEvent * e );
	void wheelEvent ( QWheelEvent * e );
	void keyPressEvent ( QKeyEvent * e );
	void keyReleaseEvent ( QKeyEvent * e );

    void contentsDragEnterEvent( QDragEnterEvent *e );
    void contentsDragMoveEvent( QDragMoveEvent *e );
    void contentsDragLeaveEvent( QDragLeaveEvent *e );
    void contentsDropEvent( QDropEvent *e );
    void mouseMoveEvent( QMouseEvent *e );
    void mouseReleaseEvent( QMouseEvent *e );

private:
	GLuint makeReferenceLines();
	GLuint makeObject();
	GLuint makeAxis();
	GLuint makeLabels();
	GLuint makeLegend();
	GLvoid makeSquare(GLenum mode, double d_left, double d_top, double d_width, double d_height);
	void setDefaultColors(NMGGraphicObject *nmggo_graphicObject);
	void makeGraphicObject(NMGGraphicObject *nmggo_graphicObject);
	void repaintObject();
	void makeLineObject(NMGGraphicObject *nmggo_graphicObject);
	void makeAreaObject(NMGGraphicObject *nmggo_graphicObject);
	void makeDotsObject(NMGGraphicObject *nmggo_graphicObject);
	void makePolygonsObject(NMGGraphicObject *nmggo_graphicObject);
	void makePolygon(double d_coordX, double d_coordY, NMGGraphicObject *nmggo_graphicObject);
	bool needMakeObject(bool b_onlyPolygons);

	int xCoordsToPixels(double f_coord);
	int yCoordsToPixels(double f_coord);
	double xPixelsToCoords(int i_pixel);
	double yPixelsToCoords(int i_pixel);

	void paintReferenceLabels(void);
	int decimalDigits(float f_interval);
	void enableDisablePopupOptions(void);
	void recalculateAttributes();

    GLuint m_ui_reference;
    GLuint m_ui_object;
	GLuint m_ui_axis;
	GLuint m_ui_labels;
	GLuint m_ui_legend;

	double m_d_minCoordsWidth;
	double m_d_maxCoordsRight;
	double m_d_minCoordsLeft;
	double m_d_currentCoordsLeft;
	double m_d_currentCoordsWidth;

	double m_d_minCoordsHeight;
	double m_d_maxCoordsTop;
	double m_d_minCoordsBottom;
	double m_d_currentCoordsBottom;
	double m_d_currentCoordsHeight;

	float m_f_zoomFactor;
	float m_f_xZoom;
	float m_f_yZoom;

	int m_i_xLabelsMargin;
	int m_i_yLabelsMargin;
	double m_d_rightMargin;
	double m_d_topMargin;

	QPtrList<NMGGraphicObject> m_nmggol_graphicObjects;

	QFont m_fnt_font;
	QString m_str_XAxisLabel;
	QString m_str_YAxisLabel;
	QString m_str_GraphLabel;

	int m_i_lastDefaultColor;
	int m_i_lastDefaultPolygon;
	int m_i_defaultDotSize;
	NMGGraphicObject::NMVertexPolygon getDefaultPolygon();
	QColor m_col_referenceLinesColor;
	NMRefrenceLinesMode m_i_xReferenceLinesMode;
	NMRefrenceLinesMode m_i_yReferenceLinesMode;
	int m_i_numXReferenceLines;
	int m_i_numYReferenceLines;
	bool m_b_showXReferenceLines;
	bool m_b_showYReferenceLines;

	int m_i_legendXPos;
	int m_i_legendYPos;

	// Popup options menu
	QPopupMenu m_pm_graphOptions;
	QPopupMenu m_pm_zoomOptions;
	int m_i_idShowYLines;
	int m_i_idHideYLines;
	int m_i_idShowXLines;
	int m_i_idHideXLines;
	int m_i_idFitToX; 
	int m_i_idFitToY; 
	int m_i_idKeepAspectRatio;
	int m_i_idRestore;
	int m_i_idZoom;

	void createPopupOptions();
	bool m_b_showReferenceLinesOptionsPopup;
	bool m_b_showZoomOptionsPopup;

	bool m_b_shiftPressed;
	bool m_b_ctrlPressed;

	bool m_b_needToResize;
	void calculateNewSize(bool b_keepZoom=true);

	GLvoid glPrint(char *text);
	GLvoid buildFont(GLvoid);

	QString m_str_fontPath;
	FTFont* m_ftfnt_font;

	int m_i_legendPixelsLeft;
	int m_i_legendPixelsTop;
	int m_i_legendPixelsWidth;
	int m_i_legendPixelsHeight;

    int m_i_legendOldPixelsLeft;
	int m_i_legendOldPixelsTop;
    QPoint m_p_presspos;
    bool m_b_mousePressed;

	int m_i_currentWidth;
	int m_i_currentHeight;
	bool m_b_resizing;
};

#endif
