/***************************************************************************
 *   Copyright (C) 2005 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmggraphwidget.h"

#include <qnamespace.h>
#include <qgroupbox.h>
#include <qlistview.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qcursor.h>
#include <qcolordialog.h>
#include <qcombobox.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qaction.h>
#include <qmessagebox.h>
#include <qfiledialog.h>

#include <math.h>
#include <iostream>

#include "../nmgmoduleutils.h"
#include "nmgmgengraphdataloader.h"
#include <nmgpreferencesmanager.h>
#include <messagebar.h>

using namespace std;

NMGGraphWidget::NMGGraphWidget(QWidget *parent, const char *name, NMGModulePreferencesManager * prefMan)
 : NMGGraphWidgetBase(parent, name)
{
	preferencesManager = prefMan;

	m_pm_flowAdminMenu = 0;
	m_tb_toolBar = 0;

	m_gw_TheGraph = 0;
	m_flv_FlowList = 0;

	m_gdl_dataLoader = 0;

	// TODO: Initialize as NONE and get the value from the data file
	m_selectedModule = MGEN;

	createFlowAdminMenu();

	timer = new QTimer(this);
	connect( timer, SIGNAL(timeout()), this, SLOT(updateProgress()) );
}


NMGGraphWidget::~NMGGraphWidget()
{
	delete m_flv_FlowList;

	if (m_gw_TheGraph != 0)
		delete m_gw_TheGraph;
}

void NMGGraphWidget::paintEvent ( QPaintEvent * )
{
	if (m_gw_TheGraph!=0)
	{
		m_gw_TheGraph->setGeometry(1,1,gbGraphContainer->width()-2,gbGraphContainer->height()-2);
		m_flv_FlowList->setGeometry(5,25,gbFlowList->width()-10,gbFlowList->height()-30);
	}
}

void NMGGraphWidget::setModuleType(moduleType selectedModule)
{
	m_selectedModule = selectedModule;
}

void NMGGraphWidget::createFlowAdminMenu()
{
	m_pm_flowAdminMenu = new QPopupMenu(this,"flowAdmin");
	m_pm_flowAdminMenu->insertItem("Add flow",this,SLOT(addFlow()),CTRL+Key_N);
	m_pm_flowAdminMenu->insertItem("&Delete selected flow",this,SLOT(deleteFlow()),CTRL+Key_D);
	m_pm_flowAdminMenu->insertItem("&Delete all flows",this,SLOT(deleteAllFlows()),SHIFT+CTRL+Key_D);
	m_pm_flowAdminMenu->insertSeparator();
	
	m_pm_graphTypeMenu = new QPopupMenu(this,"graphType");
	
	m_i_idMenuGraphType = m_pm_flowAdminMenu->insertItem("Graph type",m_pm_graphTypeMenu);
	m_pm_flowAdminMenu->setItemEnabled(m_i_idMenuGraphType,false);
	
	
	m_str_graphFontPath = preferencesManager->getValue("GraphicRenderFont");
	
	if (m_str_graphFontPath.isEmpty())
	{
		QMessageBox::warning(0,"Render font not configured","Please, select a graphic representation's font (*.ttf) ");
		selectGraphicFont();
	}
	
}

void NMGGraphWidget::selectGraphicFont()
{
	m_str_graphFontPath = QFileDialog::getOpenFileName("/usr/share/fonts/", "TrueType Font (*.ttf)", this, "render font dialog", "Select render font");
	preferencesManager->addValue("GraphicRenderFont",m_str_graphFontPath);
	if(m_gw_TheGraph!=0)
		m_gw_TheGraph->setFontFile(m_str_graphFontPath);
}

/**
 * Slot which adds a new flow
 */
void NMGGraphWidget::addFlow()
{
	if (m_flv_FlowList==0)
	{
		m_flv_FlowList = new NMGFlowListView(gbFlowList,"Flow List");
		m_flv_FlowList->addColumn("Global Id",0);
		m_flv_FlowList->addColumn("Name");
		m_flv_FlowList->addColumn("Rate");
		m_flv_FlowList->addColumn("Size");
		m_flv_FlowList->addColumn("Source");
		m_flv_FlowList->addColumn("Destination");
		m_flv_FlowList->addColumn("File");
		m_flv_FlowList->addColumn("Id");
		m_flv_FlowList->setGeometry(5,25,gbFlowList->width()-10,gbFlowList->height()-30);
		m_flv_FlowList->setAllColumnsShowFocus(TRUE);
		m_flv_FlowList->show();
		m_flv_FlowList->setEnabled(true);

		connect(m_flv_FlowList,SIGNAL(itemMove(uint,uint)),this,SLOT(itemMove(uint,uint)));
		connect(m_flv_FlowList,SIGNAL(itemHide(uint,bool)),this,SLOT(itemHide(uint,bool)));
		connect(m_flv_FlowList,SIGNAL(itemDelete(uint)),this,SLOT(itemDelete(uint)));
		connect(m_flv_FlowList,SIGNAL(itemChangeLineColor(uint)),this,SLOT(itemChangeLineColor(uint)));
		connect(m_flv_FlowList,SIGNAL(itemChangeAreaColor(uint)),this,SLOT(itemChangeAreaColor(uint)));
		connect(m_flv_FlowList,SIGNAL(itemAverage(uint,bool)),this,SLOT(itemAverage(uint,bool)));
		connect(m_flv_FlowList,SIGNAL(itemAccumulate(uint,bool)),this,SLOT(itemAccumulate(uint,bool)));
		connect(m_flv_FlowList,SIGNAL(itemShowPolygons(uint,bool)),this,SLOT(itemShowPolygons(uint,bool)));
		connect(m_flv_FlowList,SIGNAL(itemShowLines(uint,bool)),this,SLOT(itemShowLines(uint,bool)));
		connect(m_flv_FlowList,SIGNAL(itemShowArea(uint,bool)),this,SLOT(itemShowArea(uint,bool)));
		connect(m_flv_FlowList,SIGNAL(itemPolygonSquare(uint)),this,SLOT(itemPolygonSquare(uint)));
		connect(m_flv_FlowList,SIGNAL(itemPolygonTriangle(uint)),this,SLOT(itemPolygonTriangle(uint)));
		connect(m_flv_FlowList,SIGNAL(itemPolygonRhomb(uint)),this,SLOT(itemPolygonRhomb(uint)));
		connect(m_flv_FlowList,SIGNAL(itemPolygonInvertedTriangle(uint)),this,SLOT(itemPolygonInvertedTriangle(uint)));
		connect(m_flv_FlowList,SIGNAL(itemDotSize(uint,int)),this,SLOT(itemDotSize(uint,int)));
	}
	
	if (m_gw_TheGraph==0)
	{
		m_gw_TheGraph = new NMGGraphicWidget(gbGraphContainer, "graphic widget", m_str_graphFontPath);
		m_gw_TheGraph->setReferenceLines(lightGray, NMGGraphicWidget::VARIABLEREFLINE, 10, NMGGraphicWidget::VARIABLEREFLINE, 10, 36, 20);

		// Could not has the correct size, so we set it
		m_gw_TheGraph->setGeometry(1,1,gbGraphContainer->width()-2,gbGraphContainer->height()-2);

		connect(m_gw_TheGraph,SIGNAL(viewMoved(float,float)),this,SLOT(viewMoved(float,float)));
		connect(m_gw_TheGraph,SIGNAL(selectGraphicFont(void)),this,SLOT(selectGraphicFont(void)));

		slMoveXView->setEnabled(true);
		btnXZoomIn->setEnabled(true);
		btnXZoomOut->setEnabled(true);
		slMoveYView->setEnabled(true);
		btnYZoomIn->setEnabled(true);
		btnYZoomOut->setEnabled(true);
	}
	
	loadGraphData();
}

/**
 * Slot which deletes the selected item (if any)
 */
void NMGGraphWidget::deleteFlow()
{
	if (m_flv_FlowList!=0 && m_flv_FlowList->childCount()!=0 && m_flv_FlowList->selectedItemIndex()>=0)
	{
		itemDelete(m_flv_FlowList->selectedItemIndex());
		if (m_flv_FlowList->childCount()==0)
			deactivateGraph();
	}
	else 
		QMessageBox::warning(0,"No flow selected","You have to select something to delete");
}

void NMGGraphWidget::deactivateGraph()
{
	// Activate sliders and zoom buttons
	slMoveXView->setEnabled(false);
	btnXZoomIn->setEnabled(false);
	btnXZoomOut->setEnabled(false);
	slMoveYView->setEnabled(false);
	btnYZoomIn->setEnabled(false);
	btnYZoomOut->setEnabled(false);
	
	delete m_gw_TheGraph;
	m_gw_TheGraph = 0;
}

void NMGGraphWidget::deleteAllFlows()
{
	if (m_flv_FlowList==0 || m_flv_FlowList->childCount()==0)
	{
		return;
	}
	else
	{
		int reply = QMessageBox::QMessageBox::question(0,"Confirm operation",
				"Do you really want to delete all flows?", QMessageBox::Yes|QMessageBox::Default,
				QMessageBox::No|QMessageBox::Escape);
		if(reply==QMessageBox::No) return;
		
		while (m_flv_FlowList->childCount()>0)
			m_flv_FlowList->menuDeleteGraph();
		deactivateGraph();
	}
}

QPopupMenu * NMGGraphWidget::getPopupMenu()
{
	return m_pm_flowAdminMenu;
}

void NMGGraphWidget::mousePressEvent(QMouseEvent *e)
{
	if (e->button() == RightButton)
		m_pm_flowAdminMenu->popup(e->globalPos());
	e->accept();
}

QToolBar * NMGGraphWidget::getToolBar()
{
	if(!m_tb_toolBar)
	{
		m_tb_toolBar = new QToolBar(NMGModuleUtils::getMainWindow());
		QAction * action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("button_ok.png")), tr("Add flow"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(addFlow()));
		action->addTo(m_tb_toolBar);

		action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("delete.png")), tr("Delete flow"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(deleteFlow()));
		action->addTo(m_tb_toolBar);
		
		action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("edittrash.png")), tr("Delete all flows"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(deleteAllFlows()));
		action->addTo(m_tb_toolBar);
	}
	return m_tb_toolBar;
}

void NMGGraphWidget::zoomXIn()
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->zoomX(true);
	// recalculate the slider position
	slMoveXView->setValue((int)nearbyintf(m_gw_TheGraph->getRelativePosX()*(float)slMoveXView->maxValue()));

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::zoomXOut()
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->zoomX(false);
	// recalculate the slider position  
	slMoveXView->setValue((int)nearbyintf(m_gw_TheGraph->getRelativePosX()*(float)slMoveXView->maxValue()));

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::moveXView(int iSliderPosition)
{
	m_gw_TheGraph->moveViewX((float)iSliderPosition/(float)slMoveXView->maxValue());
}

void NMGGraphWidget::zoomYIn()
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->zoomY(true);
	// recalculate the slider position
	slMoveYView->setValue(slMoveYView->maxValue()-((int)nearbyintf(m_gw_TheGraph->getRelativePosY()*(float)slMoveYView->maxValue())));

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::zoomYOut()
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->zoomY(false);
	// recalculate the slider position  
	slMoveYView->setValue(slMoveYView->maxValue()-((int)nearbyintf(m_gw_TheGraph->getRelativePosY()*(float)slMoveYView->maxValue())));

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::moveYView(int iSliderPosition)
{
	m_gw_TheGraph->moveViewY((float)(slMoveYView->maxValue()-iSliderPosition)/(float)slMoveYView->maxValue());
}

void NMGGraphWidget::itemMove(uint i_itemFrom, uint i_itemTo)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->moveGraph(i_itemFrom, i_itemTo);

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemHide(uint i_itemPos, bool b_hideItem)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->hideGraph(i_itemPos, b_hideItem);

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemDelete(uint i_itemPos)
{
	setCursor(QCursor(Qt::WaitCursor));
	
	m_gdl_dataLoader->removeSelectedGraph(m_gw_TheGraph, m_flv_FlowList);

	delete ((NMGFlowListItem*)m_flv_FlowList->currentItem());

	if (m_flv_FlowList->childCount()==0)
		deactivateGraph();
	else
		m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemChangeLineColor(uint i_itemPos)
{
	QColor col_lineColor, col_areaColor;

	// get current line color
	if (m_gw_TheGraph->getColors(i_itemPos,&col_lineColor,&col_areaColor))
	{
		// get user line color
		col_lineColor = QColorDialog::getColor(col_lineColor,this,"Graph line color");
		// set line color
		m_gw_TheGraph->setColors(i_itemPos,col_lineColor,col_areaColor);
		((NMGFlowListItem*)m_flv_FlowList->currentItem())->setColors(col_lineColor,col_areaColor);
		m_gw_TheGraph->repaintScene();
	}
}

void NMGGraphWidget::itemChangeAreaColor(uint i_itemPos)
{
	QColor col_lineColor, col_areaColor;

	// get current area color
	if (m_gw_TheGraph->getColors(i_itemPos,&col_lineColor,&col_areaColor))
	{
		// get user area color
		col_areaColor = QColorDialog::getColor(col_areaColor,this,"Graph area color");
		// set line color
		m_gw_TheGraph->setColors(i_itemPos,col_lineColor,col_areaColor);
		((NMGFlowListItem*)m_flv_FlowList->currentItem())->setColors(col_lineColor,col_areaColor);
		m_gw_TheGraph->repaintScene();
	}
}

void NMGGraphWidget::itemAverage(uint i_itemPos, bool b_averageItem)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->setAverage(i_itemPos, b_averageItem);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemAccumulate(uint i_itemPos, bool b_accumulateItem)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->setAccumulate(i_itemPos, b_accumulateItem);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemShowPolygons(uint i_itemPos, bool b_show)
{
	setCursor(QCursor(Qt::WaitCursor));
	
	m_gw_TheGraph->setShowPolygons(i_itemPos, b_show);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemShowLines(uint i_itemPos, bool b_show)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->setShowLines(i_itemPos, b_show);
	if (!b_show)	// It has no sense an area without a line around it
		m_gw_TheGraph->setShowAreas(i_itemPos, false);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemShowArea(uint i_itemPos, bool b_show)
{
	setCursor(QCursor(Qt::WaitCursor));

	if (b_show)	// It has no sense an area without a line around it
		m_gw_TheGraph->setShowLines(i_itemPos, true);

	m_gw_TheGraph->setShowAreas(i_itemPos, b_show);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemPolygonSquare(uint i_itemPos)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->setPolygons(i_itemPos, NMGGraphicObject::VPSQUARE);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemPolygonTriangle(uint i_itemPos)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->setPolygons(i_itemPos, NMGGraphicObject::VPTRIANGLE);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemPolygonRhomb(uint i_itemPos)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->setPolygons(i_itemPos, NMGGraphicObject::VPRHOMB);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemPolygonInvertedTriangle(uint i_itemPos)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->setPolygons(i_itemPos, NMGGraphicObject::VPINVTRIANGLE);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::itemDotSize(uint i_itemPos, int iSize)
{
	setCursor(QCursor(Qt::WaitCursor));

	m_gw_TheGraph->setDotsSize(i_itemPos, iSize);
	m_gw_TheGraph->repaintScene();

	setCursor(QCursor(Qt::ArrowCursor));
}

void NMGGraphWidget::viewMoved(float f_relPosX, float f_relPosY)
{
	slMoveXView->setValue((int)roundf(f_relPosX*(float)slMoveXView->maxValue()));
	slMoveYView->setValue(slMoveYView->maxValue() - (int)roundf(f_relPosY*(float)slMoveYView->maxValue()));
}

/**
 * Slot which Cancels the data load
 */
void NMGGraphWidget::loadCancelled(ProgressItem *) 
{
	if (m_gdl_dataLoader)
		m_gdl_dataLoader->cancelLoad();
	progressItem->setComplete();
	progressItem = 0;
	MessageBar::longMessage("Load cancelled");
}

/**
 * Slot for reporting the finishing of the data load
 */
void NMGGraphWidget::loadFinished(ProgressItem *) 
{
	progressItem = 0;
	if (m_gdl_dataLoader->isFinished())
		MessageBar::longMessage("Load finished successfully");
}

void NMGGraphWidget::loadGraphData()
{
	QString str_lastTestPath = preferencesManager->getValue("GraphicTestPath");
	
	QString str_fileName = QFileDialog::getOpenFileName(str_lastTestPath, "XML Flow Format (*.xff);;XML (*.xml);;All files (*.*)", this, "open file dialog", "Select test file");

	progressItem = ProgressManager::createProgressItem("Add Flow");
	
	if (!str_fileName.isEmpty())
	{
		int i_lastPos = str_fileName.findRev("/");
		preferencesManager->addValue("GraphicTestPath",str_fileName.left(i_lastPos));
		
		progressItem->setTotalItems(100);
		progressItem->setStatus("Loading data...");
	
		connect(progressItem, SIGNAL(progressItemCanceled( ProgressItem* )), this, SLOT( loadCancelled( ProgressItem * ) ) );
		connect(progressItem, SIGNAL(progressItemCompleted( ProgressItem* )), this, SLOT( loadFinished( ProgressItem * ) ) );
		
		if (m_gdl_dataLoader==0)
		{
			switch(m_selectedModule)
			{
				case MGEN:
					m_gdl_dataLoader = new NMGMGENGraphDataLoader();
					break;
				default:
					m_gdl_dataLoader = new NMGGraphDataLoader();
					// TODO: Get the value from the metadata of the data file and load the associated loader
			}
		}
	
		m_gdl_dataLoader->loadGraphData(str_fileName);
		timer->start(100);      // Well, this is because we can't have a thread updating the GUI, so put a timer for updating the %
	}
	else
	{
		loadCancelled(progressItem);
	}
}

void NMGGraphWidget::endLoadGraphData()
{
	int i_numLoadedGraphs = m_gdl_dataLoader->endLoadGraphData(m_gw_TheGraph, m_flv_FlowList);
	if (i_numLoadedGraphs>0)
	{
		if (!m_pm_flowAdminMenu->isItemEnabled(m_i_idMenuGraphType))
		{ // First time --> read the graph types and show them in the menu
			QString *str_label;
			QPtrList<QString> strl_graphTypes;
			strl_graphTypes = m_gdl_dataLoader->getGraphTypeLabels();
 			
			str_label = strl_graphTypes.first();
			int i=0;
			while (str_label!=0)
			{
				int id = m_pm_graphTypeMenu->insertItem(*str_label,this,SLOT(changeGraphType(int)));
				m_pm_graphTypeMenu->setItemParameter( id, i );
 
				i++;
				str_label = strl_graphTypes.next();
			}
 					
			m_pm_flowAdminMenu->setItemEnabled(m_i_idMenuGraphType,true);
 
			m_pm_graphTypeMenu->setItemChecked(m_pm_graphTypeMenu->idAt(m_gdl_dataLoader->getGraphType()),true);
			m_pm_graphTypeMenu->setItemEnabled(m_pm_graphTypeMenu->idAt(m_gdl_dataLoader->getGraphType()),false);
 
 			// create the graph options menu
			m_pm_graphTypeMenu->insertSeparator();
			m_pm_graphTypeOptionsMenu = new QPopupMenu(this,"graphTypeOptions");
			m_i_idMenuGraphTypeOptions = m_pm_graphTypeMenu->insertItem("Graph options",m_pm_graphTypeOptionsMenu);
 			// Show options, if any
			loadGraphTypeOptions();
			
			// Activate sliders and zoom buttons
			slMoveXView->setEnabled(true);
			btnXZoomIn->setEnabled(true);
			btnXZoomOut->setEnabled(true);
			slMoveYView->setEnabled(true);
			btnYZoomIn->setEnabled(true);
			btnYZoomOut->setEnabled(true);
		}
		
		m_gw_TheGraph->show();
	}
}


void NMGGraphWidget::loadGraphTypeOptions()
{
	QString *str_label;
	QPtrList<QString> strl_graphTypeOptions;
	strl_graphTypeOptions = m_gdl_dataLoader->getGraphTypeOptionsLabels();
			
	m_pm_graphTypeOptionsMenu->clear();
			
	str_label = strl_graphTypeOptions.first();
	int i=0;
	while (str_label!=0)
	{
		int id = m_pm_graphTypeOptionsMenu->insertItem(*str_label,this,SLOT(setGraphTypeOption(int)));
		m_pm_graphTypeOptionsMenu->setItemParameter( id, i );

		i++;
		str_label = strl_graphTypeOptions.next();
	}
					
	// if any option found enable the options menu; else disable it
	m_pm_graphTypeMenu->setItemEnabled(m_i_idMenuGraphTypeOptions,i>0);
}

void NMGGraphWidget::setGraphTypeOption(int i_graphTypeOption)
{
	m_gdl_dataLoader->setGraphTypeOption(i_graphTypeOption);
	m_gdl_dataLoader->reloadAllGraphs(m_gw_TheGraph, m_flv_FlowList);
	loadGraphTypeOptions();
}

void NMGGraphWidget::changeGraphType(int i_graphTypeSelected)
{
	// Search the old graph type and enable and uncheck it...
	for (int i=0;i<m_pm_graphTypeMenu->count();i++)
	{
		if (!m_pm_graphTypeMenu->isItemEnabled(m_pm_graphTypeMenu->idAt(i)))
		{
			m_pm_graphTypeMenu->setItemEnabled(m_pm_graphTypeMenu->idAt(i),true);
			m_pm_graphTypeMenu->setItemChecked(m_pm_graphTypeMenu->idAt(i),false);
		}
	}
	// check and disable the selected graph type...
	m_pm_graphTypeMenu->setItemChecked(m_pm_graphTypeMenu->idAt(i_graphTypeSelected),true);
	m_pm_graphTypeMenu->setItemEnabled(m_pm_graphTypeMenu->idAt(i_graphTypeSelected),false);
	
	// and change the graph type
	m_gdl_dataLoader->setGraphType(i_graphTypeSelected);
	m_gdl_dataLoader->reloadAllGraphs(m_gw_TheGraph, m_flv_FlowList);
	
	// Show options in the menu, if any
	loadGraphTypeOptions();
}

void NMGGraphWidget::updateProgress()
{
	QString mess;
	int percent;
	bool ended;

	m_gdl_dataLoader->getLoadStatus(mess, &percent, &ended);
	if (ended && progressItem) {
		timer->stop();
		progressItem->setComplete();
		progressItem = 0;
		return;
	}
	if (progressItem) {
		progressItem->setStatus(mess);
		progressItem->setCompletedItems(percent);
		progressItem->updateProgress();
	}

	
	if (m_gdl_dataLoader->getError()!=0)
	{
		timer->stop();
		m_gdl_dataLoader->processError(m_gdl_dataLoader->getError(), m_gdl_dataLoader->getErrorString());
		if (m_gdl_dataLoader->getError()==0) // loader relaunched -> start the timer again
			timer->start(100);
		else // loader aborted
		{
			progressItem->setComplete();
			progressItem = 0;
			MessageBar::longMessage("Load aborted");
			m_gdl_dataLoader->resetError();
			return;
		}
	}

	if (m_gdl_dataLoader->isFinished())
		endLoadGraphData();	
}

	
#include "nmggraphwidget.moc"
