/***************************************************************************
 *   Copyright (C) 2006 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGGRAPHWIDGET_H
#define NMGGRAPHWIDGET_H

#include <qdockwindow.h>
#include <qlistbox.h>

#include <progressmanager.h>
#include "nmggraphwidgetbase.h"
#include "nmggraphicwidget.h"
#include "nmgflowlistview.h"
#include "nmggraphdataloader.h"
#include "../nmgmodulepreferencesmanager.h"


/**
@author David Lacort
*/
class NMGGraphWidget : public NMGGraphWidgetBase
{
Q_OBJECT
public:
    NMGGraphWidget(QWidget *parent = 0, const char *name = 0, NMGModulePreferencesManager * prefMan = 0);

    ~NMGGraphWidget();

	QPopupMenu * getPopupMenu();
    QToolBar * getToolBar();

	// Allowed Modules
	enum moduleType{NONE,MGEN};

	void setModuleType(moduleType selectedModule);

private slots:
    void addFlow();
    void deleteFlow();
    void deleteAllFlows();

	void zoomXIn();
	void zoomXOut();
	void moveXView(int);
	void zoomYIn();
	void zoomYOut();
	void moveYView(int);

	void itemMove(uint i_itemFrom, uint i_itemTo);
	void itemHide(uint i_itemPos, bool b_hideItem);
	void itemDelete(uint i_itemPos);
	void itemChangeLineColor(uint i_itemPos);
	void itemChangeAreaColor(uint i_itemPos);
	void itemAverage(uint i_itemPos, bool b_averageItem);
	void itemAccumulate(uint i_itemPos, bool b_accumulateItem);
	void itemShowPolygons(uint i_itemPos, bool b_show);
	void itemShowLines(uint i_itemPos, bool b_show);
	void itemShowArea(uint i_itemPos, bool b_show);
	void itemPolygonSquare(uint i_itemPos);
	void itemPolygonTriangle(uint i_itemPos);
	void itemPolygonRhomb(uint i_itemPos);
	void itemPolygonInvertedTriangle(uint i_itemPos);
	void itemDotSize(uint i_itemPos, int iSize);

	void viewMoved(float f_relPosX, float f_relPosY);

	void mousePressEvent(QMouseEvent *);

	void changeGraphType(int i_graphTypeSelected);
	void setGraphTypeOption(int i_graphTypeOption);

	void graphDataLoaded(int i_numLoadedGraphs);

    void updateProgress(void);
    void loadCancelled(ProgressItem *progressItem);
    void loadFinished(ProgressItem *progressItem);

	void selectGraphicFont();


protected:
	void paintEvent ( QPaintEvent * );

private:
	QPopupMenu * m_pm_flowAdminMenu;
	QPopupMenu * m_pm_graphTypeMenu;
	QPopupMenu * m_pm_graphTypeOptionsMenu;
	int m_i_idMenuGraphType;
	int m_i_idMenuGraphTypeOptions;
	QToolBar * m_tb_toolBar;

	void createFlowAdminMenu();
	QString m_str_graphFontPath;

	int i_lastElement;
	NMGGraphicWidget *m_gw_TheGraph;
	NMGFlowListView *m_flv_FlowList;
	NMGGraphDataLoader *m_gdl_dataLoader;

// 	NMGGLoaderThread *m_lt_loaderThread;

	moduleType m_selectedModule;

	void loadGraphData();
	void endLoadGraphData();
	void loadGraphTypeOptions();
	void deactivateGraph();

    QTimer *timer;
    ProgressItem * progressItem;
	NMGModulePreferencesManager * preferencesManager;
};

#endif
