/***************************************************************************
 *   Copyright (C) 2005 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgmgengraphdataloader.h"

#include <qmessagebox.h>
#include <iostream>
#include <math.h>

#include "nmgflowlistview.h"
#include <qfile.h>
#include <qxml.h>


using namespace std;

NMGMGENGraphDataLoader::NMGMGENGraphDataLoader()
 : NMGGraphDataLoader()
{
	QString *str_graphType;
	// the graph types MUST be in the same order as the enum mgenGraphs
	str_graphType = new QString("One-Way Delay");
	m_strl_graphTypes.append(str_graphType);
	str_graphType = new QString("One-Way Delay Distribution");
	m_strl_graphTypes.append(str_graphType);
	str_graphType = new QString("Inter-Packet Delay Variation");
	m_strl_graphTypes.append(str_graphType);
	str_graphType = new QString("IPDV Distribution");
	m_strl_graphTypes.append(str_graphType);

	m_b_showSeqId = false;
	m_i_newFlowId = 0;
	
	m_lt_loaderThread = 0;	
	m_i_error = 0;
}


NMGMGENGraphDataLoader::~NMGMGENGraphDataLoader()
{
}

QPtrList<QString> NMGMGENGraphDataLoader::getGraphTypeOptionsLabels()
{
	m_strl_graphTypeOptions.clear();
	switch(m_i_graphType)
	{
		case MGEN_OWDOT:
		case MGEN_IPDV:
		{
			QString *str_graphTypeOption;
			
			if (m_b_showSeqId)
				str_graphTypeOption = new QString("Show time from start");
			else
				str_graphTypeOption = new QString("Show packet sequence number");
			m_strl_graphTypeOptions.append(str_graphTypeOption);
		}
	}
	
	return m_strl_graphTypeOptions;
}

void NMGMGENGraphDataLoader::setGraphTypeOption(int i_option)
{
	switch(i_option)
	{
		case 0:
			m_b_showSeqId = !m_b_showSeqId;
	}
}

void NMGMGENGraphDataLoader::loadGraphData(QString str_fileName)
{
	m_b_loadFinished = false;
	m_str_fileName = str_fileName;

	if(str_fileName and str_fileName!="")
	{
		m_lt_loaderThread = new NMGGLoaderThread();
		m_lt_loaderThread->loadGraphData(str_fileName);
		m_lt_loaderThread->start();
	}
}

int NMGMGENGraphDataLoader::endLoadGraphData(NMGGraphicWidget *gw_TheGraph, NMGFlowListView *flv_FlowList)
{
	QPtrList<double> XDataList;
	QPtrList<double> YDataList;
	XDataList.setAutoDelete(true);
	YDataList.setAutoDelete(true);

	int i_numFlowsLoaded=0;
	
	if (m_lt_loaderThread>0)
	{
		NMGMGENReadFlowData *rfd_readGraph;
		QPtrList<NMGMGENReadFlowData> *rfdl_readGraphData=0;

		bool b_abort = false;
		
		rfdl_readGraphData = m_lt_loaderThread->getReadData();

		if(rfdl_readGraphData==0)
		{
			delete m_lt_loaderThread;
			m_lt_loaderThread = 0;
			m_i_error = -3;
			m_str_error = "Error reading xff file. No data found";
			return 0;
		}

		rfd_readGraph = rfdl_readGraphData->first();
		
		// If we only have 1 test get its name as the graph title. Else only the test type
		if (m_rfdl_graphData.count()==0) // only the current loading
			gw_TheGraph->setGraphLabel( rfd_readGraph->getTestName() + " - " + getGraphLabel() );
		else
			gw_TheGraph->setGraphLabel( getGraphLabel() );
		
		gw_TheGraph->setXAxisLabel( getXAxisLabel() );
		gw_TheGraph->setYAxisLabel( getYAxisLabel() );
		
		if(m_i_graphType==MGEN_OWDOT || m_i_graphType==MGEN_IPDV)
		{
			flv_FlowList->enableAverage(true);
			flv_FlowList->enableAccumulate(false);
			flv_FlowList->viewAverage(true);
			flv_FlowList->viewAccumulate(false);
		}
		else if(m_i_graphType==MGEN_OWDD || m_i_graphType==MGEN_IPDVD)
		{
			flv_FlowList->enableAverage(false);
			flv_FlowList->enableAccumulate(false);
			flv_FlowList->viewAverage(false);
			flv_FlowList->viewAccumulate(true);
		}
		
		while(rfd_readGraph!=0 && !b_abort)
		{
			rfd_readGraph->setGlobalFlowId(m_i_newFlowId);
			if (rfd_readGraph->getName().isNull())
				rfd_readGraph->setName("Flow "+QString::number(m_i_newFlowId));
			m_i_newFlowId++;
			m_rfdl_graphData.append(rfd_readGraph);
			
			if (loadMgenGraphDataFromXml(rfd_readGraph, XDataList, YDataList))
			{
				QColor col_textColor, col_bgColor;
				QString str_graphName;

				str_graphName = rfd_readGraph->getName();
//				str_graphName += " \nrate=" + rfd_readGraph->getRate();
//				str_graphName += "\nsize=" + rfd_readGraph->getSize();

				gw_TheGraph->addGraph(str_graphName, XDataList, YDataList);
				gw_TheGraph->setDefaultDotsSize(2);
				
				rfd_readGraph->setTestFileName(m_str_fileName);
						
				flv_FlowList->setUpdatesEnabled( FALSE );
				QListViewItem *lvi_Flow = new NMGFlowListItem(flv_FlowList,
						QString::number(rfd_readGraph->getGlobalFlowId()),
						rfd_readGraph->getName(),
						rfd_readGraph->getRate(),
						rfd_readGraph->getSize(),
						rfd_readGraph->getTestSrcAddress(),
						rfd_readGraph->getTestDstAddress(),
						rfd_readGraph->getTestFileName(),
						rfd_readGraph->getId());
				
				setGraphVisualization(gw_TheGraph, (NMGFlowListItem*)lvi_Flow);
				
				flv_FlowList->setCurrentItem(lvi_Flow);
				flv_FlowList->setUpdatesEnabled( TRUE );
				if (gw_TheGraph->getColors(0, &col_textColor, &col_bgColor))
					((NMGFlowListItem*)lvi_Flow)->setColors(col_textColor,col_bgColor);
				((NMGFlowListItem*)lvi_Flow)->setDotSize(2);
				flv_FlowList->repaintItem(flv_FlowList->lastItem());
				flv_FlowList->triggerUpdate();
				i_numFlowsLoaded++;

				XDataList.clear();
				YDataList.clear();
			}
			else
			{
				if (QMessageBox::critical(0,"Flow data error",m_lt_loaderThread->getErrorString(), QMessageBox::Abort, QMessageBox::Ignore, QMessageBox::NoButton) == QMessageBox::Abort)
					b_abort = true;
				// else continue;
			}
			rfd_readGraph = rfdl_readGraphData->next();
		}

		if (rfdl_readGraphData!=0)
		{
			delete rfdl_readGraphData;	// only the list. Elements have been appended to m_rfdl_graphData
		}		

		// show all in the graph
		gw_TheGraph->repaintScene();
		gw_TheGraph->restore();
		
		delete m_lt_loaderThread;
		m_lt_loaderThread = 0;
	}
	
	m_b_loadFinished = true;
	
	return i_numFlowsLoaded;
}


int NMGMGENGraphDataLoader::getError()
{
	if (m_i_error!=0)
		return m_i_error;
	else if (m_lt_loaderThread)
		return m_lt_loaderThread->getError();
	else return 0;
}

QString NMGMGENGraphDataLoader::getErrorString()
{
	if (m_i_error!=0)
		return m_str_error;
	else if (m_lt_loaderThread)
		return m_lt_loaderThread->getErrorString();
	else return "";
}

void NMGMGENGraphDataLoader::processError(int i_error, QString str_error)
{
	m_i_error = i_error;
	m_str_error = str_error;
	
	delete m_lt_loaderThread;
	m_lt_loaderThread = 0;
	
	switch(m_i_error)
	{
		case -1:	// fatal error
			QMessageBox::critical(0,"Fatal error",m_str_error, QMessageBox::Ok, QMessageBox::NoButton);
			break;
		case -2:	// Negative delays detected
			if (QMessageBox::warning(0,"Invalid data",m_str_error+".\nReload data anyway?", QMessageBox::Abort, QMessageBox::Yes, QMessageBox::NoButton) == QMessageBox::Yes)
			{
				m_b_loadFinished = false;
				m_i_error = 0;
				m_str_error = "";
				if(m_str_fileName and m_str_fileName!="")
				{
					m_lt_loaderThread = new NMGGLoaderThread();
					m_lt_loaderThread->acceptInvalidData(true);
					m_lt_loaderThread->loadGraphData(m_str_fileName);
					m_lt_loaderThread->start();
				}
			}
			break;
		case -3:	// flow data error
			QMessageBox::warning(0,"Flow data error",m_str_error, QMessageBox::Ok, QMessageBox::NoButton);
			break;
		default:
			QMessageBox::critical(0,"Undefined error",m_str_error, QMessageBox::Ok, QMessageBox::NoButton);
	}
}

/*
void NMGMGENGraphDataLoader::resetError()
{
	m_i_error = 0;
	m_str_error = "";
}
*/
QString NMGMGENGraphDataLoader::getXAxisLabel()
{
	switch(m_i_graphType)
	{
		case MGEN_OWDOT:
			// One-Way Delay 
			if (m_b_showSeqId)
				return "packet sequence number";
			else
				return "time from start (msec)";
			break;
		case MGEN_OWDD:
			// One-Way Delay Distribution 
			return "delay (msec)";
			break;
		case MGEN_IPDV:
			// IP Delay Variation
			if (m_b_showSeqId)
				return "packet sequence number";
			else
				return "time from start (msec)";
			break;
		case MGEN_IPDVD:
			// IP Delay Variation Distribution
			return "ipdv (msec)";
			break;
	}
	return "";
}

QString NMGMGENGraphDataLoader::getYAxisLabel()
{
	switch(m_i_graphType)
	{
		case MGEN_OWDOT:
			// One-Way Delay 
			return "delay (msec)";
			break;
		case MGEN_OWDD:
			// One-Way Delay Distribution 
			return "% of packets";
			break;
		case MGEN_IPDV:
			// IP Delay Variation
			return "ipdv (msec)";
			break;
		case MGEN_IPDVD:
			// IP Delay Variation Distribution
			return "% of packets";
			break;
	}
	return "";
}

bool NMGMGENGraphDataLoader::isFinished()
{
	if (m_lt_loaderThread!=0)
		return m_lt_loaderThread->isFinished();
	else
		return m_i_error==0;
}

void NMGMGENGraphDataLoader::cancelLoad()
{
	if (m_lt_loaderThread!=0)
		m_lt_loaderThread->kill();
}

void NMGMGENGraphDataLoader::getLoadStatus(QString str_message, int *i_percent, bool *b_finished)
{
	// 90% of precessing assigned to parsing the file. 10% to GL loading
	if (m_lt_loaderThread!=0)
	{
		if (!m_lt_loaderThread->isFinished())
			str_message = m_lt_loaderThread->getLoadStatus();
		else
			str_message = "Loading data to GL";
		
		*i_percent = m_lt_loaderThread->getPercentCompleted() * 0.9;
		*b_finished = false;
	}
	else if (m_b_loadFinished)
	{
		// load finished.
		str_message = "Data loaded";
		*i_percent = 100;
		*b_finished = true;
	}
	else
	{
		str_message = "Unknown status";
		*i_percent = 0;
		*b_finished = false;
	}
}

bool NMGMGENGraphDataLoader::reloadAllGraphs(NMGGraphicWidget *gw_TheGraph, NMGFlowListView *flv_FlowList)
{

	int i_graphPos = 0;
	QPtrList<double> XDataList;
	QPtrList<double> YDataList;
	XDataList.setAutoDelete(true);
	YDataList.setAutoDelete(true);

	if (m_rfdl_graphData.count()>0)
	{
		// If we only have 1 test get its name as the graph title. Else only the test type
		if (m_rfdl_graphData.count()==1) // only 1 test
			gw_TheGraph->setGraphLabel( m_rfdl_graphData.first()->getTestName() + " - " + getGraphLabel() );
		else
			gw_TheGraph->setGraphLabel( getGraphLabel() );
		
		gw_TheGraph->setXAxisLabel( getXAxisLabel() );
		gw_TheGraph->setYAxisLabel( getYAxisLabel() );
	
		if(m_i_graphType==MGEN_OWDOT || m_i_graphType==MGEN_IPDV)
		{
			flv_FlowList->enableAverage(true);
			flv_FlowList->enableAccumulate(false);
			flv_FlowList->viewAverage(true);
			flv_FlowList->viewAccumulate(false);
		}
		else if(m_i_graphType==MGEN_OWDD || m_i_graphType==MGEN_IPDVD)
		{
			flv_FlowList->enableAverage(false);
			flv_FlowList->enableAccumulate(false);
			flv_FlowList->viewAverage(false);
			flv_FlowList->viewAccumulate(true);
		}

		for(QListViewItem *lvi_Flow = flv_FlowList->firstChild(); lvi_Flow!=0; lvi_Flow = lvi_Flow->nextSibling())
		{
			// Read the name from the 3rd column and the flow id from the 4th one
			int i_currentFlowGlobalId = lvi_Flow->text(0).toInt();
		
			bool b_abort = false;
			// search the flow in the file and add it to the graph
			m_rfdl_graphData.first();
			do
			{
				if (m_rfdl_graphData.current()->getGlobalFlowId()==i_currentFlowGlobalId)
				{
					if (loadMgenGraphDataFromXml(m_rfdl_graphData.current(), XDataList, YDataList))
					{
						gw_TheGraph->setData(i_graphPos, XDataList, YDataList);
						setGraphVisualization(gw_TheGraph, (NMGFlowListItem*)lvi_Flow, i_graphPos);
					}
					else
					{
						if (QMessageBox::critical(0,"Flow data error","Error loading flow data", QMessageBox::Abort, QMessageBox::Ignore, QMessageBox::NoButton) == QMessageBox::Abort)
							b_abort = true;
					}
					
					XDataList.clear();
					YDataList.clear();
				}
			}
			while(m_rfdl_graphData.next() && !b_abort);
	
			i_graphPos++;
		}

		gw_TheGraph->repaintScene();
		gw_TheGraph->restore();
	}

	return true;
}

void NMGMGENGraphDataLoader::removeSelectedGraph(NMGGraphicWidget *gw_TheGraph, NMGFlowListView *flv_FlowList)
{
	bool b_found = false;
	QListViewItem *lvi_Flow = flv_FlowList->currentItem();
	int i_currentItemGlobalFlowId = lvi_Flow->text(0).toInt();
	
	m_rfdl_graphData.first();
	while(m_rfdl_graphData.current() && !b_found)
	{
		if (m_rfdl_graphData.current()->getGlobalFlowId() == i_currentItemGlobalFlowId)
		{
			m_rfdl_graphData.remove();
			b_found = true;
		}
		m_rfdl_graphData.next();
	}
	
	gw_TheGraph->removeGraph(flv_FlowList->selectedItemIndex());

}

bool NMGMGENGraphDataLoader::loadMgenGraphDataFromXml(NMGMGENReadFlowData *rfd_readGraph,QPtrList<double> &XDataList,QPtrList<double> &YDataList)
{
	bool b_allOk = false;

	switch(m_i_graphType)
	{
		case MGEN_OWDOT:
		{
			// One-Way delay over time [One-Way delay (Transitory)]
			if (m_b_showSeqId)
				copyList(XDataList, rfd_readGraph->m_dl_seqIdList);
			else
				copyList(XDataList, rfd_readGraph->m_dl_timestampList);
			b_allOk = !XDataList.isEmpty();
			copyList(YDataList, rfd_readGraph->m_dl_delayList);
			b_allOk = b_allOk && !YDataList.isEmpty();
			break;
		}
		case MGEN_OWDD:
		{
			// One-Way Delay Distribution 
			QPtrList<double> delaysList;
			delaysList.setAutoDelete(true);
			copyList(delaysList, rfd_readGraph->m_dl_delayList);
			b_allOk = !delaysList.isEmpty();
			b_allOk = b_allOk && generateDistributionLists(delaysList,XDataList,YDataList);
			break;
		}
		case MGEN_IPDV:
		{
			// Inter-Packet Delay Variation
			QPtrList<double> delaysList;
			delaysList.setAutoDelete(true);
			if (m_b_showSeqId)
				copyList(XDataList, rfd_readGraph->m_dl_seqIdList);
			else
				copyList(XDataList, rfd_readGraph->m_dl_timestampList);
			b_allOk = !XDataList.isEmpty();
			copyList(delaysList, rfd_readGraph->m_dl_delayList);
			b_allOk = b_allOk && !delaysList.isEmpty();
			b_allOk = b_allOk && generateIPDVlist(delaysList,YDataList);
			break;
		}
		case MGEN_IPDVD:
		{
			// IPDVD Distribution
			QPtrList<double> seqIdList;
			QPtrList<double> delaysList;
			QPtrList<double> ipdvList;
			seqIdList.setAutoDelete(true);
			delaysList.setAutoDelete(true);
			ipdvList.setAutoDelete(true);
			copyList(seqIdList, rfd_readGraph->m_dl_seqIdList);
			b_allOk = !seqIdList.isEmpty();
			copyList(delaysList, rfd_readGraph->m_dl_delayList);
			b_allOk = b_allOk && !delaysList.isEmpty();
			b_allOk = b_allOk && generateIPDVlist(delaysList,ipdvList);
			b_allOk = b_allOk && generateDistributionLists(ipdvList,XDataList,YDataList);
			break;
		}
	}

	return (b_allOk && !YDataList.isEmpty());
}


void NMGMGENGraphDataLoader::copyList(QPtrList<double> &destList, QPtrList<double> &sourceList)
{
	double *d_currentElem;
	
	d_currentElem = sourceList.first();
	while(d_currentElem!=0)
	{
		double *d_newElem = new double;
		*d_newElem = *d_currentElem;
		destList.append(d_newElem);
		
		d_currentElem = sourceList.next();
	}
}
		
int NMGMGENGraphDataLoader::searchCurrent(QPtrList<double> &list,double d_value,int i_left,int i_right)
{
	if (i_right < i_left)
		return i_left;
	int i_mid = floor((double)((i_left+i_right)/2));
	if (d_value > *list.at(i_mid))
		return searchCurrent(list, d_value, i_mid+1, i_right);
	else if (d_value < *list.at(i_mid))
		return searchCurrent(list, d_value, i_left, i_mid-1);
	else
		return i_mid;
}

bool NMGMGENGraphDataLoader::generateDistributionLists(QPtrList<double> &dataList,QPtrList<double> &valueList,QPtrList<double> &percentList)
{
	double *d_currentData;

 	d_currentData = dataList.first();
	if (d_currentData!=0)
	{
		// insert the first value in the result lists
		double *d_value;
		double *d_counter;
		d_value = new double;
		d_counter = new double;
		*d_value = *d_currentData;
		*d_counter = 1.0;
		
		valueList.append(d_value);
		percentList.append(d_counter);
		d_currentData = dataList.next();

	}
	else
		return false;
	
	while (d_currentData!=0)
	{
		// Search the value of the current delay in XDataList
		int i_index = searchCurrent(valueList,*d_currentData,0,valueList.count()-1);
		
		double *d_currentValue = valueList.at(i_index);

		if (d_currentValue!=0 && (*d_currentValue==*d_currentData))
		{	// if the current delay is found increase the associated counter
			double *d_counter, *d_currentCounter;
			d_currentCounter = percentList.at(i_index);
			d_counter = new double;
			*d_counter = *d_currentCounter + 1.0;
			percentList.replace(i_index, d_counter);
		}
		else
		{	// if the current delay is not found create it and the counter
			double *d_value;
			double *d_counter;
			d_value = new double;
			d_counter = new double;
			*d_value = *d_currentData;
			*d_counter = 1.0;
			
			valueList.insert(i_index, d_value);
			percentList.insert(i_index, d_counter);
		}
				
		d_currentData = dataList.next();
	}

	// transform counters into per cents
	double *d_currentCounter = percentList.first();
	// search in XDataList the value of the current delay
	while ( d_currentCounter!=0)
	{
		int i_index = percentList.at();
		double *d_percent;
		d_percent = new double;
		*d_percent = *d_currentCounter / dataList.count() * 100.0;
		percentList.replace(i_index, d_percent);
		d_currentCounter = percentList.next();
	}

	return true;
}

void NMGMGENGraphDataLoader::insertListItem(QPtrList<double> &list, int i_curPos, double d_value)
{
	double *d_element;
	d_element = new double;
	*d_element = d_value;
	list.insert(i_curPos, d_element);
}

bool NMGMGENGraphDataLoader::generateIPDVlist(QPtrList<double> &delaysList,QPtrList<double> &ipdvList)
{
	double *d_currentDelay;
	double *d_previousDelay;
	
	d_previousDelay = delaysList.first();

	if (d_previousDelay!=0)
	{
		d_currentDelay = delaysList.next();
		if (d_currentDelay!=0)
		{
			double *d_ipdv;
			d_ipdv = new double;
			*d_ipdv = 0.0;
			
			ipdvList.append(d_ipdv);
		}
		else
		{
			return false;
		}
	}
	else
	{
		return false;
	}
	
	while (d_currentDelay!=0)
	{
		// calculate and insert the first ipdv in the result list
		// operations are calculated in microseconds (int) because of double precision problems
		int i_ipdv; 
		i_ipdv = (int)(*d_currentDelay * 1000) - (int)(*d_previousDelay * 1000);
		
		double *d_ipdv;
		d_ipdv = new double;
		*d_ipdv = ((double)i_ipdv) / 1000.0;
		
		ipdvList.append(d_ipdv);
		
		// advance to the next delay
		d_previousDelay = d_currentDelay;
		d_currentDelay = delaysList.next();
	}
	
	return true;
}

void NMGMGENGraphDataLoader::setGraphVisualization(NMGGraphicWidget *gw_TheGraph, NMGFlowListItem *lvi_Flow, int i_graphPos)
{
	switch(m_i_graphType)
	{
		case MGEN_OWDOT:
			gw_TheGraph->setRepresentation(i_graphPos, NM_LINES);
			gw_TheGraph->setAverage(i_graphPos, true);
			gw_TheGraph->setAccumulate(i_graphPos, false);
			gw_TheGraph->setDrawZeroIfNoData(i_graphPos, false);
			lvi_Flow->setAveraged(true);
			lvi_Flow->setAccumulated(false);
			lvi_Flow->setPolygonsShowed(false);
			lvi_Flow->setLinesShowed(true);
			lvi_Flow->setAreaShowed(false);
			break;
		case MGEN_OWDD:
			gw_TheGraph->setRepresentation(i_graphPos, NM_LINES_AREA);
			gw_TheGraph->setAverage(i_graphPos, false);
			gw_TheGraph->setAccumulate(i_graphPos, true);
			gw_TheGraph->setDrawZeroIfNoData(i_graphPos, true);
			lvi_Flow->setAveraged(false);
			lvi_Flow->setAccumulated(true);
			lvi_Flow->setPolygonsShowed(false);
			lvi_Flow->setLinesShowed(true);
			lvi_Flow->setAreaShowed(true);
			break;
		case MGEN_IPDV:
			gw_TheGraph->setRepresentation(i_graphPos, NM_DOTS);
			gw_TheGraph->setAverage(i_graphPos, false);
			gw_TheGraph->setAccumulate(i_graphPos, false);
			gw_TheGraph->setDrawZeroIfNoData(i_graphPos, false);
			lvi_Flow->setAveraged(false);
			lvi_Flow->setAccumulated(false);
			lvi_Flow->setPolygonsShowed(false);
			lvi_Flow->setLinesShowed(false);
			lvi_Flow->setAreaShowed(false);
			break;
		case MGEN_IPDVD:
			gw_TheGraph->setRepresentation(i_graphPos, NM_LINES_AREA);
			gw_TheGraph->setAverage(i_graphPos, false);
			gw_TheGraph->setAccumulate(i_graphPos, true);
			gw_TheGraph->setDrawZeroIfNoData(i_graphPos, true);
			lvi_Flow->setAveraged(false);
			lvi_Flow->setAccumulated(true);
			lvi_Flow->setPolygonsShowed(false);
			lvi_Flow->setLinesShowed(true);
			lvi_Flow->setAreaShowed(true);
			break;
	}	
}

