/***************************************************************************
 *   Copyright (C) 2005 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGMGENGRAPHDATALOADER_H
#define NMGMGENGRAPHDATALOADER_H

#include <qtimer.h>
#include "nmggraphdataloader.h"
#include "nmgmgenreadflowdata.h"
#include "nmggloaderthread.h"

/**
	@author David Lacort <david.lacort@gmail.com>
*/

class NMGMGENGraphDataLoader : public NMGGraphDataLoader
{
public:
    NMGMGENGraphDataLoader();
    ~NMGMGENGraphDataLoader();

	enum mgenGraphs {MGEN_OWDOT,MGEN_OWDD,MGEN_IPDV,MGEN_IPDVD};

	void loadGraphData(QString str_fileName);
	int endLoadGraphData(NMGGraphicWidget *gw_TheGraph, NMGFlowListView *flv_FlowList);

	bool reloadAllGraphs(NMGGraphicWidget *gw_TheGraph, NMGFlowListView *flv_FlowList);
	void removeSelectedGraph(NMGGraphicWidget *gw_TheGraph, NMGFlowListView *flv_FlowList);

	QPtrList<QString> getGraphTypeOptionsLabels();
	void setGraphTypeOption(int i_option);

	bool isFinished();
	int getError();
	QString getErrorString();
	void processError(int i_error, QString str_error);
	void cancelLoad();
	void getLoadStatus(QString str_message, int *i_percent, bool *b_finished);

private:
	bool loadMgenGraphDataFromXml(NMGMGENReadFlowData *rfd_readGraph,QPtrList<double> &XDataList,QPtrList<double> &YDataList);
	void setGraphVisualization(NMGGraphicWidget *gw_TheGraph, NMGFlowListItem *lvi_Flow, int i_graphPos=0);

	void sortLists(QPtrList<double> &dataListOne, QPtrList<double> &dataListTwo);
	int searchCurrent(QPtrList<double> &list,double d_searched,int i_firstIndex,int i_lastIndex);


	bool generateDistributionLists(QPtrList<double> &dataList,QPtrList<double> &valueList,QPtrList<double> &percentList);
	bool generateIPDVlist(QPtrList<double> &delaysList,QPtrList<double> &ipdvList);

	void insertListItem(QPtrList<double> &list, int i_curPos, double f_value);
	void copyList(QPtrList<double> &destList, QPtrList<double> &sourceList);

	QString getXAxisLabel();
	QString getYAxisLabel();

	bool m_b_showSeqId;

	QString m_str_fileName; 

	QPtrList<NMGMGENReadFlowData> m_rfdl_graphData;

	int m_i_newFlowId;

	NMGGLoaderThread *m_lt_loaderThread;

	bool m_b_loadFinished;
};

#endif
