/***************************************************************************
 *   Copyright (C) 2005 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgmgenxmlflowformatparser.h"

#include <iostream>
#include <qptrlist.h>

#include "../libnmgmgen/nmgmgenxmltags.h"

using namespace std;

NMGMGENXMLFlowFormatParser::NMGMGENXMLFlowFormatParser()
{
	m_b_invalidDataAccepted = false;
	m_str_status = "Creating reader thread";
}

bool NMGMGENXMLFlowFormatParser::startDocument()
{
	m_md_metadata = 0;
	m_rfd_currentReadData = 0;
	m_rfdl_ReadFlows = 0;

	b_metadata = false;
	b_source = false;;
	b_destination = false;
	b_management = false;
	b_conf_mgen = false;
	b_conf_drec = false;
	
	b_result = false;
	b_flow = false;
	b_seq = false;
			
	m_b_finished = false;
	m_i_errorCode = 0;
	m_b_stopParsing	= false;
	
	m_i_currentPacket = 0;
	m_i_numPacketsAllFlows = 0;

	m_str_status = "Reading data from file";
	
	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::endDocument()
{
	m_b_finished = true;

	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::startElement( const QString&, const QString&, const QString& qName, const QXmlAttributes &attributes )
{
	if (m_b_stopParsing)
	{
		m_str_status = "Stopped";
		m_str_error = "Parsing cancelled by user";
		cout << m_str_error << endl;
		deleteObjects();
		return FALSE;
	}

	str_tagName = qName;
	if (b_metadata)
	{
		startElement_metadata( qName, attributes );
	}
	else if (b_result)
	{
		if (!startElement_result( qName, attributes ))
			return FALSE;
	}
	else // if (!b_metadata && !b_result)
	{
		m_str_status = "Parsing file data";
		if (qName == "metadata")
		{
			if (!m_md_metadata)
				m_md_metadata = new NMGMGENMetadata();
			
			b_metadata = true;
		}
		else if (qName == "result")
		{
			b_result = true;
		}
	}
	
	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::startElement_metadata(const QString &qName, const QXmlAttributes &attributes)
{
	if (b_source)
	{
		if (qName == "management")
			b_management = true;
	}		
	else if (b_destination)
	{
		if (qName == "management")
			b_management = true;
	}
	else if (b_conf_mgen)
	{
		if (qName == TAG_ON)
		{
			m_fmd_currentFlowMetadata = new NMGMGENFlowMetadata();
			
			for (int i=0; i<attributes.count();i++)
			{
				if (attributes.qName(i)==ATTR_RATE)
					m_fmd_currentFlowMetadata->setRate(attributes.value(i));
				else if (attributes.qName(i)==ATTR_DEST)
					m_fmd_currentFlowMetadata->setDest(attributes.value(i));
				else if (attributes.qName(i)==ATTR_SIZE)
					m_fmd_currentFlowMetadata->setSize(attributes.value(i));
				else if (attributes.qName(i)==ATTR_FLOW_ID)
					m_fmd_currentFlowMetadata->setId(attributes.value(i));
				else if (attributes.qName(i)==ATTR_PATTERN)
					m_fmd_currentFlowMetadata->setPattern(attributes.value(i));
				else if (attributes.qName(i)==ATTR_NAME)
					m_fmd_currentFlowMetadata->setName(attributes.value(i));
				else if (attributes.qName(i)==ATTR_TIME)
					m_fmd_currentFlowMetadata->setTimeOn(attributes.value(i));
			}
			
			m_md_metadata->addFlowMetadata(m_fmd_currentFlowMetadata);
			m_fmd_currentFlowMetadata = 0;
		}
		else if (qName == TAG_OFF)
		{
			if (m_md_metadata!=0)
			{
				QString str_flowId="";
				QString str_timeOff="";
				
				for (int i=0; i<attributes.count();i++)
				{
					if (attributes.qName(i)==ATTR_TIME)
						str_timeOff = attributes.value(i);
					else if (attributes.qName(i)==ATTR_FLOW_ID)
						str_flowId = attributes.value(i);
				}
				
				if (str_flowId!="" && str_timeOff!="")
				{
					// search the flow and set the time off
					m_fmd_currentFlowMetadata = m_md_metadata->getFlowMetadata(str_flowId);
					if (m_fmd_currentFlowMetadata!=0)
						m_fmd_currentFlowMetadata->setTimeOff(str_timeOff);
					m_i_numPacketsAllFlows += (((m_fmd_currentFlowMetadata->getTimeOff().toInt() - m_fmd_currentFlowMetadata->getTimeOn().toInt())*m_fmd_currentFlowMetadata->getRate().toInt())/1000);
					m_fmd_currentFlowMetadata = 0;
				}
			}
		}
	}
	else if (b_conf_drec)
	{
		// nothing to get
	}
	else
	{
		if (qName == TAG_SOURCE)
			b_source = true;
		else if (qName == TAG_DESTINATION)
			b_destination = true;
		else if (qName == TAG_CONF_MGEN)
			b_conf_mgen = true;
		else if (qName == TAG_CONF_DREC)
			b_conf_drec = true;
	}
	
	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::startElement_result(const QString &qName, const QXmlAttributes &attributes)
{
	if (b_flow)
	{
		if (b_seq)
		{
			if (qName == TAG_SOURCE)
				b_source = true;
			else if (qName == TAG_DESTINATION)
				b_destination = true;
		}
		else // if (!b_seq)
		{
			if (qName == "seq")
			{
				str_seqId = "";
				str_txTime = "";
				str_rxTime = "";

				for (int i=0; i<attributes.count();i++)
				{
					if (attributes.qName(i)=="id")
					{
						str_seqId = attributes.value(i);
					}
				}
				b_seq = true;
			}
			if (qName == TAG_SOURCE)
				b_source = true;
			else if (qName == TAG_DESTINATION)
				b_destination = true;
		}
	}
	else // if (!b_flow)
	{
		if (qName == "flow")
		{
			str_startTime = "";
			
			if (m_rfd_currentReadData)
				delete m_rfd_currentReadData;
			m_rfd_currentReadData = 0;
			
			for (int i=0; i<attributes.count();i++)
			{
				if (attributes.qName(i)=="id")
				{
					m_rfd_currentReadData = new NMGMGENReadFlowData(m_md_metadata, m_md_metadata->getFlowMetadata(attributes.value(i)));
				}
				if(m_rfd_currentReadData->getId().isNull())
					m_rfd_currentReadData->setId(attributes.value(i));
			}
			
			if (m_rfd_currentReadData)
				b_flow = true;
			else
			{
				m_str_error = "Flow id not found or dynamic memory error";
				m_i_errorCode = -1;
				cerr << m_str_error << endl;
				deleteObjects();
				return FALSE;
			}
		}
	}
	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::characters ( const QString & ch )
{
	if (m_b_stopParsing)
	{
		m_str_error = "Parsing cancelled by user";
		cout << m_str_error << endl;
		deleteObjects();
		return FALSE;
	}

	if (b_metadata)
	{
		characters_metadata(ch);
	}
	else if (b_result)
	{
		characters_result(ch);
	}
	
	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::characters_metadata(const QString & ch)
{
	if (b_source)
	{
		if (!b_management)
		{
			if(str_tagName==TAG_ADDRESS) 
				m_md_metadata->setTestSrcAddress(ch);
		}
	}
	else if (b_destination)
	{
		if (!b_management)
		{
			if(str_tagName==TAG_ADDRESS) 
				m_md_metadata->setTestDstAddress(ch);
		}
	}
	else if (b_conf_mgen)
	{
		// nothing to get
	}
	else if (b_conf_drec)
	{
		// nothing to get
	}
	else
	{
		if (str_tagName==ATTR_NAME)
			m_md_metadata->setTestName(ch);
		if (str_tagName==TAG_IDENTIFIER)
			m_md_metadata->setTestId(ch);
	}
	
	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::characters_result(const QString & ch)
{
	if (b_flow)
	{
		if (b_seq)
		{
			if (!b_source && !b_destination)
			{
				if(str_tagName=="tx_time") 
				{
					str_txTime = ch;
					if (str_startTime=="")
						str_startTime = ch;
				}
				else if(str_tagName=="rx_time") 
					str_rxTime = ch;
			}
		}
		else if (b_source)
		{
			if(str_tagName==TAG_ADDRESS && m_rfd_currentReadData->getTestSrcAddress().isNull()) 
				m_rfd_currentReadData->setTestSrcAddress(ch);
		}
		else if (b_destination)
		{
			if(str_tagName==TAG_ADDRESS && m_rfd_currentReadData->getTestDstAddress().isNull()) 
				m_rfd_currentReadData->setTestDstAddress(ch);
		}
	}
	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::endElement( const QString&, const QString&, const QString& qName)
{
	if (m_b_stopParsing)
	{
		m_str_error = "Parsing cancelled by user";
		cout << m_str_error << endl;
		deleteObjects();
		return FALSE;
	}

	if (b_metadata)
	{
		endElement_metadata(qName);
	}
	else if (b_result)
	{
		if (!endElement_result(qName))
			return FALSE;
	}
	
	str_tagName = "";
	
	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::endElement_metadata(const QString& qName)
{
	if (b_source)
	{
		if (b_management && qName == "management")
			b_management = false;
		else if (qName == TAG_SOURCE)
			b_source = false;
	}		
	else if (b_destination)
	{
		if (b_management && qName == "management")
			b_management = false;
		else if (qName == TAG_DESTINATION)
			b_destination = false;
	}
	else if (b_conf_mgen)
	{
		if (qName == TAG_CONF_MGEN)
			b_conf_mgen = false;
	}
	else if (b_conf_drec)
	{
		if (qName == TAG_CONF_DREC)
			b_conf_drec = false;
	}
	else
	{ 
		if (qName == "metadata")
			b_metadata = false;
	}
	
	return TRUE;
}

bool NMGMGENXMLFlowFormatParser::endElement_result(const QString& qName)
{
	if (b_flow)
	{
		if (b_seq)
		{
			if (b_source && qName == TAG_SOURCE)
				b_source = false;
			else if (b_destination && qName == TAG_DESTINATION)
				b_destination = false;
			else 
			{
				if (qName == "seq")
				{
					b_seq = false;
					
					if (str_seqId!="" && str_txTime!="" && str_rxTime!="")
					{
						double *d_seqId;
						double *d_timestamp;
						double *d_delay;
						
						d_seqId = new double();
						d_timestamp = new double();
						d_delay = new double();
						
						*d_seqId = str_seqId.toDouble();
						*d_timestamp = timeStringDiff(str_startTime, str_txTime);
						*d_delay = timeStringDiff(str_txTime,str_rxTime);
						if (!m_b_invalidDataAccepted && (*d_delay < 0.0))	// invalid data
						{
							m_i_errorCode = -2;
							m_str_error = "Error: The XML file has negative delays in flow " + m_rfd_currentReadData->getName() + " (id " + m_rfd_currentReadData->getId() + ")";
							cerr << m_str_error << endl;
							// delete read data
							deleteObjects();
							return FALSE;
						}
						
						m_rfd_currentReadData->m_dl_seqIdList.append(d_seqId);
						m_rfd_currentReadData->m_dl_timestampList.append(d_timestamp);
						m_rfd_currentReadData->m_dl_delayList.append(d_delay);
						
						m_i_currentPacket++;
					}
					else
					{
						m_i_errorCode = -1;
						m_str_error = "Incomplete <seq> element";
						cerr << m_str_error << endl;
						deleteObjects();
						return FALSE;
					}
				}
			}
		}
		else if (b_source && qName == TAG_SOURCE)
			b_source = false;
		else if (b_destination && qName == TAG_DESTINATION)
			b_destination = false;
		else
		{
			if (qName == "flow")
			{
				b_flow = false;
				
				if (m_rfdl_ReadFlows==0)
					m_rfdl_ReadFlows = new QPtrList<NMGMGENReadFlowData>;
				if (m_rfd_currentReadData!=0)
				{
					sortCurrentLists();
					fillWithLostPackets();
					m_rfdl_ReadFlows->append(m_rfd_currentReadData);
					// set m_rfd_currentReadData pointer to 0 
					m_rfd_currentReadData = 0;
				}
				
			}
		}
	}
	else // if (!b_flow)
	{
		if (qName == "result")	// xml end
		{
			delete m_md_metadata;
			m_md_metadata = 0;
			b_result = false;
			m_i_numPacketsAllFlows = m_i_currentPacket;
		}
	}
	return TRUE;
}

double NMGMGENXMLFlowFormatParser::timeStringDiff(const QString &str_timeFrom, const QString &str_timeTo)
{
	int i_hoursFrom,i_minsFrom,i_secsFrom,i_microsecsFrom;
	int i_hoursTo,i_minsTo,i_secsTo,i_microsecsTo;
	int i_totalSecsFrom, i_totalSecsTo;
	long long ll_microsecsDiff;
	double d_result;
	
	sscanf(str_timeFrom.ascii(),"%d:%d:%d.%d",&i_hoursFrom,&i_minsFrom,&i_secsFrom,&i_microsecsFrom);
	sscanf(str_timeTo.ascii(),"%d:%d:%d.%d",&i_hoursTo,&i_minsTo,&i_secsTo,&i_microsecsTo);

	i_totalSecsFrom = (i_hoursFrom*60*60)+(i_minsFrom*60)+i_secsFrom;
	i_totalSecsTo = (i_hoursTo*60*60)+(i_minsTo*60)+i_secsTo;

	// operations are calculated in microseconds (int) because of float precision problems
	ll_microsecsDiff = ((long long)i_totalSecsTo-(long long)i_totalSecsFrom)*1000000 + ((long long)i_microsecsTo-(long long)i_microsecsFrom);
	// but always showing miliseconds
	d_result = (double)ll_microsecsDiff / 1000.0;

	return d_result;
}


void NMGMGENXMLFlowFormatParser::sortCurrentLists()
{
	double *d_prevElem;
	double *d_currentElem;
	int i_indexCurrent;
	
	d_prevElem = m_rfd_currentReadData->m_dl_seqIdList.first();
	d_currentElem = m_rfd_currentReadData->m_dl_seqIdList.next();
	i_indexCurrent = m_rfd_currentReadData->m_dl_seqIdList.at();
	while (d_currentElem !=0)
	{
		if (*d_currentElem < *d_prevElem) // have to sort
		{
			double *d_currentElemAux;
			// Take the element and move it to the new position.
			d_currentElem = m_rfd_currentReadData->m_dl_seqIdList.take();
			d_prevElem = m_rfd_currentReadData->m_dl_seqIdList.prev();
			
			do
			{
				d_prevElem = m_rfd_currentReadData->m_dl_seqIdList.prev();
			}
			while(d_prevElem!=0 && *d_currentElem < *d_prevElem);
			// new position found. 
			int i_newIndex = m_rfd_currentReadData->m_dl_seqIdList.at()+1;
			
			m_rfd_currentReadData->m_dl_seqIdList.insert(i_newIndex,d_currentElem);
			
			d_currentElemAux = m_rfd_currentReadData->m_dl_timestampList.take(i_indexCurrent);
			m_rfd_currentReadData->m_dl_timestampList.insert(i_newIndex,d_currentElemAux);

			d_currentElemAux = m_rfd_currentReadData->m_dl_delayList.take(i_indexCurrent);
			m_rfd_currentReadData->m_dl_delayList.insert(i_newIndex,d_currentElemAux);
		}
		d_prevElem = m_rfd_currentReadData->m_dl_seqIdList.at(i_indexCurrent);
		d_currentElem = m_rfd_currentReadData->m_dl_seqIdList.next();
		i_indexCurrent++;
	}
}

void NMGMGENXMLFlowFormatParser::fillWithLostPackets()
{
	double *d_nextElem;
	double *d_currentElem;
	double *d_currentTimestamp;
	double *d_nextTimestamp;
	int i_indexCurrent, i;
	double avgTimestamp, diffSeqID;
	
	d_currentElem = m_rfd_currentReadData->m_dl_seqIdList.first();
	d_nextElem = m_rfd_currentReadData->m_dl_seqIdList.next();

	d_currentTimestamp = m_rfd_currentReadData->m_dl_timestampList.first();
	d_nextTimestamp = m_rfd_currentReadData->m_dl_timestampList.next();

	i_indexCurrent = m_rfd_currentReadData->m_dl_seqIdList.at();

	while (d_nextElem !=0)
	{
		diffSeqID = (*d_nextElem - *d_currentElem);
		if ( diffSeqID != 1)  { // have to add lost packets
			avgTimestamp = (*d_nextTimestamp - *d_currentTimestamp) / diffSeqID;
			i = 1;
			do
			{
				double * n_seqId = new double();
				double * n_timestamp = new double();
				double * n_delay = new double();
				* n_seqId = *d_currentElem + i;
				* n_timestamp = *d_currentTimestamp + (avgTimestamp*i);
				* n_delay = 0.0;
				m_rfd_currentReadData->m_dl_seqIdList.insert(i_indexCurrent, n_seqId);
				m_rfd_currentReadData->m_dl_timestampList.insert(i_indexCurrent, n_timestamp);
				m_rfd_currentReadData->m_dl_delayList.insert(i_indexCurrent, n_delay);
				++i;
				i_indexCurrent++;
				cout << "ID " << i_indexCurrent << endl;
			}
			while(i < diffSeqID);
		}
		d_currentElem = m_rfd_currentReadData->m_dl_seqIdList.at(i_indexCurrent);
		d_currentTimestamp = m_rfd_currentReadData->m_dl_timestampList.at(i_indexCurrent);
		i_indexCurrent++;
		d_nextElem = m_rfd_currentReadData->m_dl_seqIdList.at(i_indexCurrent);
		d_nextTimestamp = m_rfd_currentReadData->m_dl_timestampList.at(i_indexCurrent);
	}
}

void NMGMGENXMLFlowFormatParser::deleteObjects()
{
	if (m_md_metadata!=0)
		delete m_md_metadata;
	m_md_metadata = 0;

	if (m_rfd_currentReadData!=0)
		delete m_rfd_currentReadData;
	m_rfd_currentReadData = 0;

	if (m_rfdl_ReadFlows!=0)
		delete m_rfdl_ReadFlows;
	m_rfdl_ReadFlows = 0;
}

void NMGMGENXMLFlowFormatParser::stop()
{
	m_b_stopParsing = true;
}

void NMGMGENXMLFlowFormatParser::acceptInvalidData(bool b_accept)
{
	m_b_invalidDataAccepted = b_accept;
}

bool NMGMGENXMLFlowFormatParser::isFinished()
{
	return m_b_finished;
}

QPtrList<NMGMGENReadFlowData>* NMGMGENXMLFlowFormatParser::getReadData()
{
	return m_rfdl_ReadFlows;
}

QString NMGMGENXMLFlowFormatParser::getErrorString()
{
	return m_str_error;
}

int NMGMGENXMLFlowFormatParser::getError()
{
	return m_i_errorCode;
}

int NMGMGENXMLFlowFormatParser::getPercentCompleted()
{
	if (m_i_numPacketsAllFlows!=0)
		return ( m_i_currentPacket * 100 ) / m_i_numPacketsAllFlows;
	else
		return 0;
}

QString NMGMGENXMLFlowFormatParser::getLoadStatus()
{
	return QString("");
}



