/***************************************************************************
 *   Copyright (C) 2005 by David Lacort                                    *
 *   david.lacort@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGMGENXMLFLOWFORMATPARSER_H
#define NMGMGENXMLFLOWFORMATPARSER_H

#include <qxml.h>
#include <qstring.h>

#include "nmgmgenmetadata.h"

#include "nmgmgenreadflowdata.h"


/**
	@author David Lacort <david.lacort@gmail.com>
*/
class NMGMGENXMLFlowFormatParser : public QXmlDefaultHandler
{
public:
	NMGMGENXMLFlowFormatParser();

    bool startDocument();
	bool endDocument();
    bool startElement( const QString&, const QString&, const QString& ,
                       const QXmlAttributes& );
	bool characters ( const QString & ch );
    bool endElement( const QString&, const QString&, const QString& );

	bool isFinished();
	int getError();
	void stop();
	void acceptInvalidData(bool b_accept);
	QPtrList<NMGMGENReadFlowData>* getReadData();
	QString getErrorString();

	int getPercentCompleted();
	QString getLoadStatus();

private:
	bool startElement_metadata(const QString &qName, const QXmlAttributes &attributes);
	bool characters_metadata(const QString & ch);
	bool endElement_metadata(const QString& qName);

	bool startElement_result(const QString &qName, const QXmlAttributes &attributes);
	bool characters_result(const QString & ch);
	bool endElement_result(const QString& qName);

	double timeStringDiff(const QString &str_timeFrom, const QString &str_timeTo);
	void sortCurrentLists();

	void deleteObjects();

	void fillWithLostPackets();

	QString m_str_error;
	QString m_str_status;
	bool m_b_invalidDataAccepted;

	QString str_tagName;

	bool b_metadata;
	bool b_source;
	bool b_management;
	bool b_destination;
	bool b_conf_mgen;
	bool b_conf_drec;

	bool b_result;
	bool b_flow;
	bool b_seq;

	QString str_startTime;
	QString str_seqId;
	QString str_txTime;
	QString str_rxTime;

	NMGMGENMetadata *m_md_metadata;
	NMGMGENFlowMetadata *m_fmd_currentFlowMetadata;
	NMGMGENReadFlowData *m_rfd_currentReadData;

	QPtrList<NMGMGENReadFlowData> *m_rfdl_ReadFlows;

	bool m_b_finished;
	bool m_b_stopParsing;
	int m_i_errorCode;
	int m_i_currentPacket;
	int m_i_numPacketsAllFlows;
};

#endif
