/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlistview.h>
#include <qtabwidget.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qgroupbox.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qaction.h>
#include <qradiobutton.h>
#include <qdatetimeedit.h>

#include <messagebar.h>
#include "nmgmgenwidget.h"

#include <nmgpreferencesmanager.h>
#include <nmgmoduletabwidgetmanager.h>
#include <nmgmoduleutils.h>

#include "nmgmgenflow.h"
#include "nmgmgenbandwith.h"

#include <iostream>
#include <cstdlib>

using namespace std;

#define IS_ITEM_MODIFICATION(item)	(item and ((NMGMGenFlowListItem *)item)->getFlowItemType()==FLOW_LIST_ITEM_MODIFICATION)
#define IS_ITEM_FLOW(item)		(item and ((NMGMGenFlowListItem *)item)->getFlowItemType()==FLOW_LIST_ITEM_FLOW)
#define IS_ITEM_EXT_HEADER(item)	(item and ((NMGMGenFlowListItem *)item)->getFlowItemType()==FLOW_LIST_ITEM_EXTENSION_HEADER)

/**
 * Contructor of the class
 * @param parent 
 * @param name 
 * @return 
 */
NMGMGenWidget::NMGMGenWidget(NMGISender * s, NMGIModuleCommunication * mc, NMGIModuleStationManager * sm, NMGMGenFlowManager * fm, 
	NMGModTabWidgetManager * tabWidgetMan, NMGModulePreferencesManager * prefMan, QWidget *parent, const char *name)
 : NMGMGenMainWidget(parent, name), interpret(this), errorNotifier(this)
{
	lastSelectedItem = 0;
	lastFlowId = 0;
	progressItem = 0;
	toolBar = 0;
	sender = s;
	moduleCommunication = mc;
	stationManager = sm;
	tabWidgetManager = tabWidgetMan;
	flowManager = fm;
	preferencesManager = prefMan;
	updateTabWithItem(0);
	flowList->setRootIsDecorated(true);
	dateStartTime->setDate(QDate().currentDate());
	timeStartTime->setTime(QTime().currentTime());
	int randomSourcePort = rand() % 1000 + 5000;
	txtSourcePort->setValue ( randomSourcePort );
	
	connect(flowList,SIGNAL(mouseButtonClicked(int, QListViewItem*, const QPoint&, int )),this,SLOT(flowListMouseButtonClicked(int, QListViewItem*, const QPoint&, int )));
	connect(flowList,SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int )),this,SLOT(flowListRightButtonPressed(QListViewItem*, const QPoint&, int )));
	connect(this,SIGNAL(activatedFlowOrMod(QListViewItem *)),this,SLOT(showItemProperties(QListViewItem * )));
	createFlowListMenu();
	connect(flowList,SIGNAL(selectionChanged ( QListViewItem * )),this,SLOT(flowListItemSelected(QListViewItem *)));

	connect(txtFlowRate,SIGNAL(valueChanged(int)),this,SLOT(updateEthernetAndATMBAndwith(int )));
	connect(txtFlowPacketSize,SIGNAL(valueChanged(int)),this,SLOT(updateEthernetAndATMBAndwith(int )));
	connect(txtModRate,SIGNAL(valueChanged(int)),this,SLOT(updateEthernetAndATMBAndwith(int )));
	connect(txtModPacketSize,SIGNAL(valueChanged(int)),this,SLOT(updateEthernetAndATMBAndwith(int )));
	
	connect(buttonStart,SIGNAL(clicked()),this,SLOT(buttonStartClicked()));
	connect(buttonStop,SIGNAL(clicked()),this,SLOT(buttonStopClicked()));
	connect(buttonOreneta,SIGNAL(clicked()),this,SLOT(buttonOrenetaClicked()));
	
	connect(&progressTimer,SIGNAL(timeout()),this,SLOT(incrementProgressItemBar()));
	connect(comboMgenVersion,SIGNAL(activated(const QString &)),this,SLOT(activeBurstPattern(const QString &)));
	connect(this,SIGNAL(activatedMgenVersion(const QString &)),this,SLOT(activeBurstPattern(const QString &)));
	connect(checkBoxTypeOfService,SIGNAL(toggled(bool)),this,SLOT(checkRootToS(bool)));
//	connect(,SIGNAL(stationChanged(NMGStation * )),this,SLOT(sourceStationChanged(NMGStation* )));	
//	sourceStationChanged(hostSourceCombo->getStationFromCombo());
}

/**
 * Destructor of the class
 * @return 
 */
NMGMGenWidget::~NMGMGenWidget()
{
	if(progressItem) progressItem->setComplete();
	progressItem = 0;
}

/**
 * Creates the flow list menu
 */
void NMGMGenWidget::createFlowListMenu()
{
	flowListMenu = new QPopupMenu();
	flowListMenu->insertItem("Add &new flow",this,SLOT(addNewFlow()),CTRL+Key_N);
	flowListMenu->insertItem("&Delete item",this,SLOT(deleteItem()),CTRL+Key_D);
	flowListMenu->insertItem("&Delete all items",this,SLOT(deleteAllItems()),SHIFT+CTRL+Key_D);
	flowListMenu->insertItem("Add &modification",this,SLOT(addModificationToFlow()),CTRL+Key_M);
	flowListMenu->insertSeparator();
	flowListMenu->insertItem("&Load test from disk",this,SLOT(loadTestFromDisk()),CTRL+Key_L);
	flowListMenu->insertItem("&Save test to disk",this,SLOT(saveTestToDisk()),CTRL+Key_S);
}


/**
 * Returns the toolbar of the mgen widget
 * @return 
 */
QToolBar * NMGMGenWidget::getToolBar()
{
	if(!toolBar)
	{
		toolBar = new QToolBar(NMGModuleUtils::getMainWindow());
		QAction * action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("button_ok.png")), tr("Add new flow"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(addNewFlow()));
		action->addTo(toolBar);

		action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("delete.png")), tr("Delete item"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(deleteItem()));
		action->addTo(toolBar);
		
		action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("edittrash.png")), tr("Delete all items"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(deleteAllItems()));
		action->addTo(toolBar);
		
		action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("launch.png")), tr("Start test"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(buttonStartClicked()));
		action->addTo(toolBar);
	}
	return toolBar;
}

/**
 * Slots whichs adds a new flow
 */
void NMGMGenWidget::addNewFlow()
{
	updateTabContentToItem(getSelectedItem());
	NMGMGenFlow * flow = new NMGMGenFlow(flowManager, flowList, flowList->lastItem());
	flow->setId(1+lastFlowId++);
	flowList->setCurrentItem(flow->getListViewItem());
	if(!lastSelectedItem) emit activatedFlowOrMod(flow->getListViewItem());
	lastSelectedItem = flow->getListViewItem();
}

/**
 * Slots which deletes the selected item (if any)
 */
void NMGMGenWidget::deleteItem()
{
	QListViewItem * item = getSelectedItem();
	deleteItem(item);
}

/**
 * Returns the slected item of the flow list (or 0)
 * @return 
 */
QListViewItem * NMGMGenWidget::getSelectedItem()
{
	if(flowList->childCount()==1) return flowList->firstChild();
	else return flowList->selectedItem();
}

/**
 * Deletes the item of the list
 * @param item 
 */
void NMGMGenWidget::deleteItem(QListViewItem * item)
{
	QListViewItem * nextItem;
	if(item and item->itemAbove()) nextItem = item->itemAbove();
	else if(item) nextItem = item->nextSibling();
	if(IS_ITEM_FLOW(item))
	{
		if(item->childCount()>0)
		{
			for(QListViewItem * itemChild = item->firstChild(); itemChild; itemChild = itemChild->nextSibling())
				deleteItem(itemChild);
		}
		flowList->takeItem(item);
		delete item;
		lastSelectedItem = 0;
		updateFlowListIdentifier();
	}
	else if(IS_ITEM_MODIFICATION(item) or IS_ITEM_EXT_HEADER(item))
	{
		item->parent()->takeItem(item);
		delete item;
		lastSelectedItem = 0;	 
	}
	else 
	{
		QMessageBox::warning(0,"No item selected","You have to select something to delete");
		return;
	}
	
	if(nextItem)
	{
		flowList->setCurrentItem(nextItem);
		flowList->setSelected(nextItem,true);
		lastSelectedItem = nextItem;
	}
	else emit activatedFlowOrMod(0);
}

/**
 * Slot which deletes all the items of the test
 */
void NMGMGenWidget::deleteAllItems()
{
	deleteAllItemsWithResult();
}

/**
 * Deletes all the items
 * @return Returns true if all the items where deleted
 */
bool NMGMGenWidget::deleteAllItemsWithResult()
{
	if(flowList->childCount()==0) return true;
	
	int reply = QMessageBox::QMessageBox::question(0,"Confirm operation",
		"Do you really want to delete all the items?", QMessageBox::Yes|QMessageBox::Default,
		QMessageBox::No|QMessageBox::Escape);
	if(reply==QMessageBox::No) return false;

	
	if(flowList->childCount()==1)
	{
		deleteItem(flowList->firstChild());
		return true;
	}
	
	//flowList->setCurrentItem(flowList->firstChild());
	
	while(flowList->childCount()>0) deleteItem(flowList->firstChild());
	
	return true;
}

/**
 * Slots which adds a modification to an existing flow
 */
void NMGMGenWidget::addModificationToFlow()
{
	updateTabContentToItem(getSelectedItem());
	QListViewItem * item = getSelectedItem();
 	if(not IS_ITEM_FLOW(item) and not IS_ITEM_MODIFICATION(item) and not IS_ITEM_EXT_HEADER(item))
		QMessageBox::warning(0,"No flow selected","You can only add modifications to a flow");
	else
	{
		NMGMGenFlow * flow;
		if(IS_ITEM_MODIFICATION(item) or IS_ITEM_EXT_HEADER(item)) flow = (NMGMGenFlow *) item->parent();
		else flow = (NMGMGenFlow *) item;
		NMGMGenFlowModification * mod = new NMGMGenFlowModification(flow,flow->firstChild());
		item->setOpen(true);
		flowList->setCurrentItem(mod->getListViewItem());
	}
}

/**
 * Slot which load a flow from disk
 */
void NMGMGenWidget::loadTestFromDisk()
{
	QString fileName = QFileDialog::getOpenFileName(getLastTestPath(),"XML Flow Format (*.xff);;XML (*.xml);;All files (*.*)",0,0,"Select test file");
	
	if(fileName and fileName!="")
	{
		saveLastTestPathFromFilePath(fileName);
		NMGXMLConfigurationFile file;
		file.setPath(fileName);
		if(!file.readFromDisk())
		{
			QMessageBox::critical(0,"Error opening file",
				"Error while reading the document from disk.\n"+file.getError());
			cerr<<"Error while reading the document from disk.\n"<<file.getError()<<endl;
		}
		else
		{
			QDomElement root = file.getDomDocument().documentElement();
			if( not deleteAllItemsWithResult()) return;
			interpret.updateWidgetFromXMLDocument(root);
			emit activatedMgenVersion(comboMgenVersion->currentText());
		}
		
	}
}

/**
 * Slot which saves flows to disk
 */
void NMGMGenWidget::saveTestToDisk()
{
	updateTabContentToItem(getSelectedItem());
	if(flowList->childCount()==0) QMessageBox::warning(0,"No flows","There are no flows on the list.");
	else
	{
		if(hostSourceCombo->getStationFromCombo() == 0) 
			QMessageBox::warning(0,"No Source Host","Source host is not defined.");
		else if (hostDestinationCombo->getStationFromCombo() == 0)
			QMessageBox::warning(0,"No Destination Host","Destination host is not defined.");
		else {
			QString testContent = interpret.generateXMLTestDocument(false);
		
			QString fileName = QFileDialog::getSaveFileName(getLastTestPath(),"XML Flow Format (*.xff);;XML (*.xml);;All files (*.*)",0,0,"File to save the flows");
			if(fileName and fileName!="")
			{
				if(!fileName.contains(".xff")) fileName = fileName + ".xff";
				if(QFile::exists(fileName))
				{
					int reply = QMessageBox::question(0,"Overwirte the file?","The file "+fileName+" exists.\n"+
						"Do you want to overwrite it?",
						QMessageBox::Yes|QMessageBox::Default,QMessageBox::No|QMessageBox::Escape);
					if(reply==QMessageBox::No) return;
				}
				saveLastTestPathFromFilePath(fileName);
				NMGXMLConfigurationFile file;
				file.setPath(fileName);
				file.getDomDocument().setContent(testContent);
				if(!file.writeToDisk()) 
					cerr << "Error while writing the file to disk("<<file.getError()<<").";
			}
		}
	}
}

/**
 * Slot executed when the user clicks with the right button on the flow list
 * @param item Item clicked (0 if there is no such item)
 * @param point Position of the mouse
 * @param  
 */
void NMGMGenWidget::flowListRightButtonPressed ( QListViewItem * , const QPoint & point, int )
{
	flowListMenu->popup(point);
}

/**
 * Slot executed when the users clicks on the flow list
 * @param button Button clicked
 * @param item Item clicked (if any)
 * @param  
 * @param  
 */
void NMGMGenWidget::flowListMouseButtonClicked ( int button, QListViewItem * item, const QPoint & , int )
{
	if(button==LEFT_BUTTON and !item) /* to disable the tab widget */
		flowListItemSelected(item);
}

/**
 * Slot executed when the users presses space or return
 * @param item Item selected (if any)
 */
void NMGMGenWidget::flowListItemSelected(QListViewItem * item)
{
	if(lastSelectedItem!=item and flowList->childCount()>1) updateTabContentToItem(lastSelectedItem);
	if(item)
	{
		showItemProperties(item);
		lastSelectedItem = item;
	}
	else showItemProperties(0);
}

/**
 * Shows the information of the flow
 * @param flow 
 */
void NMGMGenWidget::showItemProperties(QListViewItem * item)
{
	updateTabWithItem(item);
	if(IS_ITEM_FLOW(item))
	{
		NMGMGenFlow *flow = (NMGMGenFlow *) item;
		txtFlowName->setText(flow->getName());
		txtFlowStart->setValue(flow->getStartTime());
		txtFlowDuration->setValue(flow->getDuration());
		txtFlowDestinationPort->setValue(flow->getPort());
		comboFlowPattern->setCurrentText(flow->getPattern());
		txtFlowRate->setValue(flow->getRate());
		txtFlowPacketSize->setValue(flow->getPacketSize());

		if(flow->isActiveBurst() && comboMgenVersion->currentText()=="4")
		{
			checkBoxPatternBurst->setChecked(true);
			comboInterval->setCurrentText(flow->getBurstIntervalTxt());
			txtInterval->setValue(flow->getBurstInterval());
			comboDuration->setCurrentText(flow->getBurstDurationTxt());
			txtDuration->setValue(flow->getBurstDuration());
		}
		else
		{
			checkBoxPatternBurst->setChecked(false);
			comboInterval->setCurrentText(DEFAULT_BURST_INTERVAL);
			txtInterval->setValue(0);
			comboDuration->setCurrentText(DEFAULT_BURST_DURATION);
			txtDuration->setValue(0);
		}	
		if(flow->getTypeOfService()<0)
		{
			checkBoxTypeOfService->setChecked(false);
			txtTypeOfService->setText("");
		}
		else
		{
			checkBoxTypeOfService->setChecked(true);
			txtTypeOfService->setText(QString("%1").arg(flow->getTypeOfService()));
		}
		if(flow->getRSVP().isActive())
		{
			checkBoxRSVPParameters->setChecked(true);
			txtRSVPParameters->setText(flow->getRSVP().getSourceParam());
			if(flow->getRSVP().getResvStyleAsShortString()=="WF") radioRSVPWF->setChecked(true);
			else if(flow->getRSVP().getResvStyleAsShortString()=="FF") radioRSVPFF->setChecked(true);
			else radioRSVPSE->setChecked(true);
			txtRESVParameters->setText(flow->getRSVP().getDestParam());
		}
		else
		{
			checkBoxRSVPParameters->setChecked(false);
			txtRSVPParameters->setText("");
			txtRESVParameters->setText("");
			radioRSVPWF->setChecked(true);
		}
		if(flow->getTrafficClass()<0)
		{
			checkBoxTrafficClass->setChecked(false);
			txtTrafficClass->setText("");
		}
		else
		{
			checkBoxTrafficClass->setChecked(true);
			txtTrafficClass->setText(QString("%1").arg(flow->getTrafficClass()));
		}		
	}
	else if(IS_ITEM_MODIFICATION(item))
	{
		NMGMGenFlowModification *mod = (NMGMGenFlowModification *) item;
		txtModStart->setValue(mod->getStartTime());
		txtModRate->setValue(mod->getRate());
		txtModPacketSize->setValue(mod->getPacketSize());
		comboModPattern->setCurrentText(mod->getPattern());
	
		if(mod->isActiveBurst() && comboMgenVersion->currentText()=="4")
		{
			checkBoxPatternBurstMod->setChecked(true);
			comboIntervalMod->setCurrentText(mod->getBurstIntervalTxt());
			txtIntervalMod->setValue(mod->getBurstInterval());
			comboDurationMod->setCurrentText(mod->getBurstDurationTxt());
			txtDurationMod->setValue(mod->getBurstDuration());
		}
		else
		{
			checkBoxPatternBurstMod->setChecked(false);
			comboIntervalMod->setCurrentText(DEFAULT_BURST_INTERVAL);
			txtIntervalMod->setValue(0);
			comboDurationMod->setCurrentText(DEFAULT_BURST_DURATION);
			txtDurationMod->setValue(0);
		}		
		if(mod->getTypeOfService()<0)
		{
			checkBoxTypeOfService->setChecked(false);
			txtTypeOfService->setText("");
		}
		else
		{
			checkBoxTypeOfService->setChecked(true);
			txtTypeOfService->setText(QString("%1").arg(mod->getTypeOfService()));
		}
		if(mod->getRSVP().isActive())
		{
			checkBoxRSVPParameters->setChecked(true);
			txtRSVPParameters->setText(mod->getRSVP().getSourceParam());
			if(mod->getRSVP().getResvStyleAsShortString()=="WF") radioRSVPWF->setChecked(true);
			else if(mod->getRSVP().getResvStyleAsShortString()=="FF") radioRSVPFF->setChecked(true);
			else radioRSVPSE->setChecked(true);
			txtRESVParameters->setText(mod->getRSVP().getDestParam());
		}
		else
		{
			checkBoxRSVPParameters->setChecked(false);
			txtRSVPParameters->setText("");
			txtRESVParameters->setText("");
			radioRSVPWF->setChecked(true);
		}
		if(mod->getTrafficClass()<0)
		{
			checkBoxTrafficClass->setChecked(false);
			txtTrafficClass->setText("");
		}
		else
		{
			checkBoxTrafficClass->setChecked(true);
			txtTrafficClass->setText(QString("%1").arg(mod->getTrafficClass()));
		}
	}
	else if(IS_ITEM_EXT_HEADER(item))
	{
		NMGMGenFlowExtensionHeader * header = (NMGMGenFlowExtensionHeader *) item;
		txtExtHeaderStartTime->setValue(header->getOptionStartTime());
		txtExtHeaderStopTime->setValue(header->getOptionStopTime());
		comboBoxOptionHeader->setCurrentText(header->getExtensionHeaderTypeAsString());
		txtExtHeaderOptionalParameter->setText(header->getOptionalParameter());
		/* labelExtHeaderOptionalParameter */
	}
}

/**
 * Updates the tab widget flow with a new flow
 * @param flow 
 */
void NMGMGenWidget::updateTabWithItem(QListViewItem * item)
{
	if(IS_ITEM_FLOW(item))
	{
		tabFlowProperties->setEnabled(true);
		removeAllTabFlowPages();
		tabFlowProperties->addTab(pageFlowProperties,TAB_FLOW_PROPERTIES);
		if(isProtocolIPv4()) tabFlowProperties->addTab(tabPageIPV4Options,TAB_IPV4_OPTIONS);
		else tabFlowProperties->addTab(tabPageIPV6Options,TAB_IPV6_OPTIONS);
		tabFlowProperties->setCurrentPage(tabFlowProperties->indexOf(pageFlowProperties));
		tabFlowProperties->addTab(tagPageResultFile,TAB_RESULT_FILE);
		
	}
	else if(IS_ITEM_MODIFICATION(item))
	{
		tabFlowProperties->setEnabled(true);
		removeAllTabFlowPages();
		tabFlowProperties->addTab(pageModificationProperties,TAB_MOD_FLOW_PROPERTIES);
		if(isProtocolIPv4()) tabFlowProperties->addTab(tabPageIPV4Options,TAB_IPV4_OPTIONS);
		else tabFlowProperties->addTab(tabPageIPV6Options,TAB_IPV6_OPTIONS);
		tabFlowProperties->setCurrentPage(tabFlowProperties->indexOf(pageModificationProperties));
		tabFlowProperties->addTab(tagPageResultFile,TAB_RESULT_FILE);
	}
	else if(IS_ITEM_EXT_HEADER(item))
	{
		removeAllTabFlowPages();
		tabFlowProperties->addTab(tabPageIPv6ExtensionsHeaders,TAB_IPV6_EXT_HEADERS);
		tabFlowProperties->setCurrentPage(tabFlowProperties->indexOf(tabPageIPv6ExtensionsHeaders));
		tabFlowProperties->addTab(tagPageResultFile,TAB_RESULT_FILE);
	}
	else tabFlowProperties->setEnabled(false);
}

/**
 * Removes the tab pages which are present in the tabFlowProperties
 */
void NMGMGenWidget::removeAllTabFlowPages()
{
	if(tabFlowProperties->indexOf(pageFlowProperties)>=0)
		tabFlowProperties->removePage(pageFlowProperties);

	if(tabFlowProperties->indexOf(pageModificationProperties)>=0)
		tabFlowProperties->removePage(pageModificationProperties);
			
	if(tabFlowProperties->indexOf(tabPageIPV4Options)>=0)
		tabFlowProperties->removePage(tabPageIPV4Options);
			
	if(tabFlowProperties->indexOf(tabPageIPV6Options)>=0)
		tabFlowProperties->removePage(tabPageIPV6Options);
			
	if(tabFlowProperties->indexOf(tabPageIPv6ExtensionsHeaders)>=0)
		tabFlowProperties->removePage(tabPageIPv6ExtensionsHeaders);
		
	if(tabFlowProperties->indexOf(tagPageResultFile)>=0)
		tabFlowProperties->removePage(tagPageResultFile);
		
}

/**
 * Updates the item (flow or modification from the existing data in the tab widget)
 * @param item Item to update
 */
void NMGMGenWidget::updateTabContentToItem(QListViewItem * item)
{
	if(flowList->childCount()==1 and !item) item = flowList->firstChild();
	if(!item) return;
	if(IS_ITEM_MODIFICATION(item))
	{ // item is a flow modification
		NMGMGenFlowModification * mod = (NMGMGenFlowModification *) item;
		mod->setStartTime(txtModStart->value());
		mod->setRate(txtModRate->value());
		mod->setPacketSize(txtModPacketSize->value());
		mod->setPattern(comboModPattern->currentText());
		
		if(checkBoxTypeOfService->isChecked()) 
			mod->setTypeOfService(txtTypeOfService->text().toInt(0,0));
		else mod->setTypeOfService(-1);

		if(checkBoxPatternBurstMod->isChecked() && comboMgenVersion->currentText()=="4") {
			mod->setBurstInterval(comboIntervalMod->currentText(),txtIntervalMod->value());
			mod->setBurstDuration(comboDurationMod->currentText(),txtDurationMod->value());
			mod->setBurst(true);
		}
		else {
			mod->setBurstInterval(DEFAULT_BURST_INTERVAL,0);
			mod->setBurstDuration(DEFAULT_BURST_DURATION,0);
			mod->setBurst(false);
		}
		
		if(checkBoxRSVPParameters->isChecked())
		{
			mod->getRSVP().setActive(true);
			mod->getRSVP().setSourceParam(txtRSVPParameters->text());
			mod->getRSVP().setResvStyleAsShortString(radioRSVPWF->isChecked()?"WF":radioRSVPFF->isChecked()?"FF":"SE");
			mod->getRSVP().setDestParam(txtRESVParameters->text());
		}
		else mod->getRSVP().setActive(false);

		if(checkBoxTrafficClass->isChecked()) mod->setTrafficClass(txtTrafficClass->text().toInt());
		else mod->setTrafficClass(-1);
	}
	else if(IS_ITEM_FLOW(item))
	{ //item is a flow
		NMGMGenFlow * flow = (NMGMGenFlow *) item;
		flow->setName(txtFlowName->text().utf8());
		flow->setStartTime(txtFlowStart->value());
		flow->setDuration(txtFlowDuration->value());
		flow->setPattern(comboFlowPattern->currentText());
		flow->setRate(txtFlowRate->value());
		flow->setPacketSize(txtFlowPacketSize->value());
		if(flow->getPort() != txtFlowDestinationPort->value())
		{
			if(not flow->getFlowManager()
					->isPortAvailable(txtFlowDestinationPort->value()))
			{
				QMessageBox::warning(0,"Port used","The port " +
					QString("%1").arg(txtFlowDestinationPort->value()) + 
					" is already used.");
			}
			else flow->setPort(txtFlowDestinationPort->value());
		}
		
		if(checkBoxPatternBurst->isChecked() && comboMgenVersion->currentText()=="4") {
			flow->setBurstInterval(comboInterval->currentText(),txtInterval->value());
			flow->setBurstDuration(comboDuration->currentText(),txtDuration->value());
			flow->setBurst(true);
		}
		else {
			flow->setBurstInterval(DEFAULT_BURST_INTERVAL,0);
			flow->setBurstDuration(DEFAULT_BURST_DURATION,0);
			flow->setBurst(false);
		}

		if(checkBoxTypeOfService->isChecked()) 
			flow->setTypeOfService(txtTypeOfService->text().toInt(0,0));
		else flow->setTypeOfService(-1);
		
		if(checkBoxRSVPParameters->isChecked())
		{
			flow->getRSVP().setActive(true);
			flow->getRSVP().setSourceParam(txtRSVPParameters->text());
			flow->getRSVP().setResvStyleAsShortString(radioRSVPWF->isChecked()?"WF":radioRSVPFF->isChecked()?"FF":"SE");
			flow->getRSVP().setDestParam(txtRESVParameters->text());
		}
		else flow->getRSVP().setActive(false);

		if(checkBoxTrafficClass->isChecked()) flow->setTrafficClass(txtTrafficClass->text().toInt());
		else flow->setTrafficClass(-1);	
	}
	else if(IS_ITEM_EXT_HEADER(item))
	{
	
	}
}

/**
 * Slot executed when the user clicks the start button to start a new test
 */
void NMGMGenWidget::buttonStartClicked()
{
	updateTabContentToItem(getSelectedItem());
	
	if(not allFieldsNeededCorrect()) return;
	
	QString xmlTest = interpret.generateXMLTestDocument(true);
	
	if(sender->sendCommand(Start,xmlTest))
	{
		buttonStop->setEnabled(true);
		buttonStart->setEnabled(false);
		tabWidgetManager->setTabImage("runa.png");
		maxTestTime = getMaximumFlowEndTime();	
		QString title = txtTestName->text()==""?"One way delay":txtTestName->text();
		progressItem = ProgressManager::createProgressItem(title);
		progressItem->setTotalItems(100);		
		connect(progressItem, SIGNAL(progressItemCanceled( ProgressItem* )), this, SLOT( testCancelled( ProgressItem *) ) );
		progressItem->setStatus("Preparing test...");
	}
	else MessageBar::longMessage("Command not started\nThere is no connection with the server!");

}


/**
 * Checks if all the fields are correct to send a start message to the server
 * If they are not correct a message box is shown to the server
 * @return 
 */
bool NMGMGenWidget::allFieldsNeededCorrect()
{
	QString errorMessage = "";
	
	if(hostSourceCombo->getStationAddressFromCombo()=="") 
		errorMessage += "The source address is not an optional field.\n";
	else
	{
		NMGStation * sourceStation = hostSourceCombo->getStationFromCombo();
		if(!sourceStation)
			errorMessage += "The source host is not optional.\n";
		else
		{
			switch ((comboMgenVersion->currentText()).toInt()) {
				case 3:
					if(sourceStation->getStationFieldsManager().getField(getStationManager()->getModuleStationKey(),ID_MGEN_PATH)=="")
						errorMessage += "Mgen path is not an optional field for the source host.\n";
					break;
				case 4:
					if(sourceStation->getStationFieldsManager().getField(getStationManager()->getModuleStationKey(),ID_MGEN4_PATH)=="")
						errorMessage += "Mgen path for version 4 is not an optional field for the source host.\n";
					break;
			}
			if(sourceStation->getManagementUsername()=="")
				errorMessage += "Management username is not an optional field for the source host.\n";
		}
	}
	
		
	if(hostDestinationCombo->getStationAddressFromCombo()=="") 
		errorMessage += "The destination address is not an optional field.\n";
	else
	{
		NMGStation * destionationStation = hostDestinationCombo->getStationFromCombo();
		if(!destionationStation)
			errorMessage += "The destination host is not optional.\n";
		else
		{
			switch ((comboMgenVersion->currentText()).toInt()) {
				case 3:
					if(destionationStation->getStationFieldsManager().
						getField(getStationManager()->getModuleStationKey(),ID_DREC_PATH)=="")
						errorMessage += "Drec path is not an optional field for the destination host.\n";
					if(destionationStation->getStationFieldsManager().getField(getStationManager()->getModuleStationKey(),ID_DECODE_PATH)=="")
						errorMessage += "Decode path is not an optional field for the destination host.\n";
					break;
				case 4: 
					if(destionationStation->getStationFieldsManager().getField(getStationManager()->getModuleStationKey(),ID_MGEN4_PATH)=="")
						errorMessage += "Mgen path for version 4 is not an optional field for the destination host.\n";
					break;
			}
			if(destionationStation->getManagementUsername()=="")
				errorMessage += "Management username is not an optional field for the destination host.\n";
		}
	}
	
	
	if(flowList->childCount()==0)
		errorMessage += "There are no flows in the test.\n";
		
	if(errorMessage!="")
		QMessageBox::warning(0,"Command not started",errorMessage);
	return errorMessage==""?true:false;
}

/**
 * Test cancelled using the progress bar widget
 * @param item 
 */
void NMGMGenWidget::testCancelled( ProgressItem * )
{
	buttonStopClicked();
}

/**
 * Slot executed when the user clicks the stop button to stop a test which is being executed
 */
void NMGMGenWidget::buttonStopClicked()
{
	sender->sendCommand(Stop,"");
	updateWidgetBecauseOfTestEnd();
}

/**
 * Updates the widgets becuse the end has ended
 */
void NMGMGenWidget::updateWidgetBecauseOfTestEnd()
{
	buttonStop->setEnabled(false);
	buttonStart->setEnabled(true);
	tabWidgetManager->removeTabImage();
	
	if(progressItem) progressItem->setComplete();
	progressItem = 0;
}

/**
 * Slot executed when the users clkick the Oreneta button
 */
void NMGMGenWidget::buttonOrenetaClicked()
{
	if(!moduleCommunication->existsModule(ORENETA_MODULE_NAME))
		QMessageBox::warning(0,"Oreneta is not loaded","Oreneta module is not loaded ito memory.\nPlease load the module if you want to execute it.");
	else
	{
		
		if(hostSourceCombo->getStationAddressFromCombo()=="" or 
			hostDestinationCombo->getStationAddressFromCombo()=="") 
		{
			QMessageBox::warning(0,"Insufficient parameters",
				"Source and destination address are needed by Oreneta module.");
			return;
		}

		
		QString data = "<data><source>"+hostSourceCombo->getStationAddressFromCombo()+
			"</source><destination>"+hostDestinationCombo->getStationAddressFromCombo()+"</destination></data>";
		QString id;
		if((id=moduleCommunication->subscribe(ORENETA_MODULE_NAME,data))!="") cout << "Execution of Oreneta Succesful(id: "<<id<<")."<<endl;
		else cerr <<"Error while subscribing Oreneta to the MGEn Module"<<endl;
	}
}
 
/**
 * Slot executed when the source station changes
 * @param station 
 */
void NMGMGenWidget::sourceStationChanged(NMGStation * station)
{
	txtTestDestination->setText(station?station->getManagementHost().getAddress():"");
}
   
/**
 * Updates the bandwiths of ATM and Ehernet
 * @param  
 */
void NMGMGenWidget::updateEthernetAndATMBAndwith(int )
{
	/* flow tab widgets  */
	int bwATM = NMGMGenBandwith::getATMBandwith(txtFlowRate->value(),txtFlowPacketSize->value());
	int bwEth = NMGMGenBandwith::getEthernetBandwith(txtFlowRate->value(),txtFlowPacketSize->value());

	txtBandwithATM->setText(QString("%1").arg(bwATM)+" kbps");
	txtBandwithEthernet->setText(QString("%1").arg(bwEth)+" kbps");
	
	/* mod tab widgets  */
	bwATM = NMGMGenBandwith::getATMBandwith(txtModRate->value(),txtModPacketSize->value());
	bwEth = NMGMGenBandwith::getEthernetBandwith(txtModRate->value(),txtModPacketSize->value());

	txtModBandwithATM->setText(QString("%1").arg(bwATM)+" kbps");
	txtModBandwithEthernet->setText(QString("%1").arg(bwEth)+" kbps");


}

/**
 * Returns true if the protocol used in the test is IPv4
 * @return 
 */
bool NMGMGenWidget::isProtocolIPv4()
{
	NMGStation * station = hostDestinationCombo->getStationFromCombo();
	if(!station) return true;
	else return station->getTestHost().isHostIPv4();
}

/**
 * Updates the flows identifiers
 */
void NMGMGenWidget::updateFlowListIdentifier()
{
	lastFlowId = 1;
	for(QListViewItem * itemChild = flowList->firstChild(); itemChild; itemChild = itemChild->nextSibling())
		((NMGMGenFlow *) itemChild)->setId(lastFlowId++);
}

/**
 * Returns the op up menu of the widget
 * @return 
 */
QPopupMenu * NMGMGenWidget::getPopupMenu()
{
	return flowListMenu;
}

/**
 * Returns the station manager
 * @return 
 */
NMGIModuleStationManager * NMGMGenWidget::getStationManager()
{
	return stationManager;
}

/**
 * Returns the flow manager
 * @return 
 */
NMGMGenFlowManager * NMGMGenWidget::getFlowManager()
{
	return flowManager;
}

/**
 * Returns the last test path (or "" if undefined)
 * @return 
 */
QString NMGMGenWidget::getLastTestPath()
{
	QString lastTestPath = preferencesManager->getValue(ID_PREFERENCE_LAST_TEST_PATH);
	if(!lastTestPath or lastTestPath=="") lastTestPath = "";
	return lastTestPath;
}

/**
 * Saves the last test path from the filename path of the test
 * @param filePath 
 */
void NMGMGenWidget::saveLastTestPathFromFilePath(const QString & filePath)
{
	if(filePath and filePath!="")
	{
		int lastPos = filePath.findRev("/");
		preferencesManager->addValue(ID_PREFERENCE_LAST_TEST_PATH,filePath.left(lastPos));
	}
}

/**
 * Notifies the arrival of the reply of the start message
 * @param data XML message to be parsed
 */
void NMGMGenWidget::notifyResult(QString data)
{
	QString decodeFile = interpret.getResultFilePathFromResult(data);
	if(decodeFile=="")
	{
		MessageBar::longMessage( "The execution of the Mgen test wasn't succesful.");
		updateWidgetBecauseOfTestEnd();
	}
	else
	{
		updateWidgetBecauseOfTestEnd();
		MessageBar::longMessage("MGen test finished; Decode result at " + decodeFile + ".");
	}
}

/**
 * Notifies the error of the core module
 * @param errorType Type of the error (fatal = 1, user = 5, warning = 10, debug = 20)
 * @param errorMessage Error Message
 */
void NMGMGenWidget::notifyError(int errorType, QString errorMessage)
{
	if(errorType>=1 and errorType<=5) updateWidgetBecauseOfTestEnd();
	
	errorNotifier.notifyErrorToUser(errorType,errorMessage);
}

/**
 * Notifies an status reply form the core
 * @param status XMl document with root data
 */
void NMGMGenWidget::notifyStatus(QString status)
{
	#ifdef DEBUG
		cout << "NMGMGenWidget::notifyStatus " << status << endl;
	#endif
	
	typedef enum{NotStarted=0, SendingDrecScriptFile=10, DrecStarted=20, SendingMgenScriptFile=30, 
		MgenStarted=40, MgenFinished = 50, DecodeStarted = 60, DecodeFinished = 70,
		ReceivingDecodeResultFile=80, TestFinished = 90, TestError=100} MgenStatusReply;
	
	if(!progressItem) cerr << "Status arrived without item at the progress bar!"<<endl;
	else
	{
		int statusNum = interpret.parseStatusMessage(status);
		if(statusNum<0) cerr << "Error at status message." << endl;
		else
		{
			MgenStatusReply reply = (MgenStatusReply) statusNum;
			switch(reply)
			{
				case NotStarted: //0%
					progressItem->setStatus("Test not started");
					break;
				case SendingDrecScriptFile: //1%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					if (comboMgenVersion->currentText()=="4")
						progressItem->setStatus("Sending Mgen (DST) script file");
					else
						progressItem->setStatus("Sending Drec script file");
					break;
				case DrecStarted: //2%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					if (comboMgenVersion->currentText()=="4")
						progressItem->setStatus("Mgen (DST) started listening");
					else
						progressItem->setStatus("Drec started");
					break;
				case SendingMgenScriptFile: //3%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					runningMgenAndDrec = false;
					progressItem->setStatus("Sending Mgen script file");
					break;
				case MgenStarted: // 4% to 95% (91 increments to do)
					if(!runningMgenAndDrec)
					{ //start the timer to increase the progress bar
						stepTime=maxTestTime/91;
						leftTime = maxTestTime;
						progressTimer.start(stepTime);
						runningMgenAndDrec = true;
					}
					if (comboMgenVersion->currentText()=="4")
						progressItem->setStatus("Mgen running in SRC and DST");
					else
						progressItem->setStatus("Mgen and Drec running");
					break;
				case MgenFinished://96%
					progressTimer.stop();
					runningMgenAndDrec = false;
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Mgen finished");
					break;
				case DecodeStarted://97%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Decode started");
					break;
				case DecodeFinished: //98%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Decode finished");
					break;
				case ReceivingDecodeResultFile: //99%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Receiving decode result file");
					break;
				case TestFinished: //100%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Test finished");
					progressItem->setComplete();
					progressItem = 0;
					break;
				case TestError:
					progressTimer.stop();
					runningMgenAndDrec = false;
					progressItem->setStatus("Error executing test");
					break;
				default:
					progressItem->setStatus("Unknown status!");
			}
		}
	}
}

/**
 * Increments the progress bar
 */
void NMGMGenWidget::incrementProgressItemBar()
{
	if(progressItem and leftTime > 1)
	{
		leftTime -= stepTime;
		progressItem->incCompletedItems();
		progressItem->updateProgress();
		if (comboMgenVersion->currentText()=="4")
			progressItem->setStatus("Mgen (SRC and DST) running ("+getLeftTimeTestString()+" left)");
		else
			progressItem->setStatus("Mgen and Drec running ("+getLeftTimeTestString()+" left)");
	}
}

/**
 * Active checkbox for burst pattern
 */
void NMGMGenWidget::activeBurstPattern(const QString & mgenVersion)
{
	if(mgenVersion == "4") {
		checkBoxPatternBurst->setEnabled(true);
		checkBoxPatternBurstMod->setEnabled(true);
	} else {
		checkBoxPatternBurst->setEnabled(false);
		checkBoxPatternBurstMod->setEnabled(false);
   	 	if(checkBoxPatternBurst->isChecked()) {
			checkBoxPatternBurst->setChecked(false);
			txtInterval->setEnabled(false);
			txtDuration->setEnabled(false);
			comboInterval->setEnabled(false);
			comboDuration->setEnabled(false);
			
		}
		if(checkBoxPatternBurstMod->isChecked()) {
			checkBoxPatternBurstMod->setChecked(false);
			txtIntervalMod->setEnabled(false);
			txtDurationMod->setEnabled(false);
			comboIntervalMod->setEnabled(false);
			comboDurationMod->setEnabled(false);
			
		}
	}
}

/**
 * Check if user is "root" in order to change ToS
 */
void NMGMGenWidget::checkRootToS(bool on)
{
	if(on && hostSourceCombo->getStationFromCombo()!=0) {
		if(hostSourceCombo->getStationFromCombo()->getManagementUsername()!="root")
			QMessageBox::warning(0,"ToS Warning","To set ToS you must be root.");
	} else if(on && hostSourceCombo->getStationFromCombo()==0)
		QMessageBox::warning(0,"ToS Warning","To set ToS you must be root.");
}

/**
 * Returns the string corresponding to the left execution time of the test (in a friendly way hoiur, min, ...);
 * @return 
 */
QString NMGMGenWidget::getLeftTimeTestString()
{
	if(leftTime<=0) leftTime = 1;
	if(leftTime<1000) //less than a second --> ms
		return QString("%1").arg(leftTime)+" ms";
	else if (leftTime<60*1000) //less than a minute --> sec
		return QString("%1").arg(leftTime/1000)+" sec";
	else if(leftTime<60*60*1000) //less than an hour --> minutes 
		return QString("%1").arg(leftTime/(60*1000))+" min " + QString("%1").arg(leftTime%(60*1000)/1000)+" sec";
	else return QString("%1").arg(leftTime/(60*60*1000))+" h " + QString("%1").arg(leftTime%(60*60*1000)/(60*1000))+" min";;
	
}


/**
 * Returns the maximum end time of all the flows of the test
 * @return 
 */
int NMGMGenWidget::getMaximumFlowEndTime()
{
	int max = 0;
	for(QListViewItem * itemChild = flowList->firstChild(); itemChild; itemChild = itemChild->nextSibling())
	{
		NMGMGenFlow * flow = (NMGMGenFlow *) itemChild;
		if(max<(flow->getStartTime()+flow->getDuration())) 
			max = flow->getStartTime()+flow->getDuration();
        }	
	return max;
}
	
/**
 * Returns true if the test is running
 * @return 
 */
bool NMGMGenWidget::isTestRunning()
{
	return buttonStop->isEnabled();
}

/**
 * Checks if QLineEdit->text() is valid
 * @return true if OK, false otherwise
 */
// bool NMGMGenWidget::checkValid()
// {
//     QRegExp rx( "[1-9]\\d{0,3}" );
//     // the validator treats the regexp as "^[1-9]\\d{0,3}$"
//     QRegExpValidator v( rx, 0 );
//     QString s;
//     
//    
// }

#include "nmgmgenwidget.moc"
