/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qdatetimeedit.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qcheckbox.h>

#include "nmgmgenxmlinterpret.h"
#include "nmgmgenwidget.h"
#include "nmgmgenxmltags.h"
#include "nmgpreferencesmanager.h"

#include <iostream>
using namespace std;

#include <nmghostinformationcombowidget.h>

/**
 * Constructor of the class
 * @param w 
 * @return 
 */
NMGMGenXMLInterpret::NMGMGenXMLInterpret(NMGMGenWidget * w)
{
	widget = w;
	lastFlowId = 0;
}

/**
 * Destroyer of the class
 * @return 
 */
NMGMGenXMLInterpret::~NMGMGenXMLInterpret()
{
}

/**
 * Parses an xml reply form a start
 * @param xmlMessage XML message with tags data as root
 * @return Path of the decode file, or "" if there have been an error while parsing
 */
QString NMGMGenXMLInterpret::getResultFilePathFromResult(const QString & xmlMessage)
{
	QDomDocument doc;
	if(!doc.setContent(xmlMessage))
	{
		cerr << "Error: the document is not well formed." << endl;
		return "";
	}
	QDomElement root = doc.documentElement();
	QString resultPath = "";
	
	if(root.tagName()!=TAG_DATA) cerr << "Error root element at NMGMGenXMLInterpret different from data ("<<root.tagName()<<")."<< endl;
	else
	{
		for(QDomNode node = root.firstChild(); !node.isNull(); node = node.nextSibling())
		{
			if(node.isElement())
			{
				QDomElement elem = node.toElement();
				if(elem.tagName()==TAG_DECODE_FILE)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					//if(!id.isNull()) resultPath = id.nodeValue();
					//else cerr << "The decode result path node has no value!" << endl;
				}
				else if(elem.tagName()==TAG_RESULT_FILE)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					
				}
				else if(elem.tagName()==TAG_TEST_FILE)
				{
					//QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) resultPath = id.nodeValue();
					else cerr << "The decode result path node has no value!" << endl;
				}
				else if(elem.tagName()==TAG_RESULT_GRAPH_OWDD)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					
				}
				else if(elem.tagName()==TAG_RESULT_GRAPH_IPDV)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					
				}
				else if(elem.tagName()==TAG_RESULT_GRAPH_IPDVD)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					
				}
				else cerr << "Tag not correct in Mgen XML reply (" << elem.tagName()<<")." << endl;
			}
		}
	}
	return resultPath;
}

/**
 * Parses the status message and returns the integer value of the status
 * @param xmlMessage 
 * @return Status value or -1 if error
 */
int NMGMGenXMLInterpret::parseStatusMessage(const QString & xmlMessage)
{
	QDomDocument doc;
	if(!doc.setContent(xmlMessage))
	{
		cerr << "Error: the document is not well formed." << endl;
		return -1;
	}
	QDomElement root = doc.documentElement();
	
	if(root.tagName()!=TAG_DATA) cerr << "Error root element at NMGMGenXMLInterpret different from data ("<<root.tagName()<<")."<< endl;
	else
	{
		for(QDomNode node = root.firstChild(); !node.isNull(); node = node.nextSibling())
		{
			if(node.isElement())
			{
				QDomElement elem = node.toElement();
				if(elem.tagName()==TAG_STATUS)
				{
					QDomText id = elem.firstChild().toText();
					if(!id.isNull())
					{
						bool ok;
						int num = id.nodeValue().toInt(&ok);
						if(ok) return num;
					}
					else cerr << "The status node has no value!" << endl;
				}
//				else cerr << "Tag not correct in Mgen Status XML (" << elem.tagName()<<")." << endl;
			}
		}
	}
	return -1;
}

/**
 * Generates the xml document of the test with the information of the widget
 * @param putUserAndPass Indicates if the user and password of the management hosts must be in the XML
 * @return Returns the xml document
 */
QString NMGMGenXMLInterpret::generateXMLTestDocument(bool putUserAndPass)
{
	NMGStation * sourceStation = widget->hostSourceCombo->getStationFromCombo();
	NMGStation * destinationStation = widget->hostDestinationCombo->getStationFromCombo();
	int mgenVersion;

	QDomDocument doc(TAG_TEST);
	QDomElement root = doc.createElement(TAG_TEST);
	doc.appendChild(root);

	QDomElement testMgen = doc.createElement(TAG_METADATA);
	root.appendChild(testMgen);
	QDomElement resultMgen = doc.createElement(TAG_RESULT);
	root.appendChild(resultMgen);
	
	QDomElement testName = doc.createElement(TAG_NAME);
	testMgen.appendChild(testName);
	QDomText testNameText = doc.createTextNode(widget->txtTestName->text().utf8());
	testName.appendChild(testNameText);
	
	QDomElement testId = doc.createElement(TAG_IDENTIFIER);
	testMgen.appendChild(testId);
	QDomText testIdText = doc.createTextNode(widget->txtTestIdentifier->text().utf8());
	testId.appendChild(testIdText);

	QDomElement testMgenVersion = doc.createElement(TAG_MGEN_VERSION);
	testMgen.appendChild(testMgenVersion);
	QDomText testMgenVersionText = doc.createTextNode(widget->comboMgenVersion->currentText());
	testMgenVersion.appendChild(testMgenVersionText);
	mgenVersion = (widget->comboMgenVersion->currentText()).toInt();

	/* Result Directory Path */
	QDomElement testResultPath = doc.createElement(TAG_RESULT_PATH);
	testMgen.appendChild(testResultPath);
	QDomText testResultPathText = doc.createTextNode(NMGPreferencesManager::self()->getResultDirectory());
	testResultPath.appendChild(testResultPathText);
	
	/* Source station parameters */
	QDomElement sourceNode = doc.createElement(TAG_SOURCE);
	testMgen.appendChild(sourceNode);
	QDomElement sourceAddress = doc.createElement(TAG_ADDRESS);
	sourceNode.appendChild(sourceAddress);
	QString sourceAddresStr;
	if (sourceStation)
		sourceAddresStr = sourceStation->getTestHost().resolveAddress();
	if(sourceAddresStr=="") sourceAddresStr = sourceStation->getTestHost().getAddress();
	QDomText sourceAddressText = doc.createTextNode(sourceAddresStr);
		
	sourceAddress.appendChild(sourceAddressText);
	QDomElement sourceMgenPath = doc.createElement(TAG_MGEN_PATH);
	sourceNode.appendChild(sourceMgenPath);
	QDomText sourceMgenPathText;
	if (mgenVersion == 4)
		sourceMgenPathText = doc.createTextNode(sourceStation->getStationFieldsManager().		
			getField(widget->getStationManager()->getModuleStationKey(),ID_MGEN4_PATH));
	else 
		sourceMgenPathText = doc.createTextNode(sourceStation->getStationFieldsManager().		
			getField(widget->getStationManager()->getModuleStationKey(),ID_MGEN_PATH));
	sourceMgenPath.appendChild(sourceMgenPathText);
	
	
	QDomElement sourceManagement = doc.createElement(TAG_MANAGEMENT);
	sourceNode.appendChild(sourceManagement);
	QDomElement sourceManagementAddress = doc.createElement(TAG_ADDRESS);
	sourceManagement.appendChild(sourceManagementAddress);
	QDomText sourceManagementAddressText =  doc.createTextNode(sourceStation->getManagementHost().getAddress());
	
	sourceManagementAddress.appendChild(sourceManagementAddressText);
	QDomElement sourceManagementAddressPort = doc.createElement(TAG_PORT);
	sourceManagement.appendChild(sourceManagementAddressPort);
	QDomText sourceManagementAddressPortText =  doc.createTextNode(QString("%1").arg(sourceStation->getManagementAddressPort()));
	sourceManagementAddressPort.appendChild(sourceManagementAddressPortText);
	
	if(putUserAndPass)
	{
		QDomElement sourceManagementUsername = doc.createElement(TAG_USERNAME);
		sourceManagement.appendChild(sourceManagementUsername);
		QDomText sourceManagementUsernameText = doc.createTextNode(sourceStation->getManagementUsername());
		sourceManagementUsername.appendChild(sourceManagementUsernameText);
		QDomElement sourceManagementPassword = doc.createElement(TAG_PASSWORD);
		sourceManagement.appendChild(sourceManagementPassword);
		QDomText sourceManagementPasswordText = doc.createTextNode(sourceStation->getManagementPassword());
		sourceManagementPassword.appendChild(sourceManagementPasswordText);
	}
	
	
	/* Destination station parameters */
	
	QDomElement destinationNode = doc.createElement(TAG_DESTINATION);
	testMgen.appendChild(destinationNode);
	
	QDomElement destAddress = doc.createElement(TAG_ADDRESS);
	destinationNode.appendChild(destAddress);
	QString destAddressStr = destinationStation->getTestHost().resolveAddress();
	if(destAddressStr=="") destAddressStr = destinationStation->getTestHost().getAddress();
	QDomText destAddressText = doc.createTextNode(destAddressStr);
	destAddress.appendChild(destAddressText);
	QDomElement sourceDrecPath = doc.createElement(TAG_DREC_PATH);
	destinationNode.appendChild(sourceDrecPath);
	QDomText sourceDrecPathText;
	if (mgenVersion == 4)
		sourceDrecPathText = doc.createTextNode(destinationStation->getStationFieldsManager().
			getField(widget->getStationManager()->getModuleStationKey(),ID_MGEN4_PATH));
	else
		sourceDrecPathText = doc.createTextNode(destinationStation->getStationFieldsManager().
			getField(widget->getStationManager()->getModuleStationKey(),ID_DREC_PATH));
	sourceDrecPath.appendChild(sourceDrecPathText);
	QDomElement sourceDecodePath = doc.createElement(TAG_DECODE_PATH);
	destinationNode.appendChild(sourceDecodePath);
	QDomText sourceDecodePathText;
	if (mgenVersion == 4)
		sourceDecodePathText = doc.createTextNode(destinationStation->getStationFieldsManager().
			getField(widget->getStationManager()->getModuleStationKey(),ID_MGEN4_PATH));
	else
		sourceDecodePathText = doc.createTextNode(destinationStation->getStationFieldsManager().
			getField(widget->getStationManager()->getModuleStationKey(),ID_DECODE_PATH));
	sourceDecodePath.appendChild(sourceDecodePathText);
	
	QDomElement destManagement = doc.createElement(TAG_MANAGEMENT);
	destinationNode.appendChild(destManagement);
	QDomElement destManagementAddress = doc.createElement(TAG_ADDRESS);
	destManagement.appendChild(destManagementAddress);
	QDomText destManagementAddressText =  doc.createTextNode(destinationStation->getManagementHost().getAddress());
	destManagementAddress.appendChild(destManagementAddressText);
	QDomElement destManagementAddressPort = doc.createElement(TAG_PORT);
	destManagement.appendChild(destManagementAddressPort);
	QDomText destManagementAddressPortText =  doc.createTextNode(QString("%1").arg(destinationStation->getManagementAddressPort()));
	destManagementAddressPort.appendChild(destManagementAddressPortText);
	if(putUserAndPass)
	{
		QDomElement destManagementUsername = doc.createElement(TAG_USERNAME);
		destManagement.appendChild(destManagementUsername);
		QDomText destManagementUsernameText = doc.createTextNode(destinationStation->getManagementUsername());
		destManagementUsername.appendChild(destManagementUsernameText);
		QDomElement destManagementPassword = doc.createElement(TAG_PASSWORD);
		destManagement.appendChild(destManagementPassword);
		QDomText destManagementPasswordText = doc.createTextNode(destinationStation->getManagementPassword());
		destManagementPassword.appendChild(destManagementPasswordText);
	}
	
	if(putUserAndPass and widget->txtDistrLocalPath->text()!="")
	{
		QDomElement localNode = doc.createElement(TAG_LOCAL);
		testMgen.appendChild(localNode);

	
		QDomElement distrPath = doc.createElement(TAG_DISTR_PATH);
		localNode.appendChild(distrPath);
		QDomText distrPathText = doc.createTextNode(widget->txtDistrLocalPath->text());
		distrPath.appendChild(distrPathText);
	}
	
	createXMLConfMGen(doc,testMgen);
	createXMLConfDrec(doc,testMgen);
	
	return doc.toString();
}

/**
 * Creates the conf mgen part of the xml
 * @param doc 
 * @param root 
 */
void NMGMGenXMLInterpret::createXMLConfMGen(QDomDocument & doc, QDomElement & root)
{
	QDomElement confMgen = doc.createElement(TAG_CONF_MGEN);
	root.appendChild(confMgen);

	if(widget->checkStartTime->isChecked())
	{
	
		QString startTime = widget->timeStartTime->time().toString("hh:mm:ss");
		
		QString gmt = "";
		if(widget->comboGMTOrLocalTime->currentText()=="GMT") gmt = " GMT";
		
		QDomElement start = doc.createElement(TAG_START);
		confMgen.appendChild(start);
		QDomText startText = doc.createTextNode(startTime+gmt);
		start.appendChild(startText);
	}
	
	QDomElement port = doc.createElement(TAG_PORT);
	confMgen.appendChild(port);
	QDomText portText = doc.createTextNode(QString("%1").arg(widget->txtSourcePort->value()));
	port.appendChild(portText);
	
	
	NMGXMLFlowSortedList sortedList;
	QString testAddress = widget->hostDestinationCombo->getStationFromCombo()->getTestHost().resolveAddress();
	if(testAddress=="") testAddress = widget->hostDestinationCombo->getStationFromCombo()->getTestHost().getAddress();;
	for(QListViewItem * itemChild = widget->flowList->firstChild(); itemChild; itemChild = itemChild->nextSibling())
	{
		NMGMGenFlow * flow = (NMGMGenFlow *) itemChild;
		if (widget->comboMgenVersion->currentText() != "4") {
			flow->setBurstInterval(DEFAULT_BURST_INTERVAL,0);
			flow->setBurstDuration(DEFAULT_BURST_DURATION,0);
			flow->setBurst(false);
		}
		flow->addFlowToXMLNode(sortedList,doc,confMgen,testAddress,flow->getId());
        }

	sortedList.sort();
	for(QDomElement * elem = sortedList.first(); elem; elem = sortedList.next())
		confMgen.appendChild(*elem);
}

/**
 * Creates the conf drec part of the xml
 * @param doc 
 * @param root 
 */
void NMGMGenXMLInterpret::createXMLConfDrec(QDomDocument & doc, QDomElement & root)
{
	QDomElement confDrec = doc.createElement(TAG_CONF_DREC);
	root.appendChild(confDrec);
	
	QDomElement drecPorts = doc.createElement(TAG_PORTS);
	confDrec.appendChild(drecPorts);
	QString ports = "";
	for(QListViewItem * itemChild = widget->flowList->firstChild(); itemChild; itemChild = itemChild->nextSibling())
	{
		NMGMGenFlow * flow = (NMGMGenFlow *) itemChild;
		ports += QString("%1").arg(flow->getPort());
		if(itemChild->nextSibling()) ports += ",";
        }
	QDomText drecPortsText = doc.createTextNode(ports);
	drecPorts.appendChild(drecPortsText);
	
	NMGXMLFlowSortedList sortedList;
	QString testAddress = widget->hostDestinationCombo->getStationFromCombo()->getTestHost().resolveAddress();
	if(testAddress=="") testAddress = widget->hostDestinationCombo->getStationFromCombo()->getTestHost().getAddress();
	for(QListViewItem * itemChild = widget->flowList->firstChild(); itemChild; itemChild = itemChild->nextSibling())
	{
		NMGMGenFlow * flow = (NMGMGenFlow *) itemChild;
		flow->addFlowRSVPDestinationToXMLNode(sortedList,doc,confDrec,testAddress);
        }
	
	sortedList.sort();
	for(QDomElement * elem = sortedList.first(); elem; elem = sortedList.next())
		confDrec.appendChild(*elem);
}


/**
 * Updates the widget from a xml document (of a test)
 * @param root 
 */
void NMGMGenXMLInterpret::updateWidgetFromXMLDocument(QDomElement & root)
{
	widget->txtTestName->setText("");
	widget->txtTestIdentifier->setText("");
	
	for(QDomNode node = root.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement elem = node.toElement();
			if(elem.tagName()==TAG_METADATA)
			{
				updateWidgetFromXMLDocument(elem);
			}
			else if(elem.tagName()==TAG_NAME) 
				widget->txtTestName->setText(elem.text());
			else if(elem.tagName()==TAG_IDENTIFIER)
				widget->txtTestIdentifier->setText(elem.text());
			else if(elem.tagName()==TAG_MGEN_VERSION)
				widget->comboMgenVersion->setCurrentText(elem.text());
			else if(elem.tagName()==TAG_SOURCE)
				updateWidgetFromXMLNodeSource(elem);
			else if(elem.tagName()==TAG_DESTINATION)
				updateWidgetFromXMLNodeDestination(elem);
			else if(elem.tagName()==TAG_CONF_MGEN)
				updateWidgetFromXMLNodeConfMgen(elem);
			else if(elem.tagName()==TAG_CONF_DREC)
				updateWidgetFromXMLNodeDrec(elem);
			else if(elem.tagName()==TAG_RESULT);
			else if(elem.tagName()==TAG_LOCAL);
			else if(elem.tagName()==TAG_RESULT_PATH);
			else cerr << "Unknown tag at mgen test: " << elem.tagName() << endl;
		}
	}
}

/**
 * Updates the widget from the source station node
 * @param elem 
 */
void NMGMGenXMLInterpret::updateWidgetFromXMLNodeSource(QDomElement & elem)
{
	for(QDomNode node = elem.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement child = node.toElement();
			if(child.tagName()==TAG_ADDRESS)
			{
				NMGHostInformationComboWidget *combo = widget->hostSourceCombo;
				combo->setAddressOfStation(child.text());
			}
		}
	}
}

/**
 * Updates the widget from the destination station node
 * @param elem 
 */
void NMGMGenXMLInterpret::updateWidgetFromXMLNodeDestination(QDomElement & elem)
{
	for(QDomNode node = elem.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement child = node.toElement();
			if(child.tagName()==TAG_ADDRESS)
			{
				NMGHostInformationComboWidget *combo = widget->hostDestinationCombo;
				combo->setAddressOfStation(child.text());
			}
		}
	}
}

/**
 * Updates the widget from the conf mgen node
 * @param elem 
 */
void NMGMGenXMLInterpret::updateWidgetFromXMLNodeConfMgen(QDomElement & elem)
{
	widget->checkStartTime->setChecked(false);
	widget->txtSourcePort->setValue(5000);
	
	for(QDomNode node = elem.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement child = node.toElement();
			if(child.tagName()==TAG_PORT)
			{
				QString port = child.text();
				widget->txtSourcePort->setValue(port.toInt());
			}
			else if(child.tagName()==TAG_START)
			{
				widget->checkStartTime->setChecked(true);
				QString startTime = child.text();
				if(startTime.right(3)=="GMT") 
					widget->comboGMTOrLocalTime->setCurrentText("GMT");
				else widget->comboGMTOrLocalTime->setCurrentText("Local");
				QString hour = startTime.left(2);
				QString minutes = startTime.mid(3,2);
				QString seconds = startTime.mid(6,2);
				widget->timeStartTime->setTime(QTime(hour.toInt(),minutes.toInt(),seconds.toInt()));
			}
			else if(child.tagName()==TAG_ON)
			{
				NMGMGenFlow * flow = new NMGMGenFlow(widget->getFlowManager(),
					widget->flowList, widget->flowList->lastItem());
				flow->setParametersFromOnNode(child);
			}
			else if(child.tagName()==TAG_MOD)
			{
				QString dest = child.attribute(ATTR_DEST);
				int posPort=-1, port=-1;
				if(dest) posPort = dest.find("/");
				if(dest) port = dest.mid(posPort+1).toInt();
				NMGMGenFlow * flow = widget->getFlowManager()->getFlowByPort(port);
				if(flow)
				{
					NMGMGenFlowModification * mod = 
						new NMGMGenFlowModification(flow,flow->firstChild());
					flow->getListViewItem()->setOpen(true);
					mod->setParametersFromModNode(child,flow);
				}
				else cerr << "panic: no flow for mod with port "<<port << endl;
			}
			else if(child.tagName()==TAG_OFF)
			{
				QString flowId = child.attribute(ATTR_FLOW_ID);
				NMGMGenFlow * flow = widget->getFlowManager()->getFlowById(flowId.toInt());
				if(flow)
				{
					flow->setDuration(child.attribute(ATTR_TIME).toInt()
						-flow->getStartTime());
				}
				else cerr << "panic: no flow for id  "<<flowId<< endl;
			}
			else cerr << "unknow tag at conf_mgen document " << child.tagName() << endl;
		}
	}

}

/**
 * Updates the widget from the conf drec node
 * @param elem 
 */
void NMGMGenXMLInterpret::updateWidgetFromXMLNodeDrec(QDomElement & elem)
{
	for(QDomNode node = elem.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement child = node.toElement();
			if(child.tagName()==TAG_RESV)
			{
				QString dest = child.attribute(ATTR_DEST);
				int posPort = dest.find(":");
				int port = dest.mid(posPort+1).toInt();
				NMGMGenFlow * flow = widget->getFlowManager()->getFlowByPort(port);
				if(flow)
				{
					QString style = child.attribute(ATTR_STYLE);
					QString spec = child.attribute(ATTR_SPEC);
					int time = child.attribute(ATTR_TIME).toInt();
					if(flow->getRSVP().isActive() and flow->getStartTime()==time and
						flow->getRSVP().getDestParam()=="")
					{
						flow->getRSVP().setDestParam(spec);
						flow->getRSVP().setResvStyleAsShortString(style);
					}
					else
					{// RESV d'alguna modificació del flow
						for(QListViewItem * itemChild = flow->firstChild();
							itemChild; itemChild = itemChild->nextSibling())
						{

							NMGMGenFlowListItem *item = (NMGMGenFlowListItem *) itemChild;
							if(item->getFlowItemType()==FLOW_LIST_ITEM_MODIFICATION)
							{
								NMGMGenFlowModification * mod = 
									(NMGMGenFlowModification *) item;
								if(mod->getRSVP().isActive() and
									mod->getRSVP().getDestParam()=="" and
									time==(flow->getStartTime()+
										mod->getStartTime()))
								{
									mod->getRSVP().setDestParam(spec);
									mod->getRSVP().setResvStyleAsShortString(style);
								}
							}
						}
					}
				}
				else cerr << "panic: no flow for resv with port "<<port << endl;
			}
			else if(child.tagName()==TAG_UNRESV or child.tagName()==TAG_PORTS)
			{
			
			}
			else cerr << "unknow tag at conf_drec document " << child.tagName() << endl;
		}
	}
}
