/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "nmgmoswidget.h"
#include "mos.h"
#include "../nmgmodulemanager.h"

#include <qpushbutton.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

#include <nmgpreferencesmanager.h>
#include <nmgmoduleutils.h>
#include <messagebar.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <strings.h>
#include <math.h>

#define CHUNK 5000

#include <iostream>
using namespace std;

#define ID_PREFERENCE_LAST_TEST_PATH	"MGenLastTestPath"

/**
 * Contructor of the class
 * @param parent
 * @param name
 * @return
 */
NMGMosWidget::NMGMosWidget ( NMGModulePreferencesManager * prefMan, QWidget *parent, const char *name )
		: NMGMosWidgetBase ( parent, name )
{
	progressItem = 0;
	preferencesManager = prefMan;
	interpretThread = 0;
	sourceChanged = FALSE;
	for ( uint i = 0; i < NUM_CODECS - 1; i++ )
		cmbCodec->insertItem ( codecs[i].reference + QString ( " -- Rate %1 kbit/s" ).arg ( codecs[i].rate ) );

	connect ( btnStart,SIGNAL ( clicked() ),this,SLOT ( buttonStartClicked() ) );
	connect ( btnStop,SIGNAL ( clicked() ),this,SLOT ( buttonStopClicked() ) );
	connect ( btnSourceFile,SIGNAL ( clicked() ),this,SLOT ( buttonSourceFileClicked() ) );

	progressTimer = new QTimer();
	connect ( progressTimer,SIGNAL ( timeout() ),this,SLOT ( updateProgress() ) );
	connect ( txtSourceFile,SIGNAL ( textChanged ( const QString & ) ), this, SLOT ( setSourceChanged() ) );
}


/**
 * Destructor of the class
 * @return
 */
NMGMosWidget::~NMGMosWidget()
{
	if ( progressItem ) progressItem->setComplete();
	progressItem = 0;
}


void NMGMosWidget::buttonStartClicked()
{
	if ( txtSourceFile->text() != "" )
	{
		cout << "NMGMosWidget::buttonStartClicked" << endl;

		QString fileName = txtSourceFile->text();

		progressItem = ProgressManager::createProgressItem ( "Loading Test" );

		/* //TIP: This section would be used if we wanted to store the results in a file
		int pathPos = fileName.findRev("/") + 1;

		QString fileNameResult = fileName.left(pathPos) + MOSSTATISTICS_FILE_PREFFIX 
			+ fileName.right( fileName.length() - pathPos );
				
		if( QFile::exists( fileNameResult ) )
		{
			int reply = QMessageBox::question(0,"Overwirte the file?","The file "+fileNameResult+" exists.\n"+
					"Do you want to overwrite it?",
					QMessageBox::Yes|QMessageBox::Default,QMessageBox::No|QMessageBox::Escape);
			if(reply==QMessageBox::No) {
				loadCancelled(progressItem);	
				return;
			}
		}*/

		if ( sourceChanged )
		{
			sourceChanged = FALSE;
			interpretThread = new NMGStatisticThread();
			interpretThread->loadFileName ( fileName );
			interpretThread->activateResultFile ( FALSE );
			interpretThread->start();
			progressItem->setTotalItems ( 100 );
			progressItem->setStatus ( "Loading data..." );
		}

		connect ( progressItem, SIGNAL ( progressItemCanceled ( ProgressItem* ) ), this, SLOT ( loadCancelled ( ProgressItem * ) ) );
		connect ( progressItem, SIGNAL ( progressItemCompleted ( ProgressItem* ) ), this, SLOT ( loadFinished ( ProgressItem * ) ) );
		progressTimer->start ( 1000 );
	}
	else {
		QMessageBox::information(0,"No Source File","Test Source File must be specified", QMessageBox::Ok|QMessageBox::Default|QMessageBox::Escape);
		cout << "No Source File" << endl;
	}
}

void NMGMosWidget::buttonStopClicked()
{
	cout << "Stop" << endl;
	if ( interpretThread )
		interpretThread->kill();
	progressTimer->stop();
}

void NMGMosWidget::buttonSourceFileClicked()
{
	QString fileName = QFileDialog::getOpenFileName ( ".","XML Flow Format (*.xff);;XML (*.xml);;All files (*.*)",0,0,"Select test file" );

	if ( fileName and fileName!="" )
	{
		if ( fileName != txtSourceFile->text() )
		{
			saveLastTestPathFromFilePath ( fileName );
			txtSourceFile->setText ( fileName );
		}
	}
}

/**
 * Test cancelled using the progress bar widget
 * @param item
 */
void NMGMosWidget::loadCancelled ( ProgressItem * )
{
	if ( interpretThread )
		interpretThread->kill();
	progressTimer->stop();
	progressItem->setComplete();
	progressItem = 0;
	MessageBar::longMessage ( "Load cancelled" );
}

/**
 * Slot for reporting the finishing of the data load
 */
void NMGMosWidget::loadFinished ( ProgressItem * )
{
	progressItem = 0;
	if ( interpretThread->isFinished() )
	{
		MessageBar::longMessage ( "Load finished successfully" );
		updateWidgetBecauseOfTestEnd();
	}
}

/**
 * Updates the widgets becuse the end has ended
 */
void NMGMosWidget::updateWidgetBecauseOfTestEnd()
{
	progressItem = 0;
	MessageBar::longMessage ( "Test Finished" );
	/** main **/

	int sel_opt, n_elements, ok;

	double my_rate = codecs[cmbCodec->currentItem() ].rate;
	int my_Ie;
	double my_Bpl;
	char my_codec[CODEC_LEN] = "\0";
	for ( int t = 0; t < CODEC_LEN; t++ )
	{
		my_codec[t] = codecs[cmbCodec->currentItem() ].reference[t];
	}

	int i, tmp_seq, n_seq;
	uint n_flows;
	long long int current;
	struct parameters mos;
	double plp, test_packets;

	i = 0;

	my_Ie = searchIe ( my_rate, my_codec );
	my_Bpl = searchBpl ( my_rate, my_codec );

	n_flows = interpretThread->getStatisticsData()->getFlowCount();

	txtResult->setText ( "" );

	for ( i = 0; i < n_flows; i++ )
	{
		NMGStatisticFlowData * nmgstfldt = interpretThread->getStatisticsData()->getFlowDataAt ( i );

// 		nmgstfldt->checkFlowDataValues();

		plp = ( double ) nmgstfldt->lostPackets();
		test_packets = ( double ) nmgstfldt->sentPackets() + nmgstfldt->duplicatePackets() + plp;
		plp = plp / test_packets;

		QString fid = nmgstfldt->getId();

		// Vigila, hack fet per evitar que si un test comen�a per seq_num != 0 hi hagi problemes
		/*p_flow primer = go_first(hList);
		plp = plp - primer->seq_num;
		plp = plp / (hList->test_packets-primer->seq_num);
		// Fi Hack original a continuacio
		//plp = plp / hList->test_packets;
		fprintf(stdout, "Losses: %d, Packets: %d, PLP: %f, OOO Packets: %d, Mean Delay: %s, ",
			hList->packet_losses, hList->test_packets, plp * 100, hList->outoforder_packets, ts_print);
		ts2char(&hList->max_delay, ts_print);
		fprintf(stdout, "Max Delay: %s, ", ts_print);
		ts2char(&hList->min_delay, ts_print);
		fprintf(stdout, "Min Delay: %s\n", ts_print);*/

		mos_init ( &mos );
		mos.Ppl = plp * 100.0;
		mos.Ie = my_Ie;
		mos.Bpl = my_Bpl;
		mos.T = nmgstfldt->averageDelay();

		txtResult->setText ( txtResult->text()
		                     + QString( "* MOS value for test (<b>flowId " )
		                     + QString ( fid+"</b>)" )
		                     + QString ( " is R_MOS = %1" ).arg ( R ( &mos ) )
		                     + QString ( " and MOS = %1<br />" ).arg ( MOS ( &mos ) ) );
		fprintf ( stdout, "R_MOS = %.3f, MOS=%.3f\n", R ( &mos ), MOS ( &mos ) );

		// For debugging purposes
		/*for (i = 0; i <= 400; i += 25) {
			mos.T = i;
			printf("%d %.3f %.2f\n", i, R(&mos), MOS(&mos));
		}
		for (i = 0; i <= 30; i ++) {
			mos.Ppl = i/100.0;
			printf("%d %.3f %.2f\n", i, Ieef(&mos), MOS(&mos));
		}*/
	}
}

void NMGMosWidget::updateProgress()
{
	int percent = interpretThread->getPercentCompleted();
	QString mess = interpretThread->getErrorString();
	bool ended = interpretThread->isFinished();
	if ( ended && progressItem )
	{
		progressTimer->stop();
		progressItem->setComplete();
		progressItem = 0;
		return;
	}
	if ( progressItem )
	{
		progressItem->setStatus ( mess );
		progressItem->setCompletedItems ( percent );
		progressItem->updateProgress();
	}

	/*if (interpretThread->getError()!=0)
	{
		progressTimer->stop();
		m_gdl_dataLoader->processError(m_gdl_dataLoader->getError(), m_gdl_dataLoader->getErrorString());
		if (m_gdl_dataLoader->getError()==0) // loader relaunched -> start the timer again
			timer->start(100);
		else // loader aborted
		{
			progressItem->setComplete();
			progressItem = 0;
			MessageBar::longMessage("Load aborted");
			m_gdl_dataLoader->resetError();
			return;
		}
	}*/

	if ( interpretThread->isFinished() )
		updateWidgetBecauseOfTestEnd();
}

/**
 * Returns the last test path (or "" if undefined)
 * @return
 */
QString NMGMosWidget::getLastTestPath()
{
	QString lastTestPath = preferencesManager->getValue ( ID_PREFERENCE_LAST_TEST_PATH );
	if ( !lastTestPath or lastTestPath=="" ) lastTestPath = "";
	return lastTestPath;
}

/**
 * Saves the last test path from the filename path of the test
 * @param filePath
 */
void NMGMosWidget::saveLastTestPathFromFilePath ( const QString & filePath )
{
	if ( filePath and filePath!="" )
	{
		int lastPos = filePath.findRev ( "/" );
		preferencesManager->addValue ( ID_PREFERENCE_LAST_TEST_PATH,filePath.left ( lastPos ) );
	}
}

void NMGMosWidget::setSourceChanged ()
{
	sourceChanged = TRUE;
}


int NMGMosWidget::searchIe ( double rate, char reference[CODEC_LEN] )
{
	int i=0;
	while ( codecs[i].rate != rate || strcmp ( codecs[i].reference, reference ) )
	{
		if ( !strcmp ( codecs[i].reference, "" ) && codecs[i].rate == 0 &&
		        codecs[i].Ie == 0 )
		{
			return -EINVAL;
		}
		i++;
	}
	return codecs[i].Ie;
}

double NMGMosWidget::searchBpl ( double rate, char reference[CODEC_LEN] )
{
	int i=0;
	while ( codecs[i].rate != rate || strcmp ( codecs[i].reference, reference ) )
	{
		if ( !strcmp ( codecs[i].reference, "" ) && codecs[i].rate == 0 &&
		        codecs[i].Bpl == 0 )
		{
			return -EINVAL;
		}
		i++;
	}
	return codecs[i].Bpl;
}

/* Temporary data */
double NMGMosWidget::No ( struct parameters *params )
{
	double Nr1, Nr2, Pro, Pr1, Nfo;
	if ( !params )
		return 0;
	Nr1 = params->Ps - params->SLR - params->Ds - 100.0 + 0.004 *
	      pow ( params->Ps - ( params->SLR + params->RLR ) - params->Ds - 14.0, 2.0 );
// rserral: hack FIXME last log(10) is from Basic code
	Pro = params->Pr + 10.0 * log ( 1.0 + pow ( 10.0, ( 10.0 - params->LSTRr ) / 10.0 ) ) / log ( 10.0 );
	Pr1 = Pro + 0.008 * pow ( Pro - 35.0, 2.0 );
	Nr2 = Pr1 - 121.0 + params->RLR;
	Nfo = params->Nfor + params->RLR;
// rserral: hack FIXME last log(10) is from Basic code
	return 10.0 * log ( pow ( 10.0, Nr1 / 10.0 ) + pow ( 10.0, Nr2 / 10.0 ) +
	                    pow ( 10.0, params->Nc / 10.0 ) + pow ( 10.0, Nfo / 10.0 ) ) / log ( 10.0 );
}

/* Formula 3 to 7 of the ITU-T G.107 Specs
 */
double NMGMosWidget::Nt ( struct parameters *params )
{
	return No ( params ) - params->RLR;
}

/* Formula 2 of the ITU-T G.107 Specs
 */
double NMGMosWidget::Ro ( struct parameters *params )
{
	if ( !params )
		return 0;
	return 15.0 - 1.5 * ( params->SLR + No ( params ) );
}

/* Formula 9 and 10 of the ITU-T G.107 Specs
 */
double NMGMosWidget::Iolr ( struct parameters *params )
{
	double Xolr;
	if ( !params )
		return 0;
	// SLR+RLR=OLR
	Xolr = params->SLR + params->RLR + 0.2 * ( 64.0 + Nt ( params ) );
	return 20.0 * ( pow ( 1.0 + pow ( Xolr / 8.0, 8.0 ), 1.0 / 8.0 ) - Xolr / 8.0 );
}

/* Formula 11 to 12 of the ITU-T G.107 Specs
 */
double NMGMosWidget::Ist ( struct parameters *params )
{
	double STMRo;
	double tmp;
	if ( !params )
		return 0.0;
// rserral: hack FIXME last log(10) is from Basic code
	STMRo = -10.0 * log ( pow ( 10.0, -params->STMR / 10.0 ) + pow ( 10.0, -params->TELR / 10.0 ) * exp ( -params->T / 4.0 ) ) / log ( 10.0 );
	tmp = 12.0 * ( pow ( 1.0 + pow ( ( STMRo - 13.0 ) / 6.0, 8.0 ), 1.0 / 8.0 ) );
	tmp = tmp - 28.0 * ( pow ( 1.0 + pow ( ( STMRo + 1.0 ) / 19.4, 35.0 ), 1.0 / 35.0 ) );
	return tmp - 13.0 * ( pow ( 1.0 + pow ( ( STMRo - 3.0 ) / 33.0, 13.0 ), 1.0 / 13.0 ) ) + 29.0;
}

/* Formula 13 to 17 of the ITU-T G.107 Specs
 */
double NMGMosWidget::Iq ( struct parameters *params )
{
	double Q;
	double G;
	if ( !params )
		return 0;
	if ( params->qdu < 1 )
		params->qdu = 1;
// rserral: hack FIXME last log(10) is from Basic code
	Q = 37.0 - 15.0 * log ( params->qdu ) / log ( 10.0 );
	G = 1.07 + 0.258 * Q + 0.0602 * pow ( Q, 2.0 );
// rserral: hack FIXME last log(10) is from Basic code
	return 15.0 * log ( 1.0 + pow ( 10.0, ( Ro ( params ) - 100.0 ) / 15.0 ) * pow ( 10.0, 46.0 / 8.4 - G / 9.0 ) +
	                    pow ( 10.0, 46.0 / 30.0 - G / 40.0 ) ) / log ( 10.0 );
}

/* Formula 8 of the ITU-T G.107 Specs
 */
double NMGMosWidget::Is ( struct parameters *params )
{
	if ( !params )
		return 0;
	return Iolr ( params ) + Ist ( params ) + Iq ( params );
}

/* Formula 22 and 23 of the ITU-T G.107 Specs
 */
double NMGMosWidget::TERV ( struct parameters *params )
{
	double tmp;
	if ( !params )
		return 0.0;
// rserral: hack FIXME last log(10) is from Basic code
	tmp = params->TELR + 6.0 * exp ( -0.3 * pow ( params->T, 2.0 ) ) - 40.0 *
	      log ( ( 1.0 + params->T / 10.0 ) / ( 1.0 + params->T / 150.0 ) ) / log ( 10.0 );
	if ( params->STMR < 9 )
		return tmp + 0.5 * Ist ( params );
	return tmp;
}

/* Formula 19, 21 and 24 of the ITU-T G.107 Specs
 */
double NMGMosWidget::Idte ( struct parameters *params )
{
	double Re, Roe, Xdt, tmp;
	if ( !params )
		return 0.0;
	if ( params->T < 1 )
		return 0.0;
	Re = 80.0 + 2.5 * ( TERV ( params ) - 14.0 );
	Roe = -1.5 * ( No ( params ) - params->RLR );
	Xdt = ( Roe - Re ) / 2.0;
	tmp = Xdt + sqrt ( pow ( Xdt, 2.0 ) + 100.0 );
	tmp = ( tmp - 1.0 ) * ( 1.0 - exp ( -params->T ) );
	if ( params->STMR > 20 )
		return sqrt ( pow ( tmp, 2.0 ) + pow ( Ist ( params ), 2.0 ) );
	return tmp;
}

/* Formula 25 and 26 of the ITU-T G.107 Specs
 */
double NMGMosWidget::Idle ( struct parameters *params )
{
	double Rle, Xdl;
	if ( !params )
		return 0.0;
	// For now let's hack a bit. FIXME
	params->Tr = params->T * 2.0;
	Rle = 10.5 * ( params->WEPL + 7.0 ) * pow ( params->Tr + 1.0, -1.0/4.0 );
	Xdl = ( Ro ( params ) - Rle ) / 2.0;
	return Xdl + sqrt ( pow ( Xdl, 2.0 ) + 169.0 );
}

/* Formula 27 and 28 of the ITU-T G.107 Specs
 */
double NMGMosWidget::Idd ( struct parameters *params )
{
	double X;
	if ( !params )
		return 0.0;
	// For now let's hack a bit. FIXME
	params->Ta = params->T;
	if ( params->Ta <= 100 )
		return 0.0;
	else
	{
		X = log ( params->Ta / 100.0 ) / log ( 2.0 );
		return 25.0 * ( pow ( 1.0 + pow ( X, 6.0 ), 1.0/6.0 ) - 3.0 * pow ( 1.0 + pow ( X / 3.0, 6.0 ), 1.0 / 6.0 ) + 2.0 );
	}
}

/* Formula Id ITU-T G.107 Specs
 */
double NMGMosWidget::Id ( struct parameters *params )
{
	if ( !params )
		return 0.0;
	/*	if (params->T > 177.3)
			return 0.024*params->T+0.11*(params->T-177.3);
		else
			return 0.024*params->T; */
	return Idte ( params ) + Idle ( params ) + Idd ( params );
}

/* Formula 29, packet loss inclusion of the ITU-T G.107 Specs
 */
double NMGMosWidget::Ieef ( struct parameters *params )
{
	if ( !params )
		return 0.0;
//	if (params->Ppl)
	return params->Ie + ( 95.0 - params->Ie ) * ( params->Ppl / ( params->Ppl / params->BurstR + params->Bpl ) );
//	else
//		return 0.0;
}

/* Formula 1, R computation of ITU-T G.107 Specs
 */
double NMGMosWidget::R ( struct parameters *params )
{
	if ( !params )
		return 0.0;
	params->R = Ro ( params ) - Is ( params ) - Id ( params ) - Ieef ( params ) + params->A;
	params->R_set = 1;
	return params->R;
}

/*double NMGMosWidget::GoB(struct parameters *params) {
	if (!params)
		return 0.0;
	if (!R_set)
		R(params);
	dd
	return Ro(params) - Is(params) - Id(params) - Ieef(params) + params->A;
}
*/
double NMGMosWidget::MOS ( struct parameters *params )
{
	double my_r;
	//int tmp;
	if ( !params )
		return 0.0;
	my_r = R ( params );
	if ( my_r < 0 ) return 1.0;
	if ( my_r > 100 ) return 4.5;
	return 1.0 + my_r * 0.035 + my_r * ( my_r - 60.0 ) * ( 100.0 - my_r ) * 0.000007;
	//return (tmp*100.0+0.5)/100.0;

}

void NMGMosWidget::mos_init ( struct parameters *params )
{
	if ( !params )
		return;
	params->SLR = 8.0;
	params->RLR = 2.0;
	params->STMRs = 15.0;
	params->Ds = 3.0;
	params->STMR = 15.0;
	params->Dr = 3.0;
	params->LSTRr = 18.0;
	params->TELR = 65.0;
	params->T = 0.0;
	params->WEPL = 110.0;
	params->Tr = 0.0;
	params->Ta = 0.0;
	params->Ie = 0.0;
	params->Bpl = 1.0;
	params->Ppl = 0.0;
	params->BurstR = 1.0;
	params->A = 0.0;
	params->Nc = -70.0;
	params->Ps = 35.0;
	params->Pr = 35.0;
	params->qdu = 1.0;
	params->Nfor = -64.0;
}

#include "nmgmoswidget.moc"
