/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGMOSWIDGET_H
#define NMGMOSWIDGET_H

#include "nmgmoswidgetbase.h"
#include "../nmgimodulecommunication.h"
// #include "../libnmgstatistic/nmgstatisticthread.h"
#include "nmgstatisticthread.h"
#include <progressmanager.h>
#include "../nmgmodulepreferencesmanager.h"
#include "../../gui/nmgxmlconfigurationfile.h"
#include "../../gui/nmgimagemanager.h"
// #include "../nmgmodtabwidgetmanager.h"

#include <qwidget.h>
#include <qptrlist.h>
#include <qtimer.h>
#include <qstring.h>

#define NUM_CODECS 25
#define CODEC_LEN 10

class NMGMosWidget : public NMGMosWidgetBase
{
		Q_OBJECT
	public:
		NMGMosWidget ( NMGModulePreferencesManager * prefMan, QWidget *parent = 0, const char *name = 0 );

		~NMGMosWidget();

	public slots:
		void buttonStartClicked();
		void buttonStopClicked();
		void buttonSourceFileClicked();
		void updateProgress();
		QString getLastTestPath();
		void saveLastTestPathFromFilePath ( const QString & filePath );
		void loadCancelled ( ProgressItem * );
		void loadFinished ( ProgressItem * );
		void setSourceChanged ();

	public:
		/* Default parameters for the MOS calculation*/
		void init(struct parameters *params);

		double No ( struct parameters *params );

		/* Formula 3 to 7 of the ITU-T G.107 Specs*/
		double Nt(struct parameters *params);

		/* Formula 2 of the ITU-T G.107 Specs*/
		double Ro(struct parameters *params);

		/* Formula 9 and 10 of the ITU-T G.107 Specs*/
		double Iolr(struct parameters *params);

		/* Formula 11 to 12 of the ITU-T G.107 Specs*/
		double Ist(struct parameters *params);

		/* Formula 13 to 17 of the ITU-T G.107 Specs*/
		double Iq(struct parameters *params);

		/* Formula 8 of the ITU-T G.107 Specs*/
		double Is(struct parameters *params);

		/* Formula 22 and 23 of the ITU-T G.107 Specs*/
		double TERV(struct parameters *params);

		/* Formula 19, 21 and 24 of the ITU-T G.107 Specs*/
		double Idte(struct parameters *params);

		/* Formula 25 and 26 of the ITU-T G.107 Specs*/
		double Idle(struct parameters *params);

		/* Formula 27 and 28 of the ITU-T G.107 Specs*/
		double Idd(struct parameters *params);

		/* Formula Id ITU-T G.107 Specs*/
		double Id(struct parameters *params);

		/* Formula 29, packet loss inclusion of the ITU-T G.107 Specs*/
		double Ieef(struct parameters *params);

		/* Formula 1, R computation of ITU-T G.107 Specs*/
		double R(struct parameters *params);

		/* Formula B.1, Normal Distribution F(Z) computation of ITU-T G.107 Specs*/
		//double NDistr(struct parameters *params);
		/* Formula B.2, Good or Better computation of ITU-T G.107 Specs*/
		//double GoB(struct parameters *params);
		/* Formula B.3, Poor or Worse computation of ITU-T G.107 Specs*/
		//double PoW(struct parameters *params);

		double MOS ( struct parameters *params );
		
		void mos_init ( struct parameters *params );
		
		/* Helper function for searching the Ie number from the codec*/
		int searchIe(double rate, char reference[CODEC_LEN]);

		double searchBpl(double rate, char reference[CODEC_LEN]);

	private:
		void updateWidgetBecauseOfTestEnd();

// 		QPopupMenu * listMenu;
// 		QToolBar * toolBar;
		NMGModulePreferencesManager * preferencesManager;
		ProgressItem * progressItem;
		QTimer * progressTimer;
		NMGStatisticThread * interpretThread;
		bool sourceChanged;
};

#endif
