/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "nmgstatisticthread.h"
#include "nmgstatisticxmltags.h"

#include <qfiledialog.h>
#include <qfile.h>
#include <nmxml.h>


NMGStatisticThread::NMGStatisticThread() : NMThread()
{
	interpret = 0;
	activateResFile = FALSE;
}


NMGStatisticThread::~NMGStatisticThread()
{
	if ( interpret )
		delete interpret;
	interpret = 0;
}


void NMGStatisticThread::loadFileName(QString str_fileName)
{
	fileName = str_fileName;
}


void NMGStatisticThread::activateResultFile(bool act) {
	activateResFile = act;
}


bool NMGStatisticThread::isActivateResultFile() {
	return activateResFile;
}


void NMGStatisticThread::xsltConversion( QString & fileName, QString & xsltPath, QString & fileNameResult ) {
	QFile file ( fileName );
	if ( !file.open ( IO_ReadOnly ) )
	{
		cerr<<"Error while reading the document from disk.\n"<<file.errorString() <<endl;
	}
	else
	{	
		NMXML::applyXsltToDoc( fileName.latin1(), xsltPath.latin1(), fileNameResult.latin1());
	}
}


void NMGStatisticThread::run() {
	QFile file ( fileName );
	if ( !file.open ( IO_ReadOnly ) )
	{
		cerr<<"Error while reading the document from disk.\n"<<file.errorString() <<endl;
	}
	else
	{
		QXmlInputSource source ( file );
		
		int pathPos = fileName.findRev("/") + 1;
		
		QString fileNameResult = fileName.left(pathPos) + STATISTICS_FILE_PREFFIX 
						+ fileName.right( fileName.length() - pathPos );
	
		QXmlSimpleReader reader;
		if ( activateResFile ) 
			interpret = new NMGStatisticXMLInterpret(fileNameResult);
		else 
			interpret = new NMGStatisticXMLInterpret();
		reader.setContentHandler( interpret );

		cout << "Statistic Analysis Started" << endl;

		if (reader.parse( source ) && activateResFile) {

			cout << "Statistic Analysis Finished Successfully" << endl;
			
			fileNameResultXslt = QString("/tmp/") + QString(STATISTICS_FILE_PREFFIX) + fileName.right( fileName.length() - pathPos );
			fileNameResultXslt.replace( fileNameResultXslt.length() - 4, 4, ".qml" );
			
			QString xsltPath = QString(XML_DIR) + QString(QMLXSLT);
							
			xsltConversion( fileNameResult, xsltPath, fileNameResultXslt );
			cout << "XSLT Conversion Finished Successfully" << endl;
		}
		else if (!activateResFile)
			cout << "Statistic Analysis Finished Successfully without writing result File" << endl;
		else
			cerr << "Statistic Analysis Finished Unsuccessfully" << endl;
	}
	file.close();
}


void NMGStatisticThread::kill()
{
	interpret->stop();
}


bool NMGStatisticThread::isFinished()
{
	if (interpret)
		return interpret->isFinished();
	else
		return false;
}


/*int NMGStatisticThread::getError()
{
	if (interpret)
		return interpret->getError();
	else
		return 0;
}*/


QString NMGStatisticThread::getErrorString()
{
	if (interpret)
		return interpret->getErrorString();
	else
		return "";
}


int NMGStatisticThread::getPercentCompleted()
{
	if (interpret)
		return interpret->getPercentCompleted();
	else
		return 0;
}


QString NMGStatisticThread::getStatus()
{
	if (!interpret)
		return "Initializing parser";
	else
		return interpret->getStatus();
}


QString NMGStatisticThread::getFileNameResultXslt()
{
	return fileNameResultXslt;
}

NMGStatisticData* NMGStatisticThread::getStatisticsData() {
	return interpret->getStatisticsData();
}
