/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "nmgstatisticxmlinterpret.h"
#include "nmgstatisticxmltags.h"
#include <nmstring.h>

#include <iostream>
using namespace std;

//TODO (rbarreda) Build a NMXML object that contains the metadata tree. 
//     		  Finally dump all in a result file, if necessary
//		  avoiding the use of xw everywhere

/**
 * Constructor of the class
 * @return
 */
NMGStatisticXMLInterpret::NMGStatisticXMLInterpret ( )
{
	xw = 0;
	resStatisticFile = 0;
	resFilePath = "";
	writeToFile = FALSE;
	statusMsg = "Creating parser thread";
}

/**
 * Constructor of the class
 * @return
 */
NMGStatisticXMLInterpret::NMGStatisticXMLInterpret ( QString & resFile )
{
	xw = 0;
	resStatisticFile = 0;
	resFilePath = resFile;
	writeToFile = TRUE;
	statusMsg = "Creating parser thread";
}

/**
 * Destroyer of the class
 * @return
 */
NMGStatisticXMLInterpret::~NMGStatisticXMLInterpret ()
{
	deleteObjects();
}

bool NMGStatisticXMLInterpret::startDocument()
{
	cleanValues ();
	isAlreadyFinished = FALSE;
// 	errorCode = 0;
	stopParsing = FALSE;
	metadata = 0;
	data = 0;
	currentFlowMetadata = 0;
	numPacketsAllFlows = 0;
	countAll = 0;
	inFlow = inSeq = inTXTime = inRXTime = inSize = inMetadata = FALSE;
	delay_list = QValueVector<long>();
	ipdv_list = QValueVector<long>();
	statusMsg = "Reading data from file";
	if (writeToFile) {
		resStatisticFile = new QFile ( resFilePath );
		if ( !resStatisticFile->open ( IO_WriteOnly ) )
		{
			cerr<<"Error while opening the document.\n"<<resStatisticFile->errorString() <<endl;
			return FALSE;
		}
		else
		{
			xw = new XmlWriter ( resStatisticFile );
			xw->writeOpenTag ( TAG_TEST );
		}
	} 
	return TRUE;
}

bool NMGStatisticXMLInterpret::startElement ( const QString&, const QString&,
        const QString& qName,
        const QXmlAttributes& atts )
{
	if ( stopParsing )
	{
		statusMsg = "Stopped";
		errorMsg = "Parsing cancelled by user";
		cout << errorMsg << endl;
		cleanValues();
		deleteObjects();
		return FALSE;
	}

	if ( qName == TAG_METADATA || inMetadata )
	{
		statusMsg = "Parsing file metadata";

		if ( qName == TAG_METADATA && !metadata )
		{
			metadata = new NMGStatisticMetadata();
		}
		else if ( qName == TAG_ON )
		{
			currentFlowMetadata = new NMGStatisticFlowMetadata();

			for ( int i=0; i<atts.count();i++ )
			{
				if ( atts.qName ( i ) ==ATTR_RATE )
					currentFlowMetadata->setRate ( atts.value ( i ) );
				else if ( atts.qName ( i ) ==ATTR_FLOW_ID )
					currentFlowMetadata->setId ( atts.value ( i ) );
				else if ( atts.qName ( i ) ==ATTR_NAME )
					currentFlowMetadata->setName ( atts.value ( i ) );
				else if ( atts.qName ( i ) ==ATTR_TIME )
					currentFlowMetadata->setTimeOn ( atts.value ( i ) );
			}

			metadata->addFlowMetadata ( currentFlowMetadata );
			currentFlowMetadata = 0;
		}
		else if ( qName == TAG_MOD )
		{

			QString flowIdMod = "";
			QString timeMod = "";
			QString rateMod = "";

			for ( int i=0; i<atts.count();i++ )
			{
				if ( atts.qName ( i ) ==ATTR_RATE )
					rateMod = atts.value ( i );
				else if ( atts.qName ( i ) ==ATTR_FLOW_ID )
					flowIdMod = atts.value ( i );
				else if ( atts.qName ( i ) ==ATTR_TIME )
					timeMod = atts.value ( i );
			}
			currentFlowMetadata = metadata->getFlowMetadata ( flowIdMod );
			if ( currentFlowMetadata!=0 )
				currentFlowMetadata->addModToList ( timeMod, rateMod );
			currentFlowMetadata = 0;
		}
		else if ( qName == TAG_OFF )
		{
			QString flowIdOff = "";
			QString timeOff = "";

			for ( int i=0; i<atts.count();i++ )
			{
				if ( atts.qName ( i ) ==ATTR_FLOW_ID )
					flowIdOff = atts.value ( i );
				else if ( atts.qName ( i ) ==ATTR_TIME )
					timeOff = atts.value ( i );
			}
			currentFlowMetadata = metadata->getFlowMetadata ( flowIdOff );
			if ( currentFlowMetadata!=0 )
			{
				currentFlowMetadata->setTimeOff ( timeOff );
				numPacketsAllFlows += currentFlowMetadata->getNumPackets();
			}
			else numPacketsAllFlows = SEQLISTSIZE;
			currentFlowMetadata = 0;
		}

		if (writeToFile) {
			AttrMap atribs;
			for ( int i = 0; i < atts.length (); i++ ) atribs.insert ( atts.qName ( i ), atts.value ( i ) );
			xw->writeOpenTag ( qName, atribs );
		}
		inMetadata = TRUE;
	}
	else if ( qName == TAG_RESULT && writeToFile)
	{
		xw->setAutoNewLine ( TRUE );
		xw->writeOpenTag ( TAG_STATISTICS );
	}
	else if ( qName == TAG_FLOWS )
	{
		if (writeToFile) xw->writeOpenTag ( TAG_FLOWS );
		data = new NMGStatisticData();
	}
	else if ( qName ==  TAG_FLOW )
	{
		statusMsg = "Parsing flow data";
		currentFlowStr = atts.value ( ATTR_ID );
		currentFlowMetadata = metadata->getFlowMetadata ( currentFlowStr );
		currentFlow = currentFlowStr.toInt();
		AttrMap atribs;
		atribs.insert ( ATTR_ID, currentFlowStr );
		if ( currentFlowMetadata!=0 ) atribs.insert ( ATTR_NAME, currentFlowMetadata->getName() );
		else atribs.insert ( ATTR_NAME, "Unknown Info::Unable to know Lost packets information" );
		if (writeToFile) xw->writeOpenTag ( qName, atribs );
		if ( currentFlowMetadata!=0 ) seqList = QBitArray ( currentFlowMetadata->getNumPackets() );
		else seqList = QBitArray ( SEQLISTSIZE );
		seqList.fill ( FALSE );
		inFlow = TRUE;
		currentFlowMetadata = 0;
	}
	else if ( inFlow )
	{
		if ( qName == TAG_SEQ )
		{
			txms = rxms = size = -1; //reset variables, a new sequences is started
			currentSeq = atts.value ( ATTR_ID ).toLongLong();
			if ( currentSeq >= SEQLISTSIZE )
				if ( !seqList.resize ( 2 * SEQLISTSIZE ) )
					cerr<<"Error in resize bitarray in NMGStatisticXMLInterpret.\n" <<endl;
				else
					cerr<<"Bitarray in NMGStatisticXMLInterpret resized.\n" <<endl;
			if ( seqList.testBit ( ( uint ) currentSeq ) ) dupl++;
			seqList.setBit ( ( uint ) currentSeq );
			inSeq = TRUE;
		}
		else if ( inSeq )
		{
			if ( qName == TAG_TX_TIME ) inTXTime = TRUE;
			else if ( qName == TAG_RX_TIME ) inRXTime = TRUE;
			else if ( qName == TAG_SIZE ) inSize = TRUE;
		}
	}

	return TRUE;
}


bool NMGStatisticXMLInterpret::characters ( const QString & ch )
{
	if ( stopParsing )
	{
		statusMsg = "Stopped";
		errorMsg = "Parsing cancelled by user";
		cout << errorMsg << endl;
		cleanValues();
		deleteObjects();
		return FALSE;
	}

	if ( inFlow )
	{
		if ( inSeq )
		{
			if ( inTXTime )
			{
				QTime qtimetx = QTime::fromString ( ch.left ( 8 ) );
				qtx = qtimetx.hour() * 3600 + qtimetx.minute() * 60 + qtimetx.second();
				txms =	ch.right ( 6 ).toLong ();
			}
			else if ( inRXTime )
			{
				QTime qtimerx = QTime::fromString ( ch.left ( 8 ) );
				qrx = qtimerx.hour() * 3600 + qtimerx.minute() * 60 + qtimerx.second();
				rxms =	ch.right ( 6 ).toLong ();
			}
			else if ( inSize )
			{
				size = ch.toInt ();
			}
		}
	}
	else if ( inMetadata && writeToFile )
		xw->writeString ( ch );

	return TRUE;
}

bool NMGStatisticXMLInterpret::endElement ( const QString&, const QString&, const QString& qName )
{
	if ( stopParsing )
	{
		statusMsg = "Stopped";
		errorMsg = "Parsing cancelled by user";
		cout << errorMsg << endl;
		cleanValues();
		deleteObjects();
		return FALSE;
	}

	if ( qName == TAG_METADATA || inMetadata )
	{
		if (writeToFile) xw->writeCloseTag ( qName );
		if ( qName == TAG_METADATA )
		{
			if (writeToFile) xw->newLine();
			inMetadata = FALSE;
		}
	}
	else if ( qName == TAG_RESULT && writeToFile )
		xw->writeCloseTag ( TAG_STATISTICS );
	else if ( qName == TAG_FLOWS  && writeToFile )
		xw->writeCloseTag ( TAG_FLOWS );
	else if ( qName ==  TAG_FLOW )
	{
		makeCalc();
		if (writeToFile) {
			createStatisticNode ();
			xw->writeCloseTag ( TAG_FLOW );
		}
		createStatisticFlowData();
		oldFlow = currentFlow;
		inFlow = FALSE;
		cleanValues();
	}
	else if ( inFlow )
	{
		if ( qName == TAG_SEQ )
		{
			if ( txms > -1 && rxms > -1 )
			{
				current_delay = ( ( ( qrx - qtx ) * NANOSECS ) + ( rxms - txms ) );
				if ( current_delay < 0 )
					negative_delay = TRUE;

				nr_delays++;
				delay_list.push_back ( current_delay );

				avg_delay += current_delay;

				if ( ( count > 0 ) && ( currentSeq == oldSeq + 1 ) )
				{
					ipdv_list.push_back ( fabs ( current_delay - old_delay ) );
					nr_ipdv++;
					avg_ipdv += fabs ( current_delay - old_delay );
				}
				old_delay = current_delay;
				count++;
				countAll++;
			}
			else
			{
				//XML bad format
				inTXTime = inRXTime = inSize = FALSE;
			}

			oldSeq = currentSeq;
			inSeq = FALSE;
		}
		else if ( inSeq )
		{
			if ( qName == TAG_TX_TIME ) inTXTime = FALSE;
			else if ( qName == TAG_RX_TIME ) inRXTime = FALSE;
			else if ( qName == TAG_SIZE ) inSize = FALSE;
		}
	}

	return TRUE;
}

bool NMGStatisticXMLInterpret::endDocument()
{
	if (writeToFile) {
		xw->writeCloseTag ( TAG_TEST );
		resStatisticFile->close();
	}
	cleanValues ();
	isAlreadyFinished = TRUE;
	return TRUE;
}


void NMGStatisticXMLInterpret::stop()
{
	stopParsing = TRUE;
}


void NMGStatisticXMLInterpret::deleteObjects()
{
	if ( metadata!=0 )
		delete metadata;
	metadata = 0;

	if ( currentFlowMetadata!=0 )
		delete currentFlowMetadata;
	currentFlowMetadata = 0;
}

bool NMGStatisticXMLInterpret::isFinished()
{
	return isAlreadyFinished;
}

QString NMGStatisticXMLInterpret::getErrorString()
{
	return errorMsg;
}

/*int NMGStatisticXMLInterpret::getError()
{
	return errorCode;
}*/

int NMGStatisticXMLInterpret::getPercentCompleted()
{
	if ( numPacketsAllFlows!=0 )
		return ( countAll * 100 ) / numPacketsAllFlows;
	else
		return 0;
}

QString NMGStatisticXMLInterpret::getStatus()
{
	return statusMsg;
}

/**
 * Makes the operations to calcule the stastics of each flow
 * @return
 */
void NMGStatisticXMLInterpret::makeCalc ()
{
	double sum, temporary;
	long long index95, index99, index999, index50;

	/** Delay **/

	qHeapSort ( delay_list );

	min_delay = delay_list.first ();
	max_delay = delay_list.last ();

	sum = 0;
	for ( QValueVector<long>::iterator itd = delay_list.begin (); itd != delay_list.end (); ++itd )
	{
		temporary = ( *itd );
		sum = sum + pow ( ( temporary - avg_delay / nr_delays ), 2 );

	}

	stDev_delay = sqrt ( sum / ( nr_delays - 1 ) );

	index95 = ( long long ) ceil ( 0.95 * nr_delays );
	index50 = ( long long ) ceil ( 0.5 * nr_delays );
	index99 = ( long long ) ceil ( 0.99 * nr_delays );
	index999 = ( long long ) ceil ( 0.999 * nr_delays );

	delay_percentile95 = delay_list.at ( index95 - 1 );
	delay_percentile99 = delay_list.at ( index99 - 1 );
	delay_percentile999 = delay_list.at ( index999 - 1 );
	delay_percentile50 = delay_list.at ( index50 - 1 );
	delay_jitter = delay_percentile95 - delay_percentile50;

	/** IPDV **/

	qHeapSort ( ipdv_list );

	min_ipdv = ipdv_list.first ();
	max_ipdv = ipdv_list.last ();

	sum = 0;
	for ( QValueVector<long>::iterator itipdv = ipdv_list.begin (); itipdv != ipdv_list.end (); ++itipdv )
	{
		temporary = ( *itipdv );
		sum = sum + pow ( ( temporary - ( avg_ipdv / nr_ipdv ) ), 2 );
	}
	stDev_ipdv = sqrt ( sum / ( nr_ipdv - 1 ) );

	index95 = ( long long ) ceil ( 0.95 * nr_ipdv );
	index50 = ( long long ) ceil ( 0.5 * nr_ipdv );
	index99 = ( long long ) ceil ( 0.99 * nr_ipdv );
	index999 = ( long long ) ceil ( 0.999 * nr_ipdv );

	ipdv_percentile95 = ipdv_list.at ( index95 - 1 );
	ipdv_percentile99 = ipdv_list.at ( index99 - 1 );
	ipdv_percentile999 = ipdv_list.at ( index999 - 1 );
	ipdv_percentile50 = ipdv_list.at ( index50 - 1 );
	ipdv_jitter = ipdv_percentile95 - ipdv_percentile50;
}


/**
 * Makes a new node with the statstic values calculated for each flow
 * @param doc: xml document
 * @param id: flow identifier
 * @return flow with its statistics
 */
void NMGStatisticXMLInterpret::createStatisticNode ()
{
	//<sent>..</sent>
	xw->writeTaggedString ( TAG_SENT, QString ( "%1" ).arg ( sentPackets () ) );

	//<lost>..</lost>
	xw->writeTaggedString ( TAG_LOST, QString ( "%1" ).arg ( lostPackets () ) );

	//<duplicated>..</duplicated>
	xw->writeTaggedString ( TAG_DUPLICATED, QString ( "%1" ).arg ( duplicatePackets () ) );

	//<delay>..
	xw->writeOpenTag ( TAG_DELAY );

	//<average>..</average>
	xw->writeTaggedString ( TAG_AVERAGE, QString ( "%1" ).arg ( averageDelay () ) );

	//<min>..</min>
	xw->writeTaggedString ( TAG_MIN, QString ( "%1" ).arg ( minDelay () ) );

	//<max>..</max>
	xw->writeTaggedString ( TAG_MAX, QString ( "%1" ).arg ( maxDelay () ) );

	//<stdev>..</stdev>
	xw->writeTaggedString ( TAG_STDEV, QString ( "%1" ).arg ( stDevDelay () ) );

	//<jitter>..</jitter>
	xw->writeTaggedString ( TAG_JITTER, QString ( "%1" ).arg ( jitterDelay () ) );

	//<p50>..</p50>
	xw->writeTaggedString ( TAG_P50, QString ( "%1" ).arg ( delay50Percentile () ) );

	//<p95>..</p95>
	xw->writeTaggedString ( TAG_P95, QString ( "%1" ).arg ( delay95Percentile () ) );

	//<p99>..</p99>
	xw->writeTaggedString ( TAG_P99, QString ( "%1" ).arg ( delay99Percentile () ) );

	//<p999>..</p999>
	xw->writeTaggedString ( TAG_P999, QString ( "%1" ).arg ( delay999Percentile () ) );

	//..</delay>
	xw->writeCloseTag ( TAG_DELAY );

	//<ipdv>..
	xw->writeOpenTag ( TAG_IPDV );

	//<average>..</average>
	xw->writeTaggedString ( TAG_AVERAGE, QString ( "%1" ).arg ( averageIPDV () ) );

	//<min>..</min>
	xw->writeTaggedString ( TAG_MIN, QString ( "%1" ).arg ( minIPDV () ) );

	//<max>..</max>
	xw->writeTaggedString ( TAG_MAX, QString ( "%1" ).arg ( maxIPDV () ) );

	//<stdev>..</stdev>
	xw->writeTaggedString ( TAG_STDEV, QString ( "%1" ).arg ( stDevIPDV () ) );

	//<jitter>..</jitter>
	xw->writeTaggedString ( TAG_JITTER, QString ( "%1" ).arg ( jitterIPDV () ) );

	//<p50>..</p50>
	xw->writeTaggedString ( TAG_P50, QString ( "%1" ).arg ( ipdv50Percentile () ) );

	//<p95>..</p95>
	xw->writeTaggedString ( TAG_P95, QString ( "%1" ).arg ( ipdv95Percentile () ) );

	//<p99>..</p99>
	xw->writeTaggedString ( TAG_P99, QString ( "%1" ).arg ( ipdv99Percentile () ) );

	//<p999>..</p999>
	xw->writeTaggedString ( TAG_P999, QString ( "%1" ).arg ( ipdv999Percentile () ) );

	//..</ipdv>
	xw->writeCloseTag ( TAG_IPDV );
}

void NMGStatisticXMLInterpret::createStatisticFlowData()
{
	NMGStatisticFlowData * currentFlowData = new NMGStatisticFlowData();

	currentFlowData->setId ( currentFlowStr );

	currentFlowData->setSentPackets ( sentPackets() );
	currentFlowData->setDuplicatePackets ( duplicatePackets() );
	currentFlowData->setLostPackets ( lostPackets() );

	currentFlowData->setAverageDelay ( averageDelay() );
	currentFlowData->setMinDelay ( minDelay() );
	currentFlowData->setMaxDelay ( maxDelay() );
	currentFlowData->setStDevDelay ( stDevDelay() );
	currentFlowData->setJitterDelay ( jitterDelay() );
	currentFlowData->setDelay99Percentile ( delay50Percentile() );
	currentFlowData->setDelay95Percentile ( delay95Percentile() );
	currentFlowData->setDelay999Percentile ( delay99Percentile() );
	currentFlowData->setDelay50Percentile ( delay999Percentile() );

	currentFlowData->setAverageIPDV ( averageIPDV() );
	currentFlowData->setMinIPDV ( minIPDV() );
	currentFlowData->setMaxIPDV ( maxIPDV() );
	currentFlowData->setStDevIPDV ( stDevIPDV() );
	currentFlowData->setJitterIPDV ( jitterIPDV() );
	currentFlowData->setIpdv99Percentile ( ipdv50Percentile() );
	currentFlowData->setIpdv95Percentile ( ipdv95Percentile() );
	currentFlowData->setIpdv999Percentile ( ipdv99Percentile() );
	currentFlowData->setIpdv50Percentile ( ipdv999Percentile() );

	data->addFlowData ( currentFlowData );
	currentFlowData = 0;
}

NMGStatisticData* NMGStatisticXMLInterpret::getStatisticsData() {
	return data;
}


long long NMGStatisticXMLInterpret::sentPackets ()
{
	return count - dupl;
}


long long NMGStatisticXMLInterpret::duplicatePackets ()
{
	return dupl;
}


long long NMGStatisticXMLInterpret::lostPackets ()
{
	for ( uint i = 0; i < seqList.size(); i++ )
		if ( !seqList.testBit ( i ) ) lost++;
	return lost;
}


double NMGStatisticXMLInterpret::averageDelay ()
{
	return ( double ) ( avg_delay / nr_delays ) / MSECS;
}


double NMGStatisticXMLInterpret::averageIPDV ()
{
	return ( double ) ( avg_ipdv / nr_ipdv ) / MSECS;
}


double NMGStatisticXMLInterpret::minDelay ()
{
	return ( double ) min_delay / MSECS;
}


double NMGStatisticXMLInterpret::minIPDV ()
{
	return ( double ) min_ipdv / MSECS;
}


double NMGStatisticXMLInterpret::maxDelay ()
{
	return ( double ) max_delay / MSECS;
}


double NMGStatisticXMLInterpret::maxIPDV ()
{
	return ( double ) max_ipdv / MSECS;
}


double NMGStatisticXMLInterpret::stDevDelay ()
{
	return stDev_delay / MSECS;
}


double NMGStatisticXMLInterpret::stDevIPDV ()
{
	return stDev_ipdv / MSECS;
}


double NMGStatisticXMLInterpret::jitterDelay ()
{
	return ( double ) delay_jitter / MSECS;
}


double NMGStatisticXMLInterpret::jitterIPDV ()
{
	return ( double ) ipdv_jitter / MSECS;
}


bool NMGStatisticXMLInterpret::delayIsNegative ()
{
	return negative_delay;
}

double NMGStatisticXMLInterpret::delay50Percentile ()
{
	return ( double ) delay_percentile50 / MSECS;
}

double NMGStatisticXMLInterpret::delay99Percentile ()
{
	return ( double ) delay_percentile99  / MSECS;
}


double NMGStatisticXMLInterpret::delay95Percentile ()
{
	return ( double ) delay_percentile95 / MSECS;
}


double NMGStatisticXMLInterpret::delay999Percentile ()
{
	return ( double ) delay_percentile999 / MSECS;
}


double NMGStatisticXMLInterpret::ipdv50Percentile ()
{
	return ( double ) ipdv_percentile50 / MSECS;
}


double NMGStatisticXMLInterpret::ipdv95Percentile ()
{
	return ( double ) ipdv_percentile95 / MSECS;
}


double NMGStatisticXMLInterpret::ipdv99Percentile ()
{
	return ( double ) ipdv_percentile99 / MSECS;
}


double NMGStatisticXMLInterpret::ipdv999Percentile ()
{
	return ( double ) ipdv_percentile999 / MSECS;
}


void NMGStatisticXMLInterpret::cleanValues ()
{
	count = 0;
	dupl = 0;
	lost = 0;

	avg_delay = 0;
	nr_delays = 0;
	avg_ipdv = 0;
	nr_ipdv = 0;

	negative_delay = FALSE;

	stDev_delay = 0;
	stDev_ipdv = 0;

	delay_jitter = 0;
	ipdv_jitter = 0;

	delay_percentile50 = 0;
	delay_percentile95 = 0;
	delay_percentile99 = 0;
	delay_percentile999 = 0;

	ipdv_percentile50 = 0;
	ipdv_percentile95 = 0;
	ipdv_percentile99 = 0;
	ipdv_percentile999 = 0;

	delay_list.clear();
	ipdv_list.clear();
}

