/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlistview.h>
#include <qtabwidget.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qgroupbox.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qaction.h>
#include <qradiobutton.h>
#include <qdatetimeedit.h>

#include <messagebar.h>
#include "nmgnetperfwidget.h"

#include <nmgpreferencesmanager.h>
#include <nmgmoduletabwidgetmanager.h>
#include <nmgmoduleutils.h>

#include <iostream>
#include <cstdlib>

using namespace std;

/**
 * Contructor of the class
 * @param parent 
 * @param name 
 * @return 
 */
NMGNetperfWidget::NMGNetperfWidget(NMGISender * s, NMGIModuleCommunication * mc, NMGIModuleStationManager * sm, NMGModTabWidgetManager * tabWidgetMan, NMGModulePreferencesManager * prefMan, QWidget *parent, const char *name)
	: NMGNetperfWidgetBase(parent, name), interpret(this), errorNotifier(this)
{
	progressItem = 0;
	toolBar = 0;
	sender = s;
	moduleCommunication = mc;
	stationManager = sm;
	tabWidgetManager = tabWidgetMan;
	preferencesManager = prefMan;
	dateStartTime->setDate(QDate().currentDate());
	timeStartTime->setTime(QTime().currentTime());
	int randomSourcePort = rand() % 1000 + 12865;
	txtSourcePort->setValue ( randomSourcePort );
	
	connect(cmdStart,SIGNAL(clicked()),this,SLOT(buttonStartClicked()));
	connect(cmdStop,SIGNAL(clicked()),this,SLOT(buttonStopClicked()));
	createFlowListMenu();
	connect(&progressTimer,SIGNAL(timeout()),this,SLOT(incrementProgressItemBar()));
}

/**
 * Destructor of the class
 * @return 
 */
NMGNetperfWidget::~NMGNetperfWidget()
{
	if(progressItem) progressItem->setComplete();
	progressItem = 0;
}

/**
 * Creates the flow list menu
 */
void NMGNetperfWidget::createFlowListMenu()
{
	flowListMenu = new QPopupMenu();
	flowListMenu->insertItem("Sta&rt test",this,SLOT(buttonStartClicked()),CTRL+Key_R);
	flowListMenu->insertSeparator();
	flowListMenu->insertItem("&Load test from disk",this,SLOT(loadTestFromDisk()),CTRL+Key_L);
	flowListMenu->insertItem("&Save test to disk",this,SLOT(saveTestToDisk()),CTRL+Key_S);
}

/**
 * Returns the toolbar of the netperf widget
 * @return 
 */
QToolBar * NMGNetperfWidget::getToolBar()
{
	if(!toolBar)
	{
		toolBar = new QToolBar(NMGModuleUtils::getMainWindow());
		QAction * action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("launch.png")), tr("Start test"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(buttonStartClicked()));
		action->addTo(toolBar);
	}
	return toolBar;
}

/**
 * Slot which load a flow from disk
 */
void NMGNetperfWidget::loadTestFromDisk()
{
	QString fileName = QFileDialog::getOpenFileName(getLastTestPath(),"XML Flow Format (*.xff);;XML (*.xml);;All files (*.*)",0,0,"Select test file");
	
	if(fileName and fileName!="")
	{
		saveLastTestPathFromFilePath(fileName);
		NMGXMLConfigurationFile file;
		file.setPath(fileName);
		if(!file.readFromDisk())
		{
			QMessageBox::critical(0,"Error opening file",
					      "Error while reading the document from disk.\n"+file.getError());
			cerr<<"Error while reading the document from disk.\n"<<file.getError()<<endl;
		}
		else
		{
			QDomElement root = file.getDomDocument().documentElement();
			int reply = QMessageBox::QMessageBox::question(0,"Confirm operation",
					"Do you really want to delete current configuration?",
					QMessageBox::Yes|QMessageBox::Default,QMessageBox::No|QMessageBox::Escape);
			if(reply == QMessageBox::No) return;
			interpret.updateWidgetFromXMLDocument(root);
		}
	}
}

/**
 * Slot which saves flows to disk
 */
void NMGNetperfWidget::saveTestToDisk()
{
	if(hostSourceCombo->getStationFromCombo() == 0) 
		QMessageBox::warning(0,"No Source Host","Source host is not defined.");
	else if (hostDestinationCombo->getStationFromCombo() == 0)
		QMessageBox::warning(0,"No Destination Host","Destination host is not defined.");
	else {
		QString testContent = interpret.generateXMLTestDocument(false);
	
		QString fileName = QFileDialog::getSaveFileName(getLastTestPath(),"XML Flow Format (*.xff);;XML (*.xml);;All files (*.*)",0,0,"File to save the flows");
		if(fileName and fileName!="")
		{
			if(!fileName.contains(".xff")) fileName = fileName + ".xff";
			if(QFile::exists(fileName))
			{
				int reply = QMessageBox::question(0,"Overwirte the file?","The file "+fileName+" exists.\n"+
						"Do you want to overwrite it?",
						QMessageBox::Yes|QMessageBox::Default,QMessageBox::No|QMessageBox::Escape);
				if(reply==QMessageBox::No) return;
			}
			saveLastTestPathFromFilePath(fileName);
			NMGXMLConfigurationFile file;
			file.setPath(fileName);
			file.getDomDocument().setContent(testContent);
			if(!file.writeToDisk()) 
				cerr << "Error while writing the file to disk("<<file.getError()<<").";
		}
	}
}


/**
 * Slot executed when the user clicks the start button to start a new test
 */
void NMGNetperfWidget::buttonStartClicked()
{
	if(not allFieldsNeededCorrect()) return;
	
	QString xmlTest = interpret.generateXMLTestDocument(true);
	
	if(sender->sendCommand(Start,xmlTest))
	{
		cmdStop->setEnabled(true);
		cmdStart->setEnabled(false);
		tabWidgetManager->setTabImage("runa.png");
		maxTestTime = getMaximumEndTime();
		QString title = txtTestName->text()==""?"Netperf":txtTestName->text();
		progressItem = ProgressManager::createProgressItem(title);
		progressItem->setTotalItems(100);		
		connect(progressItem, SIGNAL(progressItemCanceled( ProgressItem* )), this, SLOT( testCancelled( ProgressItem *) ) );
		progressItem->setStatus("Preparing test...");
	}
	else MessageBar::longMessage("Command not started\nThere is no connection with the server!");

}


/**
 * Checks if all the fields are correct to send a start message to the server
 * If they are not correct a message box is shown to the server
 * @return errorMessage 
 */
bool NMGNetperfWidget::allFieldsNeededCorrect()
{
	QString errorMessage = "";
	
	//Source check
	if(hostSourceCombo->getStationAddressFromCombo()=="") 
		errorMessage += "The source address is not an optional field.\n";
	else
	{
		NMGStation * sourceStation = hostSourceCombo->getStationFromCombo();
		if(!sourceStation)
			errorMessage += "The source host is not optional.\n";
		else
		{
			if(sourceStation->getStationFieldsManager().
						getField(getStationManager()->getModuleStationKey(),ID_NETPERF_PATH)=="")
				errorMessage += "Netperf path is not an optional field for the source host.\n";
			if(sourceStation->getManagementUsername()=="")
				errorMessage += "Management username is not an optional field for the source host.\n";
		}
	}
	
	//Destination check
	if(hostDestinationCombo->getStationAddressFromCombo()=="") 
		errorMessage += "The destination address is not an optional field.\n";
	else
	{
		NMGStation * destionationStation = hostDestinationCombo->getStationFromCombo();
		if(!destionationStation)
			errorMessage += "The destination host is not optional.\n";
		else
		{
			if(destionationStation->getStationFieldsManager().
						getField(getStationManager()->getModuleStationKey(),ID_NETSERVER_PATH)=="")
				errorMessage += "Netserver path is not an optional field for the destination host.\n";
			if(destionationStation->getManagementUsername()=="")
				errorMessage += "Management username is not an optional field for the destination host.\n";
		}
	}
	
	if(errorMessage!="")
		QMessageBox::warning(0,"Command not started",errorMessage);
	return errorMessage==""?true:false;
}

/**
 * Test cancelled using the progress bar widget
 * @param item 
 */
void NMGNetperfWidget::testCancelled( ProgressItem * )
{
	buttonStopClicked();
}

/**
 * Slot executed when the user clicks the stop button to stop a test which is being executed
 */
void NMGNetperfWidget::buttonStopClicked()
{
	sender->sendCommand(Stop,"");
	updateWidgetBecauseOfTestEnd();
}

/**
 * Updates the widgets becuse the end has ended
 */
void NMGNetperfWidget::updateWidgetBecauseOfTestEnd()
{
	cmdStop->setEnabled(false);
	cmdStart->setEnabled(true);
	
	if(progressItem) progressItem->setComplete();
	progressItem = 0;
}

/**
 * Slot executed when the source station changes
 * @param station 
 */
void NMGNetperfWidget::sourceStationChanged(NMGStation * station)
{
	//txtTestDestination->setText(station?station->getManagementHost().getAddress():"");
}
   
/**
 * Returns true if the protocol used in the test is IPv4
 * @return 
 */
bool NMGNetperfWidget::isProtocolIPv4()
{
	NMGStation * station = hostDestinationCombo->getStationFromCombo();
	if(!station) return true;
	else return station->getTestHost().isHostIPv4();
}

/**
 * Returns the op up menu of the widget
 * @return 
 */
QPopupMenu * NMGNetperfWidget::getPopupMenu()
{
	return flowListMenu;
}

/**
 * Returns the station manager
 * @return 
 */
NMGIModuleStationManager * NMGNetperfWidget::getStationManager()
{
	return stationManager;
}

/**
 * Returns the last test path (or "" if undefined)
 * @return 
 */
QString NMGNetperfWidget::getLastTestPath()
{
	QString lastTestPath = preferencesManager->getValue(ID_PREFERENCE_LAST_TEST_PATH);
	if(!lastTestPath or lastTestPath=="") lastTestPath = "";
	return lastTestPath;
}

/**
 * Saves the last test path from the filename path of the test
 * @param filePath 
 */
void NMGNetperfWidget::saveLastTestPathFromFilePath(const QString & filePath)
{
	if(filePath and filePath!="")
	{
		int lastPos = filePath.findRev("/");
		preferencesManager->addValue(ID_PREFERENCE_LAST_TEST_PATH,filePath.left(lastPos));
	}
}

/**
 * Notifies the arrival of the reply of the start message
 * @param data XML message to be parsed
 */
void NMGNetperfWidget::notifyResult(QString data)
{
	QString resultFile = interpret.getResultFilePathFromResult(data);
	if(resultFile=="")
	{
		MessageBar::longMessage( "The execution of the Netperf test wasn't succesful.");
		updateWidgetBecauseOfTestEnd();
	}
	else
	{
		updateWidgetBecauseOfTestEnd();
		MessageBar::longMessage("Netperf test finished; Result at " + resultFile + ".");
	}
}

/**
 * Notifies the error of the core module
 * @param errorType Type of the error (fatal = 1, user = 5, warning = 10, debug = 20)
 * @param errorMessage Error Message
 */
void NMGNetperfWidget::notifyError(int errorType, QString errorMessage)
{
	if(errorType>=1 and errorType<=5) updateWidgetBecauseOfTestEnd();
	
	errorNotifier.notifyErrorToUser(errorType,errorMessage);
}

/** 
 * Notifies an status reply form the core
 * @param status XMl document with root data
 */
void NMGNetperfWidget::notifyStatus(QString status)
{
#ifdef DEBUG
		cout << "NMGNetperfWidget::notifyStatus " << status << endl;
#endif
	
	typedef enum{NotStarted=0, SendingNetserverScriptFile=10, NetserverStarted=20, SendingNetperfScriptFile=30, 
		NetperfStarted=40, NetperfFinished = 50, DecodeStarted = 60, DecodeFinished = 70,
		ReceivingDecodeResultFile=80, TestFinished = 90, TestError=100} NetperfStatusReply;
	
	if(!progressItem) cerr << "Status arrived without item at the progress bar!"<<endl;
	else
{
		int statusNum = interpret.parseStatusMessage(status);
		if(statusNum<0) cerr << "Error at status message." << endl;
		else
{
			NetperfStatusReply reply = (NetperfStatusReply) statusNum;
			switch(reply)
{
				case NotStarted: //0%
					progressItem->setStatus("Test not started");
					break;
				case SendingNetserverScriptFile: //1%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Sending Netserver command");
					break;
				case NetserverStarted: //2%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Netserver started");
					break;
				case SendingNetperfScriptFile: //3%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					runningNetperfAndNetserver = false;
					progressItem->setStatus("Sending Netperf command");
					break;
				case NetperfStarted: // 4% to 95% (91 increments to do)
					if(!runningNetperfAndNetserver) { 
						//start the timer to increase the progress bar
						stepTime=maxTestTime/91;
						leftTime = maxTestTime;
						progressTimer.start(stepTime);
						runningNetperfAndNetserver = true;
					}
					progressItem->setStatus("Netperf and Netserver running");
					break;
				case NetperfFinished://96%
					progressTimer.stop();
					runningNetperfAndNetserver = false;
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Netperf finished");
					break;
				case DecodeStarted://97%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Decode started");
					break;
				case DecodeFinished: //98%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Decode finished");
					break;
				case ReceivingDecodeResultFile: //99%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Receiving decode result file");
					break;
				case TestFinished: //100%
					progressItem->incCompletedItems();
					progressItem->updateProgress();
					progressItem->setStatus("Test finished");
					progressItem->setComplete();
					progressItem = 0;
					break;
				case TestError:
					progressTimer.stop();
					runningNetperfAndNetserver = false;
					progressItem->setStatus("Error executing test");
					progressItem->setComplete();
					progressItem = 0;
					break;
				default:
					progressItem->setStatus("Unknown status!");
			}
		}
	}
}

/**
 * Increments the progress bar
 */
void NMGNetperfWidget::incrementProgressItemBar()
{
	if(progressItem and leftTime > 1)
	{
		leftTime -= stepTime;
		progressItem->incCompletedItems();
		progressItem->updateProgress();
		progressItem->setStatus("Netperf and Netserver running ("+getLeftTimeTestString()+" left)");
	}
}

/**
 * Returns the string corresponding to the left execution time of the test (in a friendly way hoiur, min, ...);
 * @return 
 */
QString NMGNetperfWidget::getLeftTimeTestString()
{
	if(leftTime<=0) leftTime = 1;
	if(leftTime<1000) //less than a second --> ms
		return QString("%1").arg(leftTime)+" ms";
	else if (leftTime<60*1000) //less than a minute --> sec
		return QString("%1").arg(leftTime/1000)+" sec";
	else if(leftTime<60*60*1000) //less than an hour --> minutes 
		return QString("%1").arg(leftTime/(60*1000))+" min " + QString("%1").arg(leftTime%(60*1000)/1000)+" sec";
	else return QString("%1").arg(leftTime/(60*60*1000))+" h " + QString("%1").arg(leftTime%(60*60*1000)/(60*1000))+" min";;
	
}


/**
 * Returns the maximum end time of all the flows of the test
 * @return 
 */
int NMGNetperfWidget::getMaximumEndTime()
{
	if (radSeconds->isChecked())
		return (txtTestDuration->value())*1000;
	else
		return (int) ((txtTestDuration->value()/8)*100); //We suppose a link of 10 Mbps
}


/**
 * Returns true if the test is running
 * @return 
 */
bool NMGNetperfWidget::isTestRunning()
{
	return cmdStop->isEnabled();
}

#include "nmgnetperfwidget.moc"
