/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGNETPERFWIDGET_H
#define NMGNETPERFWIDGET_H

#include "nmgnetperfxmltags.h"

#include <qwidget.h>
#include <qpoint.h>
#include <qpopupmenu.h>
#include <qdom.h>
#include <qfiledialog.h>
#include <qtoolbar.h>
#include <qtimer.h>


#include <progressmanager.h>
#include "nmgnetperfwidgetbase.h"
#include "nmgnetperfxmlinterpret.h"
#include "nmgnetperfwidgeterrornotifier.h"
#include "../nmgimodulestationmanager.h"
#include "../nmgmodulepreferencesmanager.h"
#include "../nmgimodulecommunication.h"
#include "../nmgisender.h"
#include "../../gui/nmgxmlconfigurationfile.h"
#include "../../gui/nmgimagemanager.h"
#include "../nmgmodtabwidgetmanager.h"

#define LEFT_BUTTON	1
#define RIGHT_BUTTON	2
#define MID_BUTTON	4

#define TAB_FLOW_PROPERTIES	"Flow properties"
#define TAB_MOD_FLOW_PROPERTIES	"Modification properties"
#define TAB_EXTENDED_ATTRIBUTES	"Extended attributes"
#define TAB_IPV4_OPTIONS	"IPv4 Options"
#define TAB_IPV6_OPTIONS	"IPv6 Options"
#define TAB_IPV6_EXT_HEADERS	"IPv6 Extension Headers"
#define TAB_RESULT_FILE		"Result file"

class NMGNetperfWidget : public NMGNetperfWidgetBase
{
		Q_OBJECT
	public:
		NMGNetperfWidget ( NMGISender * s, NMGIModuleCommunication * mc, NMGIModuleStationManager * sm,
		                   NMGModTabWidgetManager * tabWidgetManager, NMGModulePreferencesManager * prefMan,
		                   QWidget *parent = 0, const char *name = 0 );
		~NMGNetperfWidget();
		NMGIModuleStationManager * getStationManager();
		void notifyResult ( QString data );
		void notifyError ( int errorType, QString errorMessage );
		void notifyStatus ( QString status );
		bool isTestRunning();
		bool isProtocolIPv4();

	public slots:
		void loadTestFromDisk();
		void saveTestToDisk();
		void buttonStartClicked();
		void buttonStopClicked();
		void sourceStationChanged ( NMGStation * station );
		QPopupMenu * getPopupMenu();
		QToolBar * getToolBar();
		QString getLastTestPath();
		void saveLastTestPathFromFilePath ( const QString & filePath );
		void testCancelled ( ProgressItem * );
		void incrementProgressItemBar();

	private:
		void createFlowListMenu();
		bool allFieldsNeededCorrect();
		void updateWidgetBecauseOfTestEnd();
		int getMaximumEndTime();
		QString getLeftTimeTestString();

		QPopupMenu * flowListMenu;
		QToolBar * toolBar;
		NMGIModuleStationManager * stationManager;
		NMGISender * sender;
		NMGModTabWidgetManager * tabWidgetManager;
		NMGIModuleCommunication * moduleCommunication;
		NMGModulePreferencesManager * preferencesManager;
		NMGNetperfXMLInterpret interpret;
		ProgressItem * progressItem;
		int maxTestTime;
		int stepTime;
		int leftTime;
		QTimer progressTimer;
		bool runningNetperfAndNetserver;
		NMGNetperfWidgetErrorNotifier errorNotifier;
};

#endif
