/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qmessagebox.h>

#include "nmgnetperfwidgeterrornotifier.h"
#include "nmgnetperfwidget.h"

#include <iostream>

using namespace std;

/**
 * Contructor of the class
 * @return 
 */
NMGNetperfWidgetErrorNotifier::NMGNetperfWidgetErrorNotifier(NMGNetperfWidget * w)
{
	widget = w;
}


/**
 * Destructor of the class
 * @return 
 */
NMGNetperfWidgetErrorNotifier::~NMGNetperfWidgetErrorNotifier()
{
}

/**
 * Notifies an error to the user
 * @param errorType Type of the error (fatal = 1, user = 5, warning = 10, debug = 20)
 * @param errorMessage Error Message
 */
void NMGNetperfWidgetErrorNotifier::notifyErrorToUser(int errorType, QString errorMessage)
{
	if(errorType>5) 
	{ /* warning or debug */
		if(errorType==10) cout << "WARNING: " << errorMessage << endl;
		else cout << "DEBUG: " << errorMessage << endl;
		return;
	}
	
	if(errorMessage==NETPERF_PATH_WRONG)
	{
		QMessageBox::critical( 0,"Wrong Netperf path!","Netperf's path is not correct.");
		NMGHostInformationComboWidget * combo = widget->hostSourceCombo;
		combo->showHostInformationWidget(0,ID_NETPERF_PATH);
	}
	else if(errorMessage==NETSERVER_PATH_WRONG)
	{
		QMessageBox::critical( 0,"Wrong Netserver path!","Netserver's path is not correct.");
		NMGHostInformationComboWidget * combo = widget->hostDestinationCombo;
		combo->showHostInformationWidget(0,ID_NETSERVER_PATH);
	}
	else if(errorMessage==CANNOT_RESOLVE_SOURCE_ADDRESS)
	{
		QMessageBox::critical( 0,"Cannot resolve source address!","Source address couldn't be resolved.");
		NMGHostInformationComboWidget * combo = widget->hostSourceCombo;
		combo->setHostBaseWidgetFocus(NMGHostInformationComboWidget::NMGHostInfoComboManagementAddress);
		combo->showHostInformationWidget();
	}
	else if(errorMessage==CANNOT_RESOLVE_DEST_ADDRESS)
	{
		QMessageBox::critical( 0,"Cannot resolve destination address!","Destination address couldn't be resolved.");
		NMGHostInformationComboWidget * combo = widget->hostDestinationCombo;
		combo->setHostBaseWidgetFocus(NMGHostInformationComboWidget::NMGHostInfoComboManagementUsername);
		combo->showHostInformationWidget();	
	}
	else if(errorMessage==BAD_PASSWORD_SOURCE_HOST)
	{
		QMessageBox::critical( 0,"Bad source host password!","User's password at source host is not correct.");
		NMGHostInformationComboWidget * combo = widget->hostSourceCombo;
		combo->setHostBaseWidgetFocus(NMGHostInformationComboWidget::NMGHostInfoComboManagementPassword);
		combo->showHostInformationWidget();
	}
	else if(errorMessage==BAD_PASSWORD_DEST_HOST)
	{
		QMessageBox::critical( 0,"Bad destination host password!","User's password at destination host is not correct.");
		NMGHostInformationComboWidget * combo = widget->hostDestinationCombo;
		combo->setHostBaseWidgetFocus(NMGHostInformationComboWidget::NMGHostInfoComboManagementPassword);
		combo->showHostInformationWidget();
	}
	else if(errorMessage==NETPERF_ADRRESS_ALREADY_IN_USE)
		QMessageBox::critical( 0,"Netperf port already in use!",
				       "The source host has already a netperf running at this port.");
	else if(errorMessage==ERROR_RUNNING_NETPERF)
		QMessageBox::critical( 0,"Error running netperf!","An error occurred while running netperf.");
	else if(errorMessage==ERROR_RUNNING_NETSERVER)
		QMessageBox::critical( 0,"Error running netserver!","An error occurred while running netserver.");
	else if(errorMessage==NETPERF_INVALID_OPTION)
		QMessageBox::critical( 0,"Netperf invalid arguments!","Error running netperf command because of the arguments.");
	else if(errorMessage==NETPERF_ERROR_UNKNOWN)
		QMessageBox::critical( 0,"Netperf error!","An unknown netperf error occurred during execution.");
	else if(errorMessage==NETSERVER_INVALID_OPTION)
		QMessageBox::critical( 0,"Netserver invalid arguments!","Error running netserver command because of the arguments.");
	else if(errorMessage==NETSERVER_ERROR_UNKNOWN)
		QMessageBox::critical( 0,"Netserver error!","An unknown netserver error occurred during execution.");
	else if(errorMessage==NETPERF_SSH_TIMEOUT_ERROR)
		QMessageBox::critical( 0,"Source host error!","Timeout while connecting via sftp to source host.");
	else if(errorMessage==NETPERF_SSH_UNKNOWN_ERROR)
		QMessageBox::critical( 0,"Source host error!","Unknown error while connecting via sftp to source host.");
	else if(errorMessage==NETSERVER_SSH_TIMEOUT_ERROR)
		QMessageBox::critical( 0,"Destination host error!","Timeout while connecting via sftp to destination host to send netserver script.");
	else if(errorMessage==NETSERVER_ERROR_PERMISSION) {
		QMessageBox::critical( 0,"Netserver error!","Not enough permissions to run netserver in destination host.");	
		NMGHostInformationComboWidget * combo = widget->hostDestinationCombo;
		combo->setHostBaseWidgetFocus(NMGHostInformationComboWidget::NMGHostInfoComboManagementUsername);
		combo->showHostInformationWidget();	
	}
	else if(errorMessage==NETSERVER_SSH_UNKNOWN_ERROR)
		QMessageBox::critical( 0,"Destination host error!","Unknown error while connecting via sftp to destination host to send netserver script.");
	else if(errorMessage==DECODE_SFTP_TIMEOUT_ERROR)
		QMessageBox::critical( 0,"Destination host error!","Timeout while connecting via sftp to destination host to get decode result.");
	else if(errorMessage==DECODE_SFTP_IS_CONNECTED_ERROR)
		QMessageBox::critical( 0,"Destination host error!","'Is connected error' while connecting via sftp to destination host to get decode result.");
	else if(errorMessage==DECODE_SFTP_UNKNOWN_ERROR)
		QMessageBox::critical( 0,"Destination host error!","Unknown error while connecting via sftp to destination host to get decode result.");
	else cout << "Warning: Unknown error message: " << errorMessage << "." << endl;
}

    

