/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qdatetimeedit.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>

#include "nmgnetperfxmlinterpret.h"
#include "nmgnetperfwidget.h"
#include "nmgnetperfxmltags.h"
#include "nmgpreferencesmanager.h"

#include <iostream>
using namespace std;

#include <nmghostinformationcombowidget.h>

/**
 * Constructor of the class
 * @param w 
 * @return 
 */
NMGNetperfXMLInterpret::NMGNetperfXMLInterpret(NMGNetperfWidget * w)
{
	widget = w;
}

/**
 * Destroyer of the class
 * @return 
 */
NMGNetperfXMLInterpret::~NMGNetperfXMLInterpret()
{
}

/**
 * Parses an xml reply form a start
 * @param xmlMessage XML message with tags data as root
 * @return Path of the decode file, or "" if there have been an error while parsing
 */
QString NMGNetperfXMLInterpret::getResultFilePathFromResult(const QString & xmlMessage)
{
	QDomDocument doc;
	if(!doc.setContent(xmlMessage))
	{
		cerr << "Error: the document is not well formed." << endl;
		return "";
	}
	QDomElement root = doc.documentElement();
	QString resultPath = "";
	
	if(root.tagName()!=TAG_DATA) cerr << "Error root element at NMGNetperfXMLInterpret different from data ("<<root.tagName()<<")."<< endl;
	else
	{
		for(QDomNode node = root.firstChild(); !node.isNull(); node = node.nextSibling())
		{
			if(node.isElement())
			{
				QDomElement elem = node.toElement();
				if(elem.tagName()==TAG_DECODE_FILE)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					//if(!id.isNull()) resultPath = id.nodeValue();
					//else cerr << "The decode result path node has no value!" << endl;
				}
				else if(elem.tagName()==TAG_RESULT_FILE)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					
				}
				else if(elem.tagName()==TAG_TEST_FILE)
				{
					//QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) resultPath = id.nodeValue();
					else cerr << "The decode result path node has no value!" << endl;
				}
				else if(elem.tagName()==TAG_RESULT_GRAPH_OWDD)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					
				}
				else if(elem.tagName()==TAG_RESULT_GRAPH_IPDV)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					
				}
				else if(elem.tagName()==TAG_RESULT_GRAPH_IPDVD)
				{
					QString text = "";
					QDomText id = elem.firstChild().toText();
					if(!id.isNull()) text = id.nodeValue();
					
				}
				else cerr << "Tag not correct in Netperf XML reply (" << elem.tagName()<<")." << endl;
			}
		}
	}
	return resultPath;
}

/**
 * Parses the status message and returns the integer value of the status
 * @param xmlMessage 
 * @return Status value or -1 if error
 */
int NMGNetperfXMLInterpret::parseStatusMessage(const QString & xmlMessage)
{
	QDomDocument doc;
	if(!doc.setContent(xmlMessage))
	{
		cerr << "Error: the document is not well formed." << endl;
		return -1;
	}
	QDomElement root = doc.documentElement();
	
	if(root.tagName()!=TAG_DATA) cerr << "Error root element at NMGNetperfXMLInterpret different from data ("<<root.tagName()<<")."<< endl;
	else
	{
		for(QDomNode node = root.firstChild(); !node.isNull(); node = node.nextSibling())
		{
			if(node.isElement())
			{
				QDomElement elem = node.toElement();
				if(elem.tagName()==TAG_STATUS)
				{
					QDomText id = elem.firstChild().toText();
					if(!id.isNull())
					{
						bool ok;
						int num = id.nodeValue().toInt(&ok);
						if(ok) return num;
					}
					else cerr << "The status node has no value!" << endl;
				}
//				else cerr << "Tag not correct in Netperf Status XML (" << elem.tagName()<<")." << endl;
			}
		}
	}
	return -1;
}

/**
 * Generates the xml document of the test with the information of the widget
 * @param putUserAndPass Indicates if the user and password of the management hosts must be in the XML
 * @return Returns the xml document
 */
QString NMGNetperfXMLInterpret::generateXMLTestDocument(bool putUserAndPass)
{
	NMGStation * sourceStation = widget->hostSourceCombo->getStationFromCombo();
	NMGStation * destinationStation = widget->hostDestinationCombo->getStationFromCombo();

	QDomDocument doc(TAG_TEST);
	QDomElement root = doc.createElement(TAG_TEST);
	doc.appendChild(root);

	QDomElement testNetperf = doc.createElement(TAG_METADATA);
	root.appendChild(testNetperf);
	QDomElement resultNetperf = doc.createElement(TAG_RESULT);
	root.appendChild(resultNetperf);
	
	QDomElement testName = doc.createElement(TAG_NAME);
	testNetperf.appendChild(testName);
	QDomText testNameText = doc.createTextNode(widget->txtTestName->text().utf8());
	testName.appendChild(testNameText);
	
	QDomElement testId = doc.createElement(TAG_IDENTIFIER);
	testNetperf.appendChild(testId);
	QDomText testIdText = doc.createTextNode(widget->txtTestId->text().utf8());
	testId.appendChild(testIdText);

	/* Result Directory Path */
	QDomElement testResultPath = doc.createElement(TAG_RESULT_PATH);
	testNetperf.appendChild(testResultPath);
	QDomText testResultPathText = doc.createTextNode(NMGPreferencesManager::self()->getResultDirectory());
	testResultPath.appendChild(testResultPathText);
	
	/* Source station parameters */
	QDomElement sourceNode = doc.createElement(TAG_SOURCE);
	testNetperf.appendChild(sourceNode);
	QDomElement sourceAddress = doc.createElement(TAG_ADDRESS);
	sourceNode.appendChild(sourceAddress);
	QString sourceAddresStr;
	if (sourceStation)
		sourceAddresStr = sourceStation->getTestHost().resolveAddress();
	if(sourceAddresStr=="") sourceAddresStr = sourceStation->getTestHost().getAddress();
	QDomText sourceAddressText = doc.createTextNode(sourceAddresStr);
	sourceAddress.appendChild(sourceAddressText);
	QDomElement sourceNetperfPath = doc.createElement(TAG_NETPERF_PATH);
	sourceNode.appendChild(sourceNetperfPath);
	QDomText sourceNetperfPathText = doc.createTextNode(sourceStation->getStationFieldsManager().	getField(widget->getStationManager()->getModuleStationKey(),ID_NETPERF_PATH));
	sourceNetperfPath.appendChild(sourceNetperfPathText);
	
	QDomElement sourceManagement = doc.createElement(TAG_MANAGEMENT);
	sourceNode.appendChild(sourceManagement);
	QDomElement sourceManagementAddress = doc.createElement(TAG_ADDRESS);
	sourceManagement.appendChild(sourceManagementAddress);
	QDomText sourceManagementAddressText =  doc.createTextNode(sourceStation->getManagementHost().getAddress());
	
	sourceManagementAddress.appendChild(sourceManagementAddressText);
	QDomElement sourceManagementAddressPort = doc.createElement(TAG_PORT);
	sourceManagement.appendChild(sourceManagementAddressPort);
	QDomText sourceManagementAddressPortText =  doc.createTextNode(QString("%1").arg(sourceStation->getManagementAddressPort()));
	sourceManagementAddressPort.appendChild(sourceManagementAddressPortText);
	
	if(putUserAndPass)
	{
		QDomElement sourceManagementUsername = doc.createElement(TAG_USERNAME);
		sourceManagement.appendChild(sourceManagementUsername);
		QDomText sourceManagementUsernameText = doc.createTextNode(sourceStation->getManagementUsername());
		sourceManagementUsername.appendChild(sourceManagementUsernameText);
		QDomElement sourceManagementPassword = doc.createElement(TAG_PASSWORD);
		sourceManagement.appendChild(sourceManagementPassword);
		QDomText sourceManagementPasswordText = doc.createTextNode(sourceStation->getManagementPassword());
		sourceManagementPassword.appendChild(sourceManagementPasswordText);
	}
	
	
	/* Destination station parameters */
	
	QDomElement destinationNode = doc.createElement(TAG_DESTINATION);
	testNetperf.appendChild(destinationNode);
	
	QDomElement destAddress = doc.createElement(TAG_ADDRESS);
	destinationNode.appendChild(destAddress);
	QString destAddressStr = destinationStation->getTestHost().resolveAddress();
	if(destAddressStr=="") destAddressStr = destinationStation->getTestHost().getAddress();
	QDomText destAddressText = doc.createTextNode(destAddressStr);
	destAddress.appendChild(destAddressText);
	QDomElement destNetserverPath = doc.createElement(TAG_NETSERVER_PATH);
	destinationNode.appendChild(destNetserverPath);
	QDomText destNetserverPathText = 
			doc.createTextNode(destinationStation->getStationFieldsManager().getField(widget->getStationManager()->getModuleStationKey(),ID_NETSERVER_PATH));
	destNetserverPath.appendChild(destNetserverPathText);
	
	QDomElement destManagement = doc.createElement(TAG_MANAGEMENT);
	destinationNode.appendChild(destManagement);
	QDomElement destManagementAddress = doc.createElement(TAG_ADDRESS);
	destManagement.appendChild(destManagementAddress);
	QDomText destManagementAddressText =  doc.createTextNode(destinationStation->getManagementHost().getAddress());
	destManagementAddress.appendChild(destManagementAddressText);
	QDomElement destManagementAddressPort = doc.createElement(TAG_PORT);
	destManagement.appendChild(destManagementAddressPort);
	QDomText destManagementAddressPortText =  doc.createTextNode(QString("%1").arg(destinationStation->getManagementAddressPort()));
	destManagementAddressPort.appendChild(destManagementAddressPortText);
	if(putUserAndPass)
	{
		QDomElement destManagementUsername = doc.createElement(TAG_USERNAME);
		destManagement.appendChild(destManagementUsername);
		QDomText destManagementUsernameText = doc.createTextNode(destinationStation->getManagementUsername());
		destManagementUsername.appendChild(destManagementUsernameText);
		QDomElement destManagementPassword = doc.createElement(TAG_PASSWORD);
		destManagement.appendChild(destManagementPassword);
		QDomText destManagementPasswordText = doc.createTextNode(destinationStation->getManagementPassword());
		destManagementPassword.appendChild(destManagementPasswordText);
	}
	
	/*if(putUserAndPass and widget->txtDistrLocalPath->text()!="")
	{
		QDomElement localNode = doc.createElement(TAG_LOCAL);
		testNetperf.appendChild(localNode);

	
		QDomElement distrPath = doc.createElement(TAG_DISTR_PATH);
		localNode.appendChild(distrPath);
		QDomText distrPathText = doc.createTextNode(widget->txtDistrLocalPath->text());
		distrPath.appendChild(distrPathText);
	}*/
	
	createXMLConfNetperf(doc,testNetperf);
	createXMLConfNetserver(doc,testNetperf);
	
	return doc.toString();
}

/**
 * Creates the conf netperf part of the xml
 * @param doc 
 * @param root 
 */
void NMGNetperfXMLInterpret::createXMLConfNetperf(QDomDocument & doc, QDomElement & root)
{
	QDomElement confNetperf = doc.createElement(TAG_CONF_NETPERF);
	root.appendChild(confNetperf);

/*	if(widget->checkStartTime->isChecked())
	{
	
	QString startTime = widget->timeStartTime->time().toString("hh:mm:ss");
		
	QString gmt = "";
	if(widget->comboGMTOrLocalTime->currentText()=="GMT") gmt = " GMT";
		
	QDomElement start = doc.createElement(TAG_START);
	confNetperf.appendChild(start);
	QDomText startText = doc.createTextNode(startTime+gmt);
	start.appendChild(startText);
}
*/
	
	NMGStation * destinationStation = widget->hostDestinationCombo->getStationFromCombo();
	
	//<destination>...</destination>
	QDomElement destAddress = doc.createElement(TAG_DESTINATION);
	confNetperf.appendChild(destAddress);
	QString destAddressStr = destinationStation->getTestHost().resolveAddress();
	if(destAddressStr=="") destAddressStr = destinationStation->getTestHost().getAddress();
	QDomText destAddressText = doc.createTextNode(destAddressStr);
	destAddress.appendChild(destAddressText);
	
	//<port>...</port>
	QDomElement port = doc.createElement(TAG_PORT);
	confNetperf.appendChild(port);
	QDomText portText = doc.createTextNode(QString("%1").arg(widget->txtSourcePort->value()));
	port.appendChild(portText);

	//<duration>...</duration>
	QDomElement testDuration = doc.createElement(TAG_DURATION);
	if (widget->radSeconds->isChecked())
		testDuration.setAttribute(ATTR_TYPE, "seconds");
	else
		testDuration.setAttribute(ATTR_TYPE, "bytes");
	confNetperf.appendChild(testDuration);
	QDomText testDurationText = doc.createTextNode(QString("%1").arg(widget->txtTestDuration->value()));
	testDuration.appendChild(testDurationText);

	//<proto>...</proto>
	QDomElement testProto = doc.createElement(TAG_PROTO);
	QDomText protocolText;
	if (widget->chkNoDelay->isChecked())
		testProto.setAttribute(ATTR_NODELAY, "yes");
	else
		testProto.setAttribute(ATTR_NODELAY, "no");
	confNetperf.appendChild(testProto);
	if (widget->radTcp->isChecked())
		protocolText = doc.createTextNode("tcp");
	else
		protocolText = doc.createTextNode("udp");
	testProto.appendChild(protocolText);
	
	//<sms>..</sms>
	if (widget->chkSms->isChecked()) {
		QDomElement sms = doc.createElement(TAG_SMS);
		confNetperf.appendChild(sms);
		QDomText smsText = doc.createTextNode(widget->txtSms->text());
		sms.appendChild(smsText);
	}

	//<sss>...</sss>
	if (widget->chkSss->isChecked()) {
		QDomElement sss = doc.createElement(TAG_SSS);
		confNetperf.appendChild(sss);
		QDomText sssText = doc.createTextNode(widget->txtSss->text());
		sss.appendChild(sssText);	
	}
	
	//<rms>..</rms>
	if (widget->chkRms->isChecked()) {
		QDomElement rms = doc.createElement(TAG_RMS);
		confNetperf.appendChild(rms);
		QDomText rmsText = doc.createTextNode(widget->txtRms->text());
		rms.appendChild(rmsText);
	}

	//<rss>...</rss>
	if (widget->chkRss->isChecked()) {
		QDomElement rss = doc.createElement(TAG_RSS);
		confNetperf.appendChild(rss);
		QDomText rssText = doc.createTextNode(widget->txtRss->text());
		rss.appendChild(rssText);	
	}
	
}

/**
 * Creates the conf netserver part of the xml
 * @param doc 
 * @param root 
 */
void NMGNetperfXMLInterpret::createXMLConfNetserver(QDomDocument & doc, QDomElement & root)
{
	QDomElement confNetserver = doc.createElement(TAG_CONF_NETSERVER);
	root.appendChild(confNetserver);
	
	//<port>...</port>
	QDomElement port = doc.createElement(TAG_PORT);
	confNetserver.appendChild(port);
	QDomText portText = doc.createTextNode(QString("%1").arg(widget->txtSourcePort->value()));
	port.appendChild(portText);
	
	//<ipv>...</ipv>
	QDomElement ipv = doc.createElement(TAG_IPV);
	confNetserver.appendChild(ipv);
	QDomText ipvText;
	if (widget->isProtocolIPv4())
		ipvText = doc.createTextNode("4");
	else
		ipvText = doc.createTextNode("6");
	ipv.appendChild(ipvText);
}


/**
 * Updates the widget from a xml document (of a test)
 * @param root 
 */
void NMGNetperfXMLInterpret::updateWidgetFromXMLDocument(QDomElement & root)
{
	widget->txtTestName->setText("");
	widget->txtTestId->setText("");
	
	for(QDomNode node = root.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement elem = node.toElement();
			if(elem.tagName()==TAG_METADATA)
				updateWidgetFromXMLDocument(elem);
			else if(elem.tagName()==TAG_NAME) 
				widget->txtTestName->setText(elem.text());
			else if(elem.tagName()==TAG_IDENTIFIER)
				widget->txtTestId->setText(elem.text());
			else if (elem.tagName()==TAG_RESULT_PATH) //TODO
				updateWidgetFromXMLNodeSource(elem);
			else if(elem.tagName()==TAG_SOURCE)
				updateWidgetFromXMLNodeSource(elem);
			else if(elem.tagName()==TAG_DESTINATION)
				updateWidgetFromXMLNodeDestination(elem);
			else if(elem.tagName()==TAG_CONF_NETPERF)
				updateWidgetFromXMLNodeConfNetperf(elem);
			else if(elem.tagName()==TAG_CONF_NETSERVER)
				updateWidgetFromXMLNodeConfNetserver(elem);
			else if(elem.tagName()==TAG_RESULT);
			else if(elem.tagName()==TAG_LOCAL);
			else cerr << "Unknown tag at netperf test: " << elem.tagName() << endl;
		}
	}
}

/**
 * Updates the widget from the source station node
 * @param elem 
 */
void NMGNetperfXMLInterpret::updateWidgetFromXMLNodeSource(QDomElement & elem)
{
	for(QDomNode node = elem.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement child = node.toElement();
			if(child.tagName()==TAG_ADDRESS)
			{
				NMGHostInformationComboWidget *combo = widget->hostSourceCombo;
				combo->setAddressOfStation(child.text());
			}
		}
	}
}

/**
 * Updates the widget from the destination station node
 * @param elem 
 */
void NMGNetperfXMLInterpret::updateWidgetFromXMLNodeDestination(QDomElement & elem)
{
	for(QDomNode node = elem.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement child = node.toElement();
			if(child.tagName()==TAG_ADDRESS)
			{
				NMGHostInformationComboWidget *combo = widget->hostDestinationCombo;
				combo->setAddressOfStation(child.text());
			}
		}
	}
}

/**
 * Updates the widget from the conf netperf node
 * @param elem 
 */
void NMGNetperfXMLInterpret::updateWidgetFromXMLNodeConfNetperf(QDomElement & elem)
{
	widget->chkStartTime->setChecked(false);
	widget->txtSourcePort->setValue(5000);
	
	for(QDomNode node = elem.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement child = node.toElement();
			if(child.tagName()==TAG_PORT)
			{
				QString port = child.text();
				widget->txtSourcePort->setValue(port.toInt());
			}
			else if(child.tagName()==TAG_START)
			{
				widget->chkStartTime->setChecked(true);
				QString startTime = child.text();
				if(startTime.right(3)=="GMT") 
					widget->comboGMTOrLocalTime->setCurrentText("GMT");
				else widget->comboGMTOrLocalTime->setCurrentText("Local");
				QString hour = startTime.left(2);
				QString minutes = startTime.mid(3,2);
				QString seconds = startTime.mid(6,2);
				widget->timeStartTime->setTime(QTime(hour.toInt(),minutes.toInt(),seconds.toInt()));
			}
			else if(child.tagName()==TAG_DURATION)
			{
				QString testDuration = child.text();
				widget->txtTestDuration->setValue(testDuration.toInt());
				if (child.attribute(ATTR_TYPE) == "seconds") {
					widget->radSeconds->setChecked(true);
					widget->radBytes->setChecked(false);
				} else {
					widget->radSeconds->setChecked(false);
					widget->radBytes->setChecked(true);
				}
			}
			else if(child.tagName()==TAG_PROTO)
			{
				if (child.text() == "tcp") {
					widget->radTcp->setChecked(true);
					widget->radUdp->setChecked(false);
				} else {
					widget->radTcp->setChecked(false);
					widget->radUdp->setChecked(true);
				}
				if (child.attribute(ATTR_NODELAY) == "yes")
					widget->chkNoDelay->setChecked(true);
				else
					widget->chkNoDelay->setChecked(false);
			}
			else if(child.tagName()==TAG_SMS)
			{
				widget->txtSms->setText(child.text());
				widget->chkSms->setChecked(true);
			}
			else if(child.tagName()==TAG_SSS)
			{
				widget->txtSss->setText(child.text());
				widget->chkSss->setChecked(true);
			}
			if(child.tagName()==TAG_RMS)
			{
				widget->txtRms->setText(child.text());
				widget->chkRms->setChecked(true);
			}
			else if(child.tagName()==TAG_RSS)
			{
				widget->txtRss->setText(child.text());
				widget->chkRss->setChecked(true);
			}
			else if(child.tagName()==TAG_DESTINATION);
			else cerr << "unknow tag at conf_netperf document " << child.tagName() << endl;
		}
	}
}

/**
 * Updates the widget from the conf netserver node
 * @param elem 
 */
void NMGNetperfXMLInterpret::updateWidgetFromXMLNodeConfNetserver(QDomElement & elem)
{
	for(QDomNode node = elem.firstChild(); not node.isNull(); node = node.nextSibling())
	{
		if(node.isElement())
		{
			QDomElement child = node.toElement();
			if(child.tagName()==TAG_PORT);
			else if(child.tagName()==TAG_IPV);
			else cerr << "unknow tag at conf_netserver document " << child.tagName() << endl;
		}
	}
}
