/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "meterlistviewitem.h"

MeterListViewItem::MeterListViewItem(QListView *parent)
 : QListViewItem(parent)
{
	m_defaultcolor = parent->paletteBackgroundColor();
	clearColor();
}


MeterListViewItem::~MeterListViewItem()
{
}

/**
	\fn MeterListViewItem::backgroundColor()
	\brief Return the current background color
*/
QColor MeterListViewItem::backgroundColor()
{
	return m_color;
}

/**
	\fn setBackgroundColor( const QColor &color)
	\brief Set the meter background color
*/
void MeterListViewItem::setBackgroundColor( const QColor &color)
{
	m_color = color;
}

/**
	\fn MeterListViewItem::paintCell ( QPainter * p, const QColorGroup & cg, int column, int width, int align )
	\brief Paint meter with the current background color
*/
void MeterListViewItem::paintCell ( QPainter * p, const QColorGroup & cg, int column, int width, int align )
{
	QColorGroup grp(cg);
        grp.setColor(QColorGroup::Base, m_color);
        QListViewItem::paintCell(p, grp, column, width, align);
}

/**
	\fn MeterListViewItem::clearColor()
	\brief Clear the current color to default widget background color
*/
void MeterListViewItem::clearColor()
{
	m_color = m_defaultcolor;
}
