/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "nmgorenetawidget.h"

#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qlistview.h>
#include <qcursor.h>
#include <qimage.h>

#include "meterlistviewitem.h"

#include "../../gui/nmghostinformationmanager.h"

//Aix son les colummnes del metersView
#define ID_COL 0
#define ADDRESS_COL 1
#define PORT_COL 2
#define DIRECTION_COL 3
#define STATUS_COL 4
#define CONNECTED_COL 5


NMGOrenetaWidget::NMGOrenetaWidget(QWidget *parent, const QString source, const QString destination, const char *name)
    :NMGOrenetaWidgetBase(parent, name),
    m_source(source), m_destination(destination),
    m_in("in.png"), m_out("out.png"), m_inout("inout.png")
{
	//Centrem la llista meters...
	metersView->setColumnAlignment(DIRECTION_COL, Qt::AlignHCenter);
	metersView->setColumnAlignment(STATUS_COL, Qt::AlignHCenter);
	metersView->setColumnAlignment(CONNECTED_COL, Qt::AlignHCenter);

	//Informaci relaitva al hosts per utilitzar Oreneta
	NMGHostInformationManager::self()->getFieldsManager().addField("Oreneta","meter","Meter Path");
	NMGHostInformationManager::self()->getFieldsManager().addField("Oreneta","port","Meter Port");
	
	//Signal and connections
	connect( removeMeterButton, SIGNAL( clicked() ), this, SLOT( deleteMeter() ) );

	//Fiquem els primers hosts :)
	metersView->clear();
	
	addMeter(1 ,m_source);
	addMeter(2 ,m_destination);
	
}

/**
	\fn NMGOrenetaWidget::addMeter()
	\brief Add new meter
*/
void NMGOrenetaWidget::addMeter()
{
	NMGHostInformationManager::self()->showHostWidget();
}

/**
	\fn NMGOrenetaWidget::addMeter( QString address)
	\brief Add new meter with default options
	\parem address IP address 
*/
void NMGOrenetaWidget::addMeter(const int &id, const QString address)
{
	MeterListViewItem *item = new MeterListViewItem(metersView);
	
	item->setText(ID_COL, QString::number(id));
	item->setText(ADDRESS_COL, address);
	/*TODO El port l'haurem d'agafar de la BD de hosts */
	item->setText(PORT_COL, "7777");

	if( m_source == address)
		item->setPixmap(DIRECTION_COL, m_out);
	else 
		item->setPixmap(DIRECTION_COL, m_in);

	item->setPixmap(STATUS_COL, QPixmap("no.png"));
	item->setPixmap(CONNECTED_COL, QPixmap("no.png"));

	//Si el host no exsisteix a la base de dades el marquem
	if (!NMGHostInformationManager::self()->existsStation(address)) {
		//Afagem un color vermell, aix s'hauria de canviar per fer-ho mes automatic
		QColor color(255,181,181);
		item->setBackgroundColor(color);
	}
	
	metersView->insertItem(item);	
}


/**
	\fn NMGOrenetaWidget::setFilter()
	\brief Change filter in analyzer
*/
void NMGOrenetaWidget::setFilter()
{
	QString msg;

	m_filter = filterEdit->text();
	if (!m_filter.isEmpty()) {
		QString msg;
		msg = "<b>Current Filter: </b>" + m_filter;
		currentFilterLabel->setText(msg);
	}
	else currentFilterLabel->setText("");
	
}

/**
	\fn NMGOrenetaWidget::showMeterMenu( QListViewItem *item, const QPoint&, int)
	\brief Show menu options for meter item
	\param item Selected meter from metersView
*/
void NMGOrenetaWidget::showMeterMenu( QListViewItem *item, const QPoint&, int)
{
	
	if (!item) return;

	QPopupMenu *menu = new QPopupMenu(this);
	menu->insertItem(QPixmap("graphic.png"), "Info Host",  this, SLOT(infoHost()) );
	menu->insertSeparator();
	
	QImage direction = (item->pixmap(DIRECTION_COL))->convertToImage();

	if (direction != m_in.convertToImage()) 
		menu->insertItem(m_in ,"In", this, SLOT(setDirectionIn()) );
	if (direction != m_out.convertToImage()) 
		menu->insertItem(m_out ,"Out", this, SLOT(setDirectionOut()) );
	if (direction != m_inout.convertToImage()) 
		menu->insertItem(m_inout ,"In-Out", this, SLOT(setDirectionInOut()) );

	menu->insertSeparator();
	menu->insertItem(QPixmap("no.png"), "Delete",  this, SLOT(deleteMeter()) );
// 	menu->insertItem( "Disable",  this, SLOT(disableMeter() ));

	menu->exec( QCursor::pos());
	
}

/**
	\fn NMGOrenetaWidget::setDirectionIn()
	\brief Change meter direction to "In" in metersView
*/
void NMGOrenetaWidget::setDirectionIn()
{
	QListViewItem *item = metersView->selectedItem();
	item->setPixmap(DIRECTION_COL, m_in);
}

/**
	\fn NMGOrenetaWidget::setDirectionOut()
	\brief Change meter direction to "Out" in metersView
*/
void NMGOrenetaWidget::setDirectionOut()
{
	QListViewItem *item = metersView->selectedItem();
	item->setPixmap(DIRECTION_COL, m_out);
}

/**
	\fn NMGOrenetaWidget::setDirectionInOut()
	\brief Change meter direction to "InOut" in metersView
*/
void NMGOrenetaWidget::setDirectionInOut()
{
	QListViewItem *item = metersView->selectedItem();
	item->setPixmap(DIRECTION_COL, m_inout);
}


/**
	\fn NMGOrenetaWidget::deleteMeter()
	\brief Delete meter from metersView
*/
void NMGOrenetaWidget::deleteMeter()
{
	QListViewItem *item = metersView->selectedItem();
	delete item;
}

/**
	\fn NMGOrenetaWidget::infoHost()
	\brief Show information about meter host from metersView
*/
void NMGOrenetaWidget::infoHost()
{
	QString address;
	QListViewItem *item = metersView->selectedItem();

	address = item->text(ADDRESS_COL);

	NMGHostInformationManager::self()->showHostWidget(address);

}

// void NMGOrenetaWidget::disableMeter()
// {
// 	QListViewItem *item = metersView->selectedItem();
// 	item->setEnabled(false);
// }




#include "nmgorenetawidget.moc"
