/***************************************************************************
 *   Copyright (C) 2005 by Rus Andrei Bogdan                                    *
 *   bogdanrus2004@yahoo.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//here are the implementations of the slots of the slots

#include "nmgsimulatorwidget.h"

#include <nmghostinformationmanager.h>
#include <iostream>
#include <config.h>


using namespace std;

//declarations

NMGStation * srcStationObj, * destStationObj, * nsStationObj;
NMGHost srcHostObj, destHostObj, nsHostObj;


NMGSimulatorWidget::NMGSimulatorWidget(NMGISender * s, QWidget *parent, const char *name)
 : NMGSimulatorWidgetBase(parent, name)
{
	
sender=s;
//configuring the data types from the input fields

//for delay tab
delayDelay->setValidator( new QDoubleValidator(.0001,99999,4,delayDelay) );
delayJitter->setValidator( new QDoubleValidator(.0001,99999,4,delayJitter));


//for rate tab
rateRate->setValidator( new QDoubleValidator(.001,99999,4,rateRate) );
//rateMaxBurst->setValidator( new QDoubleValidator(.001,99999,4,rateMaxBurst));
rateBurst->setValidator( new QIntValidator(0,9999,rateBurst));

oldName="Flow";

root=doc.createElement("NetworkSimulatorSettings");
router=doc.createElement("router");
settings=doc.createElement("settings");
	doc.appendChild(root);
	root.appendChild(router);
	root.appendChild(settings);
}



NMGSimulatorWidget::~NMGSimulatorWidget()
{
}



void NMGSimulatorWidget::addSource()
{
	srcAddress->setAlignment(AlignRight);
	//getting the station and the host objects
	srcStationObj=NMGHostInformationManager::self()->getStation(srcHost->getStationAddressFromCombo());
	srcHostObj=srcStationObj->getTestHost();
	srcAddress->setText(srcHostObj.getAddress());
	srcAlias->setText(srcHostObj.getAlias());
	srcAlias2->setAlignment(AlignCenter);
	srcAlias2->setText(srcHostObj.getAlias());

} 





void NMGSimulatorWidget::addNS()
{
	nsAddress->setAlignment(AlignRight);
	nsStationObj=NMGHostInformationManager::self()->getStation(nsHost->getStationAddressFromCombo());

	nsHostObj=nsStationObj->getManagementHost();
	nsAddress->setText(nsHostObj.getAddress());
		
	nsUser->setAlignment(AlignRight);
	nsUser->setText(nsStationObj->getManagementUsername());
	
	nsPort->setAlignment(AlignRight);
	port.sprintf("%d",nsStationObj->getManagementAddressPort()); //convert the integer value to string
	nsPort->setText(port);

	nsHostObj=nsStationObj->getTestHost();
	nsAlias->setText(nsHostObj.getAlias());
	nsAlias2->setAlignment(AlignCenter);
	nsAlias2->setText(nsHostObj.getAlias());
	connectButton->setEnabled(1);
	srcInterface->setEnabled(0);
	pixmapLabel4->setEnabled(0);
	destInterface->setEnabled(0);
	applyButton->setEnabled(0);
	saveButton->setEnabled(0);
	loadButton->setEnabled(0);
	srcInterface->clear();
	srcInterface->insertItem("Any",-1);
	destInterface->clear();
	destInterface->insertItem("Any",-1);
        
} 


void NMGSimulatorWidget::addDestination()
{
	destAddress->setAlignment(AlignRight);
	destStationObj=NMGHostInformationManager::self()->getStation(destHost->getStationAddressFromCombo());

	destHostObj=destStationObj->getTestHost();
	destAddress->setText(destHostObj.getAddress());
	destAlias->setText(destHostObj.getAlias());
	destAlias2->setAlignment(AlignCenter);
	destAlias2->setText(destHostObj.getAlias());
        
} 



// Slot for adding a setting in the list and into the xml object
void NMGSimulatorWidget::addFlowSlot()
{
int i=0;
QListViewItem * searchedItem;
QString nr, name, radioDelay, radioDD;


//verify if the name is allready used
name.append(flowName->text());
searchedItem=flowList->findItem(flowName->text(),0);
while (searchedItem!=0)
{
	i++;
	name="";
	name.append(flowName->text());
	nr.sprintf(" %d",i);
	name.append(nr);
	searchedItem=flowList->findItem(name,0);
}
flowName->setText(name);

//create a a flow item
QListViewItem * flowItem = new QListViewItem(flowList,flowName->text());

flowItem->setSelected(TRUE);
flowList->setCurrentItem(flowItem);


//constructing the element for the xml file
QDomElement flow=doc.createElement("Flow");
	QDomElement source=doc.createElement("Source");
	QDomElement destination=doc.createElement("Destination");
	QDomElement protocols=doc.createElement("Protocols");
	QDomElement delayTab=doc.createElement("DelayTab");
	QDomElement DDTab=doc.createElement("DDTab");
	QDomElement rateTab=doc.createElement("RateTab");


settings.appendChild(flow);
flow.setAttribute("name", flowName->text());
oldName=flowName->text();

	flow.appendChild(source);
	source.setAttribute("srcAlias",srcAlias->text());
	source.setAttribute("srcAddress",srcAddress->text());
	source.setAttribute("srcMask",srcMask->text());
	source.setAttribute("srcPort",srcPort->text());
	source.setAttribute("srcInterface",srcInterface->currentItem());

	flow.appendChild(destination);
	destination.setAttribute("destAlias",destAlias->text());
	destination.setAttribute("destAddress",destAddress->text());
	destination.setAttribute("destMask",destMask->text());
	destination.setAttribute("destPort",destPort->text());
	destination.setAttribute("destInterface",destInterface->currentItem());

	flow.appendChild(protocols);
	protocols.setAttribute("l4Protocol",l4Protocol->currentItem());
	protocols.setAttribute("l3Protocol",l3Protocol->currentItem());

	flow.appendChild(delayTab);
	delayTab.setAttribute("delayDelay",delayDelay->text());
	delayTab.setAttribute("delayJitter",delayJitter->text());
	radioDelay=returnDelayGroup();
	if (radioDelay=="random")  { 
					delayTab.setAttribute("radioDelay","random"); 
					delayTab.setAttribute("radioDelayValue","random");
					}
	else if (radioDelay=="correlation") {  
						delayTab.setAttribute("radioDelayValue",delayCorrelation->text());
						delayTab.setAttribute("radioDelay","correlation"); 
						}
	     else  {  
			delayTab.setAttribute("radioDelayValue",delayDistribution->currentItem());
			delayTab.setAttribute("radioDelay","distribution"); 
			}
	
	

	flow.appendChild(DDTab);
	radioDD=returnDDGroup();
	if (radioDD=="none")  { 
					DDTab.setAttribute("radioDD","none"); 
					DDTab.setAttribute("DD","none"); 
					DDTab.setAttribute("DDCorrelation","none");
					}
	else if (radioDD=="drop") {  
						DDTab.setAttribute("radioDD","drop"); 
						DDTab.setAttribute("DD",DD->text());
						DDTab.setAttribute("DDCorrelation",DDCorrelation->text()); 
						}
	     else  {  
			DDTab.setAttribute("radioDD","duplicate"); 
			DDTab.setAttribute("DD",DD->text());
			DDTab.setAttribute("DDCorrelation",DDCorrelation->text()); 
			}

	flow.appendChild(rateTab);
	rateTab.setAttribute("rateRate",rateRate->text());
	//rateTab.setAttribute("rateMaxBurst",rateMaxBurst->text());
	rateTab.setAttribute("rateBurst",rateBurst->text());


} 


void NMGSimulatorWidget::showFlowSlot(QListViewItem * item)
{
	
int out, val;
bool ok;
QString value, newName, radioDelay, radioDD;
QListViewItem * searchedItem;

newName=flowName->text();
if ((flowList->currentItem()!=0)&(item!=0)) { 
	
//saving the values again for the eventuality that there war some changes	
					
	QDomNode oldflow = settings.firstChild();
	out=0;
	//searching the item in the xml object
	while ((!settings.isNull())&(out==0)) {
	if (oldflow.toElement().attribute("name") != oldName)
		{
		oldflow = oldflow.nextSibling();
		}
	else { out=1;}
	}

	QDomElement newflow=doc.createElement("Flow");
	QDomElement source1=doc.createElement("Source");
	QDomElement destination1=doc.createElement("Destination");
	QDomElement protocols1=doc.createElement("Protocols");
	QDomElement delayTab1=doc.createElement("DelayTab");
	QDomElement DDTab1=doc.createElement("DDTab");
	QDomElement rateTab1=doc.createElement("RateTab");


	settings.replaceChild(newflow, oldflow);
        newflow.setAttribute("name", newName);

	newflow.appendChild(source1);
	source1.setAttribute("srcAlias",srcAlias->text());
	source1.setAttribute("srcAddress",srcAddress->text());
	source1.setAttribute("srcMask",srcMask->text());
	source1.setAttribute("srcPort",srcPort->text());
	source1.setAttribute("srcInterface",srcInterface->currentItem());

	newflow.appendChild(destination1);
	destination1.setAttribute("destAlias",destAlias->text());
	destination1.setAttribute("destAddress",destAddress->text());
	destination1.setAttribute("destMask",destMask->text());
	destination1.setAttribute("destPort",destPort->text());
	destination1.setAttribute("destInterface",destInterface->currentItem());

	newflow.appendChild(protocols1);
	protocols1.setAttribute("l4Protocol",l4Protocol->currentItem());
	protocols1.setAttribute("l3Protocol",l3Protocol->currentItem());

	newflow.appendChild(delayTab1);
	delayTab1.setAttribute("delayDelay",delayDelay->text());
	delayTab1.setAttribute("delayJitter",delayJitter->text());

	radioDelay=returnDelayGroup();
	if (radioDelay=="random")  { 
					delayTab1.setAttribute("radioDelay","random"); 
					delayTab1.setAttribute("radioDelayValue","random");
					}
	else if (radioDelay=="correlation") {  
						delayTab1.setAttribute("radioDelayValue",delayCorrelation->text());
						delayTab1.setAttribute("radioDelay","correlation"); 
						}
	     else  {  
			delayTab1.setAttribute("radioDelayValue",delayDistribution->currentItem());
			delayTab1.setAttribute("radioDelay","distribution"); 
			}



	newflow.appendChild(DDTab1);
	radioDD=returnDDGroup();
	if (radioDD=="none")  { 
					DDTab1.setAttribute("radioDD","none"); 
					DDTab1.setAttribute("DD","none"); 
					DDTab1.setAttribute("DDCorrelation","none");
					}
	else if (radioDD=="drop") {  
						DDTab1.setAttribute("radioDD","drop"); 
						DDTab1.setAttribute("DD",DD->text());
						DDTab1.setAttribute("DDCorrelation",DDCorrelation->text()); 
						}
	     else  {  
			DDTab1.setAttribute("radioDD","duplicate"); 
			DDTab1.setAttribute("DD",DD->text());
			DDTab1.setAttribute("DDCorrelation",DDCorrelation->text()); 
			}

	newflow.appendChild(rateTab1);
	rateTab1.setAttribute("rateRate",rateRate->text());
	//rateTab1.setAttribute("rateMaxBurst",rateMaxBurst->text());
	rateTab1.setAttribute("rateBurst",rateBurst->text());	

	if (newName!=oldName) //chech if the item name was changed
				{
				searchedItem=flowList->findItem(oldName,0);
				if (searchedItem!=0) {
				flowList->takeItem(searchedItem);

				QListViewItem * newItem = new QListViewItem(flowList,newName);

				newItem->setSelected(TRUE);
				flowList->setCurrentItem(newItem);
				oldName=newItem->text(0);

					}
				} 
	else {

	flowName->setText(item->text(0));
	oldName=flowName->text();

	//show the values for the rest of the fields
	QDomNode node = settings.firstChild();
	out=0;
	//searching the item
	while ((!settings.isNull())&(out==0)) {
	if (node.toElement().attribute("name") != oldName)
		{
		node = node.nextSibling();
		}
	else { out=1;}
	}

	QDomNode source=node.firstChild();
	QDomNode destination=source.nextSibling();
	QDomNode protocols=destination.nextSibling();
	QDomNode delayTab=protocols.nextSibling();
	QDomNode DDTab=delayTab.nextSibling();
	QDomNode rateTab=DDTab.nextSibling();

	//input datas in source tab
	srcAlias->setText(source.toElement().attribute("srcAlias"));
	srcAlias2->setText(source.toElement().attribute("srcAlias"));
	srcAddress->setText(source.toElement().attribute("srcAddress"));
	value=source.toElement().attribute("srcMask");
	val=value.toInt(&ok, 10);
	srcMask->setValue(val);
	value=source.toElement().attribute("srcPort");
	val=value.toInt(&ok, 10);
	srcPort->setValue(val);
	value=source.toElement().attribute("srcInterface");
	val=value.toInt(&ok, 10);
	srcInterface->setCurrentItem(val);

	//input datas in destination tab
	destAlias->setText(destination.toElement().attribute("destAlias"));
	destAlias2->setText(destination.toElement().attribute("destAlias"));
	destAddress->setText(destination.toElement().attribute("destAddress"));
	value=destination.toElement().attribute("destMask");
	val=value.toInt(&ok, 10);
	destMask->setValue(val);
	value=destination.toElement().attribute("destPort");
	val=value.toInt(&ok, 10);
	destPort->setValue(val);
	value=destination.toElement().attribute("destInterface");
	val=value.toInt(&ok, 10);
	destInterface->setCurrentItem(val);

	//input datas in protocols tab
	value=protocols.toElement().attribute("l4Protocol");
	val=value.toInt(&ok, 10);
	l4Protocol->setCurrentItem(val);
	l4ProtocolChanged(l4Protocol->text(val));
	value=protocols.toElement().attribute("l3Protocol");
	val=value.toInt(&ok, 10);
	l3Protocol->setCurrentItem(val);

	//show datas in delay tab
	delayDelay->setText(delayTab.toElement().attribute("delayDelay"));
	delayJitter->setText(delayTab.toElement().attribute("delayJitter"));

	radioDelay=delayTab.toElement().attribute("radioDelay");
	if (radioDelay=="random")  { 	delayGroupChanged(radioDelay);  
					delayCorrelation->setValue(0);
					delayDistribution->setCurrentItem(0);
					}
	else  if (radioDelay=="correlation") {
						delayGroupChanged(radioDelay); 
						value=delayTab.toElement().attribute("radioDelayValue");
						val=value.toInt(&ok, 10);
						delayCorrelation->setValue(val);
						delayDistribution->setCurrentItem(0); 
						}
	     else  {  
			delayGroupChanged(radioDelay); 
			delayCorrelation->setValue(0);
			value=delayTab.toElement().attribute("radioDelayValue");
			val=value.toInt(&ok, 10);
			delayDistribution->setCurrentItem(val); 
			} 
		


	

	//input datas in DD tab
	radioDD=DDTab.toElement().attribute("radioDD");
	if (radioDD=="none")  { 	dDGroupChanged(radioDD); 
					DD->setValue(0);
					DDCorrelation->setValue(0);
					}
	else  if (radioDD=="drop") {  
						dDGroupChanged(radioDD);
						value=DDTab.toElement().attribute("DD");
						val=value.toInt(&ok, 10);
						DD->setValue(val);
						value=DDTab.toElement().attribute("DDCorrelation");
						val=value.toInt(&ok, 10);
						DDCorrelation->setValue(val);
						}
	     else  {  
			dDGroupChanged(radioDD);
			value=DDTab.toElement().attribute("DD");
			val=value.toInt(&ok, 10);
			DD->setValue(val);
			value=DDTab.toElement().attribute("DDCorrelation");
			val=value.toInt(&ok, 10);
			DDCorrelation->setValue(val);
			} 


	//input datas in rate tab
	rateRate->setText(rateTab.toElement().attribute("rateRate"));
	//rateMaxBurst->setText(rateTab.toElement().attribute("rateMaxBurst"));
	rateBurst->setText(rateTab.toElement().attribute("rateBurst"));

	}
	
 }//from the first if in the function
  
} 


void NMGSimulatorWidget::showItem(QListViewItem * itemShow)
{
int out, val;
bool ok;
QString value, radioDelay, radioDD;

	flowName->setText(itemShow->text(0));
	//show the values for the rest of the fields
	QDomNode node = settings.firstChild();
	out=0;
	//searching the item
	while ((!settings.isNull())&(out==0)) {
	if (node.toElement().attribute("name") != oldName)
		{
		node = node.nextSibling();
		}
	else { out=1;}
	}
	
	QDomNode source=node.firstChild();
	QDomNode destination=source.nextSibling();
	QDomNode protocols=destination.nextSibling();
	QDomNode delayTab=protocols.nextSibling();
	QDomNode DDTab=delayTab.nextSibling();
	QDomNode rateTab=DDTab.nextSibling();

	//input datas in source tab
	srcAlias->setText(source.toElement().attribute("srcAlias"));
	srcAlias2->setText(source.toElement().attribute("srcAlias"));
	srcAddress->setText(source.toElement().attribute("srcAddress"));
	value=source.toElement().attribute("srcMask");
	val=value.toInt(&ok, 10);
	srcMask->setValue(val);
	value=source.toElement().attribute("srcPort");
	val=value.toInt(&ok, 10);
	srcPort->setValue(val);
	value=source.toElement().attribute("srcInterface");
	val=value.toInt(&ok, 10);
	srcInterface->setCurrentItem(val);

	//input datas in destination tab
	destAlias->setText(destination.toElement().attribute("destAlias"));
	destAlias2->setText(destination.toElement().attribute("destAlias"));
	destAddress->setText(destination.toElement().attribute("destAddress"));
	value=destination.toElement().attribute("destMask");
	val=value.toInt(&ok, 10);
	destMask->setValue(val);
	value=destination.toElement().attribute("destPort");
	val=value.toInt(&ok, 10);
	destPort->setValue(val);
	value=destination.toElement().attribute("destInterface");
	val=value.toInt(&ok, 10);
	destInterface->setCurrentItem(val);

	//input datas in protocols tab
	value=protocols.toElement().attribute("l4Protocol");
	val=value.toInt(&ok, 10);
	l4Protocol->setCurrentItem(val);
	l4ProtocolChanged(l4Protocol->text(val));
	value=protocols.toElement().attribute("l3Protocol");
	val=value.toInt(&ok, 10);
	l3Protocol->setCurrentItem(val);

	//input datas in delay tab
	delayDelay->setText(delayTab.toElement().attribute("delayDelay"));
	delayJitter->setText(delayTab.toElement().attribute("delayJitter"));

	radioDelay=delayTab.toElement().attribute("radioDelay");
	if (radioDelay=="random")  { 	delayGroupChanged(radioDelay);  
					delayCorrelation->setValue(0);
					delayDistribution->setCurrentItem(0);
					}
	else  if (radioDelay=="correlation") {  
						delayGroupChanged(radioDelay); 
						value=delayTab.toElement().attribute("radioDelayValue");
						val=value.toInt(&ok, 10);
						delayCorrelation->setValue(val);
						delayDistribution->setCurrentItem(0); 
						}
	     else  {  
			delayGroupChanged(radioDelay); 
			delayCorrelation->setValue(0);
			value=delayTab.toElement().attribute("radioDelayValue");
			val=value.toInt(&ok, 10);
			delayDistribution->setCurrentItem(val); 
			} 


	//input datas in DD tab
	radioDD=DDTab.toElement().attribute("radioDD");
	if (radioDD=="none")  { 	dDGroupChanged(radioDD); 
					DD->setValue(0);
					DDCorrelation->setValue(0);
					}
	else  if (radioDD=="drop") {  		
						dDGroupChanged(radioDD);
						value=DDTab.toElement().attribute("DD");
						val=value.toInt(&ok, 10);
						DD->setValue(val);
						value=DDTab.toElement().attribute("DDCorrelation");
						val=value.toInt(&ok, 10);
						DDCorrelation->setValue(val);
						}
	     else  {  
			dDGroupChanged(radioDD);
			value=DDTab.toElement().attribute("DD");
			val=value.toInt(&ok, 10);
			DD->setValue(val);
			value=DDTab.toElement().attribute("DDCorrelation");
			val=value.toInt(&ok, 10);
			DDCorrelation->setValue(val);
			} 

	//input datas in drop tab
	rateRate->setText(rateTab.toElement().attribute("rateRate"));
	//rateMaxBurst->setText(rateTab.toElement().attribute("rateMaxBurst"));
	rateBurst->setText(rateTab.toElement().attribute("rateBurst"));
}

//delete flow setting
void NMGSimulatorWidget::delFlowSlot()
{
int out;
QListViewItem * delItem, * beforeItem, * nextItem;

	delItem=flowList->selectedItem();
	if((flowList->childCount()==0)|((delItem==0)))  {flowName->setText("Flow"); oldName="Flow";}
	else
	{
	//delete item from xml object
	QDomNode node = settings.firstChild();
	
	//searching the item by the flow name
	out=0;
	while ((!settings.isNull())&(out==0)) {
	if (node.toElement().attribute("name") != oldName)
		{
		node = node.nextSibling();
		}
	else { out=1;}
						}
	
	//removing the item from xml object
	settings.removeChild(node); 
	
	beforeItem=delItem->itemAbove();
	nextItem=delItem->nextSibling();
	//delete item from flowList
	flowList->takeItem(delItem);

	if (beforeItem!=0 ) 	{
				beforeItem->setSelected(TRUE);
				flowList->setCurrentItem(beforeItem);
				oldName=beforeItem->text(0);
				showItem(beforeItem);
				}
	else			{
				if (nextItem==0) {flowName->setText("Flow"); oldName="Flow"; clearSlot();}
				else {
				nextItem->setSelected(TRUE);
				flowList->setCurrentItem(nextItem);
				oldName=nextItem->text(0);
				showItem(nextItem);
				}
				}
	}
} 


void NMGSimulatorWidget::clearSlot()
{

	flowName->setText(oldName);

	//interfaces
	srcInterface->setCurrentItem(0);
	destInterface->setCurrentItem(0);

	//source tab
	srcMask->setValue(32);
	srcPort->setValue(0);
	srcAddress->setText("0.0.0.0");
	srcAlias->setText(". . .");
	srcAlias2->setText(". . .");
	
	//destination tab
	destMask->setValue(32);
	destPort->setValue(0);
	destAddress->setText("0.0.0.0");
	destAlias->setText(". . .");
	destAlias2->setText(". . .");

	//protocol tab
	l4Protocol->setCurrentItem(0);
	l3Protocol->setCurrentItem(0);

	//delay tab
	delayDelay->clear();
	delayJitter->clear();
	delayCorrelation->setValue(0);
	delayDistribution->setCurrentItem(0);
	delayGroupChanged("random");

	//DD tab
	DD->setValue(0);
	DDCorrelation->setValue(0);
	dDGroupChanged("none");

	//rate tab
	rateRate->clear();
	//rateMaxBurst->clear();
	rateBurst->clear();

}


void NMGSimulatorWidget::saveSettingsSlot()
{
int outCheck;
QString newName, radioDelay, radioDD;
QListViewItem * searchedItem;

   if (flowList->childCount() == 0){
	QMessageBox* error = new QMessageBox;
    	error->setCaption("Error");
    	error->setText("There are no flows in the list \nIn order to save, you have to add flows.");
    	error->setIcon(QMessageBox::Information);
    	error->setButtonText(0, "Ok");
    	error->exec();
    	delete error; 
	return;
   }


//saving current flow settings in xml object before saving the file

newName=flowName->text();
QDomNode oldflow = settings.firstChild();
	outCheck=0;
	//searching the item in the xml object
	while ((!settings.isNull())&(outCheck==0)) {
	if (oldflow.toElement().attribute("name") != oldName)
		{
		oldflow = oldflow.nextSibling();
		}
	else { outCheck=1;}
	}

	QDomElement newflow=doc.createElement("Flow");
	QDomElement source1=doc.createElement("Source");
	QDomElement destination1=doc.createElement("Destination");
	QDomElement protocols1=doc.createElement("Protocols");
	QDomElement delayTab1=doc.createElement("DelayTab");
	QDomElement DDTab1=doc.createElement("DDTab");
	QDomElement rateTab1=doc.createElement("RateTab");


	settings.replaceChild(newflow, oldflow);
        newflow.setAttribute("name", newName);

	newflow.appendChild(source1);
	source1.setAttribute("srcAlias",srcAlias->text());
	source1.setAttribute("srcAddress",srcAddress->text());
	source1.setAttribute("srcMask",srcMask->text());
	source1.setAttribute("srcPort",srcPort->text());
	source1.setAttribute("srcInterface",srcInterface->currentItem());

	newflow.appendChild(destination1);
	destination1.setAttribute("destAlias",destAlias->text());
	destination1.setAttribute("destAddress",destAddress->text());
	destination1.setAttribute("destMask",destMask->text());
	destination1.setAttribute("destPort",destPort->text());
	destination1.setAttribute("destInterface",destInterface->currentItem());

	newflow.appendChild(protocols1);
	protocols1.setAttribute("l4Protocol",l4Protocol->currentItem());
	protocols1.setAttribute("l3Protocol",l3Protocol->currentItem());

	newflow.appendChild(delayTab1);
	delayTab1.setAttribute("delayDelay",delayDelay->text());
	delayTab1.setAttribute("delayJitter",delayJitter->text());
	
	radioDelay=returnDelayGroup();
	if (radioDelay=="random")  { 
					delayTab1.setAttribute("radioDelay","random"); 
					delayTab1.setAttribute("radioDelayValue","random");
					}
	else if (radioDelay=="correlation") {  
						delayTab1.setAttribute("radioDelayValue",delayCorrelation->text());
						delayTab1.setAttribute("radioDelay","correlation"); 
						}
	     else  {  
			delayTab1.setAttribute("radioDelayValue",delayDistribution->currentItem());
			delayTab1.setAttribute("radioDelay","distribution"); 
			}



	newflow.appendChild(DDTab1);
	radioDD=returnDDGroup();
	if (radioDD=="none")  { 
					DDTab1.setAttribute("radioDD","none"); 
					DDTab1.setAttribute("DD","none"); 
					DDTab1.setAttribute("DDCorrelation","none");
					}
	else if (radioDD=="drop") {  
						DDTab1.setAttribute("radioDD","drop"); 
						DDTab1.setAttribute("DD",DD->text());
						DDTab1.setAttribute("DDCorrelation",DDCorrelation->text()); 
						}
	     else  {  
			DDTab1.setAttribute("radioDD","duplicate"); 
			DDTab1.setAttribute("DD",DD->text());
			DDTab1.setAttribute("DDCorrelation",DDCorrelation->text()); 
			}

	newflow.appendChild(rateTab1);
	rateTab1.setAttribute("rateRate",rateRate->text());
	//rateTab1.setAttribute("rateMaxBurst",rateMaxBurst->text());
	rateTab1.setAttribute("rateBurst",rateBurst->text());	

	if (newName!=oldName) //chech if the item name was changed
				{
				searchedItem=flowList->findItem(oldName,0);
				if (searchedItem!=0) {
				flowList->takeItem(searchedItem);

				QListViewItem * newItem = new QListViewItem(flowList,newName);

				newItem->setSelected(TRUE);
				flowList->setCurrentItem(newItem);
				oldName=newItem->text(0);

					}
				} 
//finish saving the settings

//selecting the file to be saved

 QString fileName = QFileDialog::getSaveFileName(QString::null, "settings (*.xml)", 		this, "Network Simulator --Save flows settings" );


if (fileName==0) { return; //if there is no fille chosen do nothing 
			}

QFile file(fileName);
QTextStream out(&file);
const int Indent=4;

	router.setAttribute("nsAddress",nsAddress->text());
	file.open(IO_WriteOnly);
	doc.save(out, Indent);



}

void NMGSimulatorWidget::loadSettingsSlot()
{
QString fileName;
QListViewItem * flowItem;



//getting the fille to be opened

fileName = QFileDialog::getOpenFileName(QString::null, "settings (*.xml)", this,"Chose a configuratin fille to be opened" );

if (fileName==0) { return; //if there is no fille chosen do nothing 
			}
//inporting the settings
	 //opening the xml fille
	QFile file(fileName);
	if(!file.open(IO_ReadOnly)) {
					QMessageBox* error = new QMessageBox;
    					error->setCaption("Error");
    					error->setText("The fille that you are trying to open is corrupt \nChose another fille.");
    					error->setIcon(QMessageBox::Information);
    					error->setButtonText(0, "Ok");
    					error->exec();
    					delete error; 
					return;
					}
	//erasing the old settings
	flowList->clear();
	doc.removeChild(root);

	//creating the xml ducument object
	if(!doc.setContent( &file )) {
					QMessageBox* error = new QMessageBox;
    					error->setCaption("Error 2");
    					error->setText("The fille that you are trying to open is corrupt \nChose another fille.");
    					error->setIcon(QMessageBox::Information);
    					error->setButtonText(0, "Ok");
    					error->exec();
    					delete error; 
					return;
					file.close();  //closing the xml file
					}

	file.close();  //closing the xml file

	//getting the rest of the elements
	root=doc.documentElement();
	router=root.firstChild().toElement();
	settings=router.nextSibling().toElement();

	//vriting the ns settings
	nsAddress->setAlignment(AlignRight);
	nsStationObj=NMGHostInformationManager::self()->getStation(router.attribute("nsAddress"));

	if (nsStationObj==0)  { 
				cout<<"*** There is no station vith ip address "<<router.attribute("nsAddress")<<endl; 
				nsAddress->setText("0.0.0.0");
				nsUser->setText(" ");
				nsPort->setText("22");
				nsAlias2->setText(". . .");
				nsAlias->setText(". . .");
				}
	else {
	nsHostObj=nsStationObj->getManagementHost();  
	nsAddress->setText(nsHostObj.getAddress());

	nsUser->setAlignment(AlignRight);
	nsUser->setText(nsStationObj->getManagementUsername());
	
	nsPort->setAlignment(AlignRight);
	port.sprintf("%d",nsStationObj->getManagementAddressPort()); //convert the integer value to string
	nsPort->setText(port);

	nsHostObj=nsStationObj->getTestHost();
	nsAlias->setText(nsHostObj.getAlias());
	nsAlias2->setText(nsHostObj.getAlias());
	}

	clearSlot();

	//listing the settings in tho flowList
	QDomNode node = settings.firstChild();
	//getting all the settings names
	while (!settings.isNull()) {
	if ( node.isElement() && node.nodeName() == "Flow" ) {
	flowItem = new QListViewItem(flowList,node.toElement().attribute("name"));
        }
	else { break; }
	node = node.nextSibling();
	}


	flowItem=flowList->firstChild();
	oldName=flowItem->text(0);
	flowItem->setSelected(TRUE);
	flowList->setCurrentItem(flowItem);
	showItem(flowItem);

}


void NMGSimulatorWidget::connectSlot()
{

	if (nsAddress->text()=="0.0.0.0"){
	QMessageBox* error = new QMessageBox;
    	error->setCaption("Error");
    	error->setText("You have to chose a router where to connect.");
    	error->setIcon(QMessageBox::Information);
    	error->setButtonText(0, "Ok");
    	error->exec();
    	delete error; 
	return;
   }

	connectButton->setEnabled(0);
	QString nsAddr = nsAddress->text();
	QString nsUsr = nsUser->text();
	QString nsPass = nsStationObj->getManagementPassword();
	QString nsPortValue = nsPort->text();	
	QString nsProtocol;

	if (nsHostObj.isHostIPv4()) {nsProtocol="IPv4";}
	else {nsProtocol="IPv6";}

	QString xmlMessage = "<data> <nsAddress>"+nsAddr+"</nsAddress> <nsUsername>"+nsUsr+"</nsUsername> <nsPassword>"+nsPass+"</nsPassword> <nsPort>"+nsPortValue+"</nsPort> <nsProtocol>"+nsProtocol+"</nsProtocol> </data>";
	
	if (sender->sendCommand(Start,xmlMessage)) //sending the command
	{
	progressItem = 0;
	progressItem = ProgressManager::createProgressItem("Network Simulator Progress:"); //creating the progressBar
	progressItem->setTotalItems(4);
	progressItem->setStatus("Connecting to the host router . . .");
	}
	else  { 
		connectButton->setEnabled(1);
		srcInterface->setEnabled(0);
		pixmapLabel4->setEnabled(0);
		destInterface->setEnabled(0);
		applyButton->setEnabled(0);
		saveButton->setEnabled(0);
		loadButton->setEnabled(0);
		srcInterface->clear();
		srcInterface->insertItem("Any",-1);
		destInterface->clear();
		destInterface->insertItem("Any",-1);
		MessageBar::longMessage("Command not started\nThere is no connection with the server!");
		}
}



void NMGSimulatorWidget::applySettingsSlot()
{
QString tag;
int index, interfNumber;

if (flowList->childCount() == 0){
	QMessageBox* error = new QMessageBox;
    	error->setCaption("Error");
    	error->setText("There are no flows in the list \nIn order to apply settins, you need to create settings.");
    	error->setIcon(QMessageBox::Information);
    	error->setButtonText(0, "Ok");
    	error->exec();
    	delete error; 
	return;
   }

	QString nsAddr = nsAddress->text();
	QString nsUsr = nsUser->text();
	QString nsPass = nsStationObj->getManagementPassword();
	QString nsPortValue = nsPort->text();	
	QString nsProtocol;

	if (nsHostObj.isHostIPv4()) {nsProtocol="IPv4";}
	else {nsProtocol="IPv6";}

	QString xmlMessage = "<data> <address>"+nsAddr+"</address> <nsUsername>"+nsUsr+"</nsUsername> <nsPassword>"+nsPass+"</nsPassword> <nsPort>"+nsPortValue+"</nsPort> <nsProtocol>"+nsProtocol+"</nsProtocol>"; 	
	xmlMessage.append("<cleanInterf>");//sending the interfaces for witch we have settings
	interfNumber=destInterface->count();
	
	for (index=1;index<interfNumber;index++)
	{
		tag.sprintf("interface%d",index);
		xmlMessage.append("<"+tag+">");
		xmlMessage.append(destInterface->text(index));
		xmlMessage.append("</"+tag+">");
	}

	tag.sprintf("%d",interfNumber-1);
	xmlMessage.append("<interfaceNr>"+tag+"</interfaceNr>");
	xmlMessage.append("</cleanInterf>");
	xmlMessage.append("</data>");//ending the data tree

	if (sender->sendCommand(Start,xmlMessage))//sending the interfaces data
	{
	//starting the progres barr
	progressItem = 0;
	progressItem = ProgressManager::createProgressItem("Network Simulator Progress:"); //creating the progressBar
	progressItem->setTotalItems(flowList->childCount()+10);
	progressItem->incCompletedItems();//incrementing the progressBar
	progressItem->updateProgress();
	progressItem->setStatus("Settings are being send . . .");

	//inactivating the buttons
	loadButton->setEnabled(0);
	saveButton->setEnabled(0);
	applyButton->setEnabled(0);
	delFlowButton->setEnabled(0);
	clearButton->setEnabled(0);
	addFlowButton->setEnabled(0);
	}
	else  { 
		connectButton->setEnabled(1);
		srcInterface->setEnabled(0);
		pixmapLabel4->setEnabled(0);
		destInterface->setEnabled(0);
		applyButton->setEnabled(0);
		saveButton->setEnabled(0);
		loadButton->setEnabled(0);
		srcInterface->clear();
		srcInterface->insertItem("Any",-1);
		destInterface->clear();
		destInterface->insertItem("Any",-1);
		MessageBar::longMessage("Setting not send\nThere is no connection with the server!");
		}


}

void NMGSimulatorWidget::sendSettingsSlot()
{
bool ok, interfTable[20], exit;
int index, i, interfNumber, outCheck, setNumber, val;
QString tag, command, newName, value, radioDelay, radioDD;
QListViewItem * searchedItem;
QDomNode node, source, destination;


//saving current flow settings in xml object before saving the file

newName=flowName->text();
QDomNode oldflow = settings.firstChild();
	outCheck=0;
	//searching the item in the xml object
	while ((!settings.isNull())&(outCheck==0)) {
	if (oldflow.toElement().attribute("name") != oldName)
		{
		oldflow = oldflow.nextSibling();
		}
	else { outCheck=1;}
	}

	QDomElement newflow=doc.createElement("Flow");
	QDomElement source1=doc.createElement("Source");
	QDomElement destination1=doc.createElement("Destination");
	QDomElement protocols1=doc.createElement("Protocols");
	QDomElement delayTab1=doc.createElement("DelayTab");
	QDomElement DDTab1=doc.createElement("DDTab");
	QDomElement rateTab1=doc.createElement("RateTab");


	settings.replaceChild(newflow, oldflow);
        newflow.setAttribute("name", newName);

	newflow.appendChild(source1);
	source1.setAttribute("srcAlias",srcAlias->text());
	source1.setAttribute("srcAddress",srcAddress->text());
	source1.setAttribute("srcMask",srcMask->text());
	source1.setAttribute("srcPort",srcPort->text());
	source1.setAttribute("srcInterface",srcInterface->currentItem());

	newflow.appendChild(destination1);
	destination1.setAttribute("destAlias",destAlias->text());
	destination1.setAttribute("destAddress",destAddress->text());
	destination1.setAttribute("destMask",destMask->text());
	destination1.setAttribute("destPort",destPort->text());
	destination1.setAttribute("destInterface",destInterface->currentItem());

	newflow.appendChild(protocols1);
	protocols1.setAttribute("l4Protocol",l4Protocol->currentItem());
	protocols1.setAttribute("l3Protocol",l3Protocol->currentItem());

	newflow.appendChild(delayTab1);
	delayTab1.setAttribute("delayDelay",delayDelay->text());
	delayTab1.setAttribute("delayJitter",delayJitter->text());
	
	radioDelay=returnDelayGroup();
	if (radioDelay=="random")  { 
					delayTab1.setAttribute("radioDelay","random"); 
					delayTab1.setAttribute("radioDelayValue","random");
					}
	else if (radioDelay=="correlation") {  
						delayTab1.setAttribute("radioDelayValue",delayCorrelation->text());
						delayTab1.setAttribute("radioDelay","correlation"); 
						}
	     else  {  
			delayTab1.setAttribute("radioDelayValue",delayDistribution->currentItem());
			delayTab1.setAttribute("radioDelay","distribution"); 
			}

	newflow.appendChild(DDTab1);
	radioDD=returnDDGroup();
	if (radioDD=="none")  { 
					DDTab1.setAttribute("radioDD","none"); 
					DDTab1.setAttribute("DD","none"); 
					DDTab1.setAttribute("DDCorrelation","none");
					}
	else if (radioDD=="drop") {  
						DDTab1.setAttribute("radioDD","drop"); 
						DDTab1.setAttribute("DD",DD->text());
						DDTab1.setAttribute("DDCorrelation",DDCorrelation->text()); 
						}
	     else  {  
			DDTab1.setAttribute("radioDD","duplicate"); 
			DDTab1.setAttribute("DD",DD->text());
			DDTab1.setAttribute("DDCorrelation",DDCorrelation->text()); 
			}

	newflow.appendChild(rateTab1);
	rateTab1.setAttribute("rateRate",rateRate->text());
	//rateTab1.setAttribute("rateMaxBurst",rateMaxBurst->text());
	rateTab1.setAttribute("rateBurst",rateBurst->text());	

	if (newName!=oldName) //chech if the item name was changed
				{
				searchedItem=flowList->findItem(oldName,0);
				if (searchedItem!=0) {
				flowList->takeItem(searchedItem);

				QListViewItem * newItem = new QListViewItem(flowList,newName);

				newItem->setSelected(TRUE);
				flowList->setCurrentItem(newItem);
				oldName=newItem->text(0);

					}
				} 
//finish saving the settings


	QString nsAddr = nsAddress->text();
	QString nsUsr = nsUser->text();
	QString nsPass = nsStationObj->getManagementPassword();
	QString nsPortValue = nsPort->text();	
	QString nsProtocol;

	if (nsHostObj.isHostIPv4()) {nsProtocol="IPv4";}
	else {nsProtocol="IPv6";}

	QString xmlMessage = "<data> <address>"+nsAddr+"</address> <nsUsername>"+nsUsr+"</nsUsername> <nsPassword>"+nsPass+"</nsPassword> <nsPort>"+nsPortValue+"</nsPort> <nsProtocol>"+nsProtocol+"</nsProtocol>"; 	
	xmlMessage.append("<rootTc>");//sending the interfaces for witch we have settings
	setNumber=flowList->childCount();
	interfNumber=destInterface->count();

	node = settings.firstChild();

	for (index=0;index<=interfNumber;index++)
	{
	interfTable[index]=FALSE;
	}

	index=1;
	exit=FALSE;
	while((index<=setNumber)&(exit==FALSE))//searching for the used interfaces
	{
	source=node.firstChild();
	destination=source.nextSibling();

	value=destination.toElement().attribute("destInterface");
	val=value.toInt(&ok, 10);
	
	if (val==0) {  //if we have a setting with the output interface Any
			for (i=0;i<interfNumber;i++)
			{interfTable[i]=TRUE;} //we consider used all the interfaces
			exit=TRUE;
			}
	else {interfTable[val]=TRUE;}
	
	node = node.nextSibling();//going to the next element
	index++;
	}

	i=0;
	for (index=1;index<interfNumber;index++)
	{
	if (interfTable[index]) 
		{
		i++;
		tag.sprintf("interface%d",i);
		xmlMessage.append("<"+tag+">");
		xmlMessage.append(destInterface->text(index));
		xmlMessage.append("</"+tag+">");

		tag.sprintf("clean%d",i);
		xmlMessage.append("<"+tag+">");
		if (cleanInterface[index]==TRUE) {
				xmlMessage.append("YES");
				}
		else { xmlMessage.append("NO"); }
		xmlMessage.append("</"+tag+">");
		}
	}

	tag.sprintf("%d",i);
	xmlMessage.append("<interfaceNr>"+tag+"</interfaceNr>");

	xmlMessage.append("</rootTc>");

	xmlMessage.append("</data>");//ending the data tree

	if (sender->sendCommand(Start,xmlMessage))//sending the interfaces data
	{
	cout<<"****************************************************************"<<endl;
	}
	else  { 
		connectButton->setEnabled(1);
		srcInterface->setEnabled(0);
		pixmapLabel4->setEnabled(0);
		destInterface->setEnabled(0);
		applyButton->setEnabled(0);
		saveButton->setEnabled(0);
		loadButton->setEnabled(0);
		srcInterface->clear();
		srcInterface->insertItem("Any",-1);
		destInterface->clear();
		destInterface->insertItem("Any",-1);
		MessageBar::longMessage("Setting not send\nThere is no connection with the server!");
		}
	

}


void NMGSimulatorWidget::nextSettingSlot(int indexSet)
{
bool ok;
int i, j, val, offset, mark, hMajor, netemIndex, protocolNumber;

QString  value, command, markString,sAddr, sMask, sPort, dAddr, dMask, dPort, l4Prot, iInterf, nextCommand, htbMajor, htbMinor, netemMajor, tag, result, responseDD;

QDomNode node, source, destination, protocols, delayTab, DDTab, rateTab;

cout<<"-----******* Sending command nr: "<<indexSet<<" *******------- "<<endl;

	//creating the xml message containing the settings
	QString nsAddr = nsAddress->text();
	QString nsUsr = nsUser->text();
	QString nsPass = nsStationObj->getManagementPassword();
	QString nsPortValue = nsPort->text();	
	QString nsProtocol;

	if (nsHostObj.isHostIPv4()) {nsProtocol="IPv4";}
	else {nsProtocol="IPv6";}

	QString xmlMessage = "<data> <address>"+nsAddr+"</address> <nsUsername>"+nsUsr+"</nsUsername> <nsPassword>"+nsPass+"</nsPassword> <nsPort>"+nsPortValue+"</nsPort> <nsProtocol>"+nsProtocol+"</nsProtocol>";

	node = settings.firstChild();
	for (i=1;i<indexSet;i++)//going to the setting on the position indexSet
	{
	node = node.nextSibling();//going to the next element
	}

	source=node.firstChild();
	destination=source.nextSibling();
	protocols=destination.nextSibling();
	delayTab=protocols.nextSibling();
	DDTab=delayTab.nextSibling();
	rateTab=DDTab.nextSibling();

	offset=1000;
	//*******************getting the iptables parameters
	sAddr=source.toElement().attribute("srcAddress");
	sMask=source.toElement().attribute("srcMask");
	sPort=source.toElement().attribute("srcPort");
	value=source.toElement().attribute("srcInterface");
	val=value.toInt(&ok, 10);
	iInterf=srcInterface->text(val);

	dAddr=destination.toElement().attribute("destAddress");
	dMask=destination.toElement().attribute("destMask");
	dPort=destination.toElement().attribute("destPort");

	value=protocols.toElement().attribute("l4Protocol");
	val=value.toInt(&ok, 10);
	l4Prot=l4Protocol->text(val);

	//Putting data in <iptablesBegin>
	command=" "; 
	xmlMessage.append("<iptablesBegin>");
	command.append("-s "+sAddr+"/"+sMask);
	command.append(" -d "+dAddr+"/"+dMask);
	if (iInterf!="Any") {
			command.append(" -i "+iInterf);
			}

	xmlMessage.append(command);
	xmlMessage.append("</iptablesBegin>");//ending the field <iptablesBegin>
	

	//Adding the protocols used
	protocolNumber=l4Protocol->count();
	if (l4Prot=="All") {
				for (i=1;i<protocolNumber;i++)
				{
				tag.sprintf("protocol%d",i);
				xmlMessage.append("<"+tag+">");
				command=" ";
				command.append("-p "+l4Protocol->text(i));
				if (l4Protocol->text(i)!="ICMP") 	{
							if (sPort!="0") {
								command.append(" --sport "+sPort);
								}

							if (dPort!="0") {
								command.append(" --dport "+dPort);
								}
							}
				xmlMessage.append(command);
				xmlMessage.append("</"+tag+">");
				}
				xmlMessage.append("<protocolNr>3</protocolNr>");
				}
	else 	{
		xmlMessage.append("<protocol1>");
		command=" ";
		command.append("-p "+l4Prot);
		if (l4Prot!="ICMP") 	{
				if (sPort!="0") {
					command.append(" --sport "+sPort);
						}

				if (dPort!="0") {
					command.append(" --dport "+dPort);
						}
					}
		xmlMessage.append(command);
		xmlMessage.append("</protocol1>");
		xmlMessage.append("<protocolNr>1</protocolNr>");
		}


	//Putting data in <iptablesEnd>
	command=" ";
	xmlMessage.append("<iptablesEnd>");
	
	mark=offset+indexSet;
	markString.sprintf("%d",mark);
	command.append("-j MARK --set-mark "+markString);

	xmlMessage.append(command);
	xmlMessage.append("</iptablesEnd>");//ending the field <iptablesEnd>
	//ENd of getting the iptables parameters*****************


	/***********Getting the htbChild parameters*******************/	
	xmlMessage.append("<htbChild>");

	hMajor=1;
	htbMajor.sprintf("%d",hMajor);
	htbMinor.sprintf("%d",indexSet);

	command=" ";
	command.append("parent "+htbMajor+": ");
	command.append("classid ");
	command.append(htbMajor+":"+htbMinor+" htb rate ");
	//setting the rate and the ceil and eventualy burst and cburst
	if (rateTab.toElement().attribute("rateRate")!="") {
		command.append(rateTab.toElement().attribute("rateRate"));
		command.append("kbit ");
		//command.append("ceil ");
		//command.append(rateTab.toElement().attribute("rateRate"));
		//command.append("kbit ");

		if (rateTab.toElement().attribute("rateBurst")!="")
			{
			command.append("burst ");
			command.append(rateTab.toElement().attribute("rateBurst"));
			command.append("b ");
			//command.append("cburst ");
			//command.append(rateTab.toElement().attribute("rateBurst"));
			//command.append("b ");
			}
	}
	else {command.append("0.01kbit ceil 10000mbps ");}//if nothing is speciffied, we put a setting that will not affect the transmission, and we ignore the burst and cburst settings
			
	xmlMessage.append(command);

	xmlMessage.append("</htbChild>");
	/*****************htbChild********************/
	/*********************************************/
	
	

	/*****************Netem Qdiscs*********************/
	//making the netem qdiscs, first the drop qdisc, then the duplicate qdisc, and then the delay qdisc. This is the order, but it can be missing any of them
	netemIndex=0;
	responseDD=DDTab.toElement().attribute("radioDD");
	if (responseDD!="none") //if we have drop or duplicate
		{
		netemIndex++;
		tag.sprintf("netem%d",netemIndex);
		xmlMessage.append("<"+tag+">");
		command=" ";
		command.append("parent "+htbMajor+":"+htbMinor);

		netemMajor.sprintf("%d%d",indexSet,netemIndex);
		result=getDDSlot(indexSet, netemMajor);
	
		command.append(result);
		xmlMessage.append(command);
		xmlMessage.append("</"+tag+">");
			
			if (delayTab.toElement().attribute("delayDelay")!="")//if we have delay settigs
			{
			netemIndex++;
			tag.sprintf("netem%d",netemIndex);
			xmlMessage.append("<"+tag+">");
			command=" ";
			command.append("parent "+netemMajor+":1");

			netemMajor.sprintf("%d%d",indexSet,netemIndex);
			result=getDelaySlot(indexSet, netemMajor);
	
			command.append(result);
			xmlMessage.append(command);
			xmlMessage.append("</"+tag+">");
			
			}
			
		}
	else 	{//if we don't have drop or duplicate
		if (delayTab.toElement().attribute("delayDelay")!="")//if we have delay settigs
			{
			netemIndex++;
			tag.sprintf("netem%d",netemIndex);
			xmlMessage.append("<"+tag+">");
			command=" ";
			command.append("parent "+htbMajor+":"+htbMinor);

			netemMajor.sprintf("%d%d",indexSet,netemIndex);
			result=getDelaySlot(indexSet, netemMajor);
	
			command.append(result);
			xmlMessage.append(command);
			xmlMessage.append("</"+tag+">");
			
			}	
	
		}
	
	
	tag.sprintf("%d",netemIndex);
	xmlMessage.append("<netemNr>"+tag+"</netemNr>");

	/**************************************************/


	/*************************************************************/
	/*****************Putting the filter parrameters*************/
	xmlMessage.append("<filter>");	
	//tc filter add dev DEV...
	command=" ";
	command.append("parent "+htbMajor+":");
	command.append(" protocol ip prio 1 handle ");
	command.append(markString);
	command.append(" fw classid "+htbMajor+":"+htbMinor);
	
	xmlMessage.append(command);
	xmlMessage.append("</filter>");
	/*****************Ending the filter parrameters*************/

	/*************************************************************/


	/*********Sending the interfaces for witch this command is for******/
	xmlMessage.append("<interfaces>");
	j=0;
	value=destination.toElement().attribute("destInterface");
	val=value.toInt(&ok, 10);
	if (val==0) {
			j=destInterface->count();
			for (i=1;i<j;i++)
			{
			tag.sprintf("interf%d",i);
			xmlMessage.append("<"+tag+">");
			xmlMessage.append(destInterface->text(i));
			xmlMessage.append("</"+tag+">");
			}
			tag.sprintf("%d",(i-1));
			xmlMessage.append("<interfNr>"+tag+"</interfNr>");
		}
	else  {
		xmlMessage.append("<interf1>"+destInterface->text(val)+"</interf1>");
		xmlMessage.append("<interfNr>1</interfNr>");
		}

	xmlMessage.append("</interfaces>");
	/*************************Interfaces********************************/
	/*******************************************************************/


	nextCommand="";
	//sending the next command or NO if this was the last command
	i=indexSet+1;
	if (indexSet!=flowList->childCount()) { 
						nextCommand.sprintf("%d",i);
						}
	else { nextCommand="NO"; } 
	xmlMessage.append("<nextCommandAvailable>"+nextCommand+"</nextCommandAvailable>");

	xmlMessage.append("</data>");//ending the settings tree

	if (sender->sendCommand(Start,xmlMessage))//sending the interfaces data
	{
	cout<<"****************************************************************"<<endl;
	}
	else  { 
		connectButton->setEnabled(1);
		srcInterface->setEnabled(0);
		pixmapLabel4->setEnabled(0);
		destInterface->setEnabled(0);
		applyButton->setEnabled(0);
		saveButton->setEnabled(0);
		loadButton->setEnabled(0);
		srcInterface->clear();
		srcInterface->insertItem("Any",-1);
		destInterface->clear();
		destInterface->insertItem("Any",-1);
		MessageBar::longMessage("Setting not send\nThere is no connection with the server!");
		}

}


//getting datas from delay tab menu
QString NMGSimulatorWidget::getDelaySlot(int currentIndex, QString netemMajor)
{
bool ok;
int i, val;
QString command, value, response;
QDomNode node, source, destination, protocols, delayTab;

	node = settings.firstChild();
	for (i=1;i<currentIndex;i++)//going to the setting on the position currentIndex
	{
	node = node.nextSibling();//going to the next element
	}

	source=node.firstChild();
	destination=source.nextSibling();
	protocols=destination.nextSibling();
	delayTab=protocols.nextSibling();

	command="";
	command.append(" handle ");
	command.append(netemMajor);
	command.append(": netem delay ");
	command.append(delayTab.toElement().attribute("delayDelay")+"ms ");

	/*******If we have jitter**************/
	//if we have jitter
	if (delayTab.toElement().attribute("delayJitter")!="")
		{
		command.append(delayTab.toElement().attribute("delayJitter"));
		command.append("ms ");

		//if we have correlation or distribution value
		response=delayTab.toElement().attribute("radioDelay");
		if (response!="random")
			if (response=="correlation")
				{//if we have correlation
				command.append(delayTab.toElement().attribute("radioDelayValue"));
				command.append("% ");
				}
			else {//if we have distribution 
				command.append("distribution ");
				value=delayTab.toElement().attribute("radioDelayValue");
				val=value.toInt(&ok, 10);
				command.append(delayDistribution->text(val));
				}

		}
	/*************************************************************/

	return command; 
}


//getting datas from drop tab menu
QString NMGSimulatorWidget::getDDSlot(int currentIndex, QString netemMajor)
{
int i;
QString command, value;
QDomNode node, source, destination, protocols, delayTab, DDTab;

	node = settings.firstChild();
	for (i=1;i<currentIndex;i++)//going to the setting on the position currentIndex
	{
	node = node.nextSibling();//going to the next element
	}

	source=node.firstChild();
	destination=source.nextSibling();
	protocols=destination.nextSibling();
	delayTab=protocols.nextSibling();
	DDTab=delayTab.nextSibling();

	command="";
	command.append(" handle ");
	command.append(netemMajor);
	command.append(": netem "+DDTab.toElement().attribute("radioDD")+" ");
	command.append(DDTab.toElement().attribute("DD")+"% ");
			
	/******************If we have correlation**********************/
	//if we have correlation value
	if (DDTab.toElement().attribute("DDCorrelation")!="0")
		{
		command.append(DDTab.toElement().attribute("DDCorrelation"));
		command.append("% ");
		}
	/***************************************************************/
	
	
	return command;
}


void NMGSimulatorWidget::delayGroupChanged()
{
delayCorrelation->setEnabled(0);
delayDistribution->setEnabled(0);

if (dRadio1->isOn()) { return; }
else if (dRadio2->isOn()) {
				delayCorrelation->setEnabled(1);
				return;
				}
	else if (dRadio3->isOn()) {
				delayDistribution->setEnabled(1);
				return;
				}

}


void NMGSimulatorWidget::delayGroupChanged(QString value)
{

delayCorrelation->setEnabled(0);
delayDistribution->setEnabled(0);
if (value=="random") { 
			dRadio1->setChecked(TRUE);
			return; 
			}
else if (value=="correlation") {
				dRadio2->setChecked(TRUE);
				delayCorrelation->setEnabled(1);
				return;
				}
	else if (value=="distribution") {
				dRadio3->setChecked(TRUE);
				delayDistribution->setEnabled(1);
				return;
				}

}



void NMGSimulatorWidget::dDGroupChanged()
{
DD->setEnabled(0);
DDCorrelation->setEnabled(0);
if (DDR1->isOn()) { 
			DDAction->setText("None");
			return; 
			}
else if (DDR2->isOn()) {
				DD->setEnabled(1);
				DDCorrelation->setEnabled(1);
				DDAction->setText("Drop");
				return;
				}
	else if (DDR3->isOn()) {
				DD->setEnabled(1);
				DDCorrelation->setEnabled(1);
				DDAction->setText("Duplicate");
				return;
				}
}


void NMGSimulatorWidget::dDGroupChanged(QString value)
{
DD->setEnabled(0);
DDCorrelation->setEnabled(0);

if (value=="none") { 	
			DDR1->setChecked(TRUE);
			DDAction->setText("None");
			return; 
			}
else if (value=="drop") {
				DDR2->setChecked(TRUE);
				DD->setEnabled(1);
				DDCorrelation->setEnabled(1);
				DDAction->setText("Drop");
				return;
				}
	else if (value=="duplicate") {
				DDR3->setChecked(TRUE);
				DD->setEnabled(1);
				DDCorrelation->setEnabled(1);
				DDAction->setText("Duplicate");
				return;
				}
}


QString NMGSimulatorWidget::returnDelayGroup()
{
QString response;
if (dRadio1->isOn()) {
			response="random";
			}
else if (dRadio2->isOn()) {
			response="correlation";
			}

      else if (dRadio3->isOn()) {
				response="distribution";
				}
return response;
}


QString NMGSimulatorWidget::returnDDGroup()
{
QString response;
if (DDR1->isOn()) {
			response="none";
			}
else if (DDR2->isOn()) {
			response="drop";
			}

      else if (DDR3->isOn()) {
				response="duplicate";
				}
return response;

}


void NMGSimulatorWidget::l4ProtocolChanged()
{

srcPort->setEnabled(1);
destPort->setEnabled(1);

if (l4Protocol->text(l4Protocol->currentItem())=="ICMP") {
				srcPort->setValue(0);
				destPort->setValue(0);
				srcPort->setEnabled(0);
				destPort->setEnabled(0);
				}

}



void NMGSimulatorWidget::l4ProtocolChanged(QString value)
{
srcPort->setEnabled(1);
destPort->setEnabled(1);

if (value=="ICMP") {
				srcPort->setValue(0);
				destPort->setValue(0);
				srcPort->setEnabled(0);
				destPort->setEnabled(0);
				}
}


#ifdef CMAKE_BS
#include "nmgsimulatorwidget.moc"
#endif
