/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgsnooperwidget.h"
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qdom.h>

#define NAME_OBJECTLIST_COL 0

MemObject::MemObject( const QString &name, const QString &parent)
    :m_name(name), m_parent(parent)
{
}

NMGSnooperWidget::NMGSnooperWidget(QWidget *parent, NMGSnooper *snooper, const char *name)
    :NMGSnopperWidgetBase(parent, name),
m_snooper(snooper)
{
	stopButton->setEnabled(FALSE);
	objectlist->setRootIsDecorated(TRUE);
}

void NMGSnooperWidget::stopSnooper()
{
	m_snooper->sendCommand(Stop, "");	

	stopButton->setEnabled(FALSE);
	startButton->setEnabled(FALSE);

	m_message = new QMessageBox("Snooper", "Wait while Snooper is closing...", QMessageBox::Information, QMessageBox::NoButton, QMessageBox::NoButton, QMessageBox::NoButton, this);
	m_message->setModal(FALSE);
	m_message->show();
}


void NMGSnooperWidget::startSnooper()
{
	QDomDocument petition("snooper");
	QDomElement root = petition.createElement("data");
 	petition.appendChild( root );

	QDomElement tag;
	QDomText data;

	tag = petition.createElement("interval");
	data = petition.createTextNode(intervalSpin->text());
	tag.appendChild(data);
	root.appendChild(tag);

	m_snooper->sendCommand( Start, petition.toString() );

	stopButton->setEnabled(TRUE);
	startButton->setEnabled(FALSE);
}


void NMGSnooperWidget::resultReceived(QString data)
{
	QDomDocument doc;
	QDomElement element;
	QDomNode node;

	doc.setContent(data);
	element = doc.documentElement();
	node = element.firstChild();

	if(node.nodeName() == "ok") { //Snooper descarregat
		startButton->setEnabled(TRUE);
		m_message->close();
		delete m_message;
		return;
	}
}


void NMGSnooperWidget::statusReceived( QString data)
{
	QDomNode nodeparent;
	QDomDocument doc;
	doc.setContent( data);
	
	QDomElement element = doc.documentElement();

	QString name;
	QString parent;

	QPtrList<MemObject> list;
	list.setAutoDelete( FALSE);

	objectlist->clear();
		
	for( QDomNode n = element.firstChild(); !n.isNull(); n = n.nextSibling() )
	{
		if(n.isElement()) {
			if(n.nodeName() == "data")
				n = n.firstChild();

			if(n.nodeName() == "objects") 
				n = n.firstChild();

			if(n.nodeName() == "object") {
				nodeparent = n;
				n = n.firstChild();
				name = n.toElement().text();
				n = n.nextSibling();
				if(!n.isNull())
					parent = n.toElement().text();
				else
					parent = "";
				//Dibuixa l'item
				//showItems( name,parent);
				list.append( new MemObject(name, parent) );
				n = nodeparent;
			}
		}
	}
	showItems( list);
}


void NMGSnooperWidget::showItems( QPtrList<MemObject> list)
{
	MemObject *obj;
	QListViewItem *item;
	
	QStrList knows;
	knows.setAutoDelete( TRUE);
	
	while ( !list.isEmpty() ) {
		for (  obj = list.first(); obj; obj = list.next() ) {
			
			if( obj->parent().isEmpty() ) {  //Objecte sense pare...
				item = new QListViewItem(objectlist);
				item->setText( NAME_OBJECTLIST_COL, obj->name());
				item->setExpandable( TRUE);
				objectlist->setOpen(item, TRUE);
				objectlist->insertItem(item);
					
				knows.append( obj->name() );
				list.remove( obj);
			}	
			else { //Objecte amb pare...
				QString objparent;
				for (  objparent = knows.first(); objparent; objparent = knows.next() ) {
					if ( objparent == obj->parent() ) {  //Pare trobat :)
						QListViewItem *parentitem = objectlist->findItem( objparent, NAME_OBJECTLIST_COL);
						if( parentitem) {
							item = new QListViewItem( parentitem);
							item->setText( NAME_OBJECTLIST_COL, obj->name());
							parentitem->setExpandable( TRUE);
							objectlist->setOpen( parentitem, TRUE);
							parentitem->insertItem( item);

							knows.append( obj->name() );
							list.remove( obj);
							break;
						}
					}
				}
			}	
		}
	}
}

#include "nmgsnooperwidget.moc"
