/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qlistview.h>
#include <qtabwidget.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qaction.h>
#include <qdatetimeedit.h>
#include <qtextbrowser.h>
#include <qcombobox.h>

#include <qfile.h>
#include <qxml.h>

#include "nmgstatisticwidget.h"
#include <nmxml.h>
#include <nmstring.h>

#include <nmgpreferencesmanager.h>
#include <nmgmoduleutils.h>
#include <messagebar.h>

#include <iostream>

using namespace std;

/**
 * Contructor of the class
 * @param parent 
 * @param name 
 * @return 
 */
NMGStatisticWidget::NMGStatisticWidget(NMGModulePreferencesManager * prefMan, QWidget *parent, const char *name)
	: NMGStatisticWidgetBase(parent, name)

{
	progressItem = 0;
	toolBar = 0;
	preferencesManager = prefMan;

	createListMenu();
	
 	progressTimer = new QTimer();
 	connect(progressTimer,SIGNAL(timeout()),this,SLOT(updateProgress()));
}

/**
 * Destructor of the class
 * @return 
 */
NMGStatisticWidget::~NMGStatisticWidget()
{
	if(progressItem) progressItem->setComplete();
	progressItem = 0;
}

/**
 * Creates the flow list menu
 */
void NMGStatisticWidget::createListMenu()
{
	listMenu = new QPopupMenu();
	listMenu->insertItem("Load Test's &Results file to start statistical analisys",this,SLOT(loadTestFromDisk()),CTRL+Key_R);
	listMenu->insertItem("Load &Statistics file from disk",this,SLOT(loadStatisticsFromDisk()),CTRL+Key_S);
	listMenu->insertSeparator();
	
	QPopupMenu * exportListMenu = new QPopupMenu();
	exportListMenu->insertItem("Export To &XHTML",this,SLOT(exportStatisticsToXHTML()),CTRL+Key_X);
	exportListMenu->insertItem("Export To &TXT",this,SLOT(exportStatisticsToTXT()),CTRL+Key_T);
	listMenu->insertItem("Export Statistics To...",exportListMenu);
	
	QPopupMenu * extractListMenu = new QPopupMenu();
	extractListMenu->insertItem("Extract &OWD per sequence",this,SLOT(extractOtherStatisticsSN()),CTRL+Key_O);
	extractListMenu->insertItem("Extract OWD per TX&Delay",this,SLOT(extractOtherStatisticsTS()),CTRL+Key_D);
	listMenu->insertItem("Extract Other statistics from file",extractListMenu);
}

/**
 * Returns the toolbar of the statistic widget
 * @return 
 */
QToolBar * NMGStatisticWidget::getToolBar()
{
	if(!toolBar)
	{
		toolBar = new QToolBar(NMGModuleUtils::getMainWindow());
		QAction * action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("mgenicon.png")), tr("Load Test's Results File"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(loadTestFromDisk()));
		action->addTo(toolBar);
		
		action = new QAction(QIconSet(*NMGModuleUtils::getPixmap("statistic.png")), tr("Load Statistics File"),0,this);
		connect(action,SIGNAL(activated()),this,SLOT(loadStatisticsFromDisk()));
		action->addTo(toolBar);
	}
	return toolBar;
}


/**
 * Returns the op up menu of the widget
 * @return 
 */
QPopupMenu * NMGStatisticWidget::getPopupMenu()
{
	return listMenu;
}


/**
 * Slot which load a flow from disk
 */
void NMGStatisticWidget::loadTestFromDisk()
{
	QString fileName = QFileDialog::getOpenFileName(getLastTestPath(),"XML Flow Format (*.xff);;XML (*.xml);;All files (*.*)",0,0,"Select test file");

	if ( fileName and fileName!="" )
	{
		progressItem = ProgressManager::createProgressItem("Loading Test");
	
		saveLastTestPathFromFilePath ( fileName );
		
		int pathPos = fileName.findRev("/") + 1;
		
		QString fileNameResult = fileName.left(pathPos) + STATISTICS_FILE_PREFFIX 
			+ fileName.right( fileName.length() - pathPos );
		fileNameResult.replace( fileNameResult.length() - 4, 4, ".xsf" );
					
		if( QFile::exists( fileNameResult ) )
		{
			int reply = QMessageBox::question(0,"Overwirte the file?","The file "+fileNameResult+" exists.\n"+
					"Do you want to overwrite it?",
					QMessageBox::Yes|QMessageBox::Default,QMessageBox::No|QMessageBox::Escape);
			if(reply==QMessageBox::No) {
				loadCancelled(progressItem);	
				return;
			}
		}

		interpretThread = new NMGStatisticThread();
		interpretThread->loadFileName( fileName );
		interpretThread->activateResultFile(TRUE);
		interpretThread->start();
 		progressItem->setTotalItems(100);
 		progressItem->setStatus("Loading data...");
	
		connect(progressItem, SIGNAL(progressItemCanceled( ProgressItem* )), this, SLOT( loadCancelled( ProgressItem * ) ) );
		connect(progressItem, SIGNAL(progressItemCompleted( ProgressItem* )), this, SLOT( loadFinished( ProgressItem * ) ) );
		progressTimer->start(1000);
	}
	else
	{
// 		loadCancelled(progressItem);
	}
}


/**
 * Slot executed when the user clicks the start button to start a new test
 */
void NMGStatisticWidget::loadStatisticsFromDisk()
{
	QString fileName = QFileDialog::getOpenFileName(getLastTestPath(true),"XML Statistic Format (*.xsf);;QML Statistic Results (*.qml);;XML (*.xml);;All files (*.*)",0,0,"Select Statistic file");
	
	if ( fileName and fileName!="" )
	{
		saveLastTestPathFromFilePath ( fileName, true );
		
		QString fileNameResult = fileName;
	
		txtBrowser->mimeSourceFactory()->setExtensionType("qml", "text/utf8");

		if ( fileName.right(3) != "qml" ) {

			int pathPos = fileName.findRev("/") + 1;

			fileNameResult = QString("/tmp/") + fileName.right( fileName.length() - pathPos );

			fileNameResult.replace( fileNameResult.length() - 4, 4, ".qml" );
			
			QString xsltPath = QString(XML_DIR) + QString(QMLXSLT);
			
			xsltConversion( fileName, xsltPath, fileNameResult );
		}
		txtBrowser->setSource( fileNameResult );
			
	}
}


void NMGStatisticWidget::exportStatisticsToXHTML()
{
	QString fileName = QFileDialog::getOpenFileName(getLastTestPath(true),"XML Statistic Format (*.xsf);;XML (*.xml);;All files (*.*)",0,0,"Select Statistic file");
	
	if ( fileName and fileName!="" )
	{
		saveLastTestPathFromFilePath ( fileName, true );
		int pathPos = fileName.findRev("/") + 1;
		QString fileNameResult = fileName.left(pathPos) + QString(EXPORT_XHTML_FILE) 
						+ fileName.right( fileName.length() - pathPos );
		fileNameResult.replace( fileNameResult.length() - 4, 4, ".html" );
		QString xsltPath = QString(XML_DIR) + QString(XHTMLXSLT);
		xsltConversion( fileName, xsltPath, fileNameResult );
		MessageBar::longMessage("File Exported Successfully to" + fileNameResult);
	}
}


void NMGStatisticWidget::exportStatisticsToTXT()
{
	QString fileName = QFileDialog::getOpenFileName(getLastTestPath(true),"XML Statistic Format (*.xsf);;XML (*.xml);;All files (*.*)",0,0,"Select Statistic file");
	
	if ( fileName and fileName!="" )
	{
		saveLastTestPathFromFilePath ( fileName, true );
		int pathPos = fileName.findRev("/") + 1;
		QString fileNameResult = fileName.left(pathPos) + QString(EXPORT_TXT_FILE) 
						+ fileName.right( fileName.length() - pathPos );
		fileNameResult.replace( fileNameResult.length() - 4, 4, ".txt" );
		QString xsltPath = QString(XML_DIR) + QString(TXTXSLT);
		xsltConversion( fileName, xsltPath, fileNameResult );
		MessageBar::longMessage("File Exported Successfully to" + fileNameResult);
	}
}


void NMGStatisticWidget::extractOtherStatisticsSN()
{
	QString fileName = QFileDialog::getOpenFileName(getLastTestPath(),"XML Flow Format (*.xff);;XML (*.xml);;All files (*.*)",0,0,"Select Test file");
	
	if ( fileName and fileName!="" )
	{
		saveLastTestPathFromFilePath ( fileName );
		int pathPos = fileName.findRev("/") + 1;
		QString fileNameResult = fileName.left(pathPos) + QString(EXTRACT_SN_FILE) 
						+ fileName.right( fileName.length() - pathPos );
		fileNameResult.replace( fileNameResult.length() - 4, 4, ".txt" );
		QString xsltPath = QString(XML_DIR) + QString(SNXSLT);
		xsltConversion( fileName, xsltPath, fileNameResult );
		MessageBar::longMessage("File Exported Successfully to" + fileNameResult);
	}
}


void NMGStatisticWidget::extractOtherStatisticsTS()
{
	QString fileName = QFileDialog::getOpenFileName(getLastTestPath(),"XML Flow Format (*.xff);;XML (*.xml);;All files (*.*)",0,0,"Select Test file");
	
	if ( fileName and fileName!="" )
	{
		saveLastTestPathFromFilePath ( fileName );
		int pathPos = fileName.findRev("/") + 1;
		QString fileNameResult = fileName.left(pathPos) + QString(EXTRACT_TS_FILE) + fileName.right( fileName.length() - pathPos );
		fileNameResult.replace( fileNameResult.length() - 4, 4, ".txt" );
		QString xsltPath = QString(XML_DIR) + QString(TSXSLT);
		xsltConversion( fileName, xsltPath, fileNameResult );
		MessageBar::longMessage("File Exported Successfully to" + fileNameResult);
	}}


void NMGStatisticWidget::xsltConversion( QString & fileName, QString & xsltPath, QString & fileNameResult ) {
	QFile file ( fileName );
	if ( !file.open ( IO_ReadOnly ) )
	{
		QMessageBox::critical ( 0,"Error opening file",
					"Error while reading the document from disk.\n"+file.errorString() );
		cerr<<"Error while reading the document from disk.\n"<<file.errorString() <<endl;
	}
	else
	{	
		NMXML::applyXsltToDoc( fileName.latin1(), xsltPath.latin1(), fileNameResult.latin1());
	}
}

/**
 * Test cancelled using the progress bar widget
 * @param item 
 */
void NMGStatisticWidget::loadCancelled( ProgressItem * )
{
	if (interpretThread)
		interpretThread->kill();
	progressItem->setComplete();
	progressItem = 0;
	MessageBar::longMessage("Load cancelled");
}

/**
 * Slot for reporting the finishing of the data load
 */
void NMGStatisticWidget::loadFinished(ProgressItem *) 
{
	progressItem = 0;
 	if (interpretThread->isFinished()) {
		MessageBar::longMessage("Load finished successfully");
		updateWidgetBecauseOfTestEnd();
	}
}


/**
 * Updates the widgets becuse the end has ended
 */
void NMGStatisticWidget::updateWidgetBecauseOfTestEnd()
{
// 	if(interpretThread) interpretThread-> setComplete();
	progressItem = 0;
	txtBrowser->mimeSourceFactory()->setExtensionType("qml", "text/utf8");
	txtBrowser->setSource( interpretThread->getFileNameResultXslt() );
}


/**
 * Returns the last test path (or "" if undefined)
 * @return 
 */
QString NMGStatisticWidget::getLastTestPath(bool isStatisticPath)
{
	QString lastTestPath;
	if ( !isStatisticPath ) lastTestPath = preferencesManager->getValue(ID_PREFERENCE_LAST_TEST_PATH);
	else lastTestPath = preferencesManager->getValue(ID_PREFERENCE_LAST_STATISTIC_PATH);
	if(!lastTestPath or lastTestPath=="") lastTestPath = "";
	return lastTestPath;
}

/**
 * Saves the last test path from the filename path of the test
 * @param filePath 
 */
void NMGStatisticWidget::saveLastTestPathFromFilePath(const QString & filePath, bool isStatisticPath)
{
	if(filePath and filePath!="")
	{
		int lastPos = filePath.findRev("/");
		if ( !isStatisticPath )
			preferencesManager->addValue(ID_PREFERENCE_LAST_TEST_PATH,filePath.left(lastPos));
		else
			preferencesManager->addValue(ID_PREFERENCE_LAST_STATISTIC_PATH,filePath.left(lastPos));

	}
}


void NMGStatisticWidget::updateProgress()
{
	int percent = interpretThread->getPercentCompleted();
	QString mess = interpretThread->getErrorString();
	bool ended = interpretThread->isFinished();
	if (ended && progressItem) {
		progressTimer->stop();
		progressItem->setComplete();
		progressItem = 0;
		return;
	}
	if (progressItem) {
		progressItem->setStatus(mess);
		progressItem->setCompletedItems(percent);
		progressItem->updateProgress();
	}

	/*if (interpretThread->getError()!=0)
	{
		progressTimer->stop();
		m_gdl_dataLoader->processError(m_gdl_dataLoader->getError(), m_gdl_dataLoader->getErrorString());
		if (m_gdl_dataLoader->getError()==0) // loader relaunched -> start the timer again
			timer->start(100);
		else // loader aborted
		{
			progressItem->setComplete();
			progressItem = 0;
			MessageBar::longMessage("Load aborted");
			m_gdl_dataLoader->resetError();
			return;
		}
	}*/

	if (ended)
		updateWidgetBecauseOfTestEnd();	
}

#include "nmgstatisticwidget.moc"
