/***************************************************************************
 *   Copyright (C) 2007 by Roberto Barreda                                 *
 *   rbarreda@ac.upc.edu                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef NMGSTATISTICXMLINTERPRET_H
#define NMGSTATISTICXMLINTERPRET_H

#include <qstring.h>
#include <qxml.h>
#include <qptrstack.h>
#include <qdatetime.h>
#include <math.h>
#include <qfile.h>
#include <qbitarray.h>
#include <xmlwriter.h>
#include <qfile.h>

#include "nmgstatisticflowmetadata.h"
#include "nmgstatisticmetadata.h"
#include "nmgstatisticflowdata.h"
#include "nmgstatisticdata.h"

#define MSECS		1000
#define NANOSECS	1000000
#define SEQLISTSIZE	1000000

class XmlWriter;

class QFile;

class QString;

class NMGStatisticXMLInterpret : public QXmlDefaultHandler
{
	public:
		NMGStatisticXMLInterpret ( );
		NMGStatisticXMLInterpret ( QString & );

		~NMGStatisticXMLInterpret();

		enum NMGStatisticAppType {MGENAPP, NETPERFAPP};

		bool startDocument();
		bool startElement ( const QString&, const QString&, const QString& ,const QXmlAttributes& );
		bool characters ( const QString & );
		bool endElement ( const QString&, const QString&, const QString& );
		bool endDocument();

		bool isFinished();
		//int getError();
		void stop();
		void deleteObjects();
		QString getErrorString();

		int getAppType();

		int getPercentCompleted();
		QString getStatus();
		NMGStatisticData* getStatisticsData();

	private:
		void makeCalc ();

		bool startParseMetadata ( const QString&, const QXmlAttributes& );
		bool startParseResult ( const QString&, const QXmlAttributes& );
		bool endParseResult ( const QString& qName );
		
		void createStatisticNode ();
		void createStatisticFlowData();

		long long sentPackets();
		long long duplicatePackets();
		long long lostPackets();

		double averageDelay();
		double minDelay();
		double maxDelay();
		double stDevDelay();
		double jitterDelay();
		double delay99Percentile();
		double delay95Percentile();
		double delay999Percentile();
		double delay50Percentile();

		double averageIPDV();
		double minIPDV();
		double maxIPDV();
		double stDevIPDV();
		double jitterIPDV();
		double ipdv99Percentile();
		double ipdv95Percentile();
		double ipdv999Percentile();
		double ipdv50Percentile();

		bool delayIsNegative();

		void cleanValues();

		QString resStatisticFilePath;
		QFile * resStatisticFile;
		XmlWriter * xw;

		int appType;

		QValueVector <long>  delay_list, ipdv_list;

		long long dupl, count, lost, nr_delays, nr_ipdv, oldSeq;
		long long currentSeq, oldFlow, currentFlow, numPacketsAllFlows, countAll;

		long qtx, qrx, txms, rxms, current_delay, old_delay;
		long min_ipdv, max_ipdv, min_delay, max_delay, avg_delay, avg_ipdv;
		double stDev_delay, stDev_ipdv, delay_jitter, ipdv_jitter;
		double delay_percentile50, delay_percentile95, delay_percentile99, delay_percentile999;
		double ipdv_percentile50, ipdv_percentile95, ipdv_percentile99, ipdv_percentile999;

		int size;

		bool negative_delay, inFlow, inSeq, inTXTime, inRXTime, inSize, inMetadata, inResult, omiteFile;

		QBitArray seqList;

		bool isAlreadyFinished;
		bool stopParsing;
		bool writeToFile;
		//int errorCode;
		QString errorMsg;
		QString statusMsg;
		QString currentFlowStr;

		NMGStatisticMetadata *metadata;
		NMGStatisticFlowMetadata *currentFlowMetadata;
		NMGStatisticData *data;

};

#endif
