/***************************************************************************
 *   Copyright (C) 2005 by Ernest Nieto                                    *
 *   ernest.nieto@gmail.com                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "nmgmodulesubscriptionlist.h"

#include "nmgmodulemanager.h"
#include "nmgmoduleinstancemanager.h"

/**
 * Constructor of the class
 * @return 
 */
NMGModuleSubscriptionList::NMGModuleSubscriptionList()
{
	setModuleId("");
}

/**
 * Contructor of the class
 * @param id Id of the module
 * @return 
 */
NMGModuleSubscriptionList::NMGModuleSubscriptionList(QString id)
{
	setModuleId(id);
}

/**
 * Destructor of the class
 * @return 
 */
NMGModuleSubscriptionList::~NMGModuleSubscriptionList()
{
}


/**
 * Returns the module id
 * @return 
 */
QString NMGModuleSubscriptionList::getModuleId()
{
	return moduleId;
}

/**
 * Sets the module id
 * @param name 
 */
void NMGModuleSubscriptionList::setModuleId(QString id)
{
	moduleId = id;
}

/**
 * Returns true if the module with id is in the subsciption
 * @param id 
 * @return 
 */
bool NMGModuleSubscriptionList::existsSubscription(QString id)
{
	for(QSubscriptionListIterator it = list.begin(); it!=list.end(); ++it)
		if(*it==id) return true;
	return false;
}

/**
 * Adds a subscription
 * @param id 
 */
void NMGModuleSubscriptionList::addSubscription(QString id)
{
	if(!existsSubscription(id)) list.append(id);
}

/**
 * Removes the subscription (if existed) of a module
 * @param id 
 */
void NMGModuleSubscriptionList::removeSubscription(QString id)
{
	for(QSubscriptionListIterator it = list.begin(); it!=list.end(); ++it)
		if(*it==id) list.remove(it);
}

/**
 * Notifies the subscriptors of a new command executed
 * @param commandType 
 */
void NMGModuleSubscriptionList::notifyExecutedCommand(NMGCommandType commandType)
{
	NMGModule * subscripted = NMGModuleManager::self()->getInstanceManager().getModule(getModuleId());
	if(!subscripted) return;
	QString subscriptedName = subscripted->getModuleName();
	
	for(QSubscriptionListIterator it = list.begin(); it!=list.end(); ++it)
	{
		QString id = *it;
		NMGModule * mod = NMGModuleManager::self()->getInstanceManager().getModule(id);
		if(mod) mod->commandReceived(subscriptedName,commandType);
	}
}

/**
 * Returns the begin iterator
 * @return 
 */
QSubscriptionListIterator NMGModuleSubscriptionList::getBeginIterator()
{
	return list.begin();
}

/**
 * Returns the end iterator
 * @return 
 */
QSubscriptionListIterator NMGModuleSubscriptionList::getEndIterator()
{
	return list.end();
}

